classes = classes or {}
GH = GH or {}

GH.assert = assert

GH.bit1 = 1
GH.bit2 = GH.bit1*2
GH.bit3 = GH.bit2*2
GH.bit4 = GH.bit3*2
GH.bit5 = GH.bit4*2
GH.bit6 = GH.bit5*2
GH.bit7 = GH.bit6*2
GH.bit8 = GH.bit7*2
GH.bit9 = GH.bit8*2
GH.bit10 = GH.bit9*2
GH.bit11 = GH.bit10*2
GH.bit12 = GH.bit11*2
GH.bit13 = GH.bit12*2
GH.bit14 = GH.bit13*2
GH.bit15 = GH.bit14*2
GH.bit16 = GH.bit15*2
GH.bit17 = GH.bit16*2
GH.bit18 = GH.bit17*2
GH.bit19 = GH.bit18*2
GH.bit20 = GH.bit19*2
GH.bit21 = GH.bit20*2
GH.bit22 = GH.bit21*2
GH.bit23 = GH.bit22*2
GH.bit24 = GH.bit23*2
GH.bit25 = GH.bit24*2
GH.bit26 = GH.bit25*2
GH.bit27 = GH.bit26*2
GH.bit28 = GH.bit27*2
GH.bit29 = GH.bit28*2
GH.bit30 = GH.bit29*2
GH.bit31 = GH.bit30*2
GH.bit32 = GH.bit31*2

--[[
This file will be run whenever a lua state is created. It's a good place to
initialize any functions you want available globally, as well as to override
default lua behaviors.
--]]

math.isInteger = function(test)
	return((test % 1)==0)
end

-- math.random doesn't really do what we want it to do (ie easily allow random
-- numbers in a floating point range) and will often do the wrong thing or
-- fail silently, so it gets removed. Keep a local reference to the function
-- to allow behavior of our choosing
math.__old_random = math.__old_random or math.random
math.random = nil

-- The GH random. math.random isn't reused in order to avoid confusion for
-- people reading the lua docs (ie it would work differently than documented
-- so make people call something else.)

-- Random function that allows two floating point numbers to be used as the
-- range. If rangeMax is not present, range is considered 1 to rangeMin

-- If both inputs are integers, return value will be a random integer,
-- otherwise, result will be a floating point number (which will be the
-- range mulitplied by the result of math.random())
math.Random = function(rangeMin, rangeMax)
	if rangeMax == nil then
		if rangeMin == nil then
			return math.__old_random()
		elseif (rangeMin >= 1) then
			rangeMin, rangeMax = 1, rangeMin
		else
			GH.assert(rangeMin >= 0, "The range must be >= 0")
			rangeMin, rangeMax = 0, rangeMin
		end
	elseif rangeMin > rangeMax then
		rangeMax, rangeMin = rangeMin, rangeMax
	end

	if math.isInteger(rangeMin) and math.isInteger(rangeMax) then
		--GH.assert(rangeMin < rangeMax)
		--GH.assert(rangeMin >= 0)
		--GH.assert(rangeMax > 0)
		return math.__old_random(rangeMin, rangeMax)
	end

	local range = rangeMax - rangeMin
	local r = math.__old_random()
	return rangeMin + (r * range)
end

function randomFromWeightedTable(t)
	GH.assert(isTable(t))
	GH.assert(table.count(t) > 0)
	local shortNotation
	for _,v in pairs(t) do
		GH.assert(isTable(v) or isNumber(v))
		if (shortNotation == nil) then
			shortNotation = isNumber(v)
		else
			GH.assert(shortNotation == isNumber(v))
		end
		if (not shortNotation) then
			GH.assert(isNumber(v.weight))
		end
	end

	-- Calculate the sum of the weights
	local num = 0
	local sum = 0
	for _,v in pairs(t) do
		sum = sum + ((shortNotation and v) or v.weight)
		num = num + 1
	end
	GH.assert(num > 0)

	if (sum == 0) then
		local value
		num = math.Random(num)
		for k,v in pairs(t) do
			value = shortNotation and k or v
			num = num - 1
			if (num == 0) then
				break
			end
		end
		return value
	end
	GH.assert(sum > 0)

	-- Get a random index
	local randomValue = math.Random(sum)
	-- This is because the random start in 1 and finish in the same number than sum
	randomValue = randomValue - 1
	GH.assert(randomValue >= 0)
	GH.assert(randomValue < sum)

	-- Calculate the value
	local value
	local counter = 0
	for k,v in pairs(t) do
		value = shortNotation and k or v
		counter = counter + ((shortNotation and v) or v.weight)
		if (counter > randomValue) then
			break
		end
	end
	return value
end

function math.scalarEqual(s1, s2)
	return math.abs(s1 - s2) < 0.00001
end

math.arrayMod = function(v, m)
	if (not isNumber(m)) then
		m = #m
	end
	return ((v-1) % m) + 1
end

math.mod = function(v, m)
	--GH.assert(math.scalarEqual(3.1 % 3.2, 3.1))
	--GH.assert(math.scalarEqual(3.2 % 3.1, 0.1))
	--GH.assert(math.scalarEqual(-3.1 % 3.2, 0.1))
	--GH.assert(math.scalarEqual(-3.2 % 3.1, 3.0))
	--GH.assert(math.scalarEqual(-3.1 % -3.2, -3.1))
	--GH.assert(math.scalarEqual(-3.2 % -3.1, -0.1))
	--GH.assert(math.scalarEqual(3.1 % -3.2, -0.1))
	--GH.assert(math.scalarEqual(3.2 % -3.1, -3.0))
	return v % m
end

math.ramp = function(f, r)
	r = r or 0.5
	return (f < r and f/r) or (1-f < r and (1-f)/r) or 1
end

math.reduceSpeed = function(time, curSpeed, accDec)
    local acc = (accDec * time) / 1000
    if (curSpeed > 0) then
        if (acc < curSpeed) then
            return curSpeed - acc
        end
    elseif (acc < -curSpeed) then
        return curSpeed + acc
    end
    return 0
end

math.calcNewSpeedAndPos = function(time, curPos, targetPos, curSpeed, maxSpeed, accInc, accDec, allowMoveBack, onlyBreak)
	local breakDistance = curSpeed * curSpeed / (2 * accDec)
	local delta = targetPos - curPos

	if (math.abs(delta) < breakDistance or onlyBreak) then
        curSpeed = math.reduceSpeed(time, curSpeed, accDec)
	else
		local acc = (accInc * time) / 1000
		if (delta > 0) then
			curSpeed = curSpeed + acc
			if (curSpeed > maxSpeed) then
				curSpeed = maxSpeed
			end
		elseif (allowMoveBack ~= false) then
			curSpeed = curSpeed - acc
			if (curSpeed < -maxSpeed) then
				curSpeed = -maxSpeed
			end
		end
	end

	local onLeftSide = (curPos < targetPos)
	curPos = curPos + (curSpeed * time) / 1000
	if ((curPos < targetPos) ~= onLeftSide) then
		curPos = targetPos
		curSpeed = 0
	end

	return curPos, curSpeed
end

function math.sign(v)
	return (v >= 0 and 1) or -1
end

function math.sign0(v)
	return (v == 0 and 0) or (v > 0 and 1) or -1
end

function math.clamp(v, min, max)
	return (v <= min and min) or (v >= max and max) or v
end

function math.linear(fact)
	return fact
end

function math.scurve(fact)
	return fact * fact * (3 - fact - fact)
end

function math.jcurve(fact)
	return fact * fact
end

function math.rcurve(fact)
	return 1 - (fact - 1) * (fact - 1)
end

function math.lerp(fact, v1, v2)
	return v1 * (1.0 - fact) + v2 * fact
end

function math.inv_lerp(v, v1, v2)
	if (v2 == v1) then
		return 0
	end
	return (v - v1) / (v2 - v1)
end

function math.distance(p1, p2)
	local deltax = p2.x - p1.x
	local deltay = p2.y - p1.y
	return math.sqrt(deltax * deltax + deltay * deltay)
end

function math.distanceSquared(p1, p2)
	local deltax = p2.x - p1.x
	local deltay = p2.y - p1.y
	return deltax * deltax + deltay * deltay
end

function math.pathLength(poly)
	local length = 0
	for i=1,#poly-1 do
		poly[i].distance = poly[i].distance or math.distance(poly[i], poly[i+1])
		length = length + poly[i].distance
	end
	return length
end

function math.positionAlong(poly, travelled)
	for i=1,#poly-1 do
		local p1,p2 = poly[i], poly[i+1]
		local distance = p1.distance or math.distance(p1, p2)
		if (travelled <= distance) then
			local frac = travelled/distance
			local dx,dy = p2.x-p1.x, p2.y-p1.y
			return {x=p1.x + frac * dx, y=p1.y + frac * dy}
		end
		travelled = travelled - distance
	end
	return {x=poly[#poly].x, y=poly[#poly].y}
end

function math.smoothened(poly, smoothFactor, sharpEndPoints)
	GH.assert(smoothFactor ~= 0)
	local out = {}

	local splineX = {}
	local splineY = {}
	local x0, x1, y0, y1
	local startPoint = 1
	local endPoint = #poly

	-- Iterating the loop counter-clockwise
	for i = startPoint,endPoint-1 do
		-- i = second and j = first are the vertices between which the spline will be computed
		-- h = prev and k = next are resp. the vertices i-1 and j+1
		-- h and k are needed for calculating the direction vectors at i and j

		local prev = poly[math.max(i - 1, 1)]
		local first = poly[i]
		local second = poly[i + 1]
		local next = poly[math.min(i + 2, endPoint)]

		if (sharpEndPoints and i == startPoint) then
			-- If the ending-points of the points-array has to have sharp edges, then don't use directional values
			x0 = 0
			y0 = 0
		else
			-- Calculate tangent of vertex
			x0 = first.x - prev.x
			y0 = first.y - prev.y
		end

		if (sharpEndPoints and i+1 == endPoint) then
			-- If the ending-points of the points-array has to have sharp edges, then don't use directional values
			x1 = 0
			y1 = 0
		else
			-- Calculate tangent of vertex
			x1 = next.x - second.x
			y1 = next.y - second.y
		end

		--table.push_back(out, {x=first.x, y=first.y})

		local ax = 2 * second.x - 2 * first.x + x0 + x1
		local bx = -3 * second.x + 3 * first.x - 2 * x0 - x1
		local dx = second.x

		local ay = 2 * second.y - 2 * first.y + y0 + y1
		local by = -3 * second.y + 3 * first.y - 2 * y0 - y1
		local dy = second.y

		for i=0,smoothFactor-1 do
			local t = (smoothFactor-i)/smoothFactor
			local tt = t*t
			local ttt = tt*t
			table.push_back(out, {
				x=ax*ttt + bx*tt + x0*t + dx,
				y=ay*ttt + by*tt + y0*t + dy})
		end

		if (i+1 == endPoint) then
			table.push_back(out, {x=second.x, y=second.y})
		end
	end

	return out
end

function _iOS(x, y)
	if (iOS) then return x end
	return y
end

function table.insert_if_unique(t, item)
	if (not table.contains(t, item)) then
		table.insert(t, item)
		return true
	end
	return false
end

function table.push_back(t, item)
	table.insert(t, item)
	return item
end

function table.push_front(t, item)
	table.insert(t, 1, item)
	return item
end

function table.count(t)
	local num = 0
	for _,_ in pairs(t) do
		num = num + 1
	end
	return num
end

function table.empty(t)
	for _,_ in pairs(t) do
		return false
	end
	return true
end

function table.iempty(t)
	return #t == 0
end

function table.is_array(t)
	local num = #t
	for k,_ in pairs(t) do
		if (not isNumber(k) or k < 1 or k > num) then
			return false
		end
	end
	return true
end

function table.pop_back(t)
	local v = t[#t]
	table.remove(t, #t)
	return v
end

function table.pop_front(t)
	local v = t[1]
	table.remove(t, 1)
	return v
end

function table.randomize(t)
	local tmp = {}
	while (#t > 0) do
		local idx = math.Random(#t)
		--GH.assert(math.isInteger(idx))
		--GH.assert(idx >= 1)
		table.insert(tmp, t[idx])
		table.remove(t, idx)
	end
	for _,v in ipairs(tmp) do
		table.insert(t, v)
	end
end

function table.append_table(dest, src)
	for _,v in ipairs(src) do
		table.insert(dest, v)
	end
end

function table.copy(t, recursive, metatables)
	return table.copy_into({}, t, recursive, metatables)
end

function table.copy_into(destTable, srcTable, recursive, overwrite, metatables)
	if type(destTable) ~= "table" then
		destTable = {}
	end
	if type(srcTable) ~= "table" then
		return destTable
	end

	local recursiveDepth = 1
	if isNumber(recursive) then
		if recursive ~= 0 then
			recursiveDepth = recursive
		end
	elseif recursive then
		recursiveDepth = -1 -- infinite
	end

	overwrite = overwrite ~= false -- Default true

	local saved = {}
	saved[srcTable] = destTable

	local _recursiveCopy
	_recursiveCopy = function(dest, src, recDepth, metaTab)
		for k,v in pairs(src) do
			if isTable(v) and recDepth ~= 0 then
				if saved[v] == nil then
					local tmp = rawget(dest, k)
					if not isTable(tmp) then
						tmp = {}
					end
					saved[v] = tmp
					_recursiveCopy(tmp, v, recDepth-1)
				end
				rawset(dest, k, saved[v])
			elseif (overwrite or rawget(dest, k) == nil) then
				rawset(dest, k, v)
			end
		end
		if metaTab and getmetatable(t) then
			if metaTab == "set" or metaTab == true then
				setmetatable(dest, getmetatable(t))
			elseif (isNumber(metaTab)) then
				local meta
				_recursiveCopy(meta, getmetatable(t), metaTab, metaTab)
				setmetatable(dest, meta)
			end
		end
	end
	_recursiveCopy(destTable, srcTable, recursiveDepth-1, metatables)
	return destTable
end

function table.contains(t, val, recursive, metatables, keys)
	if val == nil then
		return false
	end

	local checked = {}
	local _findInTable
	_findInTable = function(t)
		local _checkValue
		_checkValue = function(v)
			if not checked[v] then
				if v == val then
					return true
				end
				if recursive and type(v) == "table" and _findInTable(v) then
					return true
				end
				if metatables and _checkValue(getmetatable(t)) then
					return true
				end
				checked[v] = true
			end
			return false
		end

		for k,v in pairs(t) do
			if _checkValue(v) then
				return true
			end
			if keys and _checkValue(k) then
				return true
			end
		end

		return false
	end

	if type(t) == "table" then
		return _findInTable(t)
	end
	return false
end

function table.index_of(t, val)
	for k,v in pairs(t) do
		if (v == val) then
			return k
		end
	end
end

function table.max_index(t)
	local max
	local value
	for k,v in pairs(a) do
		if (isNumber(k) and (not max or k > max)) then
			max = k
			value = v
		end
	end
	return max, value
end

function table.remove_item(t, val, all)
	local removed = 0
	for i,v in ipairs(t) do
		if (v == val) then
			table.remove(t, i)
			if not all then return 1 end
			removed = removed + 1
		end
	end
	return removed ~= 0 and removed
end

function string.starts_with(s, sub)
	return string.sub(s, 1, #sub) == sub
end

function string.ends_with(s, sub)
	return string.sub(s, -#sub) == sub
end

string.contains = string.find

function string.split(s, delim)
	local t = {}
	local pos = 1
	if string.find("", delim, 1) then
		return t
	end
	while true do
		local p = string.find(s, delim, pos)
		if p then
			if p > 1 then -- Skip first encounter, it's empty
				table.insert(t, string.sub(s, pos, p-1))
			end
			pos = p+1
		else -- Insert last split part
			table.insert(t, string.sub(s, pos))
			break
		end
	end
	return t
end

function isTable(v)
	return type(v) == "table"
end

function isString(v)
	return type(v) == "string"
end

function isNumber(v)
	return type(v) == "number"
end

function isBoolean(v)
	return type(v) == "boolean"
end

function isCallable(v)
	if v == nil then return false end
	if type(v) == "function" then
		return true
	else
		local meta = getmetatable(v)
		if isTable(meta) and isCallable(rawget(meta, "__call")) then
			return true
		end
	end
	return false
end

function deriveLuaClass(className, superName)
	GH.assert(className ~= superName, className.." == "..superName)
	_G.classes[className] = _G.classes[className] or {}
	return setupMetatable(className, superName)
end

function getClass(className)
	return _G.classes[className]
end

function setupMetatable(className, superName)
	if _G.classes == nil then
		_G.classes = setmetatable({}, {__mode = "k"})
	end

	if not isString(className) then
		error("You need to provide a class name")
		return
	end

	if superName ~= nil and type(superName) ~= "string" then
		error("You need to provide a super class via a string")
		return
	end

	local base = _G.classes[className]
	if base == nil then
		error("This class does not exist yet: " .. className)
		return
	elseif type(base) ~= "table" then
		error("A non-table was stored... WTF?: " .. className)
		return
	end

	local super
	if superName ~= nil then
		super = _G.classes[superName]
		if type(super) ~= "table" then
			error("Could not find the 'super' table: " .. superName .. " (base: " .. className .. ")")
			return
		end
	end

	local link_rawget = function(base, super, idx)
		local super_rawget
		local base_rawget
		if super ~= nil then
			super_rawget = rawget(super, "__index_funcs")[idx]
			if super_rawget ~= nil then
				base_rawget = function(t, k)
					local v = rawget(base, k)
					if v ~= nil then
						return v
					end
					return super_rawget(t, k)
				 end
			end
		end
		if base_rawget == nil then
			base_rawget = function(t, k)
				return rawget(base, k)
			end
		end
		rawget(base, "__index_funcs")[idx] = base_rawget
		return base_rawget
	end

	local link_getters = function(base, super, idx)
		local getters = rawget(base, "__getters") or {}
		rawset(base, "__getters", getters)

		local base_get
		local super_get
		if super ~= nil then
			super_get = rawget(super, "__index_funcs")[idx]
		end
		if super_get ~= nil then
			base_get = function(t, k)
				local f = getters[k]
				if f ~= nil then
					return f(t)
				end
				return super_get(t, k)
			 end
		else
			base_get = function(t, k)
				local f = getters[k]
				if f ~= nil then
					return f(t)
				end
				return nil
			 end
		end
		rawget(base, "__index_funcs")[idx] = base_get
		return base_get
	end

	local link_setters = function(base, super, idx)
		local setters = rawget(base, "__setters") or {}
		rawset(base, "__setters", setters)

		local base_set
		local super_set
		if super ~= nil then
			super_set = rawget(super, "__newindex_funcs")[idx]
		end
		if super_set ~= nil then
			base_set = function(t, k, v)
				local f = setters[k]
				if f ~= nil then
					f(t, v)
					return true
				end
				return super_set(t, k, v)
			 end
		else
			base_set = function(t, k, v)
				local f = setters[k]
				if f ~= nil then
					f(t, v)
					return true
				end
				return false
			 end
		end
		rawget(base, "__newindex_funcs")[idx] = base_set
		return base_set
	end

	rawset(base, "className", className)
	rawset(base, "class", base)
	rawset(base, "super", super)
	setmetatable(base, { __index = super })
	if super ~= nil then
		rawset(base, "superInit", function(self, ...)
			local f = super.init
			if isCallable(f) then
				f(self, ...)
			end
		end)

		rawset(base, "classInit", function(self, ...)
			local f = self.class.init
			if isCallable(f) then
				f(self, ...)
			end
		end)

		if not rawget(base, "__tostring") then
			rawset(base, "__tostring", function(self)
				if super.__tostring then
					return super.__tostring(self)
				else
					return type(self)
				end
			end)
		end
	else
		local f = function() end
		rawset(base, "superInit", f)
		rawset(base, "classInit", f)
	end

	rawset(base, "new", function(...)
		local inst = setmetatable({ class = base }, base)
		local f = inst.init
		if isCallable(f) then
			f(inst, ...)
		end
		return inst
	end)

	local index_funcs = rawget(base, "__index_funcs") or {}
	rawset(base, "__index_funcs", index_funcs)

	local newindex_funcs = rawget(base, "__newindex_funcs") or {}
	rawset(base, "__newindex_funcs", newindex_funcs)

	if super ~= nil then
		local super_index_funcs = rawget(super, "__index_funcs") or {}
		if super_index_funcs ~= nil then
			for i = 1, #super_index_funcs do
				index_funcs[i] = super_index_funcs[i]
			end
		end

		local super_newindex_funcs = rawget(super, "__newindex_funcs") or {}
		if super_newindex_funcs ~= nil then
			for i = 1, #super_newindex_funcs do
				newindex_funcs[i] = super_newindex_funcs[i]
			end
		end
	end

	link_rawget(base, super, 1)
	link_getters(base, super, 2)

	local base_index
	local i1 = index_funcs[1]
	local i2 = index_funcs[2]
	if #index_funcs == 0 then
		base_index = rawget
	elseif #index_funcs == 2 then -- most common
		base_index = function(t, k)
			local v = rawget(t, k)
			if v ~= nil then return v end
			v = i1(t, k)
			if v ~= nil then return v end
			return i2(t, k)
		end
	else
		base_index = function(t, k)
			local v = rawget(t, k)
			if v ~= nil then return v end
			for i = 1, #index_funcs do
				v = index_funcs[i](t, k)
				if v ~= nil then return v end
			end
		end
	end

	link_setters(base, super, 1)

	local base_newindex
	local i1 = newindex_funcs[1]
	if #newindex_funcs == 0 then
		base_newindex = rawset
	elseif #newindex_funcs == 1 then -- most common
		base_newindex = function(t, k, v)
			if rawget(t, k) == nil then
				if i1(t, k, v) then
					return
				end
			end
			rawset(t, k, v)
		end
	else
		base_newindex = function(t, k, v)
			if rawget(t, k) == nil then
				for i = 1, #newindex_funcs do
					if newindex_funcs[i](t, k, v) then
						return
					end
				end
			end
			rawset(t, k, v)
		end
	end

	rawset(base, "__index", base_index)
	rawset(base, "__newindex", base_newindex)

	local getters = rawget(base, "__getters")
	local setters = rawget(base, "__setters")

	rawset(base, "setAccessors", function(k, getter, setter)
		rawset(getters, k, getter)
		rawset(setters, k, setter)
	end)

	rawset(base, "setGetter", function(k, getter) rawset(getters, k, getter) end)
	rawset(base, "setSetter", function(k, setter) rawset(setters, k, setter) end)

	return base
end

function derivesFrom(object, className)
	return forEachMetatable(
		object,
		function(meta)
			if (meta["className"] == className) then
				return true
			end
		end
	)
end

function forEachMetatable(object, f)
	local meta = getmetatable(object) or object.class or object.super
	local done = {}
	while isTable(meta) and not done[meta] do
		done[meta] = true
		local r = f(meta)
		if r ~= nil then return r end
		meta = meta.super
	end
end

function getTopMostCppClassName(forClassName)
	if not isString(forClassName) then
		return ""
	end

	local cppClassNames = GH.getCppClassNames()
	local _getRecursive
	_getRecursive = function(className)
		if cppClassNames[className] ~= nil then
			return className
		end
		local classMeta = _G.classes[className]
		if type(classMeta) == "table" then
			if classMeta.super then
				return _getRecursive(classMeta.super.className)
			else
				error("Class metatable found without a 'super' field: "..classMeta.className.." while searching for "..className)
			end
		else
			error("Could not find parent class of "..className.." (for "..forClassName..")")
		end
		return ""
	end

	return _getRecursive(forClassName)
end

function tryParseAsTable(s, ...)
	if (isString(s)) then
		if (string.find(s, "{") ~= 1) then
			s = "{"..s.."}"
		end
	end
	return tryParse(s, ...)
end

function tryParse(s, ...)
	local f = loadstring("return "..tostring(s))
	if (isCallable(f)) then
		local success, r = pcall(f, ...)
		if (success) then
			return r
		end
	end
end

lua_loadfile = lua_loadfile or loadfile
loadfile = nil
function dofile(filename, mandatory)
	local f = GH.loadfile(filename, mandatory)
	return f and f()
end

function string.naturalCompare(str1, str2)
	local compareExpression = "^(%D*)(%d*)(.*)$"
	local chars1, num1
	local chars2, num2
	while (string.len(str1) > 0) or (string.len(str2) > 0) do
		-- Extract non-digits, digits and rest of string
		chars1, num1, str1 = string.match(str1, compareExpression)
		chars2, num2, str2 = string.match(str2, compareExpression)

		-- Compare the non-digits
		if (chars1 == chars2) then
			-- If either number begins with a zero, then compare alphabetically,
			-- otherwise compare numerically
			if (num1 ~= "") and (num2 ~= "") then
				num1, num2 = tonumber(num1), tonumber(num2)
			end

			if (num1 < num2) then
				return -1
			elseif (num1 > num2) then
				return 1
			end

		elseif (chars1 < chars2) then
			return -1
		else
			return 1
		end
	end

	return 0
end

-- Recursive function quick fix -> needs elegant solution!
function table.val_to_str(v, recDone)
	if (type(v) == "string") then
		return string.format("%q", v)
	elseif (recDone and recDone[v]) then
		return "<table>"
	end
	return type(v) == "table" and table.tostring(v, recDone) or tostring(v)
end

function table.key_to_str(k, recDone)
	if (type(k) == "string" and string.match(k, "^[_%a][_%a%d]*$")) then
		return k
	elseif (recDone[k]) then
		return "[<table>]"
	end
	return "["..table.val_to_str(k, recDone).."]"
end

function unpack10(t)
    if #t <= 4 then
		return t[1], t[2], t[3], t[4]
    elseif #t <= 10 then
		return t[1], t[2], t[3], t[4], t[5], t[6], t[7], t[8], t[9], t[10]
    end
    return unpack(t)
end

function table.tostring(t, recDone)
	if (not isTable(t)) then return tostring(t) end
	recDone = recDone or {}
	recDone[t] = true
	local result, done = {}, {}
	for k,v in ipairs(t) do
		if (not recDone[k]) then
			table.insert(result, table.val_to_str(v, recDone))
			done[k] = true
		end
	end
	for k,v in pairs(t) do
		if (not done[k] and not recDone[k]) then
			table.insert(result, table.key_to_str(k, recDone).."="..table.val_to_str(v, recDone))
		end
	end
	return "{"..table.concat(result, ",").."}"
end

function serialize(name, val, allowSimpleTablesFrom, endAppend)
	if (allowSimpleTablesFrom and allowSimpleTablesFrom < 0) then
		allowSimpleTablesFrom = nil
	end

	if (not lua_reserved_words) then
		-- Lua keywords must be quoted as keys, so we make a list of them.
		lua_reserved_words = {}
		for _,v in ipairs({
			"and", "break", "do", "else", "elseif", "end", "false",
			"for", "function", "if", "in", "local", "nil", "not", "or",
			"repeat", "return", "then", "true", "until", "while"
		}) do lua_reserved_words[v] = true end
	end

    -- These must be declared before use
    local _basicSerialize, _serializeKey, _serializeTableCompact
    local _isInlineTable, _internalSerializeTableCompact, _save

	_basicSerialize = function(o, debugKey)
		if (isNumber(o) or isBoolean(o)) then
			return tostring(o)
		elseif (isString(o)) then
			return string.format("%q", o)
		end
		return string.format("%q", tostring(o))
		--error("Trying to 'Basic serialize' a "..type(o).." key="..tostring(debugKey))
	end

	_serializeKey = function(k, dot)
		if (isString(k) and
			string.find(k, "^[_%a][_%a%d]*$") and
			not lua_reserved_words[k])
		then
			return dot .. k
		end
		if (isNumber(k) or isBoolean(k)) then
			return "[" .. tostring(k) .. "]"
		elseif (isString(k)) then
			return "[" .. string.format("%q", k) .. "]"
		end
		error("Trying to serialize a key that is a "..type(k))
	end

	_isInlineTable = function(t)
		local meta = getmetatable(t)
		if (meta and meta["__allowInlineSerialize"] == false) then return false end
		if (#t > 30) then return false end
		for k,v in pairs(t) do
			--if (isTable(v) or (isString(v) and #v > 10)) then return false end
			if (isTable(v) or isString(v)) then return false end
		end
		return true
	end

	local _naturalCompareFunctor = function(a, b)
		return string.naturalCompare(a, b) == -1
	end

	_internalSerializeTableCompact = function(v, out, saved, indentedName, exIndt, extraIndent)
		if (isTable(v)) then
			if (saved[v]) then
				if (saved[v] ~= "__illegal__") then
					table.insert(out, indentedName .. saved[v])
				else
					print("Error: value marked as 'illegal' ("..table.tostring(v)..")")
				end
			else
				saved[v] = "__illegal__"
				table.insert(out, indentedName .. _serializeTableCompact(v, saved, exIndt, extraIndent))
			end
		else
			table.insert(out, indentedName .. _basicSerialize(v, indentedName))
		end
	end

	_serializeTableCompact = function(value, saved, indent, extraIndent)
		if (table.empty(value)) then
			return "{}"
		end

		if (_isInlineTable(value)) then
			local out, numericDone = {}, {}
			for k,v in ipairs(value) do
				table.insert(out, _basicSerialize(v, k))
				numericDone[k] = true
			end
			local unsortedPairsTable = {}
			for k,v in pairs(value) do
				if (not numericDone[k]) then
					table.insert(unsortedPairsTable, _serializeKey(k, "") .. "=" .. _basicSerialize(v, k))
				end
			end
			table.sort(unsortedPairsTable, _naturalCompareFunctor)
			for _,v in ipairs(unsortedPairsTable) do
				table.insert(out, v)
			end
			return "{ " .. table.concat(out, ", ") .. ", }"
		end

		local exIndt = indent .. extraIndent
		local out, numericDone = {}, {}
		for k,v in ipairs(value) do
			_internalSerializeTableCompact(v, out, saved, exIndt, exIndt, extraIndent)
			numericDone[k] = true
		end
		local unsortedPairsTable = {}
		for k,v in pairs(value) do
			if (not numericDone[k]) then
				local indentedName = exIndt .. _serializeKey(k, "") .. " = "
				_internalSerializeTableCompact(v, unsortedPairsTable, saved, indentedName, exIndt, extraIndent)
			end
		end
		table.sort(unsortedPairsTable, _naturalCompareFunctor)
		for _,v in ipairs(unsortedPairsTable) do
			table.insert(out, v)
		end
		return "{\n" .. table.concat(out, ",\n") .. ",\n" .. indent .. "}"
	end

    _save = function(ccName, value, out, saved, level)
		local indentedName = (#ccName == 0) and "" or (ccName .. " = ")

		if (isTable(value)) then
			if (saved[value]) then
				if (saved[value] ~= "__illegal__") then
					table.insert(out, indentedName .. saved[value])
				else
					print("Error: value marked as 'illegal' ("..table.tostring(value)..")")
				end
			else
				saved[value] = ccName
				if (table.empty(value)) then
					table.insert(out, indentedName .. ccName .. " or {}")
				else
					local allowCompact
					local meta = getmetatable(value)
					meta = meta and meta["__allowCompactSerialize"]
					if (meta ~= nil) then
						allowCompact = meta
					else
						allowCompact = allowSimpleTablesFrom and level >= allowSimpleTablesFrom
					end

					if (allowCompact) then
						table.insert(out, indentedName .. _serializeTableCompact(value, saved, "", "\t"))
					else
						table.insert(out, indentedName .. ccName .. " or {}")
						for k,v in pairs(value) do
							_save(ccName .. _serializeKey(k, "."), v, out, saved, level+1)
						end
					end
				end
			end
		elseif (isNumber(value) or isBoolean(value) or isString(value)) then
			table.insert(out, indentedName .. _basicSerialize(value, indentedName))

		else
			--print("Skipping '" .. ccName .. "'. Cannot save a " .. type(value))
		end
	end

	val = val or _G[name] -- Default to "name" in global namespace
	if (val == nil) then
		print("Variable '" .. tostring(name) .. "' does not exist")
		return ""
	end
	GH.assert(type(name) == "string", "Argument to serialize should be the *name* of a variable")

	local out = {} -- Output to this table
	_save(name, val, out, {}, 1) -- Do serialization

	table.sort(out, _naturalCompareFunctor)
	endAppend = endAppend or (allowSimpleTablesFrom == 0 and "") or "\n"
	return table.concat(out, "\n") .. endAppend -- Turn into a string
end

function saveStringToFile(str, fileName)
    local err
    if isString(fileName) and #fileName > 0 then
        local outFile
		GH.create_file_path(fileName)
        outFile, err = io.open(fileName, "w")
        if not err then
        	outFile:write(string.char(0xef,0xbb,0xbf)) -- UTF8 BOM
            outFile:write(str)
            outFile:close()
        end
    end
    return err
end

function serializeTable(rootName, saveTable, fileName, allowSimpleTablesFrom)
	-- copyfile
	-- the file 'fileName' is copied into 'fileName_backup.dtt'
	-- and then the new player status is saved into 'fileName'
	-- thi way if 'fileName' gets corrupted we will still have the
	-- player status (almost up to date) in 'fileName_backup.dtt'
	if #fileName > 0 then
		local backupFileName = string.gsub(fileName, ".dtt", "_backup.dtt")

		infile, err = io.open(fileName, "r")
		if not err then
			instr = infile:read("*all")
			infile:close()

			GH.create_file_path(backupFileName)
			outfile = io.open(backupFileName, "w")
			outfile:write(instr)
			outfile:close()
		end

	end
	--copyfile
	local result = serialize(rootName, saveTable, allowSimpleTablesFrom) .. "\n"
	return result, saveStringToFile(result, fileName)
end

function serializeTableReturnStyle(saveTable, fileName)
    local result = "return " .. serialize("", saveTable, 0) .. "\n"
    return result, saveStringToFile(result, fileName)
end

table.serialize = serialize -- Also store in table.

function serializeToJSON(val, addWhitespace)
	if (val == nil) then
		return "null"
	end
	local _save
	_save = function(value, check, indent)
		if (isTable(value)) then
			if (check[value]) then
				error("Trying to save a recursive table into JSON. Impossibru!")
			end
			local newCheck = table.copy(check)
			newCheck[value] = true
			local first = true
			local newIndent = addWhitespace and "\n"..indent or ""
			local newIndentTab = addWhitespace and newIndent.."\t" or ""
			local commaSpace = addWhitespace and ", " or ","
			if (table.is_array(value)) then
				local tmp = "["
				for k,v in ipairs(value) do
					if (first) then
						first = false
					else
						tmp = tmp..commaSpace
					end
   					tmp = tmp..newIndentTab.._save(v, newCheck, indent.."\t")
				end
				return tmp..newIndent.."]"
			else
				local tmp = "{"
				local tableSpace = (addWhitespace and " ") or ""
				for k,v in pairs(value) do
					if (not isString(k)) then
						error("Cannot serialize a key of type '"..type(k).."' key: '"..tostring(k))
					end
					if (first) then
						first = false
					else
						tmp = tmp..commaSpace
					end

   					tmp = tmp..newIndentTab..string.format("%q", k)..":"..tableSpace.._save(v, newCheck, indent.."\t")
				end
				return tmp..newIndent.."}"
			end
		elseif (isNumber(value) or isBoolean(value)) then
			return tostring(value)
		elseif (isString(value)) then
			return string.format("%q", value)
		end
		--error("Cannot save value '"..tostring(value).."'. It's a '"..type(value).."'.")
	end
	return _save(val, {}, "")
end

function getInterpolatedEmbedValue(t, frac, ...)
	if (isNumber(t)) then
		return t
	end
	GH.assert(isTable(t))
	local min, max
	for k,_ in pairs(t) do
		GH.assert(isNumber(k))
		if (min == nil or k < min) then min = k end
		if (max == nil or k > max) then max = k end
	end
	GH.assert(isNumber(min))
	GH.assert(isNumber(max))
	frac = (isNumber(frac) and frac) or 0
	local index = min + (max - min) * frac
	local v = t[index]
	if (v) then
		return getInterpolatedEmbedValue(v, ...)
	end
	local lo, hi
	for k,v in pairs(t) do
		if (lo == nil or (k < lo and k > index)) then lo = k end
		if (hi == nil or (k < hi and k > index)) then hi = k end
	end
	return math.lerp(math.inv_lerp(index, lo, hi),
		getInterpolatedEmbedValue(t[hi], ...),
		getInterpolatedEmbedValue(t[lo], ...))
end

function getInterpolatedEmbedValueByValue(t, val, ...)
	if (isNumber(t)) then
		return t
	end
	GH.assert(isTable(t))
	val = (isNumber(val) and val) or 1
	local lo, hi
	for k,v in pairs(t) do
		GH.assert(isNumber(k))
		if (k == val) then return v end
		if (not lo or (k < lo and lo > val) or (k > lo and k <= val)) then lo = k end
		if (not hi or (k > hi and hi < val) or (k < hi and k >= val)) then hi = k end
	end
	GH.assert(isNumber(lo))
	return math.lerp(math.inv_lerp(val, lo, hi),
		getInterpolatedEmbedValueByValue(t[lo], ...),
		getInterpolatedEmbedValueByValue(t[hi], ...))
end
