-- ************************************ Character ************************************ --

Character = getClass("Character")



function Character:isWorkActive()

	return self.workActive ~= false -- Actually false, allow nil

end



--function Character:onActorWorkStart(task, success)

--	Character.super.onActorWorkStart(self, task, success)



--	local challengeManager = level.challengeManager

--	if (challengeManager and success) then

--		challengeManager:onWorkStart(task)

--	end

--end



-- ************************************ Delicious Character ************************************ --

DelCharacter = getClass("DelCharacter")



-- ************************************ Stationary Character ************************************ --

StationaryCharacter = getClass("StationaryCharacter")

function StationaryCharacter:onLevelInitialized(desc)

	StationaryCharacter.super.onLevelInitialized(self, desc)



	local wp = self.waitPosition

	if (wp.x == self.x and wp.y == self.y) then

		self:setStationaryAppearance(true)

	end

end



function StationaryCharacter:setStationaryAppearance(stationary)

	if (stationary) then

		self:setAppearance("stationary", "stationary")

	else

		self:removeAppearance("stationary")

	end

end



function StationaryCharacter:evaluateAtStationaryPosition()

	local wp = self.waitPosition

	local atWaitPos = (self.x == wp.x and self.y == wp.y)

	self:setStationaryAppearance(atWaitPos)

	return atWaitPos

end



function StationaryCharacter:backToWaitPosition()
	local wp = self.waitPosition
	if (self.wp and self.wp.x == wp.x and wp.y == wp.y) then
		self:setStationaryAppearance(true)
		return
	end
	if (self:evaluateAtStationaryPosition()) then
		return
	end
	self.wp = wp
	self.walkBackToStationaryPositionTask =
		self:addTask({class="WalkToTask", target=wp, playDefaultOnFinish=true})
	self:addTask({class="CallTask", func=function()
		self:evaluateAtStationaryPosition()

		-- If the character is an entertainer and has the function to show
		-- the tip, call it
		if (self.isEntertainer and self.doEntertainerTutorialTip) then
			self:doEntertainerTutorialTip();
		end

		if (self.arriveAtWaitPos) then
			local atWaitPos = (self.x == wp.x and self.y == wp.y)
			self:arriveAtWaitPos(wp, atWaitPos)
		end
		self.wp = nil
	end})
	if (level and level.onStationaryWalkBack) then
		level:onStationaryWalkBack(self, wp)
	end
end

function StationaryCharacter:isWorkActive()

	return self.stationaryActive and StationaryCharacter.super.isWorkActive(self)

end



-- ************************************ Cleaner Entertainer Restocker ************************************ --

CleanerEntertainerRestocker = deriveLuaClass("CleanerEntertainerRestocker", "StationaryCharacter")



function CleanerEntertainerRestocker:init(desc)

	CleanerEntertainerRestocker.superInit(self, desc)



	self.helperType = desc.helperType or "cleaner"

	self.enterFromEntrance = self.enterFromEntrance ~= false

	--self.helperTypeChoice = self.helperTypeChoice or level.layoutId.."_helper_type"

	--self.entertainHitMaskThreshold = self.entertainHitMaskThreshold or nil



	--self:onUpgradeChoice()



	--JORDI-----------------------

	--self:updateHelperType()

	------------------------------

end



--JORDI----------------------

function CleanerEntertainerRestocker:onPreInitializeLevel()

	self:updateHelperType()

end

-----------------------------



--function CleanerEntertainerRestocker:onUpgradeChoice()

--	local helperType = player:getUpgradeSelection(self.helperTypeChoice)

--	if (helperType and helperType ~= "" and helperType ~= self.helperType) then

--		self.helperType = helperType

--		self:updateHelperType()

--	end

--end



function CleanerEntertainerRestocker:updateHelperType()

	-- Prevent old assignments
	if (self.isCleaner) then level.cleaner = nil end
	if (self.isEntertainer) then level.entertainer = nil end


	self.isCleaner = self.helperType == "cleaner"
	self.isEntertainer = self.helperType == "entertainer"
	self.isRestocker = self.helperType == "restocker"

	if (self.isRestocker) then
		self.objects = self.objects or {}
		self.stockBelowRefill = self.stockBelowRefill or 1
		self.restockAnimation = self.restockAnimation or "work"
		self.restockFrameTime = self.restockFrameTime or -1
		self.restockWhenFinished = self.restockWhenFinished or "default"
		self.isRestocking = self.isRestocking or false
		self.isWalkingToStartPosition = true
		self.walkBackTask = nil
		self.isActive = false

		self.onTick = self.onTick or function(self, time)
			self:tick(time)
			if (not level.isPlaying) then
				return
			end
			self.doRestocking = false
			for _,obj in pairs(self.objects) do
				if (self.doForceRestockAll or (not self.forceRestockAll and not self.isRestocking and obj.stock <= self.stockBelowRefill and obj.stock < obj.maxStock)) then
					if (self.walkBackTask and not self.walkBackTask.__deleted) then
						taskSystem:cancelTask(self.walkBackTask)
						self.walkBackTask = nil
					end
					self.doRestocking = true
					self.isWalkingToStartPosition = false
					GH.assert(obj.restockInterval > 0)
					local restockPos = obj.restockPos or obj.workPos
					self:addTask({class="RestockTask", obj=obj, restockPos=restockPos, actor=self, duration=0})
				end
			end

			self.doForceRestockAll = false
			if (self.doRestocking) then
				self.doRestocking = false
				self.isRestocking = true
				self.isActive = true
				self:addTask({class="SetTask", isRestocking=false})
			end
			if (not self.isRestocking and not self.isWalkingToStartPosition) then
				self.walkBackTask = self:addTask({
					class="WalkToTask",
					target={x=self.waitPosition.x, y=self.waitPosition.y},
					playDefaultOnFinish=true,
					directionOnFinish="down"
				})
				self:addTask({class="SetTask", isActive=false})
				self.isWalkingToStartPosition = true
			end
		end
	elseif (self.isEntertainer) then
		self.clickable = true
--		self.introductionTextId = "UNLOCKED_ENTERTAINER"
		level.entertainer = self
	elseif (self.isCleaner) then
--		self.introductionTextId = "UNLOCKED_CLEANER"
		level.cleaner = self
	end
end



function CleanerEntertainerRestocker:allowWork(actor)

	if (self.isEntertainer) then

		return false

	else

		return CleanerEntertainerRestocker.super.allowWork(self, actor)

	end

end



function CleanerEntertainerRestocker:onClick()

	local ret = CleanerEntertainerRestocker.super.onClick(self)

	if (self.isEntertainer) then

		self.selected = not self.selected

		if (self.walkBackToStationaryPositionTask and not self.walkBackToStationaryPositionTask.__deleted) then

			taskSystem:cancelTask(self.walkBackToStationaryPositionTask)

			self.walkBackToStationaryPositionTask = nil

			self:resetStationaryTimer()

		end

		return true

	end

	return ret

end



function CleanerEntertainerRestocker:onTaskAssigned(task)

	self:setStationaryAppearance(false)

end



function CleanerEntertainerRestocker:onActorWorkStart(task, success)

	CleanerEntertainerRestocker.super.onActorWorkStart(self, task, success)

	self:setStationaryAppearance(false)

	--if (self.isEntertainer) then

	--	if (self.entertainHitMaskThreshold) then

	--		self.normalHitMaskThreshold = self.hitMaskThreshold

	--		self.hitMaskThreshold = self.entertainHitMaskThreshold

	--	end

	--end

	if (self.isRestocker) then

		return CleanerEntertainerRestocker.super.onActorWorkStart(self, task, success)

	end

end



function CleanerEntertainerRestocker:onActorWorkFinish(task)

	--if (self.isEntertainer) then

	--	if (self.normalHitMaskThreshold) then

	--		self.hitMaskThreshold = self.normalHitMaskThreshold

	--	end

	--end

	if (self.isRestocker) then

		return CleanerEntertainerRestocker.super.onActorWorkFinish(self, task)

	end

end



function CleanerEntertainerRestocker:status_onDisplayStockChange(obj)

	if (self.isRestocker) then

		self.objects[obj] = true

	end

end



-- ************************************ Customer ************************************ --

Customer = getClass("Customer")

function Customer:leaveRoom(idx, exit, didPay)
	taskSystem:cancelAllTasksFor(self)
	self:addTask({class="PlayDefaultAnimationWithMoodTask"})
	self:addTask({class="Task", duration=idx*500 + 300})
	if (isTable(exit)) then
		self:addTask({class="ExitTask", exit=exit})
	elseif (isNumber(exit)) then
		self:addTask({class="ExitTask", exitIdx=exit})
	else
		self:addTask({class="ExitTask", exitIdx=math.Random(#level:getRoom(0).exits) - 1})
	end
end

-- ************************************ Customer Group ************************************ --

CustomerGroup = getClass("CustomerGroup")



--function CustomerGroup:acceptsSpecialTreat(prodName)

--	return self.mood < 4

--end



-- ************************************ Emily & Partick Customer Group ************************************ --

--EmPatCustomerGroup = getClass("EmPatCustomerGroup")

--

--EmPatCustomerGroup.specialIdleAnimTimeoutMin = 7000

--EmPatCustomerGroup.specialIdleAnimTimeoutMax = 15000

--EmPatCustomerGroup.specialIdleAnimations = {

--	{a="idle1", f=150, c=1},

--	{a="idle2", f=150, c=1},

--	{a="idle3", f=150, c=1},

--}



--SlideGroup = getClass("SlideGroup")

--

--function SlideGroup:onSlideStart(queueStation)

--	local main = self.main

--	local tire = nil

--	for i=1, 3 do

--		local obj = level:getObject("pooltire"..i)

--		if (obj and obj.stock == 0 and not obj.taken) then

--			tire = obj

--			tire.taken = true

--			break

--		end

--	end

--	if (main and tire) then

--

--		local slidePositions = {{x=-108, y=1188}, {x=166, y=1261}, {x=200, y=1400}}

--		main:resetQueueLink()

--

--		taskSystem:cancelAllTasksFor(main)

--		main:addTask({class="PlayDefaultAnimationTask"})

--		main:addTask({class="WalkToTask", target=queueStation, playDefaultOnFinish=true, })

--		main:addTask({class="Task", duration=700, })

--		main:addTask({class="CallTask",

--			func=function()

--				local waterslide_bend = level:getSpriteExt("waterslide_bend")

--				if (waterslide_bend) then

--					waterslide_bend:animate({startFrame=0, frameTime=100}, {frame=0})

--					waterslide_bend:playSample("pool_slide:sound")

--				end

--			end, })

--		main:addTask({class="Task", duration=1300, })

--		main:addTask({class="CallTask",

--			func=function()

--				main.location = slidePositions[1]

--				main:setAppearance("slide", "slide", 0x100000)

--				main.oldWalkSpeed = main.walkSpeed

--				main.walkSpeed = main.walkSpeed * 5

--			end, })

--		main:addTask({class="WalkToTask", target=slidePositions[2], })

--		main:addTask({class="PlayDefaultAnimationTask"})

--		main:addTask({class="CallTask",

--			func=function()

--				main:removeAppearance("slide")

--				main.walkSpeed = main.oldWalkSpeed

--				main.location = slidePositions[3]

--				local splash = level:getSpriteExt("splash")

--				if (splash) then

--					--splash.visible = true

--					splash:animate({visible=true}, {startFrame=0, frameTime=100}, {frame=0}, {visible=false})

--				end

--				splash:playSample("splash:sound")

--			end, })

--		main:addTask({class="Task", duration=600, })

--		main:addTask({class="WalkToTask", target=tire.compoundPosition, playDefaultOnFinish=true, })

--		main:addTask({class="CallTask",

--			func=function()

--				tire:playSample("drop_off_rubber_band:sound")

--				tire.stock = tire.stock + 1

--				tire.taken = false

--			end, })

--		main:addTask({class="ExitTask"})

--	end

--end

--

--function SlideGroup:handleScore(score)

--	local main = self.main

--	if (main) then

--		local pos = main.floaterPosition or {x=0, y=0}

--		level:createFloater({

--			id = "slideGroupDone",

--			textId = "EVENT_SCORE_ONE_DONE",

--			x = pos.x, y = pos.y,

--			bonusScore = score,

--			bonus = {

--				textId = "EVENT_SCORE_ONE_DONE_FMT",

--			},

--		})

--	end

--end
