-- ************************************ Level ************************************ --
Level = getClass("Level")

function Level:fireTrigger(trigger, ...)

	local args = {...}
	args[1] = args[1] or {}
	local handled = false
	if (not handled and self.event) then
		handled = self.event:handleTrigger(trigger, unpack10(args))
	end
	if (not handled) then
		handled = taskSystem:handleTrigger(trigger, unpack10(args))
	end
	if (not handled) then
		handled = self:handleTrigger(trigger, unpack10(args))
	end
	return handled
end

function Level:onRemoveProductFromTray(tray, product)
	local handled, halt
	if (product and product.source and product.source.onRemoveProductFromTray) then
		--[[ PR --]] --pb("["..self.name.."] onRemoveProductFromTray: calling remove product from tray for "..product.name)
		handled, halt = product.source:onRemoveProductFromTray(tray, product)
		if (halt) then return handled end
	end
	if (self.event and self.event.onRemoveProductFromTray) then
		--[[ PR --]] --pb("["..self.name.."] onRemoveProductFromTray for event: "..self.event.name)
		handled, halt = self.event:onRemoveProductFromTray(tray, product)
		if (halt) then return handled end
	end
	return handled
end

function Level:onWorkStart(task)
	return false
end

function Level:onWorkFinish(task)
end

function Level:onAnimationStarted(anim)
end

function musicMoodString(str)
	local musicMoodStart = anim.musicMoodStart
	if (musicMoodStart == "-") then musicMoodStart = "" end
	anim.musicMoodEnd_original = self.musicMood
	if (isCallable(self.setMusicAndMood) and musicMoodStart) then
		--print("musicMoodStart: ".. musicMoodStart)
		self:setMusicAndMood({mood=musicMoodStart, quickFade=true})
	end

	if (self.selectedObject) then
		self.wasSelected = self.selectedObject.selected
		self.selectedObject.selected = false
	end
end

function Level:onAnimationFinished(anim)
	local musicMoodEnd = anim.musicMoodEnd
	if (musicMoodEnd == "-") then musicMoodEnd = "" end
	if (not musicMoodEnd) then
		musicMoodEnd = anim.musicMoodEnd_original
	end
	if (isCallable(self.setMusicAndMood)) then
		self:setMusicAndMood({mood=musicMoodEnd, quickFade=true})
	end

	if (self.selectedObject) then
		self.selectedObject.selected = self.wasSelected
	end

	anim.resetEmotionsOnFinish = anim.resetEmotionsOnFinish or "default"
	if (anim.id ~= "postShift" and anim.resetEmotionsOnFinish ~= "none") then
		local actors = { hero, cleaner, entertainer}
		if (anim.resetEmotionsOnFinish == "all") then
			actors = anim:getActors()
		elseif (anim.resetEmotionsOnFinish ~= "default") then
			actors = {level:resolveSprite(anim.resetEmotionsOnFinish)}
		end
		for _,v in ipairs(actors) do
			if (v) then
				local emo = v.emotion
				if (isString(emo) and emo ~= "") then
					v.emotion = ""
				end
			end
		end
	end
end

-- Try to find an value given a certain value
-- with the help of a parameters table
function Level:resolveActor(v, paramTable, allowParse)
	if (isTable(paramTable) and paramTable[v] ~= nil) then
		v = paramTable[v]

		-- Return the table itself
		-- instead of creating a character
		if (isTable(v)) then
			return v
		end
	end

	-- Check if we're dealing with a string
	if (isString(v)) then
		local resolved = level:resolveSprite(v)
		if (resolved) then return resolved end

		if (characterTypes and characterTypes[v]) then
			local character = self:resolveActor({traits=v})
			if (character) then return character end
		end

		if (allowParse ~= false) then
			-- Try to parse the string
			local f = loadstring("return "..v)
			if (isCallable(f)) then
				local success, r = pcall(f)
				if (success and r ~= nil) then
					local obj = self:resolveActor(r, paramTable, false)
					if (obj) then
						return obj
					end
					return r
				end
			end
		end
	elseif (isTable(v)) then
		local obj = v.name or v.traits
		if (isString(obj)) then
			obj = level:resolveSprite(obj)
		end
		if (isTable(obj)) then return obj end

		-- Return a newly created character
		-- (which is added to the level immediately)
		local character = level:createCharacter(v)
		if (isTestingAnimation and character.x <= -9999 and level.validPosition) then
			character.location = level.validPosition
		end
		character.createdInAnimation = true
		return character
	end

	return v
end

function Level:resolveSprite(v)
	if (v == "hero") then return hero end
	if (v == "game") then return level.gameLayer end
	if (v == "level") then return level end
	return self:getSpriteExt(v)
end

function Level:createGenericFloater(textId, x, y)
	return self:createFloater({id="generic", textId=textId, x=x, y=y, align="center"})
end

function Level:getMetaDifficultyValue(t)
	-- The original value of the 0.5 was player.difficultyFrac which value was
	-- difficulty / (difficultyCount - 1)
	return getInterpolatedEmbedValue(t, 0.5, player.metaDifficultyFrac)
end

--function Level:setUpgradeSelection(upgradeGroup, upgradeChoice)
--	if (player) then
--		player:setUpgradeSelection(upgradeGroup, upgradeChoice)
--	end
--	self.gameLayer:traverse(function(_, node)
--		if (node.onUpgradeChoice) then
--			node:onUpgradeChoice(upgradeGroup, upgradeChoice)
--		end
--	end)
--end

function Level:placeHintArrow(obj, importance)
--print("Level:placeHintArrow "..serialize("obj", obj or {"nil"}))
	if (isString(obj)) then
		obj = self:resolveSprite(obj)
	end
	if (isTable(obj)) then
		obj.needsHintArrow = function(me)
			return me.visible and me.clickable
		end
		obj.onAddWorkTaskOn = function(me, task)
			if (task.className == "WorkTask") then
				me.needsHintArrow = nil
				me.onAddWorkTaskOn = me.class.onAddWorkTaskOn
			end
		end
		if (importance) then
			obj.hintArrowImportance = importance
		end
	end
end

function Level:removeHintArrow(obj)
--print("Level:placeHintArrow "..serialize("obj", obj or {"nil"}))
	if (isString(obj)) then
		obj = self:resolveSprite(obj)
	end
	if (isTable(obj)) then
		if (obj.needsHintArrow) then
			obj.needsHintArrow = nil
			if (obj.class) then
				obj.onAddWorkTaskOn = obj.class.onAddWorkTaskOn
			end
		end
	end
end

function Level:getSpritesByExactType(typeName)
	local r = {}
	self:traverse(function(_, node)
		if (not node.obsolete and node.className == typeName) then
			table.push_back(r, node)
		end
	end)
	return r
end

function Level:createCharacter(desc)
	GH.assert(isTable(characterTypes))
	GH.assert(isTable(characterTypes.default))
	local classDesc = table.copy(characterTypes.default, true)
	local traits = characterTypes[desc.traits]
	table.copy_into(classDesc, traits, true)
	table.copy_into(classDesc, desc, true)

	local character = self:add(classDesc)
	if (character.onCreated) then
		character:onCreated()
	end
	return character
end


-- ************************************ Delicious Level ************************************ --
DelLevel = getClass("DelLevel")
Tray = getClass("Tray")

function DelLevel:init(desc)
	DelLevel.superInit(self, desc)
end

function DelLevel:preInitializeLevel()
	self:traverse(function(_, node)
		if (node.onPreInitializeLevel and node ~= self and not node.obsolete) then
			node:onPreInitializeLevel()
		end
	end)
end

function DelLevel:initializeLevel()

	self.closeDelay = self.closeDelay or 4000

	self.productSettings = self.productSettings or {}
	for k,v in pairs(self.productSettings) do
		local productInfo = table.copy(self.defaultPoductSettings, true)
		table.copy_into(productInfo, v, true)
		self.productSettings[k] = productInfo
	end
	local possibleProducts = self.possibleProducts
	for _,p in ipairs(possibleProducts) do
		local product = self.productSettings[p]
		if (not product) then
			product = table.copy(level.defaultPoductSettings)
			self.productSettings[p] = product
		end
	end

-- 	self:startGame()

	self.placeChances = self.placeChances or { table = 50, counter = 50 }
end

function DelLevel:createCustomerTriggerTraitsList()

    -- Get the configuration. The % of customer per level
    self.availableCustomerTraits = self.availableCustomerTraits or self.customerChances
    GH.assert(self.availableCustomerTraits, "Level does not have table 'availableCustomerTraits'.")

    -- Get the triggers configuration. This is define in rooms inside the levels folder
 	local triggerList = self.triggers or (self.trigger and self.trigger.triggers)
	local numCustomerGroupTriggers = 0

    -- Get the amount of spawn lines
    if (triggerList) then
		for _,t in ipairs(triggerList) do
			if (t.trigger == "spawn" and not isTable(t.customers)) then
				numCustomerGroupTriggers = numCustomerGroupTriggers + 1
			end
		end
	end

    -- Could be one rooms with all the the trigger sets to explicit (customers table) customer. This check this case.
    self.customerTriggerTraitsList = {}
    if (numCustomerGroupTriggers == 0) then
        return
    end

    -- Calculate the sum of all the % inside the configuration table
    local totalWeight = 0
	for key,weight in pairs(self.availableCustomerTraits) do
		totalWeight = totalWeight + weight
	end

	local randomCustomerTraits = {}
	for traits,weight in pairs(self.availableCustomerTraits) do
		local count = math.floor(numCustomerGroupTriggers * weight / totalWeight)
		for i=1,count do
			table.push_back(self.customerTriggerTraitsList, traits)
		end
		table.push_back(randomCustomerTraits, traits)
	end

	table.randomize(randomCustomerTraits)
	while (#self.customerTriggerTraitsList < numCustomerGroupTriggers) do
		GH.assert(randomCustomerTraits[1])
		table.push_back(self.customerTriggerTraitsList, randomCustomerTraits[1])
		table.pop_front(randomCustomerTraits)
	end
	table.randomize(self.customerTriggerTraitsList)
end

function DelLevel:onLevelInitialized()
	self:traverse(function(_, node)
		if (node.onLevelInitialized and node ~= self and not node.obsolete) then
			--print("------ JORDI : DelLevel.lua : CALL onLevelInitialized FOR NODE " .. node.name .. "\n");
			node:onLevelInitialized()
		end
	end)
  self:onInitialized()

	if (self.isGamePlayLevel) then
		self.customerTriggerTraitsListIdx = 1
		self:createCustomerTriggerTraitsList()
		--print(serialize("", self.customerTriggerTraitsList))
	end

    if (hero) then
		hero.doWalkToLock = true
		hero.positionLock = hero.location
	end
    if (level.entertainer) then
		level.entertainer.doWalkToLock = true
		level.entertainer.positionLock = level.entertainer.location
	end
    if (level.cleaner) then
		level.cleaner.doWalkToLock = true
		level.cleaner.positionLock = level.cleaner.location
	end
end

function DelLevel:onLevelEnd()
end

function DelLevel:onProductsInitialized()
	self:traverse(function(_, node)
		if (node.onProductsInitialized and node ~= self and not node.obsolete) then
			node:onProductsInitialized()
		end
	end)

	--if (testOrder_productCount) then
	--	local testOrder_hor = 1
	--	testOrder_hor = 9
	--	local complexityCount = 5
	--	local w = GH.getUnitWidth()/(testOrder_hor+1.5)
	--	local h = GH.getUnitHeight()/(complexityCount+1)
	--	for x=1,testOrder_hor do
	--		for complexity=1,complexityCount do
	--			local orderDescription = self:generateOrderDescription(nil, testOrder_productCount, {place="table", tablePlace="table", main={complexityClass=complexity}})
	--			orderDescription.class = "Order"
	--			local orderOverhead = self:addChild(getClass("Order").createOrderBalloonWithOrderDescription(orderDescription))
	--			orderOverhead.x = x * w - w/3
	--			orderOverhead.y = complexity * h + h/2
	--		end
	--	end
	--end
end

function DelLevel:getNumTimedProductOrdered(productName)
	local groups = self.customerGroups
	--print("DelLevel:hasOrderWithProduct(productName)")
	local numTimesOrdered = 0
	for _,v in ipairs(groups) do
		if (v.order) then
			numTimesOrdered = numTimesOrdered + v.order:countProduct(productName)
		end
	end
	return numTimesOrdered
end

function DelLevel:getObjectInLevelThatCanMakeIngredient(ingredientName, product, unfinishedQueuedProducts)
	local resNode = self.gameLayer:traverse(function(_, node)
		if (node.getObjectThatCanMakeIngredient) then
			node = node:getObjectThatCanMakeIngredient(ingredientName, unfinishedQueuedProducts)
			if (node) then
				return node:getFirstObjectInUseChain(product)
			end
		end
	end)

	if (resNode) then
		return resNode
	end
end


--function DelLevel:getIngredientContainingName(ingredientName)
--	return self.gameLayer:traverse(function(_, node)
--
--		if (node.ingredientName) then
--			print(node.ingredientName)
--		end
--	end)
--end

function DelLevel:onDeliver(order, productsDelivered, owner)
	--[[ PR --]] --pr("["..ts(owner.name).."] Receiving order:")
	--[[ PR --]] --pi()
	--[[ PR --]] --for _,v in ipairs(productsDelivered) do
	--[[ PR --]] --	pb("Received "..v)
	--[[ PR --]] --end
	--[[ PR --]] --local orderedProducts = order.currentStep.productNames
	--[[ PR --]] --local deliveredProducts = order.currentStep.deliveredProductNames
	--[[ PR --]] --for _,v in ipairs(orderedProducts) do
	--[[ PR --]] --	if (not table.contains(deliveredProducts, v)) then
	--[[ PR --]] -- 		pb("Did not yet receive "..v)
	--[[ PR --]] --	end
	--[[ PR --]] --end
	--[[ PR --]] --pd()

	if (#productsDelivered > 0) then
		self:onProductsDelivered(order, productsDelivered, owner)
	end

	if (event and event.onDeliver) then
		local n = event:onDeliver(order, productsDelivered, owner)
		if (n) then return n end
	end
end

--function DelLevel:onOrderCompleted(order, owner)
--end

function DelLevel:onCustomerGroupStepDone(orderStep, group)
	--local stepProducts = orderStep.deliveredProductNames
	--if (isBeta) then
	--	print("["..ts(group.name).."] Done with step:")
	--	--[[ PR --]] --pi()
	--	for _,v in ipairs(stepProducts) do
	--		pb(v)
	--	end
	--	pd()
	--end
end

function DelLevel:resolveSprite(v)
	if (v == "cleaner") then return self.cleaner end
	if (v == "entertainer") then return self.entertainer end
	return DelLevel.super.resolveSprite(self, v)
end

--[[
function DelLevel:createCharacter(desc)
	GH.assert(isTable(characterTypes))
	GH.assert(isTable(characterTypes.default))
	local classDesc = table.copy(characterTypes.default, true)
	local traits = characterTypes[desc.traits]
	table.copy_into(classDesc, traits, true)
	table.copy_into(classDesc, desc, true)

	local character = self:add(classDesc)
	if (character.onCreated) then
		character:onCreated()
	end
	return character
end
--]]

function DelLevel:shouldCreateTrigger(trigger, meta)
	--if (meta and meta.giftGroup and player) then
	--	local giftLevel = player:getGiftLevel(meta.giftGroup)
	--	if (isNumber(meta.giftLevel)) then
	--		if (giftLevel ~= meta.giftLevel) then
	--			return false
	--		end
	--	elseif (giftLevel == 0) then
	--		return false
	--	end
	--end

	if (g_animationMode) then
		return false
	end

	if (meta and meta.isBonusCharacter and not self.valueAdders.spawnBonusCharacters) then
		return false
	end
end

function DelLevel:spawnCustomerGroup(meta)
	if (g_animationMode) then
		return
	end

	-- Build a class to spawn a customer

	-- Determine room
	local room = nil
	if (isNumber(self.areaId)) then
		room = self:getRoom(self.areaId)
	end

	-- Determine entrance
	local entrance = {x=512, y=300}


	--If there's a meta entrance, use it
	if (meta.entrance) then
		entrance = meta.entrance
	else
		local entrances = room and room.entrances
		if (entrances) then
			GH.assert(#entrances > 0, "The room has no entrances!")
			entrance = entrances[math.Random(#entrances)]
			GH.assert(entrance.x and entrance.y)
		end
	end

	-- Set a unique name for this group
	self.customerGroupId = (self.customerGroupId or 0) + 1

	-- Build class description
	-- These properties might be overwritten below
	local classDesc = {
		class = meta.class or "CustomerGroup",
		name = meta.name or "CustomerGroup "..tostring(self.customerGroupId),
		quickBonus = true,
		customers = {},
	}

	if (not self:placeExists(meta.place)) then
		return nil
	end

	-- Check for predefined customers
	if (isTable(meta.customers)) then
		for _,explicitCustomer in pairs(meta.customers) do
			if (isString(explicitCustomer)) then
				explicitCustomer = {traits=explicitCustomer}
			end
			GH.assert(isTable(explicitCustomer))
			table.push_back(classDesc.customers, table.copy(explicitCustomer))
		end
	else
		-- Determine the maximum possible size
		GH.assert(isString(meta.place), "Please set a place for this customer group")

		--Check if the place exists (only for the guitarqueue)

		local groupSizeChances = self.groupSizeChances[meta.place] or self.groupSizeChances["counter"]
		local groupSize = meta.size
		if (groupSize) then
			GH.assert(isNumber(groupSizeChances[meta.size]), "The explicit size of this group ("..meta.size..") is not an option for this place='"..meta.place.."'")
		else
			local maxGroupSize = self:maxGroupSizeForHandler(meta.place)
			if (maxGroupSize <= 0) then
				if (not meta.skipWhenFull and ASSERT_GROUP_PLACE ~= false) then error("The maximum group size was 0 at place='"..meta.place.."'") end
				return
			end
			local availableSizes = {}
			local hasNonZeroChance = false
			for k,v in pairs(groupSizeChances) do
				if (isNumber(k) and isNumber(v) and k <= maxGroupSize) then
					availableSizes[k] = v
					if (v > 0) then
						hasNonZeroChance = true
					end
				end
			end

			if (hasNonZeroChance) then
				groupSize = randomFromWeightedTable(availableSizes)
			else
				groupSize = math.Random(maxGroupSize)
				-- print("Warning! Could not determine group size. Max: "..maxGroupSize..", "..serialize("groupSizeChances", groupSizeChances, 0))
			end

			if (hasNonZeroChance) then
				groupSize = randomFromWeightedTable(availableSizes)
			end

			--print(serialize("groupSizeChances", groupSizeChances, 0))
			--print(serialize("availableSizes", availableSizes, 0))
			--print("groupSize: "..ts(groupSize))
			--print("maxGroupSize: "..ts(maxGroupSize))
		end
		GH.assert(self.customerTriggerTraitsListIdx <= #self.customerTriggerTraitsList)
		-- Select customerType from the pregenerated list
		local chosenCharacterTraits = self.customerTriggerTraitsList[self.customerTriggerTraitsListIdx]

        if (self.customerTriggerTraitsListIdx < #self.customerTriggerTraitsList) then
            self.customerTriggerTraitsListIdx = self.customerTriggerTraitsListIdx + 1
        end

		GH.assert(isString(chosenCharacterTraits))

		-- Create N customer descriptions
		for i=1,groupSize do
			table.push_back(classDesc.customers, {traits=chosenCharacterTraits})
		end
	end

	-- Set the customers' class
	for _,v in ipairs(classDesc.customers) do
		GH.assert(isTable(v))
		if (not v.class) then
			v.class = "Customer"
		end
	end

	-- Copy the default values over all the customers
	if (isTable(meta.defaultValues)) then
		for _,v in ipairs(classDesc.customers) do
			table.copy_into(v, meta.defaultValues, true, true) -- 4th argument 'true' means "please overwrite"
		end
	end

	-- Pick genders in a random order
	local genderConfig = {"male", "female"}
	table.randomize(genderConfig)

	GH.assert(isTable(classDesc.customers[1]), "Could not create the customers with this data: "..serialize("meta", meta, 0))
	GH.assert(isString(classDesc.customers[1].traits), "Could not create the customers with this data: "..serialize("meta", meta, 0))
	GH.assert(isTable(characterTypes[classDesc.customers[1].traits]), "Customer type not found: '"..classDesc.customers[1].traits.."'")
	local traits = characterTypes[classDesc.customers[1].traits]

	for k,v in ipairs(classDesc.customers) do
		GH.assert(isString(v.traits))
		GH.assert(not v.__used)
		v.__used = true

		-- Fill character definitions with default values.
		local characterTraits = characterTypes[v.traits]
		GH.assert(isTable(characterTraits), "Could not find character traits type: '"..v.traits.."'");
		table.copy_into(v, characterTraits, true, false)
		table.copy_into(v, characterTypes.default, true, false) -- Because of no-overwrite, this goes 2nd.

		v.class = v.class or "Customer" -- Override!
		v.x = entrance.x
		v.y = entrance.y
		v.name = v.name or ("Group "..self.customerGroupId..", Customer #"..k)
		v.gender = meta.gender or (level.genders and level.genders[v.traits]) or v.gender or genderConfig[k] or randomFromWeightedTable(meta.genderChances or self.genderChances or {male=50, female=50})

		if (v.gender == "male") then v.male = true
		elseif (v.gender == "female") then v.female = true end

		-- Keep using the same gender for this level (memory optimization)
		if (self.useSingleGender) then
			level.genders = level.genders or {}
			level.genders[v.traits] = v.gender
		end

		if (meta.place == "table" or meta.place == "pooltable" or meta.place == "stool") then
			v.edible = true -- Table customers only order edible products
		end
	end

	-- Set the initial mood
	classDesc.startMood = self:calcValueI("startMood")

	-- Merge with trigger
	-- This overwrites existing properties
	if (meta) then
		for k, v in pairs(meta) do
			if (k ~= "trigger" and
				k ~= "triggers" and
				k ~= "delay" and
				k ~= "type" and
				k ~= "defaultValues" and
				k ~= "customers" and
				k ~= "root" and
				k ~= "params" and
				k ~= "__deleted" and
				k ~= "__cpp")
			then
				if (isTable(v)) then
					classDesc[k] = {}
					classDesc[k] = table.copy_into(classDesc[k], v, true, false, true)
				else
					classDesc[k] = v
				end
			end
		end
	end

	classDesc.size = #classDesc.customers

	--if (not self.generatedCustomerGroups) then
	--	self.generatedCustomerGroups = ""--"Delay,Place,Group size\n"
	--end
	--self.generatedCustomerGroups = self.generatedCustomerGroups..tostring(meta.delay)..","..(classDesc.place or "")..","..tostring(classDesc.size).."\n"

	-- Add CustomerGroup
    --print("######## End \"DelLevel:spawnCustomerGroup\" trace ########")
	return self.systemLayer:newChild(classDesc)
end

function DelLevel:generateOrderDescription(orderSteps, orderSize, customerGroup)
	local verbose = false
    if (verbose) then print(">>>>>> DelLevel:generateOrderDescription(orderSteps, orderSize, customerGroup) <<<<<<<<<") end

	if (isTable(customerGroup.orders) and isTable(customerGroup.orders[1])) then
        --print("I have cutomerGroup.orders and customerGroup.orders[1]")
		return customerGroup.orders[1]
	end
	if (isTable(customerGroup.customOrder)) then
		orderSteps = table.count(customerGroup.customOrder["steps"])
	end

	local order = {}
	local place = customerGroup.place or ""
	local tablePlace = customerGroup.tablePlace or ""
	local defaultOrderSize = self[tablePlace.."OrderSize"] or self[place.."OrderSize"] or self.tableOrderSize

    --[[
    print("self["..tablePlace.."OrderSize] = " .. self[tablePlace.."OrderSize"])
    print("self["..place.."OrderSize] = " .. self[place.."OrderSize"])
    print("self.tableOrderSize = " .. self.tableOrderSize)
    print("defaultOrderSize = " .. defaultOrderSize)
    --]]

--[[ORDER--]] --print("defaultOrderSize: "..defaultOrderSize)
	if (place ~= "counter") then
		local groupSize = customerGroup.size
--		if (customerGroup == empat) then
--			groupSize = 2
--		end
		orderSize = orderSize or (math.min(defaultOrderSize, groupSize or 1))
		orderSteps = orderSteps or self.orderSteps or 1
	else
		orderSize = orderSize or defaultOrderSize
		orderSteps = 1
	end

	-- Last resort fallback
	if (orderSize <= 0) then
		orderSize = 2
--[[ORDER--]] --print("orderSize = 2")
	end

	local order = {}
	order.steps = {}
	for stepsIdx = 1, orderSteps do
		local alreadyChosenProducts = {}
		local step = table.push_back(order.steps, {})
		local product
		if (verbose) then print("----\nStep "..stepsIdx) end
		if (customerGroup.customOrder and customerGroup.customOrder["steps"][stepsIdx][1] ~= "any" ) then
			--this is a real custom order
			for ingIdx=1,table.count(customerGroup.customOrder["steps"][stepsIdx]) do
				product = customerGroup.customOrder["steps"][stepsIdx][ingIdx]
				if (verbose) then print("\t-must\t add product"..product) end
				table.push_back(alreadyChosenProducts, product)
				table.push_back(step, product)
			end
		else
			customerGroup.onlyOneProductCount = 0
			local complexityLimits = table.copy(level.complexityLimits[customerGroup.main.complexityClass][orderSize])
			for ingIdx=1, orderSize do
	            --repeat
					product = self:getNewProductFor(customerGroup, alreadyChosenProducts, orderSize, ingIdx,complexityLimits)
					--complexityLimits[product.complexity] = complexityLimits[product.complexity] - 1
				--until (not selfExclusive or not table.contains(alreadyChosenProducts, product))
				if (verbose) then print("\t-chances\t add product"..product) end
				table.push_back(alreadyChosenProducts, product)
				table.push_back(step, product)
			end
		end
	end
	return order
end

-- Get a product using the settings in the customer and the productInfo
function DelLevel:getNewProductFor(customerGroup, alreadyChosenProducts, orderSize, ingIdx,complexityLimits)
	local verbose = false
	local gameplayProducts = {}
	local zeroChanceProducts = {}
	local products = {}
	local customer = customerGroup.main
	local possibleProducts = self.possibleProducts

	GH.assert(not table.empty(possibleProducts), "No products found AT ALL!")
	GH.assert(isNumber(customer.complexityClass), "Customer does not have a 'complexity class' number")

	if (verbose) then print("function DelLevel:getNewProductFor(customerGroup, alreadyChosenProducts, orderSize, ingIdx,complexityLimits)") end

	local complexityNumber = customer.complexityClass
	local complexityChances = nil

	--local complexityChances = level.complexityChances[complexityNumber][orderSize][ingIdx]
	if(level:isMetadifficultyActivated())then
		complexityChances = level:getMetadifficultyTableForComplexityClass("complexityChancesWithMetadifficultyLevels",complexityNumber)
	else
		complexityChances = level.complexityChances[complexityNumber][orderSize][ingIdx]
	end


	if (verbose) then print("Customer Complexity class " ..customer.complexityClass.. "// order size: "..orderSize..serialize("*complexityLimits",complexityLimits)) end
	--customer.complexityLimits = customer.complexityLimits or table.copy(level.complexityLimits[complexityNumber][orderSize])

--[[ORDER--]] --print("##############################################################")
--[[ORDER--]] --print("Order for: "..tostring(customerGroup.tablePlace))
	for _,productName in ipairs(possibleProducts) do
--[[ORDER--]] --print("============================================")
--[[ORDER--]] if (verbose) then print("Product: "..productName) end

		GH.assert(self.productSettings)
		local product = self.productSettings[productName]
		GH.assert(product, "Product not found in the product settings: '"..productName.."'")

		local weight = 1
		--print(serialize("product",product))
		if (not product or not product.orderable) then

--[[ORDER--]] if (verbose) then print("\tNot added: not orderable.") end
		end
		--else
		if (product and product.orderable) then

			-- CHECK CUSTOMERGROUP PLACE IF THE PRODUCT CAN BE ORDERED
			if (isTable(product.mustOrderPlaces)) then
--[[ORDER--]] --print(serialize(productName..".mustOrderPlaces", product.mustOrderPlaces))
				local alreadyChosen = table.contains(alreadyChosenProducts, productName)
				if (not alreadyChosen) then
--[[ORDER--]] if (verbose) then print("\tNot already chosen ") end
					local mustOrder = table.contains(product.mustOrderPlaces, "all") or table.contains(product.mustOrderPlaces, customerGroup.tablePlace)
					if (mustOrder) then
--[[ORDER--]] if (verbose) then print("Must order! ("..productName..")") end
						return productName
					end
				end
			end

			GH.assert(isString(product.complexity), "This product(" .. productName .. ") has no 'complexity' set")
			local forbiddenByBoolean = false

			if (not forbiddenByBoolean) then
				if (complexityLimits[product.complexity]) then
					if (verbose) then print("\t\t>customer.complexityLimits[product.complexity] "..complexityLimits[product.complexity] ) end
					forbiddenByBoolean = complexityLimits[product.complexity] <= 0
--[[ORDER--]] if (verbose and forbiddenByBoolean) then print("\nNot added. Complexity '"..product.complexity.."' limit reached.") end
				end
			end

			if (not forbiddenByBoolean) then
				if (isTable(product.canOrderPlaces)) then
					GH.assert(product.cantOrderPlaces == nil, "You're using both 'canOrderPlaces' AND 'cantOrderPlaces'. That doesn't make sense!")
					forbiddenByBoolean = not table.contains(product.canOrderPlaces, customerGroup.tablePlace)
--[[ORDER--]] if (verbose) then print("\tCan order at '"..customerGroup.tablePlace.."'? "..ts(not forbiddenByBoolean).."\n"..serialize("canOrderPlaces", product.canOrderPlaces)) end

				elseif (isTable(product.cantOrderPlaces)) then
					forbiddenByBoolean = table.contains(product.cantOrderPlaces, "all") or table.contains(product.cantOrderPlaces, customerGroup.tablePlace)
--[[ORDER--]] 		if (verbose) then print("\tForbidden at '"..customerGroup.tablePlace.."'? "..ts(forbiddenByBoolean)) end

				elseif (product.cantOrderPlaces == "all") then
					forbiddenByBoolean = true
--[[ORDER--]] 		if (verbose) then print("\tNot added. Forbidden to order everywhere.") end
				end

				if (product.unique and table.contains(alreadyChosenProducts, productName)) then
--[[ORDER--]] 		if (verbose) then print("\tNot added. Already chosen this UNIQUE product.") end
					forbiddenByBoolean = true
				end
			end

			if (not forbiddenByBoolean) then
--[[ORDER--]] --print("\tProduct property VS customer value")
				for k,v in pairs(product) do
					local cv = customer[k]
--[[ORDER--]] --print("\t\t"..k.." = "..tostring(v).." <=> "..tostring(cv))
					if (isBoolean(v) and isBoolean(cv) and cv ~= v) then
						forbiddenByBoolean = true
--[[ORDER--]] --print("\tNot added: forbidden by setting.")
					elseif (isNumber(v) and (k == "chance" or cv == true)) then
						weight = weight * (v / 100) -- Percentage
					end
				end
			end

			gameplayProducts[product.complexity] = gameplayProducts[product.complexity] or {}
			gameplayProducts[product.complexity][productName] = 1000
			if (verbose) then print("\tNew product in gameplayProducts[".. product.complexity .."]+ "..productName) end
			if (not forbiddenByBoolean) then
				if (weight <= 0) then
--[[ORDER--]] 		if (verbose) then print("\tNot added: '".. productName .."'.") end
					zeroChanceProducts[product.complexity] = zeroChanceProducts[product.complexity] or {}
					zeroChanceProducts[product.complexity][productName] = 1000
				else
--[[ORDER--]] 		if (verbose) then print("\tAdded '"..productName.."'.") end
					products[product.complexity] = products[product.complexity] or {}
					products[product.complexity][productName] = weight * 1000
				end
--[[ORDER--]] else
--[[ORDER--]] if (verbose) then print("\tNot added: forbidden by boolean.") end
			end
		end
	end
	if (verbose) then print(serialize("products", products)) end

	local complexityProducts = {}
	if (not table.empty(products)) then
		complexityProducts = self:getProductsFromArraysWithComplexity(products, complexityChances)
		if (verbose) then print("\tSet complexityProducts from products\n"..serialize("complexityProducts", complexityProducts)) end
	else
		-- print("Warning: no allowed products found.\n\tCustomer place: "..tostring(customerGroup.tablePlace)) --.."\n"..serialize("customer", customer))
		if (not table.empty(zeroChanceProducts)) then
			-- print("\tFalling back to products with zero chance:\n"..serialize("products", complexityProducts))
			complexityProducts = self:getProductsFromArraysWithComplexity(zeroChanceProducts, complexityChances)
			if (verbose) then print("\tSet complexityProducts from zeroChanceProducts\n"..serialize("complexityProducts", complexityProducts)) end
		elseif (not table.empty(gameplayProducts)) then
			 if (verbose) then print("\tError. Falling back to ALL products valid for gameplay:\n"..serialize("complexityProducts", complexityProducts)) end
			-- First we check the products that can be ordered in this station
			local cantBeOrdered = true
			local chosenProduct
			for _,p in ipairs(possibleProducts) do
				chosenProduct = self.productSettings[p]
				if (isTable(chosenProduct.cantOrderPlaces)) then
					cantBeOrdered = (table.contains(chosenProduct.cantOrderPlaces, "all")) or
									(table.contains(chosenProduct.cantOrderPlaces, customerGroup.tablePlace))
				end
				if (chosenProduct.orderable and not cantBeOrdered) then
					complexityProducts[p] = 1000
				end
			end
			if (table.empty(complexityProducts)) then
				complexityProducts = self:getValidProductsForPlace(gameplayProducts, customerGroup.tablePlace)
				if (verbose) then print("\tSet complexityProducts from gameplayProducts\n"..serialize("complexityProducts", complexityProducts)) end
			end
		else
			 if (verbose) then print("\tPanic! Falling back to ALL products:\n"..serialize("products", products)) end
			for _,p in ipairs(possibleProducts) do complexityProducts[p] = 1000 end
		end
	end

--[[ORDER--]] --print("Available products with '"..chosenComplexity.."' complexity: '"..table.serialize("prods", complexityProducts, 0).."'.")

	GH.assert(not table.empty(complexityProducts))

	local resultProductName
--[[ORDER--]] if (verbose) then print(serialize("complexityProducts", complexityProducts)) end
	if(table.contains(complexityProducts, "exeption")) then
		resultProductName = complexityProducts[2]
	else
		resultProductName = randomFromWeightedTable(complexityProducts)
	end
	local product = self.productSettings[resultProductName]
	if (product and product.complexity and complexityLimits[product.complexity]) then
		complexityLimits[product.complexity] = complexityLimits[product.complexity] - 1
 	end

--[[ORDER--]] if (verbose) then print("\t\t\tresultProductName: "..resultProductName) end
	return resultProductName
end

function DelLevel:getProductsFromArraysWithComplexity(products, complexityChances)
	local verbose = false
	if (verbose) then print("\n--DelLevel:getProductsFromArraysWithComplexity(products, complexityChances)--\n") end
	local complexityChancesCopy = table.copy(complexityChances)
	local tries = table.count(complexityChancesCopy)
	while (tries > 0) do
		if (verbose) then print("tries "..tries..serialize(" complexityChancesCopy",complexityChancesCopy)) end
		local chosenComplexity = randomFromWeightedTable(complexityChancesCopy)
		if (verbose) then print(serialize(" > chosenComplexity",chosenComplexity)) end
		if (products[chosenComplexity] and not table.empty(products[chosenComplexity])) then
			return products[chosenComplexity]
		else
			-- there are no products of chosenComplexity --> delete the chosenComplexity entry from complexityChancesCopy
			-- to avoid choosing this complexity again
			if (verbose) then print("No, intentem borrar l'entrada "..chosenComplexity .." de la taula complexityChancesCopy"..table.count(complexityChancesCopy)) end
			complexityChancesCopy[chosenComplexity] = nil
			tries = tries - 1
		end
	end

	-- In case there are no products of the needed complexity, we select a random complexity
	-- from the ones we have.
	if (verbose) then print("there are no products of the needed complexity,we select a random complexity") end
	local result = {}
	local availableComplexities = {}
	local i = 1
	for key,value in pairs(products) do
		availableComplexities[i] = key
		i = i + 1
	end
	if (not table.empty(availableComplexities)) then
		local r = math.Random(table.getn(availableComplexities))
		if (verbose) then print("randomly selected complexity "..availableComplexities[r]) end
		return products[availableComplexities[r]]
	end
end

function DelLevel:getValidProductsForPlace(products, place)
	-- This function selects the products that can be ordered in a place
	local validProducts = {}
	local settings
	for _,value in pairs(self.possibleProducts) do
		settings = self.productSettings[value]
		local isForbidden = table.contains(settings.cantOrderPlaces, "all") or table.contains(settings.cantOrderPlaces, place)
		if (not isForbidden) then
			validProducts[value] = 500
		end
	end
	local r = math.Random(table.getn(validProducts))
	return validProducts
end

function DelLevel:handleTrigger(trigger, meta)

	if (trigger == "spawn") then

		if (isString(meta.params)) then
			local r = tryParse(meta.params)
--[[PARSE--]] --print("params: "..tostring(meta.params))
--[[PARSE--]] --print("r: "..tostring(r))
			if (isTable(r)) then
				table.copy_into(meta, r)
			end
		end

		local clockSpriteSuccess = (not meta.clockSprite) or meta.clockSprite.reservationSuccess
		if (clockSpriteSuccess) then
			local customerGroup = self:spawnCustomerGroup(meta)
			if (customerGroup) then
				customerGroup:onInitialized()
			end
		end

	elseif (trigger == "call") then
		meta:func(self)

	elseif (trigger == "object") then
		if (not meta.name) then
			DelLevel.super.handleTrigger(self, trigger, meta)

		else
			local object = self:resolveSprite(meta.name)
			if (object and isCallable(object.handleTrigger)) then
				object:handleTrigger(meta.type or trigger, meta)
			end
		end

	elseif (trigger == "startShift") then
		self:shiftStart()

	elseif (trigger == "finishShift") then
		self:finishShift()

	elseif (trigger == "showTray") then
		local tray = self:getTray(hero)
		GH.assert(tray, "Actor '"..tostring(actor).."' does not have a tray!")
		if (tray) then
			tray:animate({alpha=1, duration=meta.duration or 0})
		end

	elseif (trigger == "hideTray") then
		local tray = self:getTray(hero)
		GH.assert(tray, "Actor '"..tostring(actor).."' does not have a tray!")
		if (tray) then
			tray:animate({alpha=0, duration=meta.duration or 0})
		end

--	elseif (trigger == "memorableMomentEnd") then
--		if (empat) then
--			empat.state = "IdlingAtTable"
--		end

	else
		DelLevel.super.handleTrigger(self, trigger, meta)
	end
end

function DelLevel:getDirtyStateImage(customer, productSettings)
--print("==============================")
	local anim = customer.currentAnimation
--print("anim: "..anim)
	local res = self.dirtyStateImages and (self.dirtyStateImages[anim] or self.dirtyStateImages.default)
	if (res) then
--print("res: "..res)
		if (res ~= "[product]") then
--print("return "..res)
			return res
		end
		local p = productSettings and productSettings.dirty;
--print("dirty state image: "..tostring(p))
		if (p) then return p end
	end
	return "plate"
end

function DelLevel:stopAllSoundLoops()
	level.gameLayer:traverse(function(_, node)
		if (node.stopLoopSample) then
			node:stopLoopSample()
		end
	end)
end

function DelLevel:removeSpecialEmotions(animation)
	local animTable = taskSystem:getAnimationDesc(animation)
	if (animTable and isTable(animTable.actors)) then
		for k,v in pairs(animTable.actors) do
			local actor = self:resolveActor(v)
			if (actor) then
				--if (empat and empat.patrick and actor.name == empat.patrick.name) then
				--	empat.patrick.showCustomerMoodAppearance = false
				--	empat.patrick:removeAppearance("restless")
				--	empat.patrick:removeAppearance("relaxed")
				--end
				--
				--if (empat and empat.emily and actor.name == empat.emily.name) then
				--	empat.emily.showCustomerMoodAppearance = false
				--	empat.emily:removeAppearance("restless")
				--	empat.emily:removeAppearance("relaxed")
				--end

				if (self.cleaner and actor.name == self.cleaner.name) then
					self.cleaner:removeAppearance("helper")
					self.cleaner:removeAppearance("cleaner")
				end
			end
		end
	end
end

function DelLevel:createPostShiftAnimation()
	local doPostShiftAnimation = true

	if (self.stopAllSoundLoopsOnLevelEnd ~= false) then -- Default true
		self:stopAllSoundLoops()
	end

	if (player and player.skipAllStory) then
		doPostShiftAnimation = false

	elseif (self.isGamePlayLevel) then
		doPostShiftAnimation = level.score >= level.star1Score
	end

	local anim
	if (event) then
		if (event.onShiftFinish) then
			anim = event:onShiftFinish()
		end
	end
	anim = anim or GH.anim.animation()

	if (not anim or not anim.atEnd) then
		anim = GH.anim.animation()
		GH.assert(anim.atEnd, "1. the famous atEnd error!")
	end

	if (anim and anim.atEnd) then
		anim:atEnd({class="CallTask", func=function()
			if (doPostShiftAnimation) then
				local anim2 = taskSystem:createAnimation("postShift")
				if (anim2) then
					if (paige) then
						paige:removeAppearance("mood")
					end
					self:removeSpecialEmotions("postShift")

					self.levelDeleted = true -- Nasty var to keep track of this level...
					anim2:atEnd({class="CallTask", func=function()
						if (level.levelDeleted ~= nil) then
							--if (debug) then
							--	print("OnShiftFinishEnd via createPostShiftAnimation")
							--end
							level:onShiftFinishEnd()
						end
					end})
					return anim2
				end
			end
			level:shiftPendingFinish()
		end})
	end
	return anim
end

function DelLevel:onWinFinishDay()
	self:traverse(function(_, node)
		if (node.onWinFinishDay and node ~= self) then
			node:onWinFinishDay()
		end
	end)
end

function DelLevel:onShiftStartEnd()
	DelLevel.OnShiftStartEnd(self)

	self:traverse(function(_, node)
		if (node.onShiftStartEnd and node ~= self) then
			node:onShiftStartEnd()
		end
	end)

	-- Create empat customerGroup
	--if (not self.empat and level.isGamePlayLevel) then
	--	local groupDesc = {}
	--	groupDesc.class = "EmPatCustomerGroup"
	--	groupDesc.allowConstructEmpty = true
	--	self.empat = self.systemLayer:newChild(groupDesc)
	--
	--	local patrick = self.empatCustomerPatrick or "patrick"
	--	patrick = (#patrick > 0) and self:resolveSprite(patrick)
	--	if (patrick and patrick.className == "Customer") then
	--		self.empat:addCustomer(patrick)
	--		self.empat.patrick = patrick
	--		patrick.emotion = ""
	--	end
	--
	--	local emily = self.empatCustomerEmily or "emily"
	--	emily = (#emily > 0) and self:resolveSprite(emily)
	--	if (emily and emily.className == "Customer") then
	--		self.empat:addCustomer(emily)
	--		self.empat.emily = emily
	--		emily.emotion = ""
	--	end
	--end
	--
	--if (self.empat) then
	--	self.empat:onInitialized()
	--	self.empat:enterForAnotherRound()
	--end

	--if (not self.paigeGroup and level.isGamePlayLevel) then
	--	local groupDesc = {}
	--	groupDesc.class = "PaigeCustomerGroup"
	--	groupDesc.allowConstructEmpty = true
	--	self.paigeGroup = self.systemLayer:newChild(groupDesc)
	--
	--	local paige = self.paigeGroupPaige or "paige"
	--	paige = (#paige > 0) and self:resolveSprite(paige)
	--	if (paige and (paige.className == "Customer" or paige.className == "Paige")) then
	--		self.paigeGroup:addCustomer(paige)
	--		self.paigeGroup.paige = paige
	--		paige.emotion = ""
	--	end
	--end
	--
	--if (self.paigeGroup) then
	--	self.paigeGroup:onInitialized()
	--	self.paigeGroup:enterForAnotherRound()
	--end
end

--function DelLevel:onShiftFinishStart()
--	if (event) then
--		event:onShiftFinishStart()
--	end
--end

function DelLevel:onCancelChain()
end

function DelLevel:debugHeroWalkTo(x, y)
	if (level.beforeAddWorkTaskOnClick) then
		level:beforeAddWorkTaskOnClick(nil, hero)
	end
	if (hero) then
		hero:addTask({class="WalkToTask", target={x=x, y=y}, playDefaultOnFinish=true})
	end
end

function DelLevel:isReservationProduct(product)
--print(product.name)
	settings = self.productSettings and self.productSettings[product.name]
--print(serialize("settings", settings))
	return settings and settings.reservation or false
end

function DelLevel:onReservationCreated(reservationSize, clockSprite, ringDuration)
	if (isString(self.reservationPhone)) then
		self.reservationPhone = self:getSpriteExt(self.reservationPhone)
	end

	local overhead = "reservation"..reservationSize
	local ringDesc = {
		reservationSize=reservationSize,
		clockSprite=clockSprite,
		ringDuration=ringDuration,
		ringingOverhead="exclamation:gamescene",
		ringingOverheadNeedsBalloon = false,
		chatBalloons={
			{overhead, "phone"},
			{overhead, "actor"},
		},
		onWorkStart=function()
			clockSprite.reservationSuccess = true
		end,
		onWorkFinish=function()
			if (clockSprite.animate) then
				clockSprite:animate({alpha=1, duration=100})
			end
			self:onReservationCallFinished(self.reservationPhone)
		end,
		onCallMissed=function()
			self:onReservationCallMissed(self.reservationPhone)
		end
	}
	self.reservationPhone:doRing(ringDesc)
	if (event and event.onReservationCallCreated) then
		event:onReservationCallCreated(self.reservationPhone, ringDesc)
	end
end

function DelLevel:onReservationCallMissed(phone)
	if (event and event.onReservationCallMissed) then
		event:onReservationCallMissed(phone)
	end
	local pos = phone.floaterPosition
	level:createGenericFloater("RESERVATION_CALL_MISSED", pos.x, pos.y)
end

function DelLevel:onReservationCallFinished(phone)
	if (event and event.onReservationCallFinished) then
		event:onReservationCallFinished(phone)
	end
end

function DelLevel:onReservationFailed(group)
	if (event and event.onReservationFailed) then
		event:onReservationFailed(group)
	end
	local pos = group.main.floaterPosition
	level:createGenericFloater("RESERVATION_FAILED", pos.x, pos.y)
end

function DelLevel:onReservationSuccess(group, table)
	if (event and event.onReservationSuccess) then
		event:onReservationSuccess(group, table)
	end
	local pos = table.floaterPosition
	local score = group.main:calcValue("reservationTip")
	level:createFloater({
		id = "reservationSuccess",
		textId = "RESERVATION_SUCCESS",
		x = pos.x, y = pos.y,
		bonusScore = self.counterKeyScore,
		bonus = {
			textId = "RESERVATION_SUCCESS_FMT",
		},
	})
end

function DelLevel:createWaterDrip(startX, startY, endY, worldFloor, doSplash, fallSpeed, suspendedFrameTime, suspendedLoops, splashFrameTime)
	if (level) then
		worldFloor = worldFloor or 2000
		fallSpeed = fallSpeed or 0.8
		suspendedFrameTime = suspendedFrameTime or 100
		suspendedLoops = suspendedLoops or 1
		splashFrameTime = splashFrameTime or 200

		local delta = endY - startY
		local duration = delta / fallSpeed
		local drip = level:add({
			class="SpriteExt",
			baseAnimation="waterdrip",
			defaultSection="room4_icons",
			anchor="center",
			fixedFloor=true,
			x=startX, y=startY,
			defaultCycles=1,
		})
		drip:playAnimation("", 0)
		drip.floor = worldFloor

		local anims = {}
		if (suspendedLoops > 0) then
			table.push_back(anims, {call=function() drip:playAnimation("suspended", 0) end})
			table.push_back(anims, {startFrame=0, frameTime=suspendedFrameTime, type="pingpong", loops=suspendedLoops*2-1})
		end
		table.push_back(anims, {call=function() drip:playAnimation("", 0) end})
		table.push_back(anims, {startFrame=0, frameTime=duration/drip.frameCount, y=endY, sytle="easeIn", duration=duration})
		if (doSplash) then
			table.push_back(anims, {call=function() drip:playAnimation("splash", 0) end})
			table.push_back(anims, {startFrame=0, frameTime=splashFrameTime})
		end
		table.push_back(anims, {remove=true})
		drip:animate(anims)
	end
end

function DelLevel:placeExists(place)
	-- For the moment to this only in the guitarqueue, it's the only station
	-- that can be bought
	if (place == "guitarqueue") then
		local queueObject = level:getObject(place)
		if (queueObject and queueObject.hasDestinyTable) then
			return queueObject:hasDestinyTable()
		else
			return false
		end
	end
	return true
end
