Task = getClass("Task")

LuaTask = getClass("LuaTask")



function Task:evaluateActorSetting(settingName)

	local r = self[settingName]

--print("settingName: "..settingName)

	if (r == nil) then return true end

--print("Task:evaluateActorSetting")

	if (isString(r)) then

		r = tryParse("function(actor, task) return "..r.." end")

	end

	if (isCallable(r)) then

--print("self.actor: ".. serialize("actor", self.actor))

		return r(self.actor, self) ~= false

	end

	return r

end



-- START CUSTOM D10 TASKS

-- ************************************ Play Movie Task ************************************ '--

PlayMovieTask = deriveLuaClass("PlayMovieTask", "LuaTask")



function PlayMovieTask:init(desc)

	PlayMovieTask.superInit(self, desc)

	self.videoEnded = false

	self.clickTimes = 0

	self.maxClickTimes = 10

end



function PlayMovieTask:start()

	GH.assert(self.movie, "Need to add movie")

	-- If the bundle version is low don't play the video because the memory crash the
	if (GH.getBundleQuality() == "low") then
		self.videoEnded = true
		return false;
	end


	self.video = scene:addChild({class="TheoraVideo", name="video", file=self.movie, autoSize=false, width=2048, height=math.ceil(2048 * (9/16))})

    if (self.audio) then
        self.video.customStream = self.audio
    end

	self.video:play()

	self.video.onEndReached = function()

		self.videoEnded = true

		self.video:animate({alpha=0.0, duration=600}, {remove=true})

		self.video = nil

	end

	self.video.layout = function(self)

		local w, h = GH.getUnitWidth(), GH.getUnitHeight()
		local tempExtraPadding = 0 -- it's a temporal patch to fix some width missing

		self.scale = w/(self.width - tempExtraPadding)

		local yOffset = (h - self.height * self.scaleY)/2

		self.y = yOffset

	end

	self.video:layout()

	return true

end



function PlayMovieTask:click()

	self.clickTimes = self.clickTimes + 1

end



function PlayMovieTask:skip()

	if (self.video) then

		self.video.obsolete = true

	end

	self.videoEnded = true

end



function PlayMovieTask:update(time)

	if (self.maxClickTimes and self.clickTimes >= self.maxClickTimes) then

		self:skip()

	end

	return self.videoEnded

end



-- END CUSTOM D10 TASKS



-- ************************************ Start Story Task ************************************ --

StartStoryTask = deriveLuaClass("StartStoryTask", "LuaTask")

function StartStoryTask:init(desc)

	StartStoryTask.superInit(self, desc)

	self.barsAnimated = self.barsAnimated ~= false

	self.showBars = self.showBars ~= false

	self.playSound = self.playSound ~= false

	self.duration = self.duration or storyBarsDuration

	self.animName = desc.animName or "notFound"

end



function StartStoryTask:start()

	if (level) then

		level:startStory(self.barsAnimated, self.showBars, self.playSound)
		level:startStoryTaskStart(self.animName)
	end

	-- Let the duration determine the time.

	return self.wait

end



-- ************************************ End Story Task ************************************ --

EndStoryTask = deriveLuaClass("EndStoryTask", "LuaTask")

function EndStoryTask:init(desc)
	EndStoryTask.superInit(self, desc)

	self.barsAnimated = self.barsAnimated ~= false
	self.duration = self.duration or storyBarsDuration
	self.animName = desc.animName or "notFound"
	self.showBars = self.showBars ~= false
	self.sendCompleteEvent = desc.sendCompleteEvent ~= false
end

function EndStoryTask:start()
	if (level) then
		level:endStory(self.barsAnimated, self.showBars)
		level:endStoryTaskStart(self.animName, self.sendCompleteEvent)
	end
	-- Let the duration determine the time.
	return self.wait
end

-- ************************************ Remove Task ************************************ --

RemoveTask = deriveLuaClass("RemoveTask", "LuaTask")

function RemoveTask:start()

	if (self.actor and not isString(self.actor.__cpp)) then

		if (self.actor.removeFromParent) then

			self.actor:removeFromParent()

		elseif (self.actor.remove) then

			self.actor:remove()

		end

	end

	return false

end



-- ************************************ Trigger Task ************************************ --

TriggerTask = deriveLuaClass("TriggerTask", "LuaTask")

function TriggerTask:start()

	self.root = self

	level:fireTrigger(self.trigger or "", self)

	return self.duration > 0

end



-- ************************************ Set Task ************************************ --

SetTask = deriveLuaClass("SetTask", "LuaTask")

function SetTask:start()

	local target = self.tgt and level:resolveSprite(self.tgt) or self.actor

	GH.assert(isTable(target), "Must provide a target to put values table on: '"..tostring(self.values).."' "..serialize("self", self, 0))

	for k,v in pairs(self) do

		if (not table.contains({"class", "actor", "__cpp", "values", "tgt"}, k)) then

			--print(">>>> SetTask: "..tostring(k).." = "..tostring(v).."\n")

			target[k] = v

		end

	end

	local val = self.values

	if (isString(val)) then

		local r = tryParseAsTable(val, target)

		if (isTable(r)) then

			val = r

		end

	end

	if (isTable(val)) then

		for k,v in pairs(val) do

			if (not table.contains({"class", "actor", "__cpp"}, k)) then

				--print(">>>> SetTask: "..tostring(k).." = "..tostring(v).."\n")

				target[k] = v

			end

		end

	end
	return false

end



-- ************************************ Call Task ************************************ --

CallTask = getClass("CallTask")

function CallTask:start()

	local object = self.object or self.actor

	local f = self.func or self.call

	local func = f

	if (isString(f)) then

		func = tryParse("function(actor, task, ...) "..f.." end")

	end



	if (not isCallable(func)) then

		if (isTable(object)) then

			func = object[f]

		end

		if (not isCallable(func)) then

			func = self[f]

			if (not isCallable(func)) then

				func = _G[f]

			end

		end

	end



	GH.assert(isCallable(func), "Not callable function in CallTask: "..tostring(func))

	func(object, self, self.params and unpack10(self.params))

	return false

end



-- ************************************ Complex Task ************************************ --

ComplexTask = getClass("ComplexTask")

function ComplexTask:start()

	local object = self.object or self.actor



	local duration = self.duration

	local rotation = self.rotation

	local translationX = self.translationX

	local translationY = self.translationY

	local scale = self.scale

	local pingR = self.pingR

	local pingX = self.pingX

	local pingY = self.pingY

	local pingS = self.pingS





	local scale_tmp = scale

	local pingS = true



	if(rotation ~= nil and rotation ~= 0) then

		if pingR == true then

			object.Rotate = function() object:animate({rotation=rotation, duration=duration}, {call=object.Rotate}) rotation = rotation * -1 end object.Rotate() --, style="easeInOut"

		else

			object.Rotate = function() object:animate({rotation=rotation, duration=duration}) end object.Rotate()

		end

	end

	if(translationX ~= nil and translationX ~= 0) then

		if pingX == true then

			object.TransX = function() object:animate({x=object.x + translationX, duration=duration}, {call=object.TransX}) translationX = translationX * -1 end object.TransX()

		else

			object.TransX = function() object:animate({x=object.x + translationX, duration=duration}) end object.TransX()

		end

	end

	if(translationY ~= nil and translationY ~= 0) then

		if pingY == true then

			object.TransY = function() object:animate({y=object.y + translationY, duration=duration}, {call=object.TransY}) translationY = translationY * -1 end object.TransY()

		else

			object.TransY = function() object:animate({y=object.y + translationY, duration=duration}) end object.TransY()

		end

	end

	if(scale ~= nil and scale ~= 0) then

		if pingS == true then

			object.Scale =

			function() object:animate({scale = scale_tmp, duration=duration}, {call=object.Scale})

				if scale_tmp ~= 1 then

					scale_tmp = 1

				else scale_tmp = scale

				end

			end object.Scale()

		else

			object.Scale = function() object:animate({scale = scale, duration=duration}) end object.Scale()

		end

	end

	return false

end


-- ************************************ Minigame Task ************************************ '--
MinigameTask = deriveLuaClass("MinigameTask", "LuaTask")
function MinigameTask:start()
	self.skippable = false

	local gameClassDesc = _G[self.minigameClassName]
	--print("CARLES::: gameClassDesc " .. gameClassDesc)
	local containerClass = "Minigame"--gameClassDesc.containerClass or "Minigame"
	local minigameClassDesc = _G[containerClass]
	local gameDesc = table.copy(minigameClassDesc.gameDefaults, true)

	local variations = gameClassDesc["variations"]
	if (#variations > 0) then
		for i,v in ipairs(variations) do
			v.variation = i
		end
		local variationDesc
		if (self.variation == -1) then
			-- UNWINNABLE!!
			variationDesc = {unwinnable=true, extra=true, minDecoys=10, maxDecoys=10, penaltyTime=150, props={box={"bottle1","bottle3","bottle5","bottle6","box1","box3","box5","box7","tube1","tube2"}}, hearts={4800,3800,2800,2000}}
		elseif (self.variation) then
			variationDesc = variations[math.clamp(self.variation, 1, #variations)]
		else
			variationDesc = randomFromWeightedTable(variations, "chance")
		end
		table.copy_into(gameDesc, variationDesc, true, true)
	end

	if (isString(self.patient)) then
		self.patient = level:getObject(self.patient)
		assert(self.patient)
	end

	gameDesc["class"] = self.minigameClassName
	gameDesc["minigameClassName"] = self.minigameClassName

    local containerDesc = table.copy(minigameClassDesc["containerDefaults"])
	containerDesc["class"] = containerClass
    containerDesc["gameDesc"] = gameDesc
	containerDesc["timed"] = self.timed
	--containerDesc["patient"] = self.patient
	self.minigame = scene:openDialog(containerDesc)

	--scene:openDialog(containerDesc)

	return true
end

function MinigameTask:update()
	return not self.minigame or self.minigame.__deleted or self.minigame.finishTask
end

function MinigameTask:skip()
		level:hideClickToContinueMinigame()
		level:hideSkipButtonMinigame()
    if (self.minigame and not self.minigame.__deleted) then
        self.minigame:remove()
        self.minigame = nil
    end
end

---- ************************************ Halt Task ************************************ --
HaltTask = deriveLuaClass("HaltTask", "LuaTask")

function HaltTask:start()
	self.skippable = false
	self.haltCheck = tryParse("function(self, actor) return "..self.haltCondition.." end")
end

function HaltTask:update(time)
	return not self:haltCheck(self.actor)
end

-- ************************************ Parenting Task ************************************ --

ParentingTask = getClass("ParentingTask")

function ParentingTask:start()

	local object = self.object or self.actor

	local newParent = self.newParent

	local reallocate = self.reallocate or false

	local offsetX = self.offsetX or 0

	local offsetY = self.offsetY or 0



	local tmpX = object.x

	local tmpY = object.y



	if(offsetX ~= nil and offsetX ~= 0) then

		tmpX = object.x + offsetX

	end

	if(offsetY ~= nil and offsetY ~= 0) then

		tmpY = object.y + offsetY

	end



	if(newParent ~= nil) then

		object = level:resolveActor(object)

		newParent = level:resolveActor(newParent)

		if(reallocate == false) then

			object:becomeChildOf(newParent)

		end

		object.x = tmpX - newParent.x

		object.y = tmpY - newParent.y

	end



end

---- ************************************ Skip Story Task ************************************ --
SkipButtonTask = deriveLuaClass("SkipButtonTask", "LuaTask")

function SkipButtonTask:start()
	self.hideSkipButton = self.hideSkipButton or false
	if(self.hideSkipButton == true) then
		level:hideSkipFromTask()
	elseif(self.hideSkipButton == false) then
		level:showSkipFromTask()
	end
end

-- ************************************ Play Sample Task ************************************ --

PlaySoundTask = deriveLuaClass("PlaySoundTask", "LuaTask")

function PlaySoundTask:start()

	GH.assert(self.sound, "Need to add sound")

	if (not taskSystem.skipping) then

		SpriteExt.playSample(self.actor, self)

	end

	return false

end



-- ************************************ Loop Sample Task ************************************ --

LoopSoundTask = deriveLuaClass("LoopSoundTask", "LuaTask")

function LoopSoundTask:start()

	GH.assert(self.sound, "Need to add sound")

	if (self.volume == nil) then
		self.volume = 1.0
	end
	--self.actor:loopSample(self.sound,self)

	SpriteExt.loopSample(self.actor,self)

	return false

end



-- ************************************ Stop Loop Sample Task ************************************ --

StopLoopSoundTask = deriveLuaClass("StopLoopSoundTask", "LuaTask")

function StopLoopSoundTask:start()

	SpriteExt.stopLoopSample(self.actor)

	return false

end



-- ************************************ Set Position Task ************************************ --

PositionTask = deriveLuaClass("PositionTask", "LuaTask")

function PositionTask:init(desc)

	PositionTask.superInit(self, desc)

	self.target = desc.target or desc.position or {x=500,y=500}



	if (not self.ease) then

		self.ease = math.linear

	elseif (not isCallable(self.ease)) then

		local options = {easeIn=math.jcurve, easeOut=math.rcurve, easeInOut=math.scurve}

		self.ease = options[self.ease] or math.linear

	end

end



function PositionTask:start()

	GH.assert(self.actor)--, "Actor not found for this task. "..tostring(self.debugInformation))

	GH.assert(self.target)--, "Target not found for this task. (actor='"..self.actor.name.."') "..tostring(self.debugInformation))

	if (self.timer.duration <= 0) then

		self:finish()

		return false

	end

	self.startLocation = {x=self.actor.x, y=self.actor.y}

end



function PositionTask:update(time)

	GH.assert(self.actor)--, "Actor not found for this task. "..tostring(self.debugInformation))

	local finished = PositionTask.super.update(self, time)

	local f = self.ease(self.timer.fraction)

	local endLocation = Level.targetToWorldPos(self.target)

	if (self.actor) then

		self.actor.location = {

			x=self.startLocation.x * (1-f) + endLocation.x * f,

			y=self.startLocation.y * (1-f) + endLocation.y * f,

		}

	end

	return finished

end



function PositionTask:finish()

	GH.assert(self.actor)--, "Actor not found for this task. "..tostring(self.debugInformation))

	if (self.actor) then

		self.actor.locationByTarget = self.target

	end

end

-- ************************************ Gesture Task ************************************ --

GestureTask = deriveLuaClass("GestureTask", "LuaTask")

function GestureTask:init(desc)
	GestureTask.superInit(self, desc)
	self.actor.clickable=true
	self.actor.completeDuration = 500
	level.inputAlwaysEnabled = true
	self.actor.isInStory = true
	self.skippable = true
	self.storyActorName = self.storyActorName
	self.storyCompleteAnimation = self.storyCompleteAnimation
	self.storyCompleteLoop = self.storyCompleteLoop or 1
	self.storyCompleteFrameTime = self.storyCompleteFrameTime or 150
	self.storyCompleteMode = self.storyCompleteMode or "Default"
	self.storyIdleAnimation = self.storyIdleAnimation
	self.storyIdleLoop = self.storyIdleLoop or -1
	self.storyIdleFrameTime = self.storyIdleFrameTime or 150
	self.storyIdleMode = self.storyIdleMode or "Default"
	self.storyActiveAnimation = self.storyActiveAnimation
	self.storyActiveLoop = self.storyActiveLoop or -1
	self.storyActiveFrameTime = self.storyActiveFrameTime or 150
	self.storyActiveMode = self.storyActiveMode or "Default"
	self.gestureEnableSound = self.gestureEnableSound
	self.gestureIdleSound = self.gestureIdleSound
	self.gestureActiveSound = self.gestureActiveSound
	self.gestureUnlockedSound = self.gestureUnlockedSound
	self.completeSound = self.completeSound
end


function GestureTask:start()
	self.actor:reset()
	self.actor.storyActorName = self.storyActorName
	self.actor.storyCompleteAnimation = self.storyCompleteAnimation
	self.actor.storyCompleteLoop = self.storyCompleteLoop
	self.actor.storyCompleteFrameTime = self.storyCompleteFrameTime
	self.actor.storyCompleteMode = self.storyCompleteMode
	self.actor.storyIdleAnimation = self.storyIdleAnimation
	self.actor.storyIdleLoop = self.storyIdleLoop
	self.actor.storyIdleFrameTime = self.storyIdleFrameTime
	self.actor.storyIdleMode = self.storyIdleMode
	self.actor.storyActiveAnimation = self.storyActiveAnimation
	self.actor.storyActiveLoop = self.storyActiveLoop
	self.actor.storyActiveFrameTime = self.storyActiveFrameTime
	self.actor.storyActiveMode = self.storyActiveMode
	self.actor.gestureEnableSound = self.gestureEnableSound
	self.actor.gestureIdleSound = self.gestureIdleSound
	self.actor.gestureActiveSound = self.gestureActiveSound
	self.actor.gestureUnlockedSound = self.gestureUnlockedSound
	self.actor.completeSound = self.completeSound
	self.actor:showGraphic(true)

	local objs = level:getObjects()
	for _,v in ipairs(objs) do
			v:removeOverhead()
	end

end

function GestureTask:disableAllObjects()
	local objs = level:getObjects()
	for _,v in ipairs(objs) do
			if (v.old_clickable == nil) then
				v.old_clickable = v.clickable == true or v.additionalClick1 ~= nil
			end
			if(v.name ~= self.actor.name) then
				v.clickable = false
			end
	end
end

function GestureTask:restoreAllObjects()
	local objs = level:getObjects()
	for _,v in ipairs(objs) do
			v.clickable = true
	end

	local tray = level:getTray(hero)
	if (tray) then
		tray.clickable = true
	end
end

function GestureTask:update(time)
end

function GestureTask:finish()
end

function GestureTask:skip()
	self.actor.obsolete=true
end

-- ************************************ (Set) View Position Task ************************************ --

ViewPositionTask = deriveLuaClass("ViewPositionTask", "LuaTask")

function ViewPositionTask:init(desc)

	ViewPositionTask.superInit(self, desc)

	self.target = desc.target or desc.position or {x=500,y=500}



	if (not self.ease) then

		self.ease = math.linear

	elseif (not isCallable(self.ease)) then

		local options = {easeIn=math.jcurve, easeOut=math.rcurve, easeInOut=math.scurve}

		self.ease = options[self.ease] or math.linear

	end

end



function ViewPositionTask:start()

	GH.assert(self.target)--, "Target not found for this task. (actor='"..self.actor.name.."') "..tostring(self.debugInformation))

	if (self.timer.duration <= 0) then

		self:finish()

		return false

	end

	self.startLocation = {x=level.viewPosition.x, y=level.viewPosition.y}

end



function ViewPositionTask:update(time)

	local finished = ViewPositionTask.super.update(self, time)

	local f = self.ease(self.timer.fraction)

	level:setViewPosition(

		self.startLocation.x * (1-f) + self.target.x * f,

		self.startLocation.y * (1-f) + self.target.y * f)

	return finished

end



function ViewPositionTask:finish()

	level:setViewPosition(self.target.x, self.target.y)

end



-- ************************************ Interpolate Value Task ************************************ --

InterpolateTask = deriveLuaClass("InterpolateTask", "LuaTask")

function InterpolateTask:init(desc)
	PositionTask.superInit(self, desc)
	self.interpolate = self.interpolate or "alpha"

	self.useStartValue = (self.useStartValue ~= false) -- Default true
	self.startValue = self.startValue or 0
	self.endValue = self.endValue or 1
	self.resetOnEndStory = desc.resetOnEndStory ~= false

	if (not self.ease) then
		self.ease = math.linear
	elseif (not isCallable(self.ease)) then
		local options = {easeIn=math.jcurve, easeOut=math.rcurve, easeInOut=math.scurve}
		self.ease = options[self.ease] or math.linear
	end
end

function InterpolateTask:start()
	if (isString(self.interpolate)) then
		local r = tryParse("function(actor, value) "..self.interpolate.." end")
		if (isCallable(r)) then
			self.interpolate = r
		end
	end

	if (self.duration == 0) then
		self:finish()
		return false
	end

	if (self.useStartValue) then
		self:set(self.startValue or 0)
	else
		self.startValue = self:get()
	end

	if (isTable(self.startValue)) then
		self.startValue = self.startValue.x or self.startValue[1]
	end
	GH.assert(isNumber(self.startValue))
	if (level and self.resetOnEndStory) then
		level:addResetElement(self.actor.name, self.interpolate, self.startValue)
	end
end

function InterpolateTask:get()
	if (isString(self.interpolate)) then
		return self.actor[self.interpolate]
	end
	return self.interpolate(self.actor, value)
end

function InterpolateTask:set(value)
	if (isString(self.interpolate)) then
		self.actor[self.interpolate] = value
	else
		return self.interpolate(self.actor, value)
	end
end

function InterpolateTask:update(time)
	local finished = InterpolateTask.super.update(self, time)
	local f = self.ease(self.timer.fraction)
	local value = self.startValue * (1-f) + self.endValue * f
	local f = self:set(value)
--print("f: "..ts(f))
	if (f ~= nil) then
--print("f: nonnil")
		finished = f
	end

	local act = level:getSpriteExt(self.actor.name)
	act:repositionShadow()

	return finished
end

function InterpolateTask:finish()
	self:set(self.endValue)
	if (self.interpolate == "floor") then
		local act = level:getSpriteExt(self.actor.name)
		if (act) then
			act:repositionShadow()
		end
	end
end

-- ************************************ Screenshake Task ************************************ --

ScreenShakeTask = deriveLuaClass("ScreenShakeTask", "LuaTask")

function ScreenShakeTask:init(desc)

	PositionTask.superInit(self, desc)

	self.strengthX = desc.strengthX or 20

	self.strengthY = desc.strengthY or 20



	if (not self.sprite) then

		self.sprite = self.actor or level

	elseif (isString(self.sprite)) then

		self.sprite = level:resolveActor(self.sprite)

	end

	GH.assert(self.sprite, "No sprite set to shake?")



	if (not self.ease) then

		self.ease = math.linear

	elseif (not isCallable(self.ease)) then

		local options = {easeIn=math.jcurve, easeOut=math.rcurve, easeInOut=math.scurve}

		self.ease = options[self.ease] or math.linear

	end



	if (self.course == "increase") then

		self.course = math.linear

	elseif (self.course == "decrease") then

		self.course = function(f) return 1-f end

	elseif (self.course == "increaseDecrease") then

		self.course = function(f) return (f > 0.5 and 2-f*2) or f*2 end

	end



	if (not self.course) then

		self.course = function() return 1 end

	end

end



function ScreenShakeTask:start()

	self.origin = self.sprite.location

end



function ScreenShakeTask:update(time)

	local finished = ScreenShakeTask.super.update(self, time)

	local f = self.ease(self.course(self.timer.fraction))

	local sx = f*self.strengthX

	local sy = f*self.strengthY

	if (sx > 0) then

		self.sprite.x = self.origin.x + math.Random(-sx, sx)

	end

	if (sy > 0) then

		self.sprite.y = self.origin.y + math.Random(-sy, sy)

	end

	if (self.sprite.stopScreenShake) then

		return true

	end

	return finished

end



function ScreenShakeTask:finish()

	if (self.origin) then

		self.sprite.location = self.origin

	end

end



-- ************************************ Emotion Task ************************************ --

EmotionTask = deriveLuaClass("EmotionTask", "LuaTask")

function EmotionTask:init(desc)

	EmotionTask.superInit(self, desc)

	self.emotion = desc.emotion or ""

end



function EmotionTask:start()

	GH.assert(self.emotion)

	GH.assert(self.actor, "EmotionTask: Actor not set!")

	self.actor.emotion = self.emotion

	return false

end



-- ************************************ Visible Task ************************************ --

VisibleTask = deriveLuaClass("VisibleTask", "LuaTask")

function VisibleTask:start()

	if (self.actor) then
		self.actor.visible = self.visible
	elseif (self.target ~= nil) then
		local targetObject = level:getChild(self.target)
		if (targetObject ~= nil) then
			targetObject.visible = self.visible
		end
	end

	return false

end



-- ************************************ Play Animation Task ************************************ --

PlayAnimationTask = getClass("PlayAnimationTask")

function PlayAnimationTask:init(desc)
	PlayAnimationTask.superInit(self, desc)
	self.cycles = desc.cycles or 0
	self.whenFinished = desc.whenFinished or "stay"
	self.frameTime = desc.frameTime or 150
	self.wait = desc.wait or false
	self.modifiedShadow = desc.modifiedShadow or ""
	self.resetOnEndStory = desc.resetOnEndStory ~= false

	if (self.playDuringStory == "default") then
		self.playDuringStory = nil
	else
		self.playDuringStory = self.playDuringStory ~= false -- Default true
	end
end

function PlayAnimationTask:modifyShadow()
	if (self.modifiedShadow ~= "") then
		if (self.resetOnEndStory) then
			level:addResetElementString(self.actor.name, "shadowResource", self.actor.shadowResource)
		end
		self.actor.shadowResource = self.modifiedShadow
	end
end

function PlayAnimationTask:start()
	--print("["..tostring(self.actor.name).."] Play Animation: "..tostring(self.actor.baseAnimation)..", anim: "..tostring(self.animation)..", cycles: "..tostring(self.cycles)..", whenFinished: "..tostring(self.whenFinished).." frameTime: "..tostring(self.frameTime))

	self.actor:playAnimation(self.animation, self.cycles, self.frameTime, self.whenFinished, self.reversed, self.playDuringStory)
	-- Here, we calculate the frame time based on the duration and the number of cycles
	if (self.frameTime < 0 and
		isNumber(self.duration) and
		self.duration > 0 and
		self.cycles > 0)
	then
		self.actor:matchFrameTimeTo(self.duration)
	end

	if (self.loopSound) then
		self.actor:loopSample(self.loopSound, self.volume, self.pan, self.pitch)
	end
end

function PlayAnimationTask:update(time)
	PlayAnimationTask.super.update(self, time)
	-- Wait until the actor has finished animating.
	if (not self.timer.infinite and not self.timer.expired) then
		return false
	end
	if (self.wait) then
		return not self.actor.isCyclingAnimation
	end
	return true
end

function PlayAnimationTask:skip()
	self.actor:finishCycleAnimation()
end

function PlayAnimationTask:finish()
	if (self.loopSound) then
		self.actor:stopLoopSample()
	end
end

-- ************************************ Message Task ************************************ --

MessageTask = deriveLuaClass("MessageTask", "LuaTask")

function MessageTask:init(desc)
	MessageTask.superInit(self, desc)
	self.msgType = self.msgType or "message"
	self.tipType = self.tipType or "point"
	self.customTip = self.customTip
	self.tipSide = self.tipSide or "center"
	self.maxLineCount = self.maxLineCount or 4
	self.keepOpen = self.keepOpen or false
	self.needsClick = self.needsClick ~= false
	self.labelWidth = self.labelWidth or defaultMessageBalloonWidth
	self.task = self
	self.portraitSection = self.portraitSection or ""
	self.portraitAnimation = self.portraitAnimation or ""
	self.skipped = false

	if (table.contains({ "ok", "yesno", "optionList", "option", "input" }, self.msgType)) then
		self:requireUserInput()
	end

	if (self.keepOpen) then
		self.skippable = true
	end
end

function MessageTask:start()
	if (not self.actor) then
		return false
	end

	if (not self.keepOpen and
		not self.needsClick and
		self.skippable and
		taskSystem.skipping)
	then
		return false
	end

	if (self.msgType == "ok") then
		self.dlg = self.actor:showOkMessage(self)
	elseif (self.msgType == "yesno") then
		self.dlg = self.actor:showYesNoMessage(self)
	-- elseif (self.msgType == "optionList") then
	--	self.dlg = self.actor:showOptionListMessage(self)
	--elseif (self.msgType == "option") then
	--	self.dlg = self.actor:showOptionMessage(self)
	elseif (self.msgType == "input") then
		self.dlg = self.actor:showInputMessage(self)
	else
		self.dlg = self.actor:showMessageDialog(self)
	end

	if (self.needsClick) then
		level:showClickToContinue(1500)
	end

	if (self.keepOpen) then
		return true
	end
end

function MessageTask:requireUserInput()
	self.needsUserInput = true
	--self.needsClick = false
	self.skippable = false
	self.duration = -1
end

function MessageTask:update(time)
	local durationEnded = MessageTask.super.update(self, time)
	local shouldCloseDialog = durationEnded
	local dialogOpen = self.actor and self.actor:findMessageDialog(self.dlg or self.textId) ~= nil

	if (self.needsUserInput or self.needsClick) then
		shouldCloseDialog = false--not dialogOpen
	end

	if (self.skipped == true) then
		self.actor:hideMessageDialog(self.dlg or self.textId)
	end

	if (shouldCloseDialog and not self.keepOpen) then
		self.actor:hideMessageDialog(self.dlg or self.textId)
	end

	local ret = not dialogOpen or self.keepOpen
	if (ret or shouldCloseDialog) then
		if (level.hideClickToContinue) then
			level:hideClickToContinue()
		end
	end
	return ret
end

function MessageTask:skip()
	local res = true
	self.skipped = true
	--if (not self.keepOpen) then
		local dlg = self.actor:findMessageDialog(self.dlg or self.textId)
		if (dlg and dlg.onTaskSkip) then
			res = dlg:onTaskSkip() ~= false
		end
		self.actor:removeMessageDialog(self.dlg or self.textId)
		level:hideClickToContinue()
	--end
	return res
end



function MessageTask:click()
	if (self.needsClick) then
		if (level and level.BM_onMessageDialogClicked) then
			local dlg = self.actor:findMessageDialog(self.dlg or self.textId)
			level:BM_onMessageDialogClicked(dlg)
		end
		self.actor:hideMessageDialog(self.textId)
		return true
	end
	return false
end

-- ************************************ Remove Message Task ************************************ --

RemoveMessageTask = deriveLuaClass("RemoveMessageTask", "LuaTask")

function RemoveMessageTask:start()

	GH.assert(isString(self.textId), "A RemoveMessageTask needs a text ID to remove that message")

	if (self.actor.hideMessageDialog) then

		self.actor:hideMessageDialog(self.textId)

	end

	return self.wait ~= nil

end



function RemoveMessageTask:update(time)

	if (self.textId == nil) then

		return true -- Guard against infinite loop

	end

	local dlg

	if (self.actor.hideMessageDialog) then

		self.actor:hideMessageDialog(self.textId)

	end

	if (self.actor.findMessageDialog) then

		dlg = self.actor:findMessageDialog(self.textId)

	end

	return dlg == nil

end



function RemoveMessageTask:skip()

	if (self.actor.removeMessageDialog) then

		self.actor:removeMessageDialog(self.textId)

	end

end



-- ************************************ Overhead Task ************************************ --

OverheadTask = deriveLuaClass("OverheadTask", "LuaTask")

function OverheadTask:start()

	self.balloon = self.balloon or false
	self.checkParent = self.checkParent or false

	if (self.removeAtEnd and taskSystem.skipping) then

		return false

	end

	local img = self.image

	if (self.section and self.section ~= "") then

		img = img..":"..self.section

	end

	self.actor:createOverhead(img, self.soundId, self.doPop, self.showDuringStory, self.balloon, self.checkParent)

	if (self.frameTime) then

		if (self.showDuringStory) then

			local s = self.actor.overhead

			if (s) then s:animate({frameTime=self.frameTime}) end

		else

			self.actor.overheadFrameTime = self.frameTime

		end

	end

	self.overhead = self.actor.overhead

	self.duration = self.duration or 0

	return self.removeAtEnd or self.duration > 0

end



function OverheadTask:finish()

	OverheadTask.super.finish(self)

	if (self.removeAtEnd and self.actor.removeOverhead) then

		self.actor:removeOverhead(self.animated ~= false and not taskSystem.skipping)

	end

end



function OverheadTask:skip()

	if (self.removeAtEnd and self.actor.removeOverhead) then

		self.actor:removeOverhead(false)

	end

	if (self.removeAtEnd and self.overhead) then

		self.overhead.obsolete = true

	end

end



-- ************************************ Remove Overhead Task ************************************ --

RemoveOverheadTask = deriveLuaClass("RemoveOverheadTask", "LuaTask")

function RemoveOverheadTask:start()

	self.overhead = self.actor.overhead

	self.actor:removeOverhead(self.animated ~= false and not taskSystem.skipping)

	return false

end



function RemoveOverheadTask:skip()

	if (self.actor.removeOverhead) then

		self.actor:removeOverhead(false)

	end

	if (self.overhead) then

		self.overhead.obsolete = true

	end

end



-- ************************************ Leave Station Task ************************************ --

LeaveStationTask = deriveLuaClass("LeaveStationTask", "LuaTask")

function LeaveStationTask:start()

	self.actor:setState("walking to station")

	local station = self.station or self.actor.station

	if (station) then

		station:leave(self.actor)

	end

	return false

end



-- ************************************ Place Customer Task ************************************ --

PlaceCustomerTask = deriveLuaClass("PlaceCustomerTask", "LuaTask")

function PlaceCustomerTask:start()

	self.target:onObjectDrop(self.actor)

	return false

end



-- ************************************ Fade Screen Task ************************************ --

FadeScreenTask = deriveLuaClass("FadeScreenTask", "LuaTask")

function FadeScreenTask:init(desc)

	FadeScreenTask.superInit(self, desc)

	self.guiWidth = 0--300
	self.fadeIn = desc.fadeIn or 0
	self.fadeOut = desc.fadeOut or 0
	self.fadeDelay = desc.fadeDelay or 0
	self.section = desc.section or ""
	self.color = desc.color or {r=0,g=0,b=0}

	-- Added 400 so when we do a zoom in the story the fade is wide enough

--	self.width = self.width or 2048 + self.guiWidth + 400
	if (not self.blueprint) then
		self.width = self.width or 2427 + self.guiWidth + 600
		self.height = self.height or 1536
	end


	if (isString(self.color)) then

		self.color = tryParse(self.color)

	end

	self.alpha = 1

	if (self.color and self.color.a) then

		self.alpha = desc.color.a

	end

	self.draw = desc.draw or (self.color and "color") or "image"

	self.removeAtEnd = desc.removeAtEnd  ~= false

end



function FadeScreenTask:start()

	if (self.removeAtEnd == true and taskSystem.skipping == true) then

		return false

	end



	local anims = {}

	local startAlpha = self.alpha

	if (self.fadeIn > 0) then

		table.insert(anims, {alpha=self.alpha, duration=self.fadeIn})

		startAlpha = 0

	end

	if (self.fadeDelay > 0) then

		table.insert(anims, {delay=self.fadeDelay})

	end

	if (self.fadeOut > 0) then

		table.insert(anims, {alpha=0, duration=self.fadeOut})

	end



	local image, section

	if (isString(self.image) and self.image ~= "") then

		image = self.image

	end

	if (isString(image) and isString(self.section) and self.section ~= "") then

		image = image..":"..self.section

	end



	if (self.spriteName) then

		self.cover = scene:getChild(self.spriteName)

	end



	if (not self.cover) then

		if (self.blueprint) then

			self.cover = scene:getDialogLayer():newChild(blueprints[self.blueprint])

		else

            self.cover = scene:getDialogLayer():newChild({class="Sprite"})

		end

	end



	local viewPos = level.floaterLayer.location


	if (not self.blueprint) then
		self.cover:setup({
			name=self.spriteName or "COVER_"..(image or ""),
			x=-viewPos.x - self.guiWidth, y=-viewPos.y, width=self.width, height=self.height,
			alpha=startAlpha, image=image,
			color=self.color, draw=self.draw,
		})
	else
		self.cover:setup({
			name=self.spriteName or "COVER_"..(image or ""),
			alpha=startAlpha, image=image,
			color=self.color, draw=self.draw,
			})
	end





--[[

	self.cover:setup({

		name=self.spriteName or "COVER_"..(image or ""),

		x=0, y=0, width=self.width, height=self.height,

		alpha=startAlpha, image=image,

		color=self.color, draw=self.draw,

	})

]]

	if (self.removeAtEnd == true) then

		table.insert(anims, {remove=true})

	end



	self.cover:animate(anims)



	self.duration = self.fadeIn + self.fadeDelay + self.fadeOut

	return self.wait

end



function FadeScreenTask:skip()

	if (self.removeAtEnd == true) then

		if (isTable(self.cover) and self.cover.remove) then

			self.cover:remove()

		end
	else
		self.fadeIn = 0

	end
	--self:addTask({class="FadeScreenTask", fadeDelay=500, fadeOut=3000, remove = true, draw = "color"})
end



-- ************************************ Arrow Task ************************************ --

--ArrowTask = deriveLuaClass("ArrowTask", "LuaTask")

--function ArrowTask:start()

--	self.position = self.position or {0, 0}

--	self.arrow = hintManager:placeArrow({type=self.arrowType, hintId=self.id,

--		parent=self.actor, x=self.position.x, y=self.position.y})

--	if (self.actor and self.removeOnClick) then

--		self.actor.ArrowTask_old_onClick = self.actor.onClick or self.actor.ArrowTask_old_onClick or function() end

--		self.actor.onClick = function(_, x, y)

--			hintManager:removeArrows(self.id or "", self.removeAnimated ~= false)

--			self.actor.onClick = self.actor.ArrowTask_old_onClick

--			self.actor:onClick(x, y)

--		end

--	end

--	return self.wait or false

--end



-- ************************************ Remove Arrow Task ************************************ --

--RemoveArrowTask = deriveLuaClass("RemoveArrowTask", "LuaTask")

--function RemoveArrowTask:start()

--	hintManager:removeArrows(self.id or "", self.animated ~= false)

--end

--

--function RemoveArrowTask:skip()

--	if (isTable(self.arrow) and self.arrow.remove) then

--		hintManager:removeArrows(self.arrow, false)

--	end

--end



-- ************************************ Play Music Task ************************************ --

PlayMusicTask = deriveLuaClass("PlayMusicTask", "LuaTask")

function PlayMusicTask:start()
	-- Avalable music: "MENU" "<roomname>"
	-- Avalable music moods: "HAPPY" "SAD" "-"
	self.loopMusic = (self.loopMusic ~= false) -- Guard against nil
	self.quickFade = (self.quickFade ~= false) -- Guard against nil
	if (self.volume == nil) then
		self.volume = 1
	end
	level:setMusicAndMood(
		self.music or level.music,
		self.mood or level.musicMood,
		self.quickFade,
		self.loopMusic,
		self.volume)
	return false
end



-- ************************************ Fade Music Task ************************************ --

FadeMusicTask = deriveLuaClass("FadeMusicTask", "LuaTask")

function FadeMusicTask:start()

	level:fadeMusic(self.volume, self.duration)

	return self.wait ~= true

end



-- ************************************ Sit Task ************************************ --

SitTask = deriveLuaClass("SitTask", "LuaTask")

function SitTask:init(desc)

	SitTask.superInit(self, desc)

	GH.assert(self.chair)

	self.chairId = self.chair

end



function SitTask:start()

	GH.assert(self.actor)

	self.chair = level:getSpriteExt(self.chair)

	GH.assert(self.chair, "Chair not found: '"..tostring(self.chairId).."'")

	self.chair:place(self.actor)

	return false

end



-- ************************************ Stand Up Task ************************************ --

StandUpTask = deriveLuaClass("StandUpTask", "LuaTask")

function StandUpTask:init(desc)

	SitTask.superInit(self, desc)

end



function StandUpTask:start()

	GH.assert(self.actor)

	if (self.actor.station) then

		self.actor.station:standUp()

	end

	return false

end



-- ************************************ Set Character Outfit Task ************************************ --

SetCharacterOutfit = deriveLuaClass("SetCharacterOutfit", "LuaTask")

function SetCharacterOutfit:init(desc)
	SetCharacterOutfit.superInit(self, desc)
end

function SetCharacterOutfit:start()
	self.actor.defaultSection = self.defaultSection or ""
	self.actor.baseAnimation = self.actor.defaultSection
	return false

end



-- ************************************ Play Default Animation Task ************************************ --

PlayDefaultAnimationTask = deriveLuaClass("PlayDefaultAnimationTask", "LuaTask")

function PlayDefaultAnimationTask:start()

	GH.assert(self.actor, "Actor not found for \"PlayDefaultAnimationTask\"")

	self.actor:playDefaultAnimation()

	return false

end

-- ************************************ Play Default Animation With Mood Task ************************************ --

PlayDefaultAnimationWithMoodTask = deriveLuaClass("PlayDefaultAnimationWithMoodTask", "LuaTask")

function PlayDefaultAnimationWithMoodTask:start()

	GH.assert(self.actor, "Actor not found for \"PlayDefaultAnimationWithMoodTask\"")

	self.actor:playDefaultAnimationWithMood()

	return false

end

-- ************************************ Walk To Stationary Position Task ************************************ --

WalkToStationaryPositionTask = deriveLuaClass("WalkToStationaryPositionTask", "WalkToTask")

function WalkToStationaryPositionTask:init(desc)

	WalkToStationaryPositionTask.superInit(self, desc)

	local actor = level:getCharacter(desc.actor)

	if (actor and actor.waitPosition) then

		self.target = actor.waitPosition

		self.playDefaultOnFinish = true

	else

		GH.assert(false, desc.actor.." : is not a stationary character")

	end

end



-- ************************************ Upgrades Walk To Task ************************************ --

UpgradesWalkToTask = deriveLuaClass("UpgradesWalkToTask", "WalkToTask")

function UpgradesWalkToTask:init(desc)

	UpgradesWalkToTask.superInit(self, desc)

	self.exitPosition = desc.exitPosition or {x=0,y=0}

	local actor = level:getCharacter(desc.actor)

	if (actor.createdInAnimation) then

		self.target = desc.exitPosition

		actor:addTask({class="RemoveTask"})

	end

end



-- ************************************ Reset Animation Walk Speed Task ************************************ --

ResetAnimationWalkSpeedTask = deriveLuaClass("ResetAnimationWalkSpeedTask", "LuaTask")

function ResetAnimationWalkSpeedTask:start()

	if (self.actor.originalAnimationWalkSpeed) then

		self.actor.animationWalkSpeed = self.actor.originalAnimationWalkSpeed

		return false

	end

	local traits = self.traits or self.actor.traits or self.actor.name

	GH.assert(traits, "Could not find 'traits' for '"..tostring(self.actor.name).."'")

	self.actor.animationWalkSpeed = characterTypes[traits].animationWalkSpeed

	return false

end



-- ************************************ Set Appearance Task ************************************ --

SetAppearanceTask = deriveLuaClass("SetAppearanceTask", "LuaTask")

function SetAppearanceTask:start()

	local selection = ""



	if (self.selectionId) then

		selection = player:getUpgradeSelection(self.selectionId)



		if (selection == "") then

			selection = self.appearanceValue

		end



		self.appearanceValue = ""

	end



	self.actor:setAppearance(self.appearanceId, (self.appearanceValue or "")..selection, self.appearanceWorth)

	return false

end



-- ************************************ Remove Appearance Task ************************************ --

RemoveAppearanceTask = deriveLuaClass("RemoveAppearanceTask", "LuaTask")

function RemoveAppearanceTask:start()

	self.actor:removeAppearance(self.appearanceId or "")

	return false

end



-- ************************************ Allow Idle Animations Task ************************************ --

AllowIdleAnimationsTask = deriveLuaClass("AllowIdleAnimationsTask", "LuaTask")

function AllowIdleAnimationsTask:start()

	self.actor.playIdleAnimations = self.playIdleAnimations -- Default FALSE

	return false

end



-- ************************************ Restock Task ************************************ --

RestockTask = deriveLuaClass("RestockTask", "LuaTask")



function RestockTask:init(desc)

	RestockTask.superInit(self, desc)



	if (isString(self.obj)) then

		self.obj = level:getObject(self.obj)

	end



	if (self.obj) then

		self.restockPos = self.restockPos or self.obj.workPos

	end

end



function RestockTask:start()



	local res = self.obj and self.obj.usesStock and self.obj.stock < self.obj.maxStock

	if (res) then

		self.obj.restockActive = true



		if (self.actor and self.actor.restockAnimation) then

			self.actor.direction = self.obj.workDirection

			local frameTime = self.actor.restockFrameTime

			self.actor:playAnimation(self.actor.restockAnimation, -1, frameTime, self.actor.restockWhenFinished)

			if ((not frameTime or frameTime < 0) and self.actor.frameCount > 0) then

				self.actor.currentFrameTime = self.obj.restockInterval / self.actor.frameCount;

			end

		end

	end



	return res

end



function RestockTask:update(time)

	return self.obj.stock >= self.obj.maxStock

end



function RestockTask:finish()

	self.obj.restockActive = false

end



function RestockTask:getWorkTarget()

	return {x=self.restockPos.x, y=self.restockPos.y}

end



-- ************************************ Option Choose Task ************************************ --

OptionChooseTask = deriveLuaClass("OptionChooseTask", "LuaTask")



function OptionChooseTask:init()

	OptionChooseTask.superInit(self, desc)

	self.skippable = false

end



function OptionChooseTask:start()

	--level:showClickToContinue(1500)

	local momentDesc = table.copy(self)

	momentDesc.__cpp = nil

	momentDesc.class = "OptionChooseDialog"

	self.moment = scene:newChild(momentDesc)

	return self.moment ~= nil

end



function OptionChooseTask:update(time)

	return

		not self.moment or

		self.moment.__deleted

end



-- ************************************ Stop Ferris Wheel Task ************************************ --

StopFerrisTask = deriveLuaClass("StopFerrisTask", "LuaTask")



function StopFerrisTask:init()

	StopFerrisTask.superInit(self, desc)

	self.target = self.target or "ferris"

	self.ferrisObject = level:getObject(self.target)

	GH.assert(self.ferrisObject)

end



function StopFerrisTask:start()

	self.ferrisObject:stopAtNearest()

	return self.wait or false

end



function StopFerrisTask:update(time)

	return self.ferrisObject.rSpeed == 0

end



function StopFerrisTask:finish()

	self.ferrisObject.r = 0

	self.ferrisObject.rTarget = 0

	self.ferrisObject.wheelState = "blocked"

	self.ferrisObject:updateCartPositions()

end



function StopFerrisTask:skip()

	self:finish()

end



---- ************************************ Poem Moment Task ************************************ --

--PoemMomentTask = deriveLuaClass("PoemMomentTask", "LuaTask")

--

--function PoemMomentTask:start()

--	GH.assert(self.momentImage)

--	GH.assert(self.momentPosition)

--

--	level:showClickToContinue(1500)

--

--	local selection = ""

--	if (self.selectionId) then

--		selection = player:getUpgradeSelection(self.selectionId)

--	end

--	if (selection == "" and self.fallbackId) then

--		selection = self.fallbackId

--	end

--

--	local poemMomentDesc = {

--		class = "PoemMoment",

--		momentImage = self.momentImage..selection,

--		momentPosition = self.momentPosition,

--		subjects = self.subjects or self.actor,

--	}

--	self.moment = scene:newChild(poemMomentDesc)

--

--	return self.moment ~= nil

--end

--

--function PoemMomentTask:update(time)

--	return

--		not self.moment or

--		self.moment.__deleted

--end

--

--function PoemMomentTask:skip()

--	if (isTable(self.moment) and self.moment.remove and not self.moment.__deleted) then

--		self.moment:hideMoment(true)

--	end

--end

--

--function PoemMomentTask:click()

--	if (isTable(self.moment) and self.moment.remove and not self.moment.__deleted) then

--		self.moment:hideMoment(false)

--	end

--	return true

--end



-- ************************************ Move Screen Task ************************************ --

MoveScreenTask = deriveLuaClass("MoveScreenTask", "LuaTask")



function MoveScreenTask:start()

	self.side = self.side or "left"

	self.direction = self.direction or "in"

	GH.assert(self.distance or self.image)



	self.axis = ((self.side == "left" or self.side == "right") and "x") or "y"



	if (self.image) then

		self.sprite = level:add({class="SpriteExt", baseAnimation=self.image, floor=self.floor})



		local sizeAxis = (self.axis == "x" and "width") or "height"

		local size = self.sprite[sizeAxis]

		local offset = -size

		if (self.side == "right") then

			offset = 1024

		elseif (self.side == "bottom") then

			offset = 768

		end

		self.sprite[self.axis] = offset



		self.distance = self.distance or size

	end



	self.startValue = self.distance

	self.endValue = 0

	if (self.direction == "out") then

		self.startValue, self.endValue = self.endValue, self.startValue

	end

	if (self.side == "right" or self.side == "bottom") then

		self.startValue, self.endValue = self.endValue, self.startValue

	end



	--if (isString(self.ease)) then

	--	local options = {easeIn=math.jcurve, easeOut=math.rcurve, easeInOut=math.scurve}

	--	self.ease = options[self.ease]

	--end

	--self.ease = self.ease or math.linear



	self.nodeToMove = level.gameLayer



	self.delay = self.delay or 0

	local delay = {delay=self.delay}

	local anim = {duration=self.duration, style=self.ease}

	anim[self.axis] = self.endValue

	self.nodeToMove[self.axis] = self.startValue

	self.nodeToMove:animate(delay,anim)

	if (self.direction == "in" and self.sprite) then

		self.sprite:animate({delay=self.duration+self.delay},{remove=true})

	end



	return self.wait -- Default don't wait

end



function MoveScreenTask:update(time)

	--local frac = self.ease(self.timer.clampedFraction)

	--self.nodeToMove[self.axis] = math.lerp(frac, self.startValue, self.endValue)

	if (self.wait and self.delay > 0) then

		self.delay = self.delay - time

		return false

	end

	return (not self.wait) or MoveScreenTask.super.update(self, time)

end



function MoveScreenTask:finish()

	if (self.wait) then

		if (self.direction == "in") then

			if (self.sprite) then

				self.sprite:remove()

				self.sprite = nil

			end

			self.nodeToMove[self.axis] = self.endValue

		end

	end

end



function MoveScreenTask:skip()

	if (self.direction == "in") then

		if (self.sprite) then

			self.sprite:remove()

			self.sprite = nil

		end

		self.nodeToMove[self.axis] = self.endValue

	end

end





-- ************************************ Start Particle Task ************************************ --

StartParticleTask = deriveLuaClass("StartParticleTask", "LuaTask")

function StartParticleTask:start()

	if (self.actor) then

		self.actor:startEffect()

	end

	if (self.duration > 0) then

		self.doStop = true

	end

end



function StartParticleTask:finish()

	if (self.doStop == true) then

		self.actor:stopEffect()

	end

end





-- ************************************ Stop Particle Task ************************************ --

StopParticleTask = deriveLuaClass("StopParticleTask", "LuaTask")

function StopParticleTask:start()

	if (self.actor) then

		self.actor:stopEffect()

	end

end





-- ************************************ Particle Task ************************************ --

ParticleTask = deriveLuaClass("ParticleTask", "LuaTask")



function ParticleTask:start()

	GH.assert(self.effect)

	if (self.parent == "level") then

		self.parent = nil

	elseif (self.parent == "topLayer") then

		self.parent = level.topLayer

	else

		if (isString(self.parent)) then

			self.parent = level:resolveSprite(self.parent)

		end

	end

	if (not self.parent) then

		level.particleNode = level.particleNode or level:add({class="SpriteExt", floor=10000})

		self.parent = level.particleNode

	end



	local parentPos = self.parent.location

	local effectPos = self.position or self.parent.floaterPosition or self.parent.location

	effectPos.x = effectPos.x - parentPos.x

	effectPos.y = effectPos.y - parentPos.y



	self.particles = self.parent:newChild({

		class="ParticleEffect",

		name=self.name or "unnamed particle task effect",

		location=effectPos,

		alpha=self.alpha,

		library=self.library or "effects:particles",

		emitter=self.effect,

		scale = self.scale or 1,

		play=self.play ~= false, -- Default true

		--emitterEnabled=self.emitterEnabled,

		removeWhenFinished=self.removeWhenFinished ~= false, -- Default true

		maxIdleTime=self.maxIdleTime,

		loop=self.loop,

		speedMultiplier=self.speedMultiplier,

	})



	if (self.initialTime) then

		self.particles:addInitialTime(self.initialTime)

	end



	if (self.neverIdle) then

		self.particles:neverMaxIdleTime()

	end



	return self.wait or false -- Default don't wait

end



function ParticleTask:update(time)

	if (self.timer.duration > 0) then

		return ParticleTask.super.update(self, time)

	end

	return not self.particles or not self.playing

end



function ParticleTask:finish()

	if (self.removeWhenFinished) then

		if (self.particles) then

			self.particles.obsolete = true

		end

	end

end



-- ************************************ Remove Particle Task ************************************ --

RemoveParticleTask = deriveLuaClass("RemoveParticleTask", "LuaTask")



function RemoveParticleTask:start()

	GH.assert(self.name)

	GH.assert(self.name ~= "")

	self.particles = level:getChild(self.name)

	GH.assert(self.particles)



	if (self.stopEmitting ~= false) then

		self.particles.emitterEnabled = false

		self.particles.removeWhenFinished = true

	end



	self.startAlpha = self.particles.alpha

	return true

end



function RemoveParticleTask:update(time)

	local finished = false

	if (self.timer.duration > 0) then

		if (self.fade) then

			local f = self.timer.clampedFraction

			self.particles.alpha = math.lerp(f, self.startAlpha, 0)

		end

		finished = RemoveParticleTask.super.update(self, time)

	end

	return finished or not self.particles or not self.playing

end



function RemoveParticleTask:finish()

	if (self.particles) then

		if (self.stopEmitting) then

			self.particles.emitterEnabled = false

			self.particles.removeWhenFinished = true

		else

			self.particles.obsolete = true

		end

	end

end



-- ************************************ Floaty Memory Task ************************************ --

FloatyMemoryTask = deriveLuaClass("FloatyMemoryTask", "LuaTask")



function FloatyMemoryTask:start()

	self.obj = level.floaterLayer:newChild({class="Sprite", image=self.image, anchor="center"})

	self.duration = self.duration or 3000

	self.angleStart = math.Random(math.pi*2)

	self.freqX = math.Random(1000, 3000)

	self.freqY = math.Random(1000, 3000)

	self.pos = self.pos or {x=0,y=0}

	self.amp = self.amp or 40

end



function FloatyMemoryTask:update(time)

	local finished = FloatyMemoryTask.super.update(self, time)

	local f = self.timer.fraction

	local amp = math.scurve(math.ramp(f)) * self.amp

	self.obj.x = self.pos.x + amp * math.sin(self.angleStart + self.timer.time/self.freqX)

	self.obj.y = self.pos.y + amp * math.cos(self.angleStart + self.timer.time/self.freqY)

	self.obj.alpha = math.scurve(math.ramp(f, 0.2))

	return finished

end



function FloatyMemoryTask:finish()

	if (self.obj) then

		self.obj:remove()

	end

end



-- ************************************ Fade Memory Task ************************************ --

FadeMemoryTask = deriveLuaClass("FadeMemoryTask", "LuaTask")



function FadeMemoryTask:start()

	self.obj = level.floaterLayer:newChild({class="Sprite", image=self.image, anchor="center"})

	self.duration = self.duration or 3000

	self.angleStart = math.Random(math.pi*2)

	self.pos = self.pos or {x=0,y=0}

	if (self.obj) then

		self.obj.x = self.pos.x

		self.obj.y = self.pos.y

	end

end



function FadeMemoryTask:update(time)

	local finished = FadeMemoryTask.super.update(self, time)

	local f = self.timer.fraction

	if (self.obj) then

		self.obj.alpha = math.scurve(math.ramp(f, 0.2))

	end

	return finished

end



function FadeMemoryTask:finish()

	if (self.obj) then

		self.obj:remove()

	end

end



-- ************************************ Lyrics Task ************************************ --

LyricsTask = deriveLuaClass("LyricsTask", "LuaTask")



function LyricsTask:init(desc)

	LyricsTask.superInit(self, desc)

	self.textId = desc.textId or "DEFAULT"

	self.doFade = desc.doFade ~= false

	self.fadeDuration = desc.fadeDuration or 500

	self.noteImage = desc.noteImage

	self.leftNoteX = desc.leftNoteX or 200

	self.rightNoteX = desc.rightNoteX or 1848

end



function LyricsTask:start()

	self.obj = level.floaterLayer:newChild({

		class="Label",

		setup=setups.outline_small,

		color0=0xffffff, color1=0x222222,

		textSize=50,

		anchor="center",

		textId=self.textId,

		floor=2048

	})



	-- Add the two notes sprites

	if (self.noteImage) then

		self.leftNote = level.floaterLayer:newChild( {

			class="Sprite",

			name="leftNote",

			x=self.leftNoteX, y=self.pos.y,

			image=self.noteImage,

			anchor="center"

		})

		self.rightNote = level.floaterLayer:newChild( {

			class="Sprite",

			name="rightNote",

			x=self.rightNoteX, y=self.pos.y,

			image=self.noteImage,

			anchor="center"

		})

	end



	self.duration = self.duration or 3000

	self.pos = self.pos or {x=0,y=0}

	if (self.obj) then

		self.obj.x = self.pos.x

		self.obj.y = self.pos.y

	end

	self.actualDuration = 0

	if (self.doFade == true) then

		self.obj.alpha = 0

		if (self.leftNote) then

			self.leftNote.alpha = 0

		end

		if (self.rightNote) then

			self.rightNote.alpha = 0

		end

	end

end



function LyricsTask:update(time)

	self.actualDuration = self.actualDuration + time



	if (self.doFade == true) then

		local alphaValue = 1

		if (self.actualDuration < self.fadeDuration) then

			alphaValue = self.actualDuration / self.fadeDuration

		elseif (self.actualDuration > self.duration - self.fadeDuration) then

			alphaValue = (self.duration - self.actualDuration) / self.fadeDuration

		end

		self.obj.alpha = alphaValue

		if (self.leftNote) then

			self.leftNote.alpha = alphaValue

		end

		if (self.rightNote) then

			self.rightNote.alpha = alphaValue

		end

	end



	local finished = LyricsTask.super.update(self, time)

	return finished

end



function LyricsTask:finish()

	if (self.obj) then

		self.obj:remove()

	end

	if (self.rightNote) then

		self.rightNote:remove()

	end

	if (self.leftNote) then

		self.leftNote:remove()

	end

end



-- ****************************** Coin ∫Animation Task ***************************** --

CoinAnimationTask = deriveLuaClass("CoinAnimationTask", "LuaTask")



function CoinAnimationTask:init(desc)

	CoinAnimationTask.superInit(self, desc)

	self.duration = 1800

end



function CoinAnimationTask:start()

	self.fountain = level:getObject("wish_fountain_controller")

	if (self.fountain) then

		self.fountain:doCoinAnimation(self.duration)

	end

end



function CoinAnimationTask:update(time)

	local finished = CoinAnimationTask.super.update(self, time)

	return finished

end



-- function CoinAnimationTask:finish()

--

-- end

-- ************************************ Show Focus Task ************************************ --

SetFocusTask = deriveLuaClass("SetFocusTask", "LuaTask")



function SetFocusTask:start()

	self.focusType = self.focusType or "Focus"

	self.focusScale = self.focusScale or 1

	self.focusSpeed = self.focusSpeed or 0.05

	self.focusAlpha = self.focusAlpha or 1

	self.followTarget = self.followTarget or nil

	if (self.followTarget == nil) then

		if (self.actor) then

			self.followTarget = self.actor.name

		elseif (hero) then

			self.followTarget = hero.name

		else

			self.followTarget = "UNKNOWN"

		end

	end



	level.getFocusLayer.setTargetScale = self.focusScale

	level.getFocusLayer.setFollowOffset = self.focusOffset

	level.getFocusLayer.setFollowTarget = self.followTarget

	level.getFocusLayer:show(self.focusType, self.focusAlpha)



	if(self.customImage and self.customImage ~= "") then

		level.getFocusLayer.setCustomImage = self.customImage

	end



	if(self.showInFront == true) then

		level.getFocusLayer:showTargetInFront()

	end

end



-- ************************************ Hide Focus Task ************************************ --

RemoveFocusTask = deriveLuaClass("RemoveFocusTask", "LuaTask")



function RemoveFocusTask:start()

	level.getFocusLayer:hide()

end



function RemoveFocusTask:skip()

	level.getFocusLayer:skip()

end


-- ************************************ Paywall Task ************************************ --
PaywallTask = deriveLuaClass("PaywallTask", "LuaTask")

function PaywallTask:init()
    PaywallTask.superInit(self, desc)
    self.skippable = false

	local productId = inAppPurchaseIds.EPISODES_PACK1
	if (player.unlockedEpisode == 3 or player.unlockedEpisode == 4) then
		productId = inAppPurchaseIds.EPISODES_PACK2
	end
	if (player.unlockedEpisode == 5 or player.unlockedEpisode == 6) then
		productId = inAppPurchaseIds.EPISODES_PACK3
	end
	local purchased = GH.inAppPurchaseIsBought(productId) or GH.inAppPurchaseIsBought(inAppPurchaseIds.EPISODES_ALL)
	if(purchased ~= true) then
		player:showPaywallFromStory(true)
	end
end

function PaywallTask:start()

end

-- ************************************ DisableBlinkTask Task ************************************ --
DisableBlinkTask = deriveLuaClass("DisableBlinkTask", "LuaTask")

function DisableBlinkTask:init()
    DisableBlinkTask.superInit(self, desc)
end

function DisableBlinkTask:start()
	if (self.actor and self.actor.disableBlink) then
		self.actor:disableBlink()
	end
end

-- ************************************ ConnectTwoPoint Task ************************************ --
ConnectTwoPointsTask = deriveLuaClass("ConnectTwoPointsTask", "LuaTask")

function ConnectTwoPointsTask:init(desc)
	ConnectTwoPointsTask.superInit(self, desc)

	self.fixedPoint = desc.fixedPoint
	self.movablePoint = desc.movablePoint
	self.targetPoint = desc.targetPoint

	if (self.actor) then
		self.actor.anchor = "bottom"
		self.actor.x = self.fixedPoint.x
		self.actor.y = self.fixedPoint.y
		self.actor.height = self:calculateDistance(self.fixedPoint.x, self.fixedPoint.y, self.movablePoint.x, self.movablePoint.y)
		self.actor.rotation = self:calculateRotation(self.fixedPoint.x, self.fixedPoint.y, self.movablePoint.x, self.movablePoint.y)
	end

	if (not self.ease) then
		self.ease = math.linear
	elseif (not isCallable(self.ease)) then
		local options = {easeIn=math.jcurve, easeOut=math.rcurve, easeInOut=math.scurve}
		self.ease = options[self.ease] or math.linear
	end
end

function ConnectTwoPointsTask:start()
	if (self.actor) then
		self.actor.location = { x=self.fixedPoint.x, y=self.fixedPoint.y, }
	end
end

function ConnectTwoPointsTask:update(time)
	local finished = ConnectTwoPointsTask.super.update(self, time)
	if (self.actor) then
		local f = self.ease(self.timer.fraction)
		local xPoint = self.movablePoint.x * (1-f) + self.targetPoint.x * f
		local yPoint = self.movablePoint.y * (1-f) + self.targetPoint.y * f

		self.actor.height = self:calculateDistance(self.fixedPoint.x, self.fixedPoint.y, xPoint, yPoint)
		self.actor.rotation = self:calculateRotation(self.fixedPoint.x, self.fixedPoint.y, xPoint, yPoint)
	end
	return finished
end

function ConnectTwoPointsTask:finish()
	if (self.actor) then
		self.actor.height = self:calculateDistance(self.fixedPoint.x, self.fixedPoint.y, self.targetPoint.x, self.targetPoint.y)
		self.actor.rotation = self:calculateRotation(self.fixedPoint.x, self.fixedPoint.y, self.targetPoint.x, self.targetPoint.y)
	end
end

function ConnectTwoPointsTask:calculateDistance(xOrig, yOrig, xPos, yPos)
    local vector = {x=xPos-xOrig, y=yPos-yOrig}
    local powDistance = vector.x*vector.x + vector.y*vector.y
    return math.sqrt(powDistance)
end

function ConnectTwoPointsTask:calculateRotation(xOrig, yOrig, xPos, yPos)
    -- Calculate the angle between the xPos, yPos vector and the vertical
    -- axis.
    local hVector = {x=xPos-xOrig, y=yPos-yOrig}
    local dVector = {x=0, y=yPos-yOrig}
    local dotProduct = hVector.x*dVector.x + hVector.y*dVector.y
    local modH = hVector.x*hVector.x + hVector.y*hVector.y
    modH = math.sqrt(modH)
    local modD = dVector.x*dVector.x + dVector.y*dVector.y
    modD = math.sqrt(modD)
    local result = dotProduct / (modH * modD)
    local angle = math.acos(result)
    angle = math.deg(angle)
    if (xOrig > xPos) then
        angle = -angle
    end
    return angle
end
