dofile("script/events/Event.lua")

-- ************************************ AnimationsEvent Event ************************************ --
AnimationsEvent = deriveLuaClass("AnimationsEvent", "Event")

function AnimationsEvent:init(desc)
	AnimationsEvent.superInit(self, desc)

	-- Status
	self.currentCheckUpId = 1
	self.checkUpStatus = "offscreen"

	self.eventObjectIds = {}
	if (desc.eventObjectId) then
		table.push_back(self.eventObjectIds, desc.eventObjectId)
	else
		for i=1,8 do
			local id = desc["eventObjectId"..i]
			if (id) then
				table.push_back(self.eventObjectIds, id)
			end
		end

		if (#self.eventObjectIds == 0) then
			table.push_back(self.eventObjectIds, "eventObject")		
		end
	end

	self.nrOfAnimations = desc.nrOfAnimations or 5
	self.minNextAnimationDuration = desc.minNextAnimationDuration or 0
	self.maxNextAnimationDuration = desc.maxNextAnimationDuration or 0

	self.noTimers = true
	if (self.maxNextAnimationDuration > 0) then
		self.noTimers = false
	end
end

function AnimationsEvent:setupChallenge()
	level.challengeManager:setupEventChallenge(self.nrOfAnimations, true)

	if (self.noTimers) then
		self:startWalkIn()
	else
		self.stateTimer = math.Random(self.minNextAnimationDuration, self.maxNextAnimationDuration)
	end
end

function AnimationsEvent:onTick(time)
	if (not self.started or self:areAllItemsDone()) then
		return
	end

	if (not self.noTimers) then
		if (self.checkUpStatus == "offscreen" and self.stateTimer) then
			self.stateTimer = self.stateTimer - time
			if (self.stateTimer <= 0) then
				self.checkUpStatus = "checking up"
				self:startWalkIn()
			end
		end
	end
end

function AnimationsEvent:startWalkIn(doAnim)
	local idx = math.min(#self.eventObjectIds, self.currentCheckUpId)

	-- Maybe self:handleTrigger needs the eventObject...
    if (idx > 0) then
        self.eventObject = level:getObject(self.eventObjectIds[idx]) or self.eventObject
    end

	local anim = taskSystem:createAnimation("eventObjectWalkIn"..self.currentCheckUpId)
	if (anim == nil) then
		anim = taskSystem:createAnimation("eventObjectWalkIn")
		if (not anim) then
			self:handleTrigger("walkInComplete", {})
		end
	end

	-- Maybe the animation has created the eventObject...
    if (idx > 0) then
        self.eventObject = level:getObject(self.eventObjectIds[idx]) or self.eventObject
    end
end

function AnimationsEvent.eventObject_onClick(obj)
	local anim = taskSystem:createAnimation("eventObjectWork"..event.currentCheckUpId)
	if (anim == nil) then
		taskSystem:createAnimation("eventObjectWork")
	end

	obj.clickable = false
	obj:removeOverhead(true)
end

function AnimationsEvent:handleTrigger(trigger, data)
	AnimationsEvent.super.handleTrigger(self, trigger, data)
	if (trigger == "checkUpComplete") then
		self.currentCheckUpId = self.currentCheckUpId + 1
		self:scoreItemDone({obj=self.eventObject})
		self:onProgress()

		self.eventObject.clickable = false
		local hasWon = self:areAllItemsDone()
		if (hasWon) then

			if (self.walkOutOnEventWOn and not self.noTimers) then
				local anim = taskSystem:createAnimation("eventObjectWalkOut"..self.currentCheckUpId)
				if (anim == nil) then
					anim = taskSystem:createAnimation("eventObjectWalkOut")
					if (not anim) then
						self.skipWalkout = true
					end
				end
			end

			self:eventWon()
		else
			self.skipWalkout = self.noTimers
			if (not self.skipWalkout) then
				local anim = taskSystem:createAnimation("eventObjectWalkOut"..self.currentCheckUpId)
				if (anim == nil) then
					anim = taskSystem:createAnimation("eventObjectWalkOut")
					if (not anim) then
						self.skipWalkout = true
					end
				end
				self.checkUpStatus = "walkingOut"
				self.eventObject.clickable = false
			end

			local idx = math.min(#self.eventObjectIds, self.currentCheckUpId)
			if (idx > 0) then
				self.eventObject = level:getObject(self.eventObjectIds[idx]) or self.eventObject
			end

			if (self.skipWalkout) then
				self.checkUpStatus = "offscreen"
				self.eventObject.clickable = true
				self.eventObject.onClick = self.eventObject_onClick
			end
		end

	elseif (trigger == "walkOutComplete") then
		self.checkUpStatus = "offscreen"
		self.stateTimer = math.Random(self.minNextAnimationDuration, self.maxNextAnimationDuration)

	elseif (trigger == "walkInComplete") then
		self.eventObject.clickable = true
		self.eventObject.onClick = self.eventObject_onClick
	end
end

function AnimationsEvent:areAllItemsDone()
	return self.currentCheckUpId > self.nrOfAnimations
end

function AnimationsEvent:getProgress()
	return (self.currentCheckUpId-1) /self.nrOfAnimations
end

function AnimationsEvent:getNumItemsTodoTotal()
	return self.nrOfAnimations
end
