dofile("script/events/Event.lua")


-- USAGE EXAMPLE
--[[
levelDesc.eventDesc =
{
	class = "AttentionTellStoryEvent",
	autoStart = true,
	scoreAllDone = 100,
	showDecreaseBar = true,
	barMaxTime = 30000,
	coolDownBar = 5000,
	targetCharacter = "mary_lynn",
}
]]--


AttentionTellStoryEvent = deriveLuaClass("AttentionTellStoryEvent", "Event")


function AttentionTellStoryEvent:init(desc)

	AttentionTellStoryEvent.superInit(self, desc)

	self.eventDone = false

	self.eventState = "Resting"

	self.barMaxTime = desc.barMaxTime or 30000
	self.barCurrentTime = desc.barMaxTime


	self.barBlueprint = "decreasingbar_requestAttention"
	self.barPosition = desc.barPosition or {x = 0, y = 0}

	self.barBlinkFadeDuration = 100
	self.fadeDuration = 200

	self.previousAnim = 3

	self.coolDownBar = desc.coolDownBar or 5000

	if (desc.attentionIcon) then

		self.attentionIcons = self.attentionIcons or {}

		if (isTable(desc.attentionIcon)) then

			for _,v in ipairs(desc.attentionIcon) do

				GH.assert(isString(v))

				table.insert(self.attentionIcons, v)

			end

		elseif (isString(desc.attentionIcon)) then

			table.insert(self.attentionIcons, desc.attentionIcon)

		end

	end


	self.targetObject = desc.targetObject

	self.targetCharacter = desc.targetCharacter

end



function AttentionTellStoryEvent:setupChallenge()

	--level.challengeManager:setupEventChallenge(math.max(1, self.targetGoal), true)

end

function AttentionTellStoryEvent:onInitialized()
	AttentionTellStoryEvent.super.onInitialized(self)
	local object = nil
	if (isTable(self.targetCharacter)) then
		if (isTable(v)) then
			object = level:createCharacter(v)
		elseif (isString(v)) then
			object = level:resolveActor(v)
		end
	elseif (isString(self.targetCharacter)) then
		object = level:resolveActor(self.targetCharacter)
	elseif (isString(self.targetObject)) then
		object = level:getObject(self.targetObject)
	end
	if (object) then
		self.targetObject = object
	end
	GH.assert(self.targetObject, "Problem fetching target object");
	self.targetObject.onClick = AttentionTellStoryEvent.targetCharacter_onClick
	self.targetObject.isEventCharacter = true
	self.targetObject.clickable = true
	self:CreateDecreasingBar()
	self:HideBar(false)

	-- Add this function for the special DisableBlinkTask
	if (self.targetObject) then
		local tobj = self.targetObject
		self.targetObject.disableBlink = function()
			tobj.blinkLikeProduct = false
		end
	end
end

function AttentionTellStoryEvent:CreateDecreasingBar()

	self.decreasingBar = self.targetObject:addChild(blueprints[self.barBlueprint])

	self.barPosition.y = self.barPosition.y - (self.targetObject.height * 0.95)

	self.decreasingBar.location = self.barPosition
	self.decreasingBar.floor = 0
	self.decreasingBar:setProgress(1)
end


function AttentionTellStoryEvent:onStart()
	AttentionTellStoryEvent.super.onStart(self)
	self:ShowBar(false, true)

end

function AttentionTellStoryEvent:onTick(time)

    if (not self.eventDone and level.isPlaying) then
	    self:CheckEventStatus()
    end

	if (self.eventDone or not self.started or level.elapsedTimeFraction >= 1) then
		return
	end

	if (self.eventState == "Resting") then
		self.barCurrentTime = self.barCurrentTime - time

		-- Progress bar actually decreasing
		if (self.barCurrentTime < self.barMaxTime) then
			if (self.decreasingBar.alpha == 0) then
				self:ShowBar(false, true)
			end

			self:SetMood(math.clamp(self.barCurrentTime / self.barMaxTime, 0, 1))

		-- Progress bar with extra cooldown (not decreasing yet)
		else
			if (self.decreasingBar.alpha == 1) then
				self:HideBar(false)
			end
		end
	end

end

function AttentionTellStoryEvent.targetCharacter_onClick(self)
	self.clickable = false
	event.targetObject.clickable = false
	-- Use self because the function targetCharacter_onClick parameter is the
	-- clickable object.
	event.eventState = "Queued"

	self:getWorkCharacter():addTask({class="CallTask",
		func=function()
			event.eventState = "Telling"
			event:onStartTelling()
		end
	})
end

function AttentionTellStoryEvent:onStartTelling()
	local animName = "eventAttentionStory"

	if (isTable(taskSystem:getAnimationDesc(animName))) then

		local anim = taskSystem:createAnimation(animName, self.sequenceThanksParams)

		anim:atEnd({class="CallTask", func=function() event:onTellingDone() end })

	else
		event:onTellingDone()
	end
end


function AttentionTellStoryEvent:onTellingDone()

	self.eventState = "Resting"

	self.barCurrentTime = self.barMaxTime + self.coolDownBar
	self.decreasingBar:setProgress(math.clamp(self.barCurrentTime / self.barMaxTime, 0, 1))
	self:SetMood(1)
end


function AttentionTellStoryEvent:getFloaterPosition(obj)

	local pos = self.targetObject.floaterPosition
	return {x = pos.x - (self.targetObject.width * 0.5), y = pos.y}

end


function AttentionTellStoryEvent:ShowBar(easingEnabled, blinkingEnabled)
  if self.decreasingBar and self.decreasingBar.alpha == 0 then
    if blinkingEnabled then
    	self:DoBarBlinking()
    end
    if easingEnabled == true then
      self.targetObject:addTask({class = "Task",
        duration = self.fadeDuration})
    else
      self.decreasingBar.alpha = 1
    end

    self.targetObject:addTask({class = "CallTask",
      func = function()
        self.decreasingBar:cancelAnimations()
        self.decreasingBar.alpha = 1
      end
    })

	self.targetObject.clickable = true
    self.decreasingBar.back.clickable = true
  end
end


function AttentionTellStoryEvent:HideBar(easingEnabled)
  if self.decreasingBar and self.decreasingBar.alpha == 1 then
    if easingEnabled == true then
      self.decreasingBar:animate({alpha = 0, duration = self.barEasingDuration})
    else
      self.decreasingBar.alpha = 0
    end
    self.decreasingBar.back.clickable = false
  end
end


function AttentionTellStoryEvent:DoBarBlinking(blinkDuration)
    if (self.decreasingBar) then
        self.targetObject:addTask({class = "CallTask",
          func = function()
            self.decreasingBar:animate({blink =
              {startAlpha = 1, endAlpha = 0.3,
              fadeIn = self.barBlinkFadeDuration,
              fadeOut = self.barBlinkFadeDuration}})
          end
        })
        self.targetObject:addTask({class = "Task",
          duration = self.barBlinkDuration})
          self.targetObject:addTask({class = "CallTask",
            func = function()
              self.decreasingBar:cancelAnimations()
              self.decreasingBar.alpha = 1
            end
          })
    end
end

function AttentionTellStoryEvent:CheckEventStatus()
    if (level.isShiftClosed and self.barCurrentTime > 0) then
		self.eventDone = true
		event.targetObject.clickable = false
		self:HideBar(false)
		self:eventWon()
    end
end

function AttentionTellStoryEvent:SetMood(mood)
	self.decreasingBar:setProgress(mood)

	if (mood == 0) then
		self:eventLost()
		self.eventDone = true
		event.targetObject.clickable = false
		self:HideBar(false)

		local animName = "eventAttentionStoryMood0"
		if (isTable(taskSystem:getAnimationDesc(animName))) then

			local anim = taskSystem:createAnimation(animName)
		end

	elseif (mood < 0.33) then
		if(self.previousAnim ~=33) then
			self.previousAnim = 33

			local animName = "eventAttentionStoryMood33"
			if (isTable(taskSystem:getAnimationDesc(animName))) then
				local anim = taskSystem:createAnimation(animName)
			end
		end

	elseif (mood < 0.66) then
		if(self.previousAnim ~=66) then
			self.previousAnim = 66

			local animName = "eventAttentionStoryMood66"
			if (isTable(taskSystem:getAnimationDesc(animName))) then

				local anim = taskSystem:createAnimation(animName)
			end
		end

	elseif(mood == 1) then
			if(self.previousAnim ~=100) then
			self.previousAnim = 100

			local animName = "eventAttentionStoryMood100"
			if (isTable(taskSystem:getAnimationDesc(animName))) then

				local anim = taskSystem:createAnimation(animName)
			end
		end
	end
end
