dofile("script/events/Event.lua")

-- ************************************ Deliver Event ************************************ --
DeliverEvent = deriveLuaClass("DeliverEvent", "Event")

-- ************************************ Deliver Button (object to grab and bring to bin) ************************************ --
DeliverButton = deriveLuaClass("DeliverButton", "IngredientButton")

function DeliverButton:init(desc)
	DeliverButton.superInit(self, desc)
	self.deactivateOnWorkFinish = true
	self.removeTrayHandler = self.removeTrayHandler or self
	self.introductionTextId = "none"
end

-- This is wrong:
--function DeliverButton:needsHintArrow()
--	return event and not event.atLeastOneDelivered and self.visible and self.clickable
--end

-- ************************************ Deliver Object (bin) ************************************ --
DeliverObject = deriveLuaClass("DeliverObject", "Object")

function DeliverEvent:init(desc)
	DeliverEvent.superInit(self, desc)

	-- Config
	self.countItemsToFind = desc.countItemsToFind ~= false -- Default true
	self.allProductsAtOnce = desc.allProductsAtOnce or false -- Default FALSE
	self.maxItemsInOrderAtTheSameTime = desc.maxItemsInOrderAtTheSameTime or 1
	self.minTimeBetweenOrders = desc.minTimeBetweenOrders or 5000
	self.maxTimeBetweenOrders = desc.maxTimeBetweenOrders or 10000
	self.doCreditsForItemDelivered = desc.doCreditsForItemDelivered ~= false -- Default true
	self.useCustomersList = desc.useCustomersList or false
	self.randomCustomers = desc.randomCustomers ~= false
	self.randomProducts = desc.randomProducts ~= false
	self.deliverProductSound = desc.deliverProductSound or "event_item_complete:sound"

	-- Status
	self.totalDeliverTargets = 0
	self.totalItemsToFind = 0
	self.remainingItemsToFind = 0
	self.customerIndex = 1
	self.newCustomer = true
	self.matchingItemType = "DeliverObject"
end

function DeliverEvent:onInitialized()
	DeliverEvent.super.onInitialized(self)

	local deliverButtonsInLevel = level:getSpritesByExactType("DeliverButton")
	for _,item in ipairs(self.items) do
		if (item.obj) then
			--[[DEBUG--]]if (isString(item.obj)) then
			--[[DEBUG--]]	assert(false, "This object is still a string: '"..item.obj.."' level: "..tostring(level:getSpriteExt(item.obj)))
			--[[DEBUG--]]end
			item.nextOrderTimerActive = true

			if (self.maxItemsInOrderBalloon) then
				item.obj.orderMaxSlotCount = self.maxItemsInOrderBalloon
			end

			item.obj.onWorkStart = self.deliver_onWorkStart
			item.obj.onWorkUpdate = nil
			item.obj.onWorkFinish = self.deliver_onWorkFinish

			item.obj.showMessageTimer = 0
			item.obj.preventOrderHints = true
			item.obj:createOrder()
			item.obj.clickable = true

			if (item.acceptedProducts) then
				item.obj.acceptedProducts = self:calculateProductsToOrder(item.acceptedProducts)
			end

			self:item_initOrder(item, deliverButtonsInLevel)
		end
	end

	self.remainingItemsToFind = self.totalItemsToFind
	self.totalDeliverTargets = #self.items
end

function DeliverEvent:setupChallenge()
	level.challengeManager:setupEventChallenge(math.max(1, self:getNumItemsTodoTotal()), true)
end

function DeliverEvent:calculateProductsToOrder(products)
	local resultList = {}
	local possibleProducts = level.possibleProducts
	for _,p1 in ipairs(products) do
		for _,p2 in ipairs(possibleProducts) do
			if (p1 == p2) then
				table.push_back(resultList, p1)
				break
			end
		end
	end
	return resultList
end

function DeliverEvent:item_initOrder(item, deliverButtonsInLevel)
	if (not self.allProductsAtOnce) then
		item.nextOrderTimer = self.timeBeforeFirstOrder or self:getNextOrderTime()
		item.itemsToOrder = item.itemsToOrder or {}
	end

	if (item.obj and item.obj.order) then
		if (item.obj.acceptedProducts) then
			for _,v in ipairs(item.obj.acceptedProducts) do
				if (v and v ~= "") then
					if (self.allProductsAtOnce) then
						item.obj.order.currentStep:addProduct(v)
					else
						table.push_back(item.itemsToOrder, v)
					end
					self.totalItemsToFind = self.totalItemsToFind + 1
				end
			end
		else
			for _,v in ipairs(deliverButtonsInLevel) do
				local ingName = v:getIngredientNameForLogic()
				if (self.allProductsAtOnce) then
					item.obj.order.currentStep:addProduct(ingName)
				else
					table.push_back(item.itemsToOrder, ingName)
				end
				self.totalItemsToFind = self.totalItemsToFind + 1
			end
		end
	end
end

function DeliverEvent:getNextOrderTime()
	return math.Random(self.minTimeBetweenOrders, self.maxTimeBetweenOrders)
end

function DeliverEvent:onStart()
	DeliverEvent.super.onStart(self)
	for _,item in ipairs(self.items) do
		if (item.obj) then
			if (self.customOverhead) then
				item.obj:createOverhead(self.customOverhead)
			else
				item.obj:updateOrderOverhead()
			end
		end
	end
end

function DeliverEvent.deliver_onWorkStart(obj, task)
	event:hideHintArrow()
	if (not event.started or
		event.completed)
	then
		return false
	end

	local tray = level:getTray(task.actor)
	if (not tray or obj.order:countItemsToDeliver(tray) <= 0) then
		obj:doWrongAnimation(task)
	else
		local duration = obj:getWorkDurationWith(task.actor, "work")
		task.duration = duration
		if (duration > 100) then

			if (obj.workSoundId and obj.workSoundId ~= "") then
				obj:loopSample(obj.workSoundId)
			end

			task.actor.direction = obj.workDirection or "up"
			task.actor:playAnimation(obj.workAnimation or "work", obj.workCycles or -1, obj.workFrameTime or 150, obj.workWhenFinished or "default")
		end
	end
end

function DeliverEvent:eventWon()
	DeliverEvent.super.eventWon(self)
    for _,item in ipairs(self.items) do
		item.obj.clickable = false
	end
end

function DeliverEvent.deliver_onWorkFinish(obj, task)
	-- Check the products
	assert(obj.order)
	local item = obj.item

	-- Check if all the products for this object are delivered
	local numProductsAtStart = obj.order.currentStep.numProductsLeft
	local tray = level:getTray(task.actor)
	local balloonEmpty = obj.order:deliver(item, tray)
	local numProductsLeft = obj.order.currentStep.numProductsLeft
	local numDelivered = numProductsAtStart - numProductsLeft
	if (numProductsLeft > 0) then
		if (event.customOverhead) then
			--obj:createOverhead(event.customOverhead)
			if (numDelivered > 0) then
				obj:removeOverhead()
			end
		else
			obj:updateOrderOverhead()
		end
	else
		obj:removeOverhead()
	end

	obj:stopLoopSample()

	if (numDelivered > 0) then
		if (event.deliverProductSound) then
			obj:playSample(event.deliverProductSound)
		end

		event.atLeastOneDelivered = true

		local totalProductsLeft = numProductsLeft
		if (item.itemsToOrder) then totalProductsLeft = totalProductsLeft + #item.itemsToOrder end
		if (event.onDeliverProducts) then
			event:onDeliverProducts(obj, item, numDelivered, totalProductsLeft, balloonEmpty)
		end
		--print(event.doCreditsForItemDelivered)
		if (event.doCreditsForItemDelivered) then
			event:creditsForItemDelivered(item, numDelivered)
		end
	else
		if (obj.showMessageTimer <= 0) then
			obj.showMessageTimer = 5000
			if (#tray.products > 0) then
				local msgId = event:getMessageId(obj)
				if (msgId) then
					task.actor:addTaskAtFront({class="MessageTask", textId=msgId, duration=2000, needsClick=false})
				end
			end
		end
	end

	if (numProductsLeft == 0) then
		if (not item.itemsToOrder or #item.itemsToOrder == 0) then
			event:onDeliverItemDone(item)
		end
	end

	task.actor:playDefaultAnimation()
end

function DeliverEvent:creditsForItemDelivered(item, numDelivered)
	local obj = item.obj

	local score = event.scoreOneDone * numDelivered

	if score > 0 then
		local pos = obj.floaterPosition
		level:createFloater({
			id = "eventDone",
			textId = "EVENT_SCORE_ONE_DONE",
			x = pos.x - (obj.width * 0.5), y = pos.y - (obj.height * 0.7),
			bonusScore = score,
			bonus = {
				textId = "EVENT_SCORE_ONE_DONE_FMT",
			},
		})
	end

	if (level.BM_eventScore) then level:BM_eventScore(score) end

	-- Inc items done
	event.remainingItemsToFind = event.remainingItemsToFind - numDelivered
	event:onProgress()
end

-- Override the scoreItemDone from Event.lua, so we don't get the double +20 message with the last product
function DeliverEvent:scoreItemDone(item)

end

function DeliverEvent:onDeliverItemDone(item)
	item.obj.clickable = false
	self:itemDone(item)
end

function DeliverEvent:getMessageId(obj)
	return obj.badDeliveryMsgId
end

function DeliverEvent:onTick(time)
	if (not self.started or self.completed) then
		return
	end

	for _,item in ipairs(self.items) do
		if (not self.allProductsAtOnce and
			item.nextOrderTimerActive and
			item.nextOrderTimer > 0 and
			#item.itemsToOrder > 0 and
			self.maxItemsInOrderAtTheSameTime > item.obj.order.currentStep.numProductsLeft)
		then
			item.nextOrderTimer = item.nextOrderTimer - time
			if (item.nextOrderTimer <= 0) then

		local anim = self:onNewOrderStart()
				anim = anim or GH.anim.animation()

				if (not anim or not anim.atEnd) then
					anim = GH.anim.animation()
					assert(anim.atEnd, "2. the famous atEnd error!")
				end

				if (anim and anim.atEnd) then
					anim:atEnd({class="CallTask", func=function()
						item.nextOrderTimer = self:getNextOrderTime()

						-- Next order
						if (#item.itemsToOrder > 0 --[[and item.obj.order.currentStep.numProductsLeft < self.maxItemsInOrderAtTheSameTime]]) then
							local newProductIdx = 1
							if(self.randomProducts) then
								newProductIdx = math.Random(#item.itemsToOrder)
							end
							local newProduct = item.itemsToOrder[newProductIdx]
							item.obj.order.currentStep:addProduct(newProduct)
							table.remove(item.itemsToOrder, newProductIdx)
							if (event.onNextOrder) then
								event:onNextOrder(item, newProduct)
							end
							if (self.customOverhead) then
								item.obj:createOverhead(self.customOverhead)
							else
								item.obj:updateOrderOverhead()
							end
						end
					end})
				end
			end
		end
		if (item.obj) then
			if (item.obj.showMessageTimer > 0) then
				item.obj.showMessageTimer = item.obj.showMessageTimer - time
			end
		end
	end
end

function DeliverEvent:onNewOrderStart()
	--print("DeliverEvent:onNewOrderStart()")
	if(self.newCustomer) then
			self:changeCustomer()
	end
	self.newCustomer = false
	return taskSystem:createAnimation("deliverOrderStart")
end

function DeliverEvent:changeCustomer()
		for _,item in ipairs(self.items) do
			if(self.useCustomersList == true) then
				if(self.randomCustomers) then
					self.customerIndex = math.Random(1, #item.obj.eventCustomersList)
				end
				item.obj.defaultSection = item.obj.eventCustomersList[self.customerIndex].defaultSection
				item.obj.baseAnimation = item.obj.eventCustomersList[self.customerIndex].baseAnimation
				self.customerIndex = self.customerIndex + 1
			end
		end
end

function DeliverEvent:onDeliverProducts(obj, item, numDelivered, totalProductsLeft, balloonEmpty)
	--print("numDelivered, totalProductsLeft: "..numDelivered .. ", " .. totalProductsLeft)
	self.newCustomer = true
	if (totalProductsLeft == 0) then
		return self:createAnimationForObject(obj, "deliverAllDelivered")
	end
	if (balloonEmpty) then
		-- Create the animation on the deliverCharacter, not the TaskSystem.
		return self:createAnimationForObject(obj, "deliverOrderDelivered")
	end
	return nil
end

function DeliverEvent:createAnimationForObject(obj, animId)
	-- TODO: ISRAEL make this correct using the animationsDesc
	local animTable = taskSystem:getAnimationDesc(animId)
	if (not isTable(animTable)) then
		return
	end
	if (not animTable.name or animTable.name == "") then
		animTable.name = animId
	end
	self:createTaskTree(obj, animId, animTable.tree, animTable.list, false)
	return true
end

function DeliverEvent:createTaskTree(obj, animId, treeDesc, list, simultaneous)
	for k,treeItem in pairs(treeDesc) do
		-- All the deliver events are always a single line of
		if (not isTable(treeItem)) then
			local desc = table.copy(list[treeItem])
			if (desc.taskEnabled ~= false) then
				obj:addTask(desc)
			end
		end
	end
end

function DeliverEvent:onItemDone(item)
	DeliverEvent.super.onItemDone(self, item)
end

function DeliverEvent:areAllItemsDone()
	if (self.countItemsToFind) then
		return self.remainingItemsToFind <= 0
	end
	return DeliverEvent.super.areAllItemsDone(self)
end

function DeliverEvent:getProgress()
	if (self.countItemsToFind and self.totalItemsToFind > 0) then
		return (self.totalItemsToFind-self.remainingItemsToFind) / self.totalItemsToFind
	else
		if (self.itemsDone < self.totalDeliverTargets) then
			return self.itemsDone / self.totalDeliverTargets
		end
		return 1
	end
end

function DeliverEvent:getNumItemsTodoTotal()
	if (self.countItemsToFind) then
		return self.totalItemsToFind
	else
		return self.totalDeliverTargets
	end
end
