-- ************************************ Event ************************************ --
Event = getClass("Event")

function Event:init(desc)
	Event.superInit(self, desc)

	-- Configuration variables
	self.autoStart = desc.autoStart or false -- Default FALSE
	self.initObjectsHidden = desc.initObjectsHidden or false -- Default FALSE
	self.showHintOnStart = desc.showHintOnStart ~= false -- Default true
	self.showItemsOnStart = desc.showItemsOnStart ~= false -- Default true
	self.hideItemsAtEnd = desc.hideItemsAtEnd or false -- Default FALSE
	self.hideItemsOnWin = desc.hideItemsOnWin or false -- Default FALSE
	self.hideItemsOnLose = desc.hideItemsOnLose or false -- Default FALSE
	self.hideItemOnDone = desc.hideItemOnDone or false -- Default FALSE
	self.showFadeInDuration = desc.showFadeInDuration or 1000
	self.foundFadeOutDuration = desc.foundFadeOutDuration or 200
	self.barFadeDuration = desc.barFadeDuration or 600
	self.floatersFromHero = desc.floatersFromHero or false -- Default FALSE
	self.blinkStartFraction = desc.blinkStartFraction or 0.16
	--self.callOutIcon = desc.callOutIcon or nil
	--self.doChallengeBarCallOut = desc.doChallengeBarCallOut or nil
	--self.animateChallengeBarOnStart = desc.animateChallengeBarOnStart ~= false -- Default TRUE

	-- Optional variables
	--self.scoreOneDone
	--self.scoreAllDone
	--self.hintObject

	-- Status variables
	self.completed = false
	self.started = false
	self.won = false
	self.items = self.items or {}
	self.itemsDone = 0
end

function Event:onInitialized()

	--challengeBar.doCallOut = self.doChallengeBarCallOut ~= false
	--if (self.callOutIcon) then
	--	challengeBar.callOut = self.callOutIcon
	--end
	--if (self.challengeBarText) then
	--	challengeBar.description = self.challengeBarText
	--elseif (player) then
	--	local visitShift = player.visitShift
	--	local challengeBarText = "CHALLENGE_"..player.episode.."_"..visitShift
	--	if (GH.hasText(challengeBarText)) then
	--		challengeBar.description = challengeBarText
	--	end
	--end

	--if (self.challengeBarCompletedTextId) then
	--	challengeBar.descriptionCompletedId = self.challengeBarCompletedTextId
	--elseif (player) then
	--	local challengeBarCompletedTextId = "CHALLENGE_"..player.episode.."_COMPLETED"
	--	if (GH.hasText(challengeBarCompletedTextId)) then
	--		challengeBar.descriptionCompletedId = challengeBarCompletedTextId
	--	end
	--end

	local upgraded = ""
	if (self.challengeBarIconUpgrade) then
		if (player:getUpgradeSelection(self.challengeBarIconUpgrade) ~= "") then
			upgraded = "_upgraded"
		end
	end

	local choice = ""
	if (self.challengeBarIconChoice) then
		choice = player:getUpgradeSelection(self.challengeBarIconChoice)
		if (choice ~= "") then choice = "_"..choice end
	end

	--local suffix = choice..upgraded
	--if (self.challengeBarImage) then
	--	challengeBar.icon = self.challengeBarImage..suffix..":"..level.defaultSection
	--elseif (player) then
	--	local challengeBarImage = "challenge-icon"..player.episode..suffix
	--	if (level:hasResource(challengeBarImage, level.defaultSection)) then
	--		challengeBar.icon = challengeBarImage..":"..level.defaultSection
	--	end
	--	print("challengeBarImage: "..challengeBarImage)
	--end

	--if (self.challengeBarProgressImage) then
	--	challengeBar:getChild("bar").image = self.challengeBarProgressImage
	--end


	if (self.matchingItemType) then
--print("self.matchingItemType: "..self.matchingItemType)
		local objects = level:getSpritesByExactType(self.matchingItemType)
		for _,obj in ipairs(objects) do
--print("Found matching item (by type) with name: "..obj.name)
			if (not self:getItemForObject(obj)) then
				table.insert(self.items, {obj=obj})
			end
		end
	end

	for _,item in ipairs(self.items) do
		if (isString(item.obj)) then
			local obj = level:getChild(item.obj)
			if (obj) then
				item.obj = obj
			end
		elseif (isTable(item.obj) and not item.obj.__cpp and isString(item.obj.traits)) then
			item.obj = level:createCharacter(item.obj)
		end

		if (self.initObjectsHidden and isTable(item.obj)) then
			item.obj.visible = false
		end
	end

	local numStepsToDo = self:getNumStepsTodo() or 1
	if (self.trackProgress == nil) then
		self.trackProgress = (numStepsToDo == 1)
	end
end

function Event:setupChallenge()
	level.challengeManager:setupEventChallenge(math.max(1, self:getNumStepsTodo()), true)
end

function Event:getNumItemsTodoTotal()
	return #self.items
end

function Event:getNumStepsTodo()
	if (self.numItemsTodo) then
		return self.numItemsTodo
	end

	local numItemsTodo = 0
	for _,item in ipairs(self.items) do
		if (item.obj and item.obj.numSteps) then
			numItemsTodo = numItemsTodo + item.obj.numSteps
		end
	end
	if (numItemsTodo == 0) then
		numItemsTodo = self:getNumItemsTodoTotal()
	end
	return numItemsTodo
end

function Event:getItemForObject(obj)
	for _,item in ipairs(self.items) do
		if (obj and item.obj == obj) then
			return item
		end
	end
end

function Event:onInactivity(levelProgress)
	if (self.inactivityHintObject) then
		level:placeHintArrow(self.inactivityHintObject)
	end
	return true
end

function Event:onShiftStart()
	if (self.autoStart) then
		level:fireTrigger("startEvent")
	end
end

function Event:onStart()
	for _,item in ipairs(self.items) do
		if (isString(item.obj)) then
			local obj = level:getChild(item.obj)
			GH.assert(obj, "Could not find event item: '"..item.obj.."'")
			item.obj = obj
		end
	end

	self.started = true
	if (self.showItemsOnStart) then
		self:showItems()
	end
	if (self.showHintOnStart) then
		self:showHint()
	end
	if (self.playStartAnimation) then
		self:doStartAnimation()
	end

	if (level.updateTableGroupSize) then
		level:updateTableGroupSize()
	end
end

function Event:doStartAnimation()
	if (self.startAnimation) then
		for _,item in ipairs(self.items) do
			if (isTable(item.obj)) then
				item.obj:playAnimation(self.startAnimation, self.startAnimationCycles, self.startAnimationFrameTime, self.startAnimationWhenFinished)
			end
		end
	end
end

function Event:preserveBool(item, k)
	local v = item.obj[k]
	local old_k = "old_"..k
	if (v == false) then
		item[old_k] = false
	elseif (v == true) then
		item[old_k] = true
	else
		item[old_k] = nil
	end
end

function Event:preserveItem(item)
	if (not item.preserved and item.obj) then
		if (isTable(item.obj)) then
			item.preserved = true
			item.done = false
			item.obj.item = item
			item.obj.numSteps = item.obj.numSteps or 1
			item.obj.stepsDone = item.obj.stepsDone or 0
			item.old_onWorkStart = item.old_onWorkStart or item.obj.onWorkStart
			item.old_onWorkUpdate = item.old_onWorkUpdate or item.obj.onWorkUpdate
			item.old_onWorkFinish = item.old_onWorkFinish or item.obj.onWorkFinish
			item.old_onWorkCancel = item.old_onWorkCancel or item.obj.onWorkCancel
			item.old_onClick = item.old_onClick or item.obj.onClick
			item.old_workAnimation = item.obj.workAnimation
			item.old_workDuration = item.obj.workDuration
			item.old_workPos = item.obj.workPos
			item.old_getWorkCharacter = item.obj.getWorkCharacter

			self:preserveBool(item, "clickable")
			self:preserveBool(item, "doHighlight")
			self:preserveBool(item, "stationaryActive")
			-- Copy all the keys in the event item table
			for k,v in pairs(item) do
				if (k ~= "obj") then
					if (isTable(v)) then
						item.obj[k] = table.copy(v, true, false, true)
					else
						item.obj[k] = v
					end
				end
			end
		end
	end
end

function Event:showItems()
	for _,item in ipairs(self.items) do
		if (item.obj) then
			self:showItem(item)
		end
	end
end

function Event:showItem(item)
	--print("item: "..ts(item))
	--print("item.shownForEvent: "..ts(item.shownForEvent))
	if (item.shownForEvent) then
		return
	end

	item.shownForEvent = true
	self:preserveItem(item)
	item.obj.clickable = true
	item.obj.doHighlight = true

	--print("item.swapObj 1: "..ts(item.swapObj))
	if (item.swapObj == nil) then
		item.swapObj = item.obj.swapObj
	end
	if (isString(item.swapObj)) then
		item.swapObj = level:getChild(item.swapObj)
	end
	--print("item.swapObj 2: "..ts(item.swapObj))
	if (isTable(item.swapObj)) then
		item.swapObj.visible = false
		item.obj.visible = true
	end
	if (not item.obj.visible) then
		item.obj.visible = true
		item.obj.alpha = 0
	end
	if (item.obj.alpha < 1) then
		if (item.obj.animate) then
			item.obj:animate({alpha=1, duration=self.showFadeInDuration})
		end
	end
	if (item.obj.onEventShow) then
		item.obj:onEventShow()
	end
end

function Event:hideItems()
	for _,item in ipairs(self.items) do
		if (item.obj) then
			self:hideItem(item)
		end
	end
end

function Event:hideItem(item)
	item.obj.clickable = false
	if (item.swapObj) then
		item.obj.visible = false
		--if (item.bar) then
			--item.bar.alpha = 0
		--end
		item.swapObj.visible = true
		if (item.swapObj.onEventItemHidden) then
			item.swapObj:onEventItemHidden(item)
		end
	else
		if (item.obj and item.obj.animate) then
			item.obj:animate({alpha=0, duration=self.foundFadeOutDuration}, {visible=false})
		end
		--if (item.bar) then
			--item.bar:animate({alpha=0, duration=self.foundFadeOutDuration})
		--end
	end
	if (item.obj.onEventHide) then
		item.obj:onEventHide()
	end
end

function Event:getProgress()
	local numItemsTodo = self:getNumStepsTodo()

	local itemsDone = 0
	for _,item in ipairs(self.items) do
		if (item.obj and item.obj.stepsDone) then
			itemsDone = itemsDone + item.obj.stepsDone
		end
	end
	if (itemsDone == 0) then
		itemsDone = self.itemsDone
	end
	return (numItemsTodo > 0 and itemsDone / numItemsTodo) or 1
end

function Event:onProgress()
	local progress = self:getProgress()
	level.challengeManager:onEventProgress(progress)
end

function Event:onStepDone(item)
	local animName = "eventThanks"..event.itemsDone
	if (item and item.obj and item.obj.stepsDone) then
		animName = animName .. "_" .. item.obj.stepsDone
	end

	if (isTable(taskSystem:getAnimationDesc(animName))) then
		return taskSystem:createAnimation(animName, self.sequenceThanksParams)
	else
		return taskSystem:createAnimation("eventThanks", self.sequenceThanksParams)
	end
end

function Event:areAllItemsDone()
	return self.itemsDone >= self:getNumItemsTodoTotal()
end

function Event:onItemDone(item)
	if (self:areAllItemsDone()) then
		return self:eventWon()
	else
		return self:onStepDone(item)
	end
end

function Event:incItemDone(item)
	self.itemsDone = self.itemsDone + 1
end

function Event:scoreItemDone(item)
	if (self.scoreOneDone and self.scoreOneDone > 0) then
		local pos = self:getFloaterPosition(item and item.obj or nil)
		level:createFloater({
			id = "generic",
			textId = "EVENT_SCORE_ONE_DONE",
			x = pos.x, y = pos.y,
			bonusScore = event.scoreOneDone,
			bonus = {
				textId = "EVENT_SCORE_ONE_DONE_FMT",
			},
		})
		if (level.BM_eventScore) then level:BM_eventScore(self.scoreOneDone) end
	end
end

function Event:stepDone(item)
	GH.assert(not item or (item.obj and item.obj.numSteps))
	if (item and not item.done) then
		item.obj.stepsDone = item.obj.stepsDone + 1
		if (item.obj.stepsDone >= item.obj.numSteps) then
			return self:itemDone(item)
		else
			self:scoreItemDone(item)
			self:onProgress()
			SpriteExt.playSample(nil, "event_item_complete")
			return self:onStepDone(item)
		end
	end
end

function Event:itemDone(item)
	if (not item.done) then
		self.lastItemDone = item

		item.done = true
		self:incItemDone()
		if (self.hideItemOnDone) then
			self:hideItem(item)
		end
		if (item.obj) then
			if (item.obj.useCustomCursor ~= nil) then
				item.obj.useCustomCursor = false
			end
			if (item.obj.onEventItemDone) then
				item.obj:onEventItemDone()
			end
		end

		if (item.bar and item.bar.finish) then
			item.bar:finish()
		end

		self:scoreItemDone(item)

		if (self.onEventStepDone) then
			self:onEventStepDone(item)
		end

		if (level.onEventStepDone) then
			level:onEventStepDone(self, item)
		end

		if (self.playDoneAnimation and self.doneAnimation) then
			if (item.obj) then
				item.obj:playAnimation(self.doneAnimation, self.doneAnimationCycles, self.doneAnimationFrameTime, self.doneAnimationWhenFinished)
			end
		end

		self:hideHintArrow()
		self:restoreItem(item)
		self:removeDecals(item)
		self:onProgress()

		SpriteExt.playSample(nil, "event_item_complete")
		return self:onItemDone(item)
	end
end

function Event:restoreItem(item)
	if (item) then
		if (item.onRestoreItem) then
			item:onRestoreItem()
		end
		if (isTable(item.obj) and item.old_onWorkStart) then -- Was preserved
			item.obj.onWorkStart = item.old_onWorkStart
			item.obj.onWorkUpdate = item.old_onWorkUpdate
			item.obj.onWorkFinish = item.old_onWorkFinish
			item.obj.onWorkCancel = item.old_onWorkCancel
			item.obj.onClick = item.old_onClick
			item.obj.clickable = item.old_clickable
			item.obj.doHighlight = item.old_doHighlight
			item.obj.stationaryActive = item.old_stationaryActive
			item.obj.workAnimation = item.old_workAnimation
			item.obj.workDuration = item.old_workDuration
			item.obj.workPos = item.old_workPos
			item.obj.getWorkCharacter = item.old_getWorkCharacter
		end
	end
end

function Event:restoreItems()
    for _,item in ipairs(self.items) do
		self:restoreItem(item)
		self:removeDecals(item)
	end
end

function Event:createProgressBar(item, initialProgress)
	if (not isTable(item.bar) and item.obj and not item.noProgressBar) then
		item.bar = level:add(blueprints[item.bar or self.bar or "progressbar_default"])
		if (item.obj.absBarPos) then
			item.bar.location = item.obj.absBarPos
		else
			item.bar.location = item.barPos or item.obj.barPos or self.barPos or {x=0,y=0}
			local pos = {
				x=(item.bar.location.x or 0) + (item.obj.x or 0),
				y=(item.bar.location.y or 0) + (item.obj.y or 0)}
			item.bar.location = pos
		end

		item.bar.floor = item.obj.barFloor or item.obj.worldFloor-item.bar.worldY+1
		item.bar.alpha = 0
		item.bar:setProgress(initialProgress or 0)
	end
end

function Event:showProgressBar(item)
	if (isTable(item.bar) and item.bar.animate) then
		item.bar:animate({alpha=1, duration=self.barFadeDuration})
	end
end

function Event:hideProgressBar(item)
	if (isTable(item.bar) and item.bar.animate) then
		item.bar:animate({alpha=0, duration=self.barFadeDuration})
	end
end

function Event:stopBlinkBar(item)
	if (item.bar and item.bar.blink) then
		GH.cancelAnimation(item.bar.blink)
		item.bar.blink = nil
		item.bar.front.alpha = 1
	end
end

function Event:blinkBar(item, fadeTime, forceNew)
	if (item.bar) then
		if (item.bar.blink and not forceNew) then
			return -- Already blinking
		end
		self:stopBlinkBar(item)
		fadeTime = fadeTime or 200
		local front = item.bar.front
		if (front and front.animate) then
			item.bar.blink = front:animate({blink={startAlpha=1,endAlpha=0.3,fadeIn=fadeTime,fadeOut=fadeTime}})
		end
	end
end

function Event:removeDecals(item, fadeTime)
	if (item and isTable(item.bar) and item.bar.animate) then
		self:stopBlinkBar(item)
		item.bar:animate({alpha=0, duration=fadeTime or self.barFadeDuration}, {remove=true})
		item.bar = nil
	end
end

function Event:determineItem(v)
	if (isNumber(v)) then
		return self.items[v]
	end

	local t = v
	if (isString(v)) then
		for _,item in ipairs(self.items) do
			if (item.name == v or (item.obj and item.obj.name == v)) then
				return item
			end
		end
		t = level:getSpriteExt(v) -- Try table...
	end

	if (isTable(t)) then
		for _,item in ipairs(self.items) do
			if (t == item) then
				return item
			end
			if (item.obj and item.obj == t) then
				return item
			end
		end
	end

	if (isTable(v)) then
		return
			self:determineItem(v.params) or
			self:determineItem(v.meta) or
			self:determineItem(v.name)
	end
end

function Event:handleTrigger(trigger, data)
	if (trigger == "startEvent") then
		local anim = taskSystem:createAnimation("eventStart", self.eventStartParams or data)
		anim = anim or GH.anim.animation()
		anim:atEnd({class="CallTask", func=function()
			self:onStart(anim)
		end})
	elseif (trigger == "eventStep") then
		self:onStep()

	elseif (trigger == "endEvent") then
		self:onShiftFinish()

	elseif (trigger == "showEventItems") then
		self:showItems()

	elseif (trigger == "showEventItem") then
		local item = self:determineItem(data)
		--GH.assert(item, "Could not determine item from: "..serialize("data", data, 0))

		--print("data.resetShowForEvent: "..ts(data.resetShowForEvent))
		if (data.resetShowForEvent) then
			item.shownForEvent = false
			item.preserved = false
			item.done = false
		end

		self:showItem(item)
		if (item.swapObj and data.cancelTasks) then
			taskSystem:abortWorkTasksFor(hero, item.swapObj)
		end

	elseif (trigger == "showHint") then
		self:showHint(data)
	end
end

function Event:showHint(obj)
	self.hintObject = obj or self.hintObject
--print("Event:showHint "..serialize("self.hintObject", self.hintObject or {"nil"}))
	if (self.hintObject) then
		level:placeHintArrow(self.hintObject)
	end
end

function Event:hideHintArrow()
	if (self.hintObject) then
		level:removeHintArrow(self.hintObject)
	end
end

function Event:getFloaterPosition(obj)
	local pos = obj and obj.floaterPosition
	if (isTable(pos)) then
		return pos
	end
	pos = self.floaterPosition
	if (isTable(pos)) then
		return pos
	end
	if (self.floatersFromHero and hero) then
		pos = hero.floaterPosition
		return {x=pos.x, y=pos.y}
	end
	if (self.lastItemDone and self.lastItemDone.obj) then
		pos = self.lastItemDone.obj.floaterPosition
		if (pos) then
			return pos
		end
		return {x=src.left + src.width/2, y=src.top}
	end
	--print("Could not figure out a floater position. Picking the middle of the screen...")
	return {x=512, y=300}
end

function Event:eventWon()
	if (self.scoreAllDone and self.scoreAllDone > 0) then
		local pos = self:getFloaterPosition()

		level:createFloater({
			delayBeforeFloaterStarts = self.scoreOneDone and self.scoreOneDone > 0 and 800 or nil,

			id = "eventDone",
			textId = "EVENT_SCORE_ALL_DONE",
			x = pos.x, y = pos.y + 60,
			bonusScore = event.scoreAllDone,
			bonus = {
				textId = "EVENT_SCORE_ALL_DONE_FMT",
			},
		})

		if (level.topLayer and level.topLayer.animate) then
			level.topLayer:animate({delay=200},{particle="effects:particles", emitter="event_done_blast", ex=pos.x, ey=pos.y})
		end
		if (level.BM_eventScore) then level:BM_eventScore(self.scoreAllDone) end
	end

	self:eventEnd()

	self:hideHintArrow()
	self.won = true
	self.completed = true
	self:restoreItems()

	level.challengeManager:onEventCompleted()

	if (self.hideItemsOnWin or self.hideItemsAtEnd) then
		self:hideItems()
	end

	SpriteExt.playSample(nil, "event_complete")

	--level:setChallengeFinished()
	if (level.updateTableGroupSize) then
		level:updateTableGroupSize()
	end

	return taskSystem:createAnimation("eventWon")
end

function Event:eventLost()
	self:eventEnd()
	self:hideHintArrow()
	self.won = false
	self.completed = true
	self:restoreItems()

	level.challengeManager:onEventFailed()

	if (self.hideItemsOnLose or self.hideItemsAtEnd) then
		self:hideItems()
	end

	if (level.updateTableGroupSize) then
		level:updateTableGroupSize()
	end

	return taskSystem:createAnimation("eventLost")
end

function Event:onShiftFinishStart()
	if (not self.completed) then
		if (self.won) then
			self:eventWon()
		else
			self:eventLost()
		end
	end
end

function Event:onShiftFinish()
	self:hideHintArrow()
	self:restoreItems()

	if (player and player.skipAllStory) then
		return
	end

	local anim
	if (self.won) then
		anim = taskSystem:createAnimation("eventWonAtEnd") or anim
		if (anim) then
			level:removeSpecialEmotions("eventWonAtEnd")
		end
	else
		anim = taskSystem:createAnimation("eventLostAtEnd") or anim
		if (anim) then
			level:removeSpecialEmotions("eventLostAtEnd")
		end
	end
	return anim
end

function Event:eventEnd()
end

function Event:onMouseDown(x, y)
	MouseDown = true
	MouseUp = false
end

function Event:onMouseUp(x, y)
	MouseDown = false
	MouseUp = true
end

function Event:onMouseMoved(x, y, lButtonDown)
end

--function Event:onGuiIconClick()
--end

function Event:onMessageClick(command, id, object)
end

function Event:getProductNames()
	return {}
end
