dofile("script/events/RequestAttentionEvent.lua")

FillBarObject = deriveLuaClass("FillBarObject", "RequestAttentionObject")

function FillBarObject:init(desc)
    FillBarObject.superInit(self, desc)

    self.ropeObjectName = desc.ropeObjectName or "well_broken_rope"
    self.pullAnimation = desc.pullAnimation or "edward_work_up"
    self.pullFrameTime = desc.pullFrameTime or 150
    self.pullCycles = desc.pullCycles or 1

    self.endAnimation = desc.endAnimation or "edward_walk"
    self.endFrameTime = desc.endFrameTime or 150
    self.endCycles = desc.endCycles or 1

    self.currentBarLevel = 0

    self.x = self.x + self.anchor.x
    self.y = self.y + self.anchor.y

    self.numberOfClicks = 0
end

function FillBarObject:onLevelInitialized()
    FillBarObject.super.onLevelInitialized(self)
    self.enableDecreasing = false

    self.ropeObject = level:getObject(self.ropeObjectName)

    self.ropeObject.clickable = true
    local selfEvent = self
    self.ropeObject.onClick = function()
        selfEvent:onClick()
    end

    self:CreateDecreasingBar()
    self:HideBar(false)
    --self:ShowBar(false, false)

    self.winEvent = false
    self.finishedPlayAnimation = true

    self.ropeObject:createOverhead("exclamation:gamescene")
    self.isFirstClick = true
end

function FillBarObject:onClick()
    -- Changed the currentBarLevel from 0..1 to 0..100, because otherwise the
    -- precision was messing with the comparison
    if (self.isFirstClick) then
        self.isFirstClick = false
        self:ShowBar(true, false)
        self.ropeObject:removeOverhead();
    end
    self.currentBarLevel = self.currentBarLevel + self.barClickIncrease
    if (self.currentBarLevel >= 100) then
        self.currentBarLevel = 100
        self.winEvent = true
        self:HideBar(true)
        self:PlayEndAnimation()
    else
        self.numberOfClicks = self.numberOfClicks + 1
        self:PlayObjectAnimation()
        if (self.eventElement) then
            self.eventElement:onProgress()
        end
    end

    self.decreasingBar:setProgress(self.currentBarLevel/100)
    self:ModifyRopeFrame(self.currentBarLevel/100)

    if (self.onClickSoundName and self.onClickSoundName ~= "") then
        self:playSample(self.barClickSoundName)
    end
end

function FillBarObject:ModifyRopeFrame(barLevel)
    if (self.ropeObject) then
        local totalFrames = self.ropeObject.frameCount
        local actualFrame = barLevel * (totalFrames - 1)
        self.ropeObject.frame = math.floor(actualFrame)
    end
end

function FillBarObject:PlayObjectAnimation()
    if (self.finishedPlayAnimation == true) then
        self.finishedPlayAnimation = false
        self.baseAnimation = self.pullAnimation
        local selfEvent = self
        self:DoAnimationRecursive()
    end
end

function FillBarObject:PlayEndAnimation()
    self.baseAnimation = self.endAnimation
    self:addTask({
        class = "PlayAnimationTask",
        animation = self.endAnimation,
        cycles = self.endCycles,
        frameTime = self.endFrameTime,
        onFinishedCallback = function()
        end
    })
end

function FillBarObject:DoAnimationRecursive()
    if (self.winEvent == false) then
        if (self.numberOfClicks > 0) then
            self.numberOfClicks = 0
            local selfEvent = self
            self:addTask({
                class = "PlayAnimationTask",
                animation = self.pullAnimation,
                cycles = self.pullCycles,
                frameTime = self.pullFrameTime,
                onFinishedCallback = function()
                    selfEvent:DoAnimationRecursive()
                end
            })
        else
            local defaultAnim = {}
            defaultAnim.animation = self.basicAnimName
            defaultAnim.cycles = self.basicAnimCycles
            defaultAnim.frameTime = self.basicAnimFrameTime
            self.baseAnimation = self.basicAnimName
            self:playAnimation(defaultAnim)
            self.finishedPlayAnimation = true
        end
    end
end

function FillBarObject:CreateDecreasingBar()
    FillBarObject.super.CreateDecreasingBar(self)
    local selfObj = self
    -- Added this to avoid the crash when the player taps on the decreasing bar
    self.decreasingBar.back.onClick = function ()
        selfObj:onClick()
    end
end

--------------------------------------------------------------------------------
FillBarEvent = deriveLuaClass("FillBarEvent", "Event")

function FillBarEvent:init(desc)
    FillBarEvent.superInit(self, desc)
end

function FillBarEvent:onInitialized()
    FillBarEvent.super.onInitialized(self)
    local eventObjectNameUnified = self.configObjectName or self.eventObjectName
    if (eventObjectNameUnified) then
        self.eventObject = level:getObject(eventObjectNameUnified)
        self.eventObject.eventElement = self
    end
    GH.assert(self.eventObject,
    "ERROR! Cannot link with the RequestAttentionEvent object in the scene! " ..
    "The RequestAttentionEvent won't work properly.")
    self.eventObject.clickable = true
    self.eventStopped = false
end

function FillBarEvent:eventWon()
    FillBarEvent.super.eventWon(self)
    self.eventStopped = true
    self.eventObject.clickable = false
    self.eventObject.ropeObject.clickable = false
end

function FillBarEvent:eventLost()
    FillBarEvent.super.eventLost(self)
    self.eventStopped = true
    self.eventObject.clickable = false
    self.eventObject.ropeObject.clickable = false
end

function FillBarEvent:onTick(time)
    if (not self.eventStopped) then
        if (level.isPlaying) then
            self:CheckEventStatus(time)
        end
    end
end

function FillBarEvent:CheckEventStatus(time)
    if (not self.eventStopped) then
        if (level.isShiftClosed) then
            self:eventLost()
        elseif (self.eventObject.winEvent) then
            self:eventWon()
        end
    end
end

function FillBarEvent:getProgress()
    local value = self.eventObject.currentBarLevel/100
	if (value < 0) then
        return 0
    elseif(value > 1) then
        return 1
    else
        return value
    end
end

function FillBarEvent:setupChallenge()
	level.challengeManager:setupEventChallenge(100, false)
end
