dofile("script/events/Event.lua")

-- ************************************ Clover Event ************************************ --
FindCloversEvent = deriveLuaClass("FindCloversEvent", "Event")

function FindCloversEvent:init(desc)
	desc.initObjectsHidden = true
	desc.showItemsOnStart = false
	FindCloversEvent.superInit(self, desc)

	-- Configuration
	--self.pluckSound = desc.pluckSound or "stock_sounds9:sound"
	--self.pluckParticle = desc.pluckParticle or "grass_pluck"
	self.prepareAnimation = desc.prepareAnimation or "prepare"
	self.useCorrespondingPositions = desc.useCorrespondingPositions ~= false --' Default true

	self.minAmountOnScreen = desc.minAmountOnScreen or 8
	self.maxAmountOnScreen = desc.maxAmountOnScreen or 8
	self.totalAmountToDo = desc.totalAmountToDo or 1
	self.minCloverDelay = desc.minCloverDelay or 1*1000
	self.maxCloverDelay = desc.maxCloverDelay or 2*1000
	self.noCloversDelay = desc.noCloversDelay or 1*1500
	self.cloverShowTime = desc.cloverShowTime or 10*1000

	-- Initialization
	self.maxAmountOnScreen = self.maxAmountOnScreen
	self.totalAmountToDo = self.totalAmountToDo
	self.matchingItemType = "Clover"

	-- Status variables
	self.possiblePositions = {}
	self.availableClovers = {}
	self.cloversAlreadyOnScreen = 0
	self:chooseNewSpawnDelay(self.initialCloverDelay)
end

function FindCloversEvent:onInitialized()
	FindCloversEvent.super.onInitialized(self)
end

function FindCloversEvent:onItemDone(item)
	FindCloversEvent.super.onItemDone(self, item)
end

function FindCloversEvent:addClover(clover)
	if (table.insert_if_unique(self.availableClovers, clover)) then
		if (clover.cloverPosition) then
			table.insert(self.possiblePositions, clover.cloverPosition)
			clover.cloverPosition = nil
		end
	end
end

function FindCloversEvent:getProgress()
	if (self.itemsDone < self.totalAmountToDo) then
		return self.itemsDone / self.totalAmountToDo
	end
	return 1
end

function FindCloversEvent:getNumItemsTodoTotal()
	return self.totalAmountToDo
end

function FindCloversEvent:chooseNewSpawnDelay(forced)
	if (forced) then
		self.nextCloverDelay = forced
	elseif (self.cloversAlreadyOnScreen < self.minAmountOnScreen) then
		self.nextCloverDelay = self.noCloversDelay
	else
		self.nextCloverDelay = math.Random(self.minCloverDelay, self.maxCloverDelay)
	end
end

function FindCloversEvent:onTick(time)
	if (not self.done and self.itemsDone < self.totalAmountToDo) then

		self.nextCloverDelay = self.nextCloverDelay - time
		if (self.nextCloverDelay <= 0) then
			self:chooseNewSpawnDelay()

--[[DEBUG CLOVERS--]] --print("self.cloversAlreadyOnScreen "..self.cloversAlreadyOnScreen)
--[[DEBUG CLOVERS--]] --print("self.maxAmountOnScreen "..self.maxAmountOnScreen)
--[[DEBUG CLOVERS--]] --print("#self.possiblePositions "..#self.possiblePositions)
--[[DEBUG CLOVERS--]] --print("#self.availableClovers "..#self.availableClovers)
			if (self.cloversAlreadyOnScreen < self.maxAmountOnScreen and
				#self.possiblePositions > 0 and
				#self.availableClovers > 0)
			then
				local cloverIdx = math.Random(1, #self.availableClovers)
				local cloverPositionIdx = cloverIdx
				if (not self.useCorrespondingPositions) then
					cloverPositionIdx = math.Random(#self.possiblePositions)
				end
				local clover = table.remove(self.availableClovers, cloverIdx)
				local cloverPosition = table.remove(self.possiblePositions, cloverPositionIdx)
--[[DEBUG CLOVERS--]] --print("clover.name: "..clover.name)

				self.cloversAlreadyOnScreen = self.cloversAlreadyOnScreen + 1

				clover.cloverPosition = cloverPosition
				clover.location = cloverPosition
				clover.floor = cloverPosition.floor
				clover.alpha = 1
				clover.visible = true
				clover.clickable = true

				if (self.pluckSound) then
					clover:playSample(self.pluckSound)
				end
				clover:playAnimation(self.prepareAnimation, 1, clover.defaultFrameTime or 35, "default")

				clover:addTask({class="Task", duration=self.cloverShowTime})
				clover:addTask({class="SetTask", clickable=false})
				clover:addTask({class="PlayAnimationTask", animation="decay", cycles=1, frameTime=clover.defaultFrameTime or 35, whenFinished="stay", wait=true})
				clover:addTask({class="SetTask", alpha=0, duration=1000})
				clover:addTask({class="CallTask", func=function() clover:hideClover() end})
			end
		end
	end
end

function FindCloversEvent:pluckClover(clover)

	if (event.pluckSound) then
		clover:addTask({class="PlaySoundTask", sound=event.pluckSound})
	end
	if (event.pluckParticle) then
		clover:newChild({
			class="ParticleEffect", x=clover.width/2, y=clover.height, scale=2,
			library="effects:particles", emitter=event.pluckParticle,
			removeWhenFinished=true, start=true})
	end
	self:animatePluckedClover(clover)
	clover:addTask({class="CallTask", func=function()
		clover:hideClover()
	end})
end

function FindCloversEvent:animatePluckedClover(clover)
	clover:animate({alpha=0, duration=1000})
	clover:addTask({class="Task", duration=1000})
end

-- ************************************ Clover Object ************************************ --
Clover = deriveLuaClass("Clover", "Object")

function Clover:init(desc)
	Clover.superInit(self, desc)
	self.shadowResource = ""

	self.showing = false
	self.visible = false
	self.clickable = true

	--self:hideClover()
end

function Clover:needsHintArrow()
	return event and not event.cloverPlucked and self.visible and self.clickable
end

function Clover:onProductsInitialized()
--[[DEBUG CLOVERS--]] --print("event "..((event and "yes") or "no"))
	if (event) then
		self.pluckType = event.pluckType
		self.cloverPosition = {x=self.x, y=self.y, floor=self.floor}
		event:addClover(self)
	end
end

function Clover:onClick()
	local clover = self
	if (event and clover.visible and clover.clickable) then
		clover.clickable = false

		taskSystem:cancelAllTasksFor(clover)
		clover:addTask({class="Task", duration=500})
		event.floaterPosition = {x=clover.x, y=clover.y}
		event.cloverPlucked = true
		event:itemDone({})
		event:pluckClover(clover)
	end
	return true
end

function Clover:hideClover()
	if (event) then
		event:addClover(self)
		event.cloversAlreadyOnScreen = event.cloversAlreadyOnScreen - 1
	end
	self.showing = false
	self.visible = false
	-- Clicked or not clicked set object to be used again
	self.clickable = true
	self.alpha = 1
end
