dofile("script/events/Event.lua")

-- ************************************ Fix Event ************************************ --
FixEvent = deriveLuaClass("FixEvent", "Event")

-- Sample usage '--
--[[

levelDesc.eventDesc =
{
	class = "FixEvent",
	autoStart = true,
	initObjectsHidden = false,
	hideItemOnDone = false,
	scoreAllDone = 150,
	scoreOneDone = 25,
	addExclamationMarks = true,
}

--]]

function FixEvent:init(desc)
	FixEvent.superInit(self, desc)

	-- Configuration variables
	self.fixAmount = desc.fixAmount or 10000
	self.fixObject_onWorkStart = desc.fixObject_onWorkStart -- or FixEvent.initfixEventOnWorkStart
	self.addExclamationMarks = desc.addExclamationMarks or false -- Default FALSE
	self.resetProgressOnCancel = desc.resetProgressOnCancel or false -- Default FALSE
	self.allowCancel = desc.allowCancel ~= false -- Default true
	self.doProgressBars = self.doProgressBars ~= false -- Default true
	self.linkedObjectForFrameIncrease = desc.linkedObjectForFrameIncrease
	self.workSoundId = desc.workSoundId
	self.fixEventObjectsRelationWithObjectsToBeShown = {}
	-- Status variables
	self.hintFired = false

	-- Override
	self.matchingItemType = "FixableObject"

	self.usinginitial = false
	--print(self.usinginitial)
end

function FixEvent:onInitialized()
	FixEvent.super.onInitialized(self)
	self.usinginitial = false
	--print(self.usinginitial)
	if (event.linkedObjectForFrameIncrease) then
		self.linkedObject = level:getObject(self.linkedObjectForFrameIncrease)
	end

	self.totalFixAmount = 0
	for _,item in ipairs(self.items) do
		item.fixAmount = item.fixAmount or item.obj.fixAmount or self.fixAmount
		if (item.obj and item.obj.numSteps) then
			item.totalFixAmount = item.fixAmount * item.obj.numSteps
		end
		item.totalFixAmount = item.totalFixAmount or item.fixAmount
		self.totalFixAmount = self.totalFixAmount + item.totalFixAmount

		item.obj.toBeShown = {}
		if self.fixEventObjectsRelationWithObjectsToBeShown ~= nil then
			for _,tableEntry in ipairs(self.fixEventObjectsRelationWithObjectsToBeShown) do
				if tableEntry.fixEventObject == item.obj.name then
					if (item.obj.toBeShown == nil) then
						item.obj.toBeShown = {}
					end
					if (item.obj.initialMarker == nil) then
						item.obj.initialMarker = true
					end
					for k,objName in ipairs(tableEntry.objectsToShow) do
						local obj = level:getObject(objName)
						obj.visible = false
						obj.name = objName
						table.insert(item.obj.toBeShown, obj)

							--print("tableEntry.fixEventObject "..tableEntry.fixEventObject)
							--print("objName "..objName)
							if (objName ~= "fix_obj1" and objName ~= "fix_obj2" and objName ~= "fix_obj3") then 
								item.obj.initialMarker=false
							end

							if (objName == "fix_obj3") then 
								item.obj.initialMarker=false
								self.usinginitial = true
							end

					end
					
				end


			end
		end





	end



end

function FixEvent:setupChallenge()
	--print("Event:setupChallenge(): "..self.totalFixAmount)
	level.challengeManager:setupEventChallenge(math.max(1, self.totalFixAmount), false)
end

FixEvent.onProgress = nil

function FixEvent:showItem(item)
	--print("function FixEvent:showItem(item)")

	FixEvent.super.showItem(self, item)
	if (item.base_animations and #item.base_animations > 0) then
		item.obj.baseAnimation = item.base_animations[1]
	end

	item.workAnimation = item.obj.workAnimation or self.workAnimation or "work"
	item.workAnimationFrameTime = item.obj.workFrameTime or self.workAnimationFrameTime or 200
	item.workDirection = item.obj.workDirection or self.workDirection or "up"

--[[
	item.fixAmount = item.fixAmount or item.obj.fixAmount or self.fixAmount
	if (item.obj and item.obj.numSteps) then
		item.totalFixAmount = item.fixAmount * item.obj.numSteps
	end
	item.totalFixAmount = item.totalFixAmount or item.fixAmount
]]
	if (self.doProgressBars ~= false) then
		item.bar = item.bar or self.bar or "progressbar_default"
	end

	item.progress = 0
	item.totalProgress = 0
	item.name = item.obj.name
	item.obj.onClick = self.fixObject_onClick
	item.obj.onWorkStart = self.fixObject_onWorkStart
	item.obj.onWorkUpdate = self.fixObject_onWorkUpdate
	item.obj.onWorkFinish = self.fixObject_onWorkFinish
	item.obj.onWorkCancel = self.fixObject_onWorkCancel
	item.obj.shouldTaskEnd = self.fixObject_shouldTaskEnd
	item.obj.onWorkEnd = self.fixObject_onWorkEnd
	item.obj.onWorkEndRestoreActor = self.fixObject_onWorkEndRestoreActor
	item.obj.updateFixAppearance = self.fixObject_updateFixAppearance


--print(self.usinginitial)


	if (self.usinginitial == true) then 
		if (self.addExclamationMarks and item.obj.initialMarker==true) then
			item.obj:createOverhead("exclamation:gamescene")
        --if (self.onExlamationMark) then
        --    self:onExlamationMark(item.obj.overhead)
        --end
		end
	end

	if (self.usinginitial == false) then
		if (self.addExclamationMarks) then
			item.obj:createOverhead("exclamation:gamescene")
		end
	end




	
	if (item.visible == false) then 

		item.obj:removeOverhead(true)
	end

	if (self.onItemShowFloater) then
		local floaterPos = self:getFloaterPosition(item.obj)
		level:createGenericFloater(self.onItemShowFloater, floaterPos.x, floaterPos.y)
	end

	item.onRestoreItem = item.onRestoreItem or function()
		local workCharacter = item.obj:getWorkCharacter()
		if (workCharacter) then
			workCharacter.direction = "down"
			workCharacter:playDefaultAnimation()
			taskSystem:abortWorkTasksFor(workCharacter, item.obj)
		end
	end

	if (self.doProgressBars ~= false) then
		self:createProgressBar(item)
	end
	if (not self.showHintOnStart) then
		self:showHint(item.obj)
	end



end

function FixEvent:onItemDone(item)
	FixEvent.super.onItemDone(self, item)
--	print(#item)
--	print(item.obj.name)
	if (item.obj) then
		if (item.base_animations and #item.base_animations > 0) then
			item.obj.baseAnimation = item.base_animations[#item.base_animations]
		else
			item.obj:showLastFrame()
		end
		item.obj:removeOverhead(true)
		item.obj.doHighlight = false
		
	end
end

function FixEvent:getProgress()
	if (self.trackProgress) then
		local num = self:getNumStepsTodo()
		if (num <= 0) then return 1 end

		local progress = 0
		for i=1,num do
			local fixAmount = self.items[i].fixAmount
			local fraction = (fixAmount > 0 and (self.items[i].progress / fixAmount)) or 1
			progress = progress + math.clamp(fraction, 0, 1)
		end
		return progress / num
	end
	return FixEvent.super.getProgress(self)
end

-- ************************************ Fix Object ************************************ --
FixableObject = deriveLuaClass("FixableObject", "EventObject")
ClickFixableObject = deriveLuaClass("ClickFixableObject", "EventObject")

function FixEvent.fixObject_onClick(self)
	if (level.event_onClick) then
		local handled = level:event_onClick(self)
		if (handled ~= nil) then return handled end
	end

	if (event.clickSoundId) then
		self:playSample(event.clickSoundId)
	end

	if (not self.item.done) then
		self.clickable = false
		self:addWorkTaskFor(self:getWorkCharacter())
		self.item.obj:removeOverhead(true)
	end
	return true
end

function FixEvent.fixObject_onWorkStart(self, task)
	if (level.event_onWorkStart) then
		local handled = level:event_onWorkStart(self, task)
		if (handled ~= nil) then return handled end
	end
	event:hideHintArrow()
	if (not self.item.done) then
		task.actor.direction = self.item.workDirection
		task.actor:playAnimation(self.item.workAnimation, -1, self.item.workAnimationFrameTime)
		task.duration = self.item.fixAmount - self.item.progress
		if (event.doProgressBars ~= false) then
			event:showProgressBar(self.item)
		end
	end

	local soundId = self.workSoundId or event.workSoundId
	if (soundId and soundId ~= "") then
		self:loopSample(soundId)
	end

	return true
end

function FixEvent.fixObject_updateFixAppearance(self)
	self.fraction = math.min(1, self.item.progress / self.item.fixAmount)

	if (self.item.matchEventAppearance ~= false and event.matchItemAppearance ~= false) then
		if (self.item.base_animations and #self.item.base_animations > 0) then
			local c = #self.item.base_animations-1
			local idx = math.floor(math.min(self.fraction * c, c)+1)
			if (self.baseAnimation ~= self.item.base_animations[idx]) then
				self.baseAnimation = self.item.base_animations[idx]
				if (self.stateChange) then
					self:stateChange()
				end
			end
		else

			local c = self.frameCount
			if (not event.hideItemOnDone) then
				c = c - 1
			end
			local frame = math.min(self.fraction * c, c)
			if (self.frame ~= frame) then
				if (event.linkedObject) then
					event.linkedObject.frame = frame
				end
				self.frame = frame
				if (self.stateChange) then
					self:stateChange()
				end
			end
		end
	end

	if (isTable(self.item.bar)) then
		self.item.bar:setProgress(self.fraction)
	end
end

function FixEvent.fixObject_shouldTaskEnd(self, task)
	local result = (self.item.progress >= self.item.fixAmount) or
		(taskSystem:hasTaskFor(task.actor) and event.allowCancel ~= false) or
		event.completed or level.hasShiftFinished

--if (result == true) then
--	print("Object_shouldTaskEnd "..self.item.progress.." "..self.item.fixAmount)
--end

	return result
end

function FixEvent.fixObject_onWorkUpdate(self, task, time)
	if (level.event_onWorkUpdate) then
		local handled = level:event_onWorkUpdate(self, task, time)
		if (handled ~= nil) then return handled end
	end

--print("function FixEvent.fixObject_onWorkUpdate(self, task, time)")

	self.item.progress = self.item.progress + time -- We cannot use the task's timer!
	self.item.totalProgress = self.item.totalProgress + time

	self:updateFixAppearance(task)
	event:onProgress()
	return self:shouldTaskEnd(task)
end

function FixEvent.fixObject_onWorkFinish(self, task)
	if (level.event_onWorkFinish) then
		local handled = level:event_onWorkFinish(self, task)
		if (handled ~= nil) then return handled end
	end
	self:onWorkEnd(task)
end

function FixEvent.fixObject_onWorkCancel(self, task)
	if (level.event_onWorkCancel) then
		local handled = level:event_onWorkCancel(self, task)
		if (handled ~= nil) then return handled end
	end
	self:onWorkEnd(task)
end

function FixEvent.fixObject_onWorkEnd(self, task)
	local isDone = (self.item.totalProgress >= self.item.totalFixAmount)
	self.clickable = not isDone
	if (isDone) then
		event:stepDone(self.item)
		if (self.stateChange) then
			self:stateChange()
		end
		if (event.workFinishSound) then
	        self:addTask({class="PlaySoundTask", sound=event.workFinishSound})
		end
		for _,item in ipairs(self.item.obj.toBeShown) do
			item.visible = true
			if (item.initialMarker == false) then
			item:createOverhead("exclamation:gamescene")
			end

		end
		if (level.event_onWorkEnd) then
			local handled = level:event_onWorkEnd(self, task)
		end
	else
		if (event.resetProgressOnCancel) then
			self.item.progress = 0
		end
	end

	self:updateFixAppearance(task)
	self:stopLoopSample()
	if (self.fixObject_onWorkEndRestoreActor) then
		self:onWorkEndRestoreActor(task)
	end
end

function FixEvent.fixObject_onWorkEndRestoreActor(self, task)
	task.actor.direction = "down"
	task.actor:playDefaultAnimation()
end
