dofile("script/events/Event.lua")

-- ************************************ FixEvent With Gestures ************************************ --
-- Sample usage
--[[
	levelDesc.eventDesc =
	{
		class = "FixWithGesturesEvent",
		autoStart = true,
		fixableObjectName = "day5_fix1",
		scoreAllDone = 0,
		scoreOneDone = 0,
		goal=4,
	}
]]--
-- Additional info: Showing/Hiding an object also will Enable/Diable its obstruction nodes

FixWithGesturesEvent = deriveLuaClass("FixWithGesturesEvent", "Event")

function FixWithGesturesEvent:init(desc)
	FixWithGesturesEvent.superInit(self, desc)

	-- Configuration variables
	self.addExclamationMarks = desc.addExclamationMarks or false
	self.doProgressBars = self.doProgressBars or false
	self.fixableObjectName = desc.fixableObjectName
	self.initialDelay = desc.initialDelay or 2000
	self.eventDone = false
	self.fixableObject = nil
	self.numSteps = 0
	self.remainingSteps = desc.goal
end

function FixWithGesturesEvent:onInitialized()
	self.fixableObject = level:getObject(self.fixableObjectName)
end

function FixWithGesturesEvent:setupChallenge()
	level.challengeManager:setupEventChallenge(self.remainingSteps, true)
	self:activateObject(self.initialDelay)
end

-- Set the onObjectFixed callback and activate the object
function FixWithGesturesEvent:activateObject(initialDelay)
	local selfObject = self
	self.fixableObject.onFixedCallback = function()
		selfObject:onObjectFixed()
	end

	self.fixableObject:activateGestureWithDelay(initialDelay)
end

-- Callback triggered when the object is fixed, it will activate the next one
function FixWithGesturesEvent:onObjectFixed()
	if (not level.hasShiftFinished) then
		if (self.fixableObject.nextObjectName and self.fixableObject.nextObjectName~= "") then
			self.fixableObject = level:getObject(self.fixableObject.nextObjectName)
			self:activateObject(self.initialDelay)
		end
	end
end

function FixWithGesturesEvent:AddStep()
	self.numSteps = self.numSteps + 1
	self:onProgress()
	SpriteExt.playSample(nil, "event_item_complete")
end

function FixWithGesturesEvent:getProgress()
	if (self.numSteps < self.remainingSteps) then
		return self.numSteps / self.remainingSteps
	end
	return 1
end

function FixWithGesturesEvent:eventLost()
	if not self.won then
		FixWithGesturesEvent.super.eventLost(self)
		self.fixableObject:OnEventLost()
	end
end
