dofile("script/events/Event.lua")

HideAndSeekEvent = deriveLuaClass("HideAndSeekEvent", "Event")

function HideAndSeekEvent:init(desc)
  HideAndSeekEvent.superInit(self, desc)

  self.hiddenObjects = {}

  self.initialized = false
  self.startTimePassed = 0.0
  self.numObjectsInLevel = 0
  self.numSteps = 0

  self.hasFinished = false
end

function HideAndSeekEvent:onInitialized()
  self:SetupConfigurationParams()
  self:DoErrorControl()

  self.currentHiddenObjects = {}
  self.currentTimes = {}
  self.currentShowDelays = {}

  for i=1,self.objectsAmount do
    self.currentTimes[i] = 0.0
    self.currentShowDelays[i] = 0
  end
end

function HideAndSeekEvent:setupChallenge()
  -- Configures the current challenge:
  level.challengeManager:setupEventChallenge(self.objectsGoal, true)
end

function HideAndSeekEvent:SetupConfigurationParams()
  local configObject = level:getObject(self.configObjectName)
  if configObject ~= nil then
    self.objectsGoal = configObject.objectsGoal or 1
    self.objectsAmount = configObject.objectsAmount or 1
    self.totalScore = configObject.totalScore or 100
    self.totalScoreFloaterPosition = configObject.totalScoreFloaterPosition or {x = 0, y = 0}
    self.totalScoreFloaterDelay = configObject.totalScoreFloaterDelay or 800
    self.startDelay = configObject.startDelay or 2000
    self.minShowDelay = configObject.minShowDelay or 2000
    self.maxShowDelay = configObject.maxShowDelay or 4000
  else
    self.objectsGoal = 1
    self.objectsAmount = 1
    self.totalScore = 100
    self.totalScoreFloaterPosition = {x = 0, y = 0}
    self.totalScoreFloaterDelay = 800
    self.startDelay = 2000
    self.minShowDelay = 2000
    self.maxShowDelay = 4000
  end
end

function HideAndSeekEvent:DoErrorControl()
  -- TODO: in phase 2
end

function HideAndSeekEvent:AddObject(obj)
  -- Adds a new hidden object from the current scene into the pool of objects:
  table.insert_if_unique(self.hiddenObjects, obj)
end

function HideAndSeekEvent:onTick(time)
  if self.eventDone == true or self.hasFinished == true then return end

  -- Runs this code the first time the event loads:
  if self.initialized == false and (self.startTimePassed >= self.startDelay) then
    self.initialized = true
    self:ChooseNextHiddenObject()
    self:EnableNextHiddenObject()
    self.startTimePassed = 0.0
  end

  -- Runs this code every time the game ticks:
  if self.initialized == true then
    if self.numObjectsInLevel < self.objectsAmount then
      self:ChooseNextHiddenObject() -- Chooses a new object randomly
      self:EnableNextHiddenObject() -- Makes it appear onto the screen
    end

    for k,v in ipairs(self.currentHiddenObjects) do
      if v.showing == false then
        if self.currentTimes[k] >= self.currentShowDelays[k] then
          self.numObjectsInLevel = self.numObjectsInLevel - 1
          table.remove(self.currentHiddenObjects, k)
          self.currentShowDelays[k] = 0
          self.currentTimes[k] = 0.0
        end
        self.currentTimes[k] = self.currentTimes[k] + time
      end
    end
  end

  -- Updates time:
  if self.initialized == false then
    self.startTimePassed = self.startTimePassed + time
  end
end

function HideAndSeekEvent:ChooseNextHiddenObject()
  local currentIndex = self.numObjectsInLevel + 1
  local rep = true
  local repCount = 0
  if #self.hiddenObjects > 0 then
    local hiddenObjectIndex
    while rep == true and repCount < #self.hiddenObjects do
      hiddenObjectIndex = math.Random(#self.hiddenObjects)
      local currentObject = self.hiddenObjects[hiddenObjectIndex]
      if currentObject.clickableOnce and currentObject.hasBeenClicked == true then
        table.remove(self.hiddenObjects, hiddenObjectIndex)
        self:ChooseNextHiddenObject()
        return
      end
      repCount = repCount + 1
      for i=1,#self.currentHiddenObjects do
        if self.currentHiddenObjects[i] == self.hiddenObjects[hiddenObjectIndex] then
          rep = true
          break
        end
        rep = false
      end
    end

    table.insert(self.currentHiddenObjects, self.hiddenObjects[hiddenObjectIndex])
    self.currentShowDelays[currentIndex] = math.floor(math.Random(self.minShowDelay, self.maxShowDelay))
    self.currentTimes[currentIndex] = 0.0
    self.numObjectsInLevel = self.numObjectsInLevel + 1
  else
    table.remove(self.currentHiddenObjects, 1)
  end
end

function HideAndSeekEvent:EnableNextHiddenObject()
  local currentIndex = self.numObjectsInLevel
  if self.currentHiddenObjects[currentIndex] ~= nil then
    self.currentHiddenObjects[currentIndex]:Show()
    self.currentHiddenObjects[currentIndex]:Appear()
  end
end

function HideAndSeekEvent:AddStep(hiddenObject)
  -- Sums up one step to the current event's progress:
  self.numSteps = self.numSteps + 1
	self:scoreItemDone({})

  if hiddenObject.stepScore > 0 then
    level:createFloater({
      id = "eventDone",
      textId = "EVENT_SCORE_ONE_DONE",
		x = hiddenObject.x, y = hiddenObject.y,
		bonusScore = hiddenObject.stepScore,
		bonus = {
			textId = "EVENT_SCORE_ONE_DONE_FMT",
		},
    })
  end


	if (level.onEventStepDone) then
		level:onEventStepDone(self, {})
	end

	self:onProgress()
	SpriteExt.playSample(nil, "event_item_complete")

	if self.numSteps == self.objectsGoal then
		self:eventWon()
		self.eventDone = true
	end
end

function HideAndSeekEvent:eventWon()
  -- TODO: Generic form for calling this!
  self.scoreAllDone = self.totalScore
  self.scoreOneDone = self.stepScore

  HideAndSeekEvent.super.eventWon(self)
end

function HideAndSeekEvent:getProgress()
	if (self.numSteps < self.objectsGoal) then
		return self.numSteps / self.objectsGoal
	end
	return 1
end

function HideAndSeekEvent:onShiftFinish()
    self.super.onShiftFinish(self)

    self.hasFinished = true
    local currentIndex = self.numObjectsInLevel
    if self.currentHiddenObjects[currentIndex] ~= nil then
      self.currentHiddenObjects[currentIndex]:Hide()
    end
end
