dofile("script/events/Event.lua")

-- ************************************ Package Delivery Event ************************************ --
PackageDeliveryEvent = deriveLuaClass("PackageDeliveryEvent", "Event")

-- Sample usage
--[[
levelDesc.eventDesc =
{
	class="PackageDeliveryEvent",
	deliveryCharacter = {traits="tashi"},
	deliveryCharacterWalks = true,
	--deliveryCharacter = {"antonio", "tashi"},
	deliveryIcon = "icon#foodbox:pool_icons",
	--deliveryIcon = {"icon#foodbox:pool_icons", "icon#refreshing_beverage:pool_icons"},

	numDeliveriesInTotal = 6,
	numDeliveriesNeeded = 4,
	--deliverySound = "balloon_pops_up:sound",
	deliveryAppearance = "carry",
	--walkToHeroPositionWhenDone = false,	-- Default FALSE
	deliveryStartPosition = {x=-65, y=145},
	deliveryPosition = {x=365, y=285},
	storagePosition = {x=1080, y=720},
	storageObject = "gift_pile",
	--pickupPosition = x=365, y=285},		-- if not set, uses deliveryPosition
	--deliveryDirection = "down",			-- default
	--deliveryAnimation = "work",			-- default
	--deliveryAnimationDirection = "up",	-- default
	startDeliveryAnimation = "work_up"
	afterDeliveryAnimation = "work_up"
	matchOfferedPackagesToProgress = true,
	timeBeforeFirstDelivery = 0,
	timeBetweenDeliveriesMin = 10000,
	timeBetweenDeliveriesMax = 12000,
	deliveryWaitTime = 3000,
}
]]--

function PackageDeliveryEvent:init(desc)
	PackageDeliveryEvent.superInit(self, desc)
	self.deliveryAnimationDirection = desc.deliveryAnimationDirection or "up"
	self.startDeliveryAnimation = desc.startDeliveryAnimation or nil
	self.afterDeliveryAnimation = desc.afterDeliveryAnimation or nil
	self.deliveryAnimation = desc.deliveryAnimation or "work"
	self.deliveryTaskDuration = desc.deliveryTaskDuration or 500
	self.deliveryTaskCycles	= desc.deliveryTaskCycles or 1
	self.deliveryFrameTime = desc.deliveryFrameTime or 100
	self.deliveryDirection = desc.deliveryDirection or "down"
	self.deliveryWhenFinished = desc.deliveryWhenFinished or "default"
	self.deliveryWhenFinishedDirection = desc.deliveryWhenFinishedDirection or "down"

	self.matchOfferedPackagesToProgress = self.matchOfferedPackagesToProgress ~= false -- Default true
	self.numDeliveriesNeeded = self.numDeliveriesNeeded or self.numDeliveriesInTotal
	self.numDeliveriesInTotal = self.numDeliveriesInTotal or self.numDeliveriesNeeded
	self.deliveryStartPosition = desc.deliveryStartPosition or {}
	self.deliveryPosition = desc.deliveryPosition or {}
	self.deliverySound = desc.deliverySound or "balloon_pops_up:sound"
	if (desc.deliveryIcon) then
		self.deliveryIcons = self.deliveryIcons or {}
		if (isTable(desc.deliveryIcon)) then
			for _,v in ipairs(desc.deliveryIcon) do
				GH.assert(isString(v))
				table.insert(self.deliveryIcons, v)
			end
		elseif (isString(desc.deliveryIcon)) then
			table.insert(self.deliveryIcons, desc.deliveryIcon)
		end
	end
	if (desc.deliveryEmotion) then
		self.deliveryEmotions = self.deliveryEmotions or {}
		if (isString(desc.deliveryEmotion)) then
			table.insert(self.deliveryEmotions, desc.deliveryEmotion)
		end
	end

	-- Status variables
	self.eventState = "none"
	self.numDeliveryAppearances = self.numDeliveryAppearances or self.numDeliveriesNeeded
	self.nextDeliveryTimer = self.timeBeforeFirstDelivery or math.Random(self.timeBetweenDeliveriesMin, self.timeBetweenDeliveriesMax)
	self.numDeliveriesLeft = self.numDeliveriesInTotal
	self.numDeliveriesRoundTripCounter = 0
	self.numDeliveriesAccepted = 0
	self.numDeliveriesAcceptedThisRound = 0
end

function PackageDeliveryEvent:onInitialized()
	PackageDeliveryEvent.super.onInitialized(self)
end

function PackageDeliveryEvent:setupChallenge()
	level.challengeManager:setupEventChallenge(math.max(1, self.numDeliveriesNeeded), true)
end

function PackageDeliveryEvent:onStart()
	PackageDeliveryEvent.super.onStart(self)

--print("===================================")
	self.deliveryCharacterWalks = self.deliveryCharacterWalks ~= false -- Default true
	self.deliveryCharacters = self.deliveryCharacters or {}
	if (isTable(self.deliveryCharacter)) then
		for _,v in ipairs(self.deliveryCharacter) do
			local char = nil
			if (isTable(v)) then
				char = level:createCharacter(v)
				char.location = self.deliveryStartPosition
			elseif (isString(v)) then
				char = level:resolveActor(v)
			end
			if (char) then
				table.insert(self.deliveryCharacters, char)
			end
		end
	elseif (isString(self.deliveryCharacter)) then
		local char = level:resolveActor(self.deliveryCharacter)
		if (char) then
			self.deliveryCharacter = char
			self.deliveryCharacter.location = self.deliveryStartPosition
			table.insert(self.deliveryCharacters, char)
			self.deliveryCharacter.isEventCharacter = true
		end
	end
	if (isTable(self.deliveryCharacterDesc)) then
		local char = level:resolveActor(self.deliveryCharacterDesc)
		if (char) then
			self.deliveryCharacter = char
			self.deliveryCharacter.location = self.deliveryStartPosition
			table.insert(self.deliveryCharacters, char)
			self.deliveryCharacter.isEventCharacter = true
		end
	end
	if (#self.deliveryCharacters == 0) then
		if (isTable(self.deliveryCharacter)) then
			self.deliveryCharacter = level:createCharacter(self.deliveryCharacter)
			self.deliveryCharacter.isEventCharacter = true
		end
		if (isTable(self.deliveryCharacter)) then
			table.insert(self.deliveryCharacters, self.deliveryCharacter)
		end
	end
	GH.assert(self.deliveryCharacters, "Problem fetching delivery character(s)");

	local tmp_idx = 1
	for _,v in ipairs(self.deliveryCharacters) do
		v.onClick = self.deliveryCharacter_onClick
		v.workPos = v.workPos or self.deliveryCharacterWorkPosOffset or {0,0}
		v.walkSpeed = v.walkSpeed or 600
		v.playIdleAnimations = false
		tmp_idx = tmp_idx + 1
	end

	local deliveryChar = self:getDeliveryCharacter()
	GH.assert(deliveryChar)
	GH.assert(deliveryChar.addTask)
	if (self.deliveryCharacterWalks and false) then
		deliveryChar:addTask({class="WalkToTask", target=self.deliveryStartPosition, playDefaultOnFinish=true})
	end
	self:carryDelivery(deliveryChar, true)
	self.eventState = "waitForNextDelivery"
	if(self.startDeliveryAnimation ~= nil) then
		deliveryChar:addTask({class="PlayAnimationTask",
			animation=self.startDeliveryAnimation, cycles=-1,	duration=-1, frameTime=150,
		})
	end
end

function PackageDeliveryEvent:onTick(time)
	if (self.allItemsDone or self.completed or not self.started) then
		return
	end

	if (self.eventState == "waitForNextDelivery") then
		-- Timer until next delivery
		self.nextDeliveryTimer = self.nextDeliveryTimer - time

		if (self.nextDeliveryTimer <= 0) then
			-- Start delivery routine
			self.eventState = "makingDelivery"

			self.numDeliveriesRoundTripCounter = self.numDeliveriesRoundTripCounter + 1
			self.numDeliveriesAcceptedThisRound = self.numDeliveriesAccepted

			local deliveryChar = self:getDeliveryCharacter()
			GH.assert(deliveryChar)
			GH.assert(deliveryChar.addTask)
			self:carryDelivery(deliveryChar, true)

			if (self.deliveryAppearances) then
				local app = self.deliveryAppearances[((self.numDeliveriesRoundTripCounter-1) % #self.deliveryAppearances) + 1]
				if (app) then
					deliveryChar:setAppearance("deliver_appearance", app)
				else
					deliveryChar:removeAppearance("deliver_appearance")
				end
				deliveryChar:removeAppearance("deliver_delivered")
			end

			local deliveryIcon = self.deliveryIcons[1]
			if (#self.deliveryIcons > 1) then
				GH.assert(#self.deliveryIcons >= self.numDeliveryAppearances, "Not enough delivery icons: "..table.serialize("self.deliveryIcons", self.deliveryIcons))
				deliveryIcon = self.deliveryIcons[self:getDeliveryAppearanceIdx()]
			end
			if (self.deliveryCharacterWalks) then
				deliveryChar:addTask({class="WalkToTask", target=self.deliveryPosition, playDefaultOnFinish=true})
			end
			deliveryChar:addTask({class="DirectionTask", direction=self.deliveryDirection})
			deliveryChar:addTask({class="OverheadTask", image=deliveryIcon, soundId=self.deliverySound, frameTime=self.deliveryIconFrameTime})
			deliveryChar:addTask({class="CallTask",
				func=function()
					deliveryChar:playSample("balloon_pops_up:sound")
					self.pickupTimer = self.deliveryWaitTime
					self.eventState = "waitingForPickup"
					if (deliveryChar.soundId) then
						SpriteExt.playSample(nil, deliveryChar.soundId)
					end
					deliveryChar.clickable = true
					if (self.showHintOnStart) then
						self.showHintOnStart = false
						self:showHint()
					end
					if (not self.pickupPosition) then
						self.pickupPosition = {self.deliveryPosition.x, self.deliveryPosition.y}
					end
					if (self.onWaitingForPickup) then
						self:onWaitingForPickup(deliveryChar)
					end
				end
			})
		end
	elseif (self.eventState == "waitingForPickup") then
		self.pickupTimer = self.pickupTimer - time
		if (self.pickupTimer <= 0) then
			self:hideHintArrow()
			self:returnFromDelivery(self:getDeliveryCharacter(), false)
		end
	end
end

function PackageDeliveryEvent:getStorageObject()
	local storageObject = event.storageObject and level:getObject(event.storageObject)
	if (not storageObject and self.storageObjects) then
		storageObject = self.storageObjects[event.numDeliveriesAccepted]
		if (storageObject) then
			storageObject = level:getObject(storageObject)
		end
	end
	return storageObject
end

function PackageDeliveryEvent.deliveryCharacter_onClick(obj)
	if (not table.contains(event.deliveryCharacters, obj)) then
		return
	end

	event.eventState = "deliveryClicked"
	event.numDeliveriesAccepted = event.numDeliveriesAccepted + 1

	obj.clickable = false
	event:hideHintArrow()

	local storageObject = event:getStorageObject()

	hero:addTask({class="WalkToTask", target=event.pickupPosition, playDefaultOnFinish=false})
	hero:addTask({class="DirectionTask", direction=event.deliveryAnimationDirection})
	hero:addTask({class="CallTask", func=function() event:onDeliveryWorkStart(obj) end})
	local dur = event.deliveryTaskDuration or 2000
	local cyc = event.deliveryTaskCycles or -1
	if (dur > 0 or cyc > 0) then
		local whenFinished = (not storageObject) and event.deliveryWhenFinished or nil
		hero:addTask({class="PlayAnimationTask",
			animation=event.deliveryAnimation,
			cycles=event.deliveryTaskCycles,
			duration=event.deliveryTaskDuration or 2000,
			frameTime=event.deliveryFrameTime,
			whenFinished=event.deliveryWhenFinished ,
			wait=true})
		if (not storageObject) then
			hero:addTask({class="DirectionTask", direction=event.deliveryWhenFinishedDirection})
		end
	end
	hero:addTask({class="CallTask", func=function() event:onDeliveryPickedUp(obj) end})

	event:carryDelivery(hero, true)

	if (storageObject) then
		hero:addTask({class="WalkToTask", target=storageObject, playDefaultOnFinish=false, directionOnFinish=storageObject.workDirection})
		hero:addTask({class="CallTask", func=function() event:onStorageObjectWorkStart(storageObject) end})
		local dur = storageObject.workDuration or 0
		local cyc = storageObject.workCycles or -1
		if (dur > 0 or cyc > 0) then
			hero:addTask({class="PlayAnimationTask",
				animation=storageObject.workAnimation or "work",
				duration=dur,
				cycles=cyc,
				frameTime=storageObject.workFrameTime or 100,
				whenFinished=storageObject.workWhenFinished or "default",
				wait=true})
		end
		hero:addTask({class="CallTask", func=function() event:onStorageObjectWorkFinish(storageObject) end})
	elseif (event.storagePosition) then
		hero:addTask({class="WalkToTask", target=event.storagePosition, playDefaultOnFinish=true})
	end
	hero:addTask({class="CallTask", func=function() event:deliveredDelivery() end})
	event:carryDelivery(hero, false)
	if (event.workFinishSound) then
		hero:addTask({class="PlaySoundTask", sound=event.workFinishSound})
	end
	hero:addTask({class="Task", duration=500})
	if (event.walkToHeroPositionWhenDone) then
		hero:addTask({class="WalkToTask", target=level.heroPosition, playDefaultOnFinish=true, directionOnFinish="down"})
	end
	hero:addTask({class="CallTask", func=function() event:onItemDone() end})
	
	obj.blinkLikeProduct = true
end

function PackageDeliveryEvent:onDeliveryWorkStart(deliveryChar)
end

function PackageDeliveryEvent:onDeliveryPickedUp(deliveryChar)
	event:itemDone({}) -- Dummy
	event:returnFromDelivery(deliveryChar, true) -- Let delivery char walk away
end

function PackageDeliveryEvent:onStorageObjectWorkStart(storageObject)
end

function PackageDeliveryEvent:onStorageObjectWorkFinish(storageObject)
	storageObject.frame = storageObject.frame + 1
end

function PackageDeliveryEvent:deliveredDelivery()
end

function PackageDeliveryEvent:returnFromDelivery(deliveryCharacter, deliverySuccess)
	deliveryCharacter.clickable = false
	self.eventState = "returningFromDelivery"
	self.nextDeliveryTimer = math.Random(self.timeBetweenDeliveriesMin, self.timeBetweenDeliveriesMax)
--print("self.timeBetweenDeliveriesMin: "..tostring(self.timeBetweenDeliveriesMin))
--print("self.timeBetweenDeliveriesMax: "..tostring(self.timeBetweenDeliveriesMax))
	deliveryCharacter:addTask({class="RemoveOverheadTask"})

	self.numDeliveriesLeft = self.numDeliveriesLeft - 1

	if (deliverySuccess) then
		if (self.deliveryDeliveredEmotion) then
			deliveryCharacter:setAppearance("deliver_delivered", self.deliveryDeliveredEmotion)
		end
	else
		if (self.deliveryMissedEmotion) then
			deliveryCharacter:setAppearance("deliver_delivered", self.deliveryMissedEmotion)
		end
	end

	if (deliverySuccess) then
		self:carryDelivery(deliveryCharacter, false)
	end
	deliveryCharacter:addTask({class="Task", duration=200})
	if (self.deliveryCharacterWalks) then
		deliveryCharacter:addTask({class="WalkToTask", target=self.deliveryStartPosition, playDefaultOnFinish=true, directionOnFinish="down"})
	end
	deliveryCharacter:addTask({class="CallTask", func=function()
		if (event.numDeliveriesAccepted < self.numDeliveriesNeeded) then
			self.eventState = "waitForNextDelivery"
		else
			self.eventState = "allDeliveriesAccepted"
		end
		if(self.afterDeliveryAnimation ~= nil) then
			deliveryCharacter:addTask({class="PlayAnimationTask",
				animation=self.afterDeliveryAnimation, cycles=-1,	duration=-1, frameTime=150,
			})
		end
	end})

	if (level.hasShiftFinished and self.numDeliveriesAccepted < self:getNumItemsTodoTotal()) then
		deliveryCharacter:addTask({class="CallTask", func=function()
			self:eventLost()
		end})
	end
	
	deliveryCharacter.blinkLikeProduct = false
end

function PackageDeliveryEvent:carryDelivery(character, carry)

	if (carry) then
		if (self.deliveryAppearance) then
			character:addTask({class="SetAppearanceTask", appearanceId=self.deliveryAppearance, appearanceValue=self.deliveryAppearance})
		end
		if (isTable(self.deliveryEmotions) and #self.deliveryEmotions > 0) then
			local emotion = self.deliveryEmotions[1]
			if (#self.deliveryEmotions > 1) then
				GH.assert(#self.deliveryEmotions >= self.numDeliveryAppearances, "Not enough delivery emotions: "..table.serialize("self.deliveryEmotions", self.deliveryEmotions))
				emotion = self.deliveryEmotions[self:getDeliveryAppearanceIdx()]
			end
			GH.assert(isString(emotion))
			character:addTask({class="EmotionTask", emotion=emotion})
		elseif (isString(self.deliveryEmotions)) then
			character:addTask({class="EmotionTask", emotion=self.deliveryEmotions})
		end
	else
		if (self.deliveryAppearance) then
			character:addTask({class="RemoveAppearanceTask", appearanceId=self.deliveryAppearance})
		end
		if (self.deliveryEmotions) then
			character:addTask({class="EmotionTask"})
		end
	end
end

function PackageDeliveryEvent:itemDone(item)
	self.itemsDone = self.itemsDone + 1

	self:scoreItemDone(item)

	if (level.onEventStepDone) then
		level:onEventStepDone(self, item)
	end

	self:onProgress()

	SpriteExt.playSample(nil, "event_item_complete")
end

function PackageDeliveryEvent:getNumItemsTodoTotal()
	return self.numDeliveriesNeeded
end

function PackageDeliveryEvent:getProgress()
	if (self.itemsDone < self.numDeliveriesNeeded) then
		return self.itemsDone / self.numDeliveriesNeeded
	end
	return 1
end

function PackageDeliveryEvent:getDeliveryAppearanceIdx()
	if (self.matchOfferedPackagesToProgress) then
		return (self.numDeliveriesAcceptedThisRound % self.numDeliveryAppearances) + 1
	end
	return ((self.numDeliveriesInTotal - self.numDeliveriesLeft) % self.numDeliveryAppearances) + 1
end

function PackageDeliveryEvent:getDeliveryCharacter()
	local charIdx = ((self.numDeliveriesInTotal - self.numDeliveriesLeft) % #self.deliveryCharacters) + 1
	return self.deliveryCharacters[charIdx] or self.deliveryCharacter
end

function PackageDeliveryEvent:getFloaterPosition()
	if (isTable(self.floaterPosition)) then
		return self.floaterPosition
	elseif (hero) then
		return hero.floaterPosition
	end
end
