dofile("script/events/Event.lua")

RequestAttentionObject = deriveLuaClass("RequestAttentionObject", "DelCharacter")

function RequestAttentionObject:init(desc)
  RequestAttentionObject.superInit(self, desc)

  self.name = desc.name or ""
  self.basicAnimSection = desc.basicAnimSection or "dev"
  self.basicAnimName = desc.basicAnimName or "IMAGE_MISSING"
  self.basicAnimCycles = desc.basicAnimCycles or -1
  self.basicAnimFrameTime = desc.basicAnimFrameTime or 0
  self.playIfNotGameplay = desc.playIfNotGameplay or false
  self.hideIfNotGameplay = desc.hideIfNotGameplay ~= false
  self.barBlueprint = desc.barBlueprint or "decreasingbar_requestAttention"
  self.barPositionAbove = desc.barPositionAbove ~= false
  self.barPosition = desc.barPosition or {x = 0, y = 0}
  self.barDecreasingMode = desc.barDecreasingMode or "By time"
  self.barClickIncrease = desc.barClickIncrease or 10.0
  self.barClickDrainDelay = desc.barClickDrainDelay or 0.0
  self.barClickSoundName = desc.barClickSoundName or "button:ui"
  self.barWarningEnabled = desc.barWarningEnabled ~= false
  self.barWarningPercent = desc.barWarningPercent or 10
  self.barWarningSound = desc.barWarningSound or "eventsomethingbad:sound"
  self.barWarningBlink = desc.barWarningBlink ~= false
  self.barEasingEnabled = desc.barEasingEnabled ~= false
  self.barEasingDuration = desc.barEasingDuration or 600
  self.barBlinkEnabled = desc.barBlinkEnabled ~= false
  self.barBlinkDuration = desc.barBlinkDuration or 100
  self.barBlinkFadeDuration = desc.barBlinkFadeDuration or 80
  self.showDecreaseBar = desc.showDecreaseBar or false
  self.parentEventObject = self
  self.initialDelay = desc.initialDelay or 2000

  if self.barDecreasingMode == "By time" then
    self.barDecreasingTime = desc.barDecreasingTime or 30000
    self.barDecreasingSpeed = -1
    self.timeModeDecreasePortionMs = self.barDecreasingTime * 0.01
  else -- By speed
    self.barDecreasingSpeed = desc.barDecreasingSpeed or 1.0
    self.barDecreasingTime = -1
  end

  if self.barClickDrainDelay < 0 then self.barClickDrainDelay = 0.0 end

  if self.hideIfNotGameplay == true then
    self.visible = false
  end

  self.firstTick = true
  self.currentBarLevel = 1
  self.timePassed = 0.0
  self.eventFinished = false
  self.canPlayWarningSound = true
end

function RequestAttentionObject:onLevelInitialized()
  RequestAttentionObject.super.onLevelInitialized(self)

  if (self.showDecreaseBar == true) then
      self:CreateDecreasingBar()
  end
  self:ConfigureMouseCallbacks()

  if self.playIfNotGameplay == true then self:PlayBasicAnimation() end

  self:HideBar(false)

  self.enableDecreasing = false
end

function RequestAttentionObject:onTick(time)
  self:tick(time)
  if (self.firstTick == true and level.isPlaying) then
    if self.hideIfNotGameplay == true then self.visible = true end
    if self.playIfNotGameplay == false then self:PlayBasicAnimation() end
    self.firstTick = false
  end

  if (level.isPlaying and
      self.eventFinished == false and
      self.enableDecreasing == true) then
    self.timePassed = self.timePassed + time

    self:ShowBar(self.barEasingEnabled, self.barBlinkEnabled)

    self:UpdateBar(time)
    self:CheckForWarning()

    if (self.decreasingBar) then
        if (self.currentBarLevel == 0) then
            self.failEvent = true
            self.clickable = false
            self.decreasingBar.back.clickable = false
        end
    end
  end
end


function RequestAttentionObject:ActivateAttention()
  if not eventFinished then
    self.enableDecreasing = true
    self.clickable = true
    self:ShowBar(true)
  end
end

function RequestAttentionObject:PlayBasicAnimation()
  self.animTask = self:addTask({class = "PlayAnimationTask",
    animation = self.basicAnimName,
    cycles = self.basicAnimCycles,
    frameTime = self.basicAnimFrameTime
  })
end

function RequestAttentionObject:CreateDecreasingBar()
  self.decreasingBar = self:addChild(blueprints[self.barBlueprint])

  local barLocation
  if self.barPositionAbove == true then
    barLocation = {x = 0.0,
                   y = -(self.height + self.decreasingBar.height * 0.75)}
  else
    local globalBarPos = self.barPosition
    barLocation = {x = globalBarPos.x,
                   y = globalBarPos.y}
  end

  self.decreasingBar.location = barLocation
  self.decreasingBar.floor = 0
  self.decreasingBar:setProgress(self.currentBarLevel)
end

function RequestAttentionObject:ShowBar(easingEnabled, blinkingEnabled)
  if self.decreasingBar and self.decreasingBar.alpha == 0 then
    if blinkingEnabled then
      self:DoBarBlinking()
    end
    if easingEnabled == true then
      self:addTask({class = "Task",
        duration = self.barBlinkDuration})
    else
      self.decreasingBar.alpha = 1
    end

    self:addTask({class = "CallTask",
      func = function()
        self.decreasingBar:cancelAnimations()
        self.decreasingBar.alpha = 1
      end
    })

    self.decreasingBar.back.clickable = true
  end
end

function RequestAttentionObject:HideBar(easingEnabled)
  if self.decreasingBar and self.decreasingBar.alpha == 1 then
    if easingEnabled == true then
      self.decreasingBar:animate({alpha = 0, duration = self.barEasingDuration})
    else
      self.decreasingBar.alpha = 0
    end
    self.decreasingBar.back.clickable = false
  end
end

function RequestAttentionObject:UpdateBar(time)
  if self.currentBarLevel > 0 then
    if (self.barDecreasingMode == "By time") then
      local timePortion = self.timeModeDecreasePortionMs
      local barPortions = self.timePassed / timePortion
      self.currentBarLevel = self.currentBarLevel - (barPortions * 0.01)
      self.currentBarLevel = math.clamp(self.currentBarLevel, 0, 1)
      self.timePassed = 0.0
    else
      self.currentBarLevel = self.currentBarLevel - (time * self.barDecreasingSpeed) / 10000
    end
    self.currentBarLevel = math.clamp(self.currentBarLevel, 0, 1)
    if (self.decreasingBar) then
        self.decreasingBar:setProgress(self.currentBarLevel)
    end
  end
end

function RequestAttentionObject:ConfigureMouseCallbacks()
  if self.decreasingBar then
    self.decreasingBar.back.onClick = function()
      self:onClick()
    end
    self.decreasingBar.back.onMouseOver = function()
      self.highlighted = true
    end
    self.decreasingBar.back.onMouseOut = function()
      self.highlighted = false
    end

    self.onMouseOver = function()
      self.decreasingBar.back.highlighted = true
    end
    self.onMouseOut = function()
      self.decreasingBar.back.highlighted = false
    end
  end
end

function RequestAttentionObject:CheckForWarning()
  if (self.barWarningEnabled == true) then
    if (self.currentBarLevel <= self.barWarningPercent * 0.01) then
      if (self.canPlayWarningSound == true) then
        self:playSample(self.barWarningSound)
        if (self.barWarningBlink) then
          self:DoBarBlinking()
        end
        self.canPlayWarningSound = false
      end
    end
  end
end

function RequestAttentionObject:onClick()
  self.currentBarLevel = self.currentBarLevel + (self.barClickIncrease * 0.01)
  self.currentBarLevel = math.clamp(self.currentBarLevel, 0, 1)
  if (self.decreasingBar) then
      self.decreasingBar:setProgress(self.currentBarLevel)
  end
  if (self.barClickSoundName and self.barClickSoundName ~= "") then
    self:playSample(self.barClickSoundName)
  end

  -- Check if we can replay the warning sound again after clicked:
  if (self.barWarningEnabled == true and
    self.canPlayWarningSound == false and
    self.currentBarLevel >= self.barWarningPercent * 0.01) then
      self.canPlayWarningSound = true
  end

  if(not self.leavingTask) then
    self.enableDecreasing = false

    self.clickable = false

    self.decreasingBar:cancelAnimations()
    self:addTask({class = "CallTask",
      func = function()
        self:HideBar(true)
      end
    })

    -- Wait a delay and then activate another random character
    self:addTask({class = "Task", duration = self.barClickDrainDelay})
    self:addTask({class = "CallTask", func =
      function()
        self.parentEventObject.eventObjects[math.Random(1, table.getn(self.parentEventObject.eventObjects))]:ActivateAttention()
      end})
  end
end

function RequestAttentionObject:getMouseOverObject()
  return self
end

function RequestAttentionObject:DoBarBlinking(blinkDuration)
    if (self.decreasingBar) then
        self:addTask({class = "CallTask",
          func = function()
            self.decreasingBar:animate({blink =
              {startAlpha = 1, endAlpha = 0.3,
              fadeIn = self.barBlinkFadeDuration,
              fadeOut = self.barBlinkFadeDuration}})
          end
        })
        self:addTask({class = "Task",
          duration = self.barBlinkDuration})
          self:addTask({class = "CallTask",
            func = function()
              self.decreasingBar:cancelAnimations()
              self.decreasingBar.alpha = 1
            end
          })
    end
end

--------------------------------------------------------------------------------
RequestAttentionEvent = deriveLuaClass("RequestAttentionEvent", "Event")

function RequestAttentionEvent:init(desc)
  RequestAttentionEvent.superInit(self, desc)

  self.timePassed = 0.0
end

function RequestAttentionEvent:onInitialized()
  RequestAttentionEvent.super.onInitialized(self)

  self:SetupConfigurationParams()
  self:DoErrorControl()
end

function RequestAttentionEvent:SetupConfigurationParams()
  self.eventObjects = {}

  -- Fill the event objects tables and set the reference to the main one
  if type(self.configObjectNames) == "table" then
    for i, v in ipairs(self.configObjectNames) do
      table.insert(self.eventObjects, level:getObject(v))
      self.eventObjects[i].parentEventObject = self
    end
  end
end

function RequestAttentionEvent:DoErrorControl()
  --// TODO: in phase 2
end

function RequestAttentionEvent:setupChallenge()
  level.challengeManager:setupEventChallenge(1, true)

  GH.assert(table.getn(self.eventObjects) > 0,
    "ERROR! Cannot link with the RequestAttentionEvent object in the scene! " ..
    "The RequestAttentionEvent won't work properly.")

  -- Activate one random character
  local randomCharacter = self.eventObjects[math.Random(1, table.getn(self.eventObjects))]
  randomCharacter:addTask({class = "Task", duration = self.initialDelay})
  randomCharacter:addTask({class = "CallTask", func =
  function()
    randomCharacter:ActivateAttention()
  end})
end

function RequestAttentionEvent:onTick(time)
    self.timePassed = self.timePassed + time
    if (not self.eventFinished) then
        if (level.isPlaying) then
            self:CheckEventStatus(time)
        end
    end
end

function RequestAttentionEvent:eventWon()
    if (not self.wonCalled) then
        self.wonCalled = true
        RequestAttentionEvent.super.eventWon(self)

        for i, v in ipairs(self.eventObjects) do
          v.eventFinished = true
          v:HideBar(true)
          if (v.StopObject) then
              v:StopObject(true)
          end
        end
    end
end

function RequestAttentionEvent:eventLost()
  if (not self.wonCalled) then
    self.wonCalled = true
    RequestAttentionEvent.super.eventLost(self)

    for i, v in ipairs(self.eventObjects) do
      v.eventFinished = true
      v:HideBar(true)
      v:LeavePosition()
    end
  end
end

function RequestAttentionEvent:CheckEventStatus(time)
    if (level.isShiftClosed) then
        self:eventWon()
    else
      for i, v in ipairs(self.eventObjects) do
        if(v.failEvent == true) then
          self:eventLost()
        end
      end
    end
end
