dofile("script/events/Event.lua")

-- ************************************ Rub Over Event ************************************ --
RubOverEvent = deriveLuaClass("RubOverEvent", "Event")

-- Sample usage '--
--[[
levelDesc.eventDesc =
{
	class = "RubOverEvent",
	autoStart = true,
	initObjectsHidden = false,
	hideItemOnDone = false,
	scoreAllDone = 150,
	scoreOneDone = 25,
	addExclamationMarks = true,
}
--]]

function RubOverEvent:init(desc)
	RubOverEvent.superInit(self, desc)

	-- Configuration variables

	self.rubAmount = desc.rubAmount or 10000
	self.thresholdWidth = desc.thresholdWidth or 20
    self.thresholdHeight = desc.thresholdHeight or 20
	self.xMargin = desc.xMargin or 300
	self.yMargin = desc.yMargin or 200
	self.addExclamationMarks = desc.addExclamationMarks or false -- Default FALSE
	self.doProgressBars = self.doProgressBars ~= false -- Default true
	self.linkedObjectForFrameIncrease = desc.linkedObjectForFrameIncrease

	-- Status variables
	self.hintFired = false

	self.targetGoal = desc.targetGoal or 10
	self.currentGoal = 0
	self.rubObjectFinished = false
	self.eventDone = false

	-- Override
	self.matchingItemType = "RubOverObject"
end

function RubOverEvent:onInitialized()
	RubOverEvent.super.onInitialized(self)
	if (event.linkedObjectForFrameIncrease) then
		self.linkedObject = level:getObject(self.linkedObjectForFrameIncrease)
	end

	self.totalFixAmount = 0
	for _,item in ipairs(self.items) do
		item.rubAmount = item.rubAmount or item.obj.rubAmount or self.rubAmount
		item.thresholdWidth = item.thresholdWidth or item.obj.thresholdWidth or self.thresholdWidth
		item.thresholdHeight = item.thresholdHeight or item.obj.thresholdHeight or self.thresholdHeight
		item.totalFixAmount = item.totalFixAmount or item.rubAmount
		self.totalFixAmount = self.totalFixAmount + item.totalFixAmount
		item.xMargin = self.xMargin
		item.yMargin = self.yMargin
		if (item.obj.xMargin) then
			item.xMargin = item.obj.xMargin --or self.xMargin
		end
		if (item.obj.yMargin) then
			item.yMargin = item.obj.yMargin --or self.xMargin
		end
	end

end

function RubOverEvent:setupChallenge()

	level.challengeManager:setupEventChallenge(math.max(1, self.targetGoal), true)
end

RubOverEvent.onProgress = nil

function RubOverEvent:showItem(item)

	RubOverEvent.super.showItem(self, item)
	if (self.doProgressBars ~= false) then
		item.bar = item.bar or self.bar or "progressbar_default"
	end

	item.progress = 0
	item.totalProgress = 0
	item.drawOutline = true
	--item.name = item.obj.name
	item.obj.onClick = self.fixObject_onClick
	item.obj.updateFixAppearance = self.fixObject_updateFixAppearance

	if (self.addExclamationMarks) then
		item.obj:createOverhead("exclamation:gamescene")
	end

	if (self.onItemShowFloater) then
		local floaterPos = self:getFloaterPosition(item.obj)
		level:createGenericFloater(self.onItemShowFloater, floaterPos.x, floaterPos.y)
	end

	item.onRestoreItem = item.onRestoreItem or function()
	end

	if (self.doProgressBars ~= false) then
		self:createProgressBar(item)
	end
	if (not self.showHintOnStart) then
		self:showHint(item.obj)
	end
end

function RubOverEvent:onItemDone(item)
	RubOverEvent.super.onItemDone(self, item)
	self.currentGoal = self.currentGoal + 1

	self:onProgress()
	SpriteExt.playSample(nil, "event_item_complete")

	if (self.currentGoal == self.targetGoal) then
		self.eventDone = true
	end
	self.visible = false
end

function RubOverEvent:getProgress()
	if (self.trackProgress) then

		local progress = 0
		local rubAmount = self.rubAmount --self.items[i].rubAmount
		local fraction = (rubAmount > 0 and (self.progress / rubAmount)) or 1
		progress = progress + math.clamp(fraction, 0, 1)

	end
	return RubOverEvent.super.getProgress(self)
end

function RubOverEvent:onShiftFinishStart()
	--Hide al GUI elements
	self.super.onShiftFinishStart(self);
	for _,item in ipairs(self.items) do
		event:hideProgressBar(item)
		item.obj:removeOverhead(true)
	end
end

-- ************************************ Rub Over Object ************************************ --
RubOverObject = deriveLuaClass("RubOverObject", "EventObject")

function RubOverObject:init(desc)
  RubOverObject.superInit(self, desc)

  self.RubObjectClicked = false
  self.RubObjectMouseIsOver = false
  self.prevX = 0.0
  self.prevY = 0.0
  self.progress = 0
  self.isMouseOver = false
  self.effectStopped = false
  self.timeSinceOut = 0
  self.eventStarted = true

end

function RubOverObject:onTick(time)

	if(self.eventStarted == true and self.isMouseOver ) then
		self.timeSinceOut = 0
		if(not self.effectPlaying and self.item.obj.effect ~= nil) then
			self.item.obj.effect:play()
			self.effectPlaying = true
			self.prevX = GH.getMouseCursorPos().x
			self.prevY = GH.getMouseCursorPos().y
		end

		mousePos = GH.getMouseCursorPos()
		self.item.obj:removeOverhead(true)
		event:showProgressBar(self.item)
		self.RubObjectMouseIsOver = true

		if(self.prevX ~= mousePos.x or self.prevY ~= mousePos.y) then
			if(math.abs(self.prevX-mousePos.x) > (self.width / self.thresholdWidth) or math.abs(self.prevY-mousePos.y) > (self.height / self.thresholdWidth)) then
				prevXInc = (mousePos.x - self.prevX)^2 --* (mousePos.x - self.prevX)
				prevYInc = (mousePos.y - self.prevY)^2 --* (mousePos.y - self.prevY)
				incrementMultiplier = math.sqrt(prevXInc+prevYInc)

				self.prevX = mousePos.x
				self.prevY = mousePos.y
				self.item.progress = self.item.progress + (time + (math.abs(incrementMultiplier)))

				self:updateFixAppearance()
				event:onProgress()
			end
		end

		if(not self.rubObjectFinished and self.item.progress >= self.item.rubAmount) then
			event:itemDone(self.item)
			event:hideProgressBar(self.item)
			--event:hideItem(self.item)
			self.rubObjectFinished = true
		end
	else
		if(self.timeSinceOut ~= nil and self.item ~= nil) then
			if(not self.effectStopped and self.item.obj.effect ~= nil and self.timeSinceOut > 1500) then
				self.item.obj.effect:stop()
				self.effectStopped = true
				self.timeSinceOut = 0
			end
			self.timeSinceOut = time + self.timeSinceOut
			self.RubObjectMouseIsOver = false
		end
	end
end

function RubOverObject:onMouseOver()
	if(self.item.obj.effect ~= nil) then
		self.item.obj.effect:play()
		self.effectStopped = false
	end
	self.isMouseOver = true
	self.timeSinceOut = 0

end

function RubOverObject:onMouseOut()
	self.isMouseOver = false
end

function RubOverObject:onMouseDown()
	if(not self.rubObjectFinished) then
		RubOverObject.super.onMouseDown(self)
		self.RubObjectClicked = true
		self.item.obj:removeOverhead(true)
		event:showProgressBar(self.item)
	end
end

function RubOverObject:onMouseUp()
	RubOverObject.super.onMouseUp(self)
	self.RubObjectClicked = false
end

function RubOverObject:isMouseInsideMargin()
	mousePos = GH.getMouseCursorPos()
	if (mousePos.x >= self.screenX - self.xMargin and mousePos.x <= self.screenX + self.xMargin) then
		if (mousePos.y >= self.screenY - self.yMargin and mousePos.y <= self.screenY + self.yMargin) then
			return true
		end
	end
	return false
end

function RubOverObject:eventEnd()
	RubOverObject.super.eventEnd(self)
end

function RubOverObject:isMouseUp()
	return MouseUp
end

function RubOverObject:isMouseDown()
	return MouseDown
end

function RubOverEvent.fixObject_onClick(self)
end

function RubOverEvent.fixObject_updateFixAppearance(self)
	self.fraction = math.min(1, self.item.progress / self.item.rubAmount)

	if (self.item.matchEventAppearance ~= false and event.matchItemAppearance ~= false) then
		if (self.item.base_animations and #self.item.base_animations > 0) then
			local c = #self.item.base_animations-1
			local idx = math.floor(math.min(self.fraction * c, c)+1)
			if (self.baseAnimation ~= self.item.base_animations[idx]) then
				self.baseAnimation = self.item.base_animations[idx]
				if (self.stateChange) then
					self:stateChange()
				end
			end
		else

			local c = self.frameCount
			if (not event.hideItemOnDone) then
				c = c - 1
			end
			local frame = math.min(self.fraction * c, c)
			if (self.frame ~= frame) then
				if (event.linkedObject) then
					event.linkedObject.frame = frame
				end
				self.frame = frame
				if (self.stateChange) then
					self:stateChange()
				end
			end
		end
	end

	if (isTable(self.item.bar)) then
		self.item.bar:setProgress(self.fraction)
	end
end
