dofile("script/events/Event.lua")

-- ************************************ Table Order Event ************************************ --
ServeCarlEvent = deriveLuaClass("ServeCarlEvent", "Event")

function ServeCarlEvent:init(desc)
	ServeCarlEvent.superInit(self, desc)

	-- Configuration variables
	self.scorePerDeliveredProduct = desc.scorePerDeliveredProduct or 0
	self.scorePerDelivery = desc.scorePerDelivery or 0
	self.randomizeCustomerOrder = desc.randomizeCustomerOrder or false
	self.doOrdersForEmptyCustomersFirst = desc.doOrdersForEmptyCustomersFirst or false
	self.timeBeforeFirstPopup = desc.timeBeforeFirstPopup or 0
	self.minimumTimeBetweenPopup = desc.minimumTimeBetweenPopup or 0
	self.maximumTimeBetweenPopup = desc.maximumTimeBetweenPopup or 0
	self.mouseShowTime = desc.mouseShowTime or 5000
	self.onlyOneOrderAtATime = self.onlyOneOrderAtATime or false
	self.numProductsPerOrder = desc.numProductsPerOrder or 1
	self.numOrdersPerCustomer = desc.numOrdersPerCustomer or 1
	self.balloonSlotCount = desc.balloonSlotCount or 4
	self.horizontalBalloonSlotCount = desc.horizontalBalloonSlotCount or 2
	self.maxNumOrdersActive = desc.maxNumOrdersActive or 100

	--self.productPool = {{"#cheese",1}, "#sausage"},

	-- Variables for items
	--{
	--	obj = "table1_1",
	--	numOrders = 1,
	-- numProductsPerOrder = 1,
	--	balloonSlotCount = 4,
	--	horizontalBalloonSlotCount = 2,
	-- productPool = {{"#cheese",1}, "#sausage"},
	--},

	-- Status variables
	self.timeBeforeFirstPopup = self.timeBeforeFirstPopup or math.Random(self.minimumTimeBetweenPopup, self.maximumTimeBetweenPopup)
	self.timeUntilNextPopup = self.timeBeforeFirstPopup
	self.nextOrderAllowed = true
	self.showItemsOnStart = false

	self.minimumTimeBetweenPopup = desc.minimumTimeBetweenPopup or 0
	self.maximumTimeBetweenPopup = desc.maximumTimeBetweenPopup or self.minimumTimeBetweenPopup
end

local function ServeCarlEvent_onWorkStart(self, task)
	event:hideHintArrow()
	if (not event.started or event.completed) then -- or self.item.order.currentStep.numProductsLeft <= 0) then
		return false
	end

	local duration = self:getWorkDurationWith(task.actor, "work")
	task.duration = duration
	if (duration > 0) then
		if (self.workDirection) then
			task.actor.direction = self.workDirection
		end
		task.actor:playAnimation(self.workAnimation or "work", self.workCycles or -1, self.workFrameTime or 150, self.workWhenFinished or "default")
	end
end

local function ServeCarlEvent_onWorkFinish(obj, task)
	GH.assert(obj.item.order)

	-- Check if all the products for this object are delivered
	local item = obj.item
	local order = item.order
	local numProductsInOrderBefore = order.currentStep.numProductsLeft
	if (numProductsInOrderBefore > 0) then
		order:deliver(item, level:getTray(task.actor))
	end
	local numProductsLeftInOrder = order.currentStep.numProductsLeft

	item.obj:updateOrderOverhead()

	if (numProductsLeftInOrder == 0) then
		if (event.onlyOneOrderAtATime) then
			event.nextOrderAllowed = true
		end
	end

	task.actor:playDefaultAnimation()

	local numDeliveredProducts = numProductsInOrderBefore - numProductsLeftInOrder
	local scoreForThisDelivery = event.scorePerDelivery + event.scorePerDeliveredProduct*numDeliveredProducts
	if (scoreForThisDelivery > 0) then
		local pos = obj.floaterPosition
		level:createFloater({
			id = "eventDone",
			textId = "EVENT_SCORE_ONE_DONE",
			x = pos.x, y = pos.y,
			bonusScore = scoreForThisDelivery,
			bonus = {
				textId = "EVENT_SCORE_ONE_DONE_FMT",
			},
		})
		if (level.BM_eventScore) then level:BM_eventScore(scoreForThisDelivery) end
	end

	taskSystem:cancelAllTasksFor(item.obj)
	item.obj:addTask({class="Task", duration=200})
	item.obj:addTask({class="PlayAnimationTask", animation="pop_up", cycles=1, frameTime=100, whenFinished="stay", wait=true, reversed=true})
	item.obj:addTask({class="VisibleTask", visible=false})
	item.obj.showing = false
	item.obj.clickable = false
end

function ServeCarlEvent:onItemDone(item)
	return ServeCarlEvent.super.onItemDone(self, item)
end

function ServeCarlEvent:onInitialized()
	ServeCarlEvent.super.onInitialized(self)

	for _,item in ipairs(self.items) do
		self:preserveItem(item)
		if (isTable(item.obj)) then
			item.obj.onWorkStart = ServeCarlEvent_onWorkStart
			item.obj.onWorkUpdate = nil
			item.obj.onWorkFinish = ServeCarlEvent_onWorkFinish

			item.numProductsPerOrder = item.numProductsPerOrder or self.numProductsPerOrder

			item.numOrdersToDo = item.numOrders or self.numOrdersPerCustomer
			item.maxSlotCount = item.balloonSlotCount or self.balloonSlotCount
			item.maxHorizontalSlotCount = item.horizontalBalloonSlotCount or self.horizontalBalloonSlotCount
			item.productPool = item.productPool or self.productPool
			item.obj.orderMaxSlotCount = item.maxSlotCount
			item.obj.orderMaxHorizontalSlotCount = item.maxHorizontalSlotCount
			item.obj.numSteps = item.numOrdersToDo
			item.order = item.obj:createOrder()

			self:hideMouse(item)
		end
	end
end

function ServeCarlEvent:getNumStepsTodo()
	return ServeCarlEvent.super.getNumStepsTodo(self)
end

function ServeCarlEvent:showItem(item)
	ServeCarlEvent.super.showItem(self, item)
end

function ServeCarlEvent:hideMouse(item)
--print("1. hideMouse: "..item.obj.name)
	item.obj.visible = false
	item.obj.showing = false
	item.obj.clickable = false

	if (item.order and item.order.currentStep) then
		local stepProducts = item.order.currentStep.productNames
		for _,v in ipairs(stepProducts) do
			--print("stepProduct: "..v)
			item.order.currentStep:removeProduct(v, false)
		end
	end
end

function ServeCarlEvent:showMouse(item)
--print("1. showMouse: "..item.obj.name)
	GH.assert(item.obj.visible == false and item.obj.showing == false, "The mouse was already showing, the show delay is lower than the hide delay.")

	item.obj.showing = true
	item.obj:addTask({class="SetTask", visible=true, showing=true, clickable=true})
	item.obj:addTask({class="PlaySoundTask", sound="mouse_squeak:sound"})
	item.obj:addTask({class="PlayAnimationTask", animation="pop_up", cycles=1, frameTime=100, whenFinished="stay", wait=true})
	item.obj:addTask({class="Task", duration=self.mouseShowTime})
	item.obj:addTask({class="CallTask", func=function()
		if (item.obj.workTaskCount <= 0) then
			item.obj.showing = false
			item.obj.clickable = false
			item.obj:addTask({class="PlayAnimationTask", animation="pop_up", cycles=1, frameTime=100, whenFinished="stay", wait=true, reversed=true})
			item.obj:addTask({class="CallTask", func=function()
				self:hideMouse(item)
				self.nextOrderAllowed = true
			end})
		end
	end})

	-- Determine set of valid products
	local validProductPool = {}
	for _,v in ipairs(item.productPool) do
		local product = isTable(v) and v[1] or v
		if (table.contains(level.possibleProducts, product)) then
			table.insert(validProductPool, v)
		end
	end
	GH.assert(#validProductPool > 0, "No valid products to choose from found!")

	GH.assert(item.order)
	local currentStep = item.order.currentStep
	local remainingProducts = currentStep.productNames
	local productCount = {}
	for _,v in ipairs(remainingProducts) do
		productCount[v] = (productCount[v] or 0) + 1
	end
	if (self.maxNumOrdersActive > #remainingProducts) then
--print("1. product added")
		for i=1,self.numProductsPerOrder do
--print("2. product added: "..self.numProductsPerOrder)
			table.randomize(validProductPool)
			local added = false
			for _,v in ipairs(validProductPool) do
				local product = v
				local maxCount = -1
				if (isTable(v)) then
					product = v[1]
					maxCount = v[2] or maxCount
				end
				local currentProductCount = productCount[product] or 0
				if (product and maxCount < 0 or currentProductCount < maxCount) then
					productCount[product] = currentProductCount + 1
					currentStep:addProduct(product)
--print("3. product added")
					added = true
					break
				end
			end
			GH.assert(added, "Could not get a valid product from: "..serialize("validProductPool", validProductPool).." with: "..serialize("productCount", productCount))
		end
		item.numOrdersToDo = item.numOrdersToDo - 1
		item.obj:updateOrderOverhead()

		if (item.event_onProductsAddedToOrder) then
			item:event_onProductsAddedToOrder()
		end
		if (item.associatedObjects) then
			for _,v in ipairs(item.associatedObjects) do
				if (v.event_onProductsAddedToOrder) then
					v:event_onProductsAddedToOrder(item)
				end
			end
		end
		if (self.onProductsAddedToOrder) then
			self:onProductsAddedToOrder(item)
		end
	end
end

function ServeCarlEvent:onTick(time)
	if (self.completed or not self.started or self.won or level.elapsedTimeFraction >= 1) then
		return
	end

	-- Determine end condition by time

	if (level.elapsedTimeFraction >= 1) then
		self:eventLost()
		return
	end

	-- Countdown for next event action
	if (self.nextOrderAllowed) then
		self.timeUntilNextPopup = self.timeUntilNextPopup - time
		if (self.timeUntilNextPopup <= 0) then
			self.timeUntilNextPopup = math.Random(self.minimumTimeBetweenPopup, self.maximumTimeBetweenPopup)
			if (self.onlyOneOrderAtATime) then
				self.nextOrderAllowed = false
			end

			local allCustomerChoices = {}
			local preferredCustomerChoices = {}
			for _,v in ipairs(self.items) do
				if (not v.done and v.order and v.numOrdersToDo > 0) then
					table.push_back(allCustomerChoices, v)
					if (self.doOrdersForEmptyCustomersFirst and v.order.currentStep.numProductsLeft == 0) then
						table.push_back(preferredCustomerChoices, v)
					end
				end
			end

			if (not table.empty(preferredCustomerChoices)) then
				allCustomerChoices = preferredCustomerChoices
			end

			if (#allCustomerChoices > 0) then
				if (self.randomizeCustomerOrder) then
					table.randomize(allCustomerChoices)
				end

				local item = allCustomerChoices[1]
				GH.assert(item)
				self:showMouse(item)
			end
		end
	end
end

function ServeCarlEvent:removeDecals(item)
	ServeCarlEvent.super.removeDecals(self, item)
	if (item and item.obj and item.order) then
		item.obj:removeOverhead(true)
		item.order = nil
	end
end

function ServeCarlEvent:onDeliver(order, matched, item)
	if (item.obj and not self.completed) then
		if (#matched > 0 or order.currentStep.numProductsLeft == 0) then
			self:onProductsDelivered(order, matched, item)

			for i=1, #matched do
				self:stepDone(item)
			end

			for _,v in ipairs(self.items) do
				if (v.done or self.completed or self.won) then
					self:removeDecals(v)
				end
			end

			self:onProgress()
		end
	end
end

function ServeCarlEvent:onProductsDelivered(order, matched, item)
	local consume = level.productSettings[matched[1]]
	item.consumeAnimation = (consume and consume.consumeAnimation) or "eat"
end
