dofile("script/events/Event.lua")

--------------------------------------------------------------------------------
-- StayAlertEvent: StayAlertPaigeObject for Restaurant 6 - PHASE 1            --
--------------------------------------------------------------------------------

StayAlertPaigeObject = deriveLuaClass("StayAlertPaigeObject", "DelCharacter")

function StayAlertPaigeObject:init(desc)
  StayAlertPaigeObject.superInit(self, desc)

  self:GatherEditorParameters(desc)
  self:SetupObject()
  self.isEventCompleted = false
end

function StayAlertPaigeObject:onTick(time)
  self:tick(time)

  if self.currentState == "waitingHero" and not self.isEventCompleted then
    if hero.x == self.workPos.x and
       hero.y == self.workPos.y then
      self:PlaySuccessAnimation()
      --event:StepDone()
      self:addTask({class = "Task", duration = self.successDuration})
      self:addTask({class = "CallTask", func = function()
          self.defaultSection = self.defaultObjectSection
          self.baseAnimation = self.defaultObjectAnimation
          self:addTask({class = "PlayAnimationTask",
                        animation = self.baseAnimation,
                        cycles = self.defaultCycles,
                        frameTime = self.defaultFrameTime,
                        wait = true,
                      })
          self:LeaveScene()
          self:WaitForNextAppearance()
          self:StartAlertCycle()
        end
      })
    end
  end

  if self.clickable == true
    and self.alertSoundName ~= "" then
    if self.alertSoundTimer >= self.alertSoundRepDelay then
      self:playSample(self.alertSoundObject)
      self.alertSoundTimer = 0.0
    else
      self.alertSoundTimer = self.alertSoundTimer + time
    end
  end
end

function StayAlertPaigeObject:GatherEditorParameters(desc)
  self.name = desc.name or ""
  self.startLocation = {x = self.x, y = self.y}
  self.requiredGoal = desc.requiredGoal or 5
  self.firstAppearanceDelay = desc.firstAppearanceDelay or 3000
  self.appearanceDelay = desc.appearanceDelay or 5000
  self.alertDuration = desc.alertDuration or 5000
  self.successDuration = desc.successDuration or 2000
  self.exactAlertsNumber = desc.exactAlertsNumber ~= false -- Default true
  self.actorAlertPositions = desc.actorAlertPositions or {}
  self.enableHeroLeavePosition = desc.enableHeroLeavePosition ~= false
  self.heroLeavePosition = desc.heroLeavePosition or {x = 0, y = 0}
  self.actorAlertAnimation = desc.actorAlertAnimation or self.baseAnimation
  self.actorAlertAnimationCycles = desc.actorAlertAnimationCycles or -1
  self.actorAlertAnimationFrameTime = desc.actorAlertAnimationFrameTime or -1
  self.actorOnClickAnimation = desc.actorOnClickAnimation or self.baseAnimation
  self.actorOnClickAnimationCycles = desc.actorOnClickAnimationCycles or -1
  self.actorOnClickAnimationFrameTime = desc.actorOnClickAnimationFrameTime or -1
  self.successAnimationSection = desc.successAnimationSection or ""
  self.successAnimationResource = desc.successAnimationResource or self.baseAnimation
  self.successAnimationCycles = desc.successAnimationCycles or -1
  self.successAnimationFrameTime = desc.successAnimationFrameTime or -1
  self.localWorkPosition = desc.localWorkPosition or {x = 0, y = 0}
  self.localWorkDirection = desc.localWorkDirection or ""
  self.successSound = desc.successSound or "balloon_pops_up:sound"
  self.stepScore = desc.stepScore or 100
  self.totalScore = desc.totalScore or 500
  self.alertSoundName = desc.alertSoundName or ""
  self.alertSoundRepDelay = desc.alertSoundRepDelay or 2000
  self.alertSoundVolume = desc.alertSoundVolume or 1.0
  self.showExclamation = desc.showExclamation or true
end

function StayAlertPaigeObject:SetupObject()
  self.defaultObjectSection = self.defaultSection
  self.defaultObjectAnimation = self.baseAnimation

  self.previousAlertPosition = {}
  self.currentAlertPosition = {x = 0, y = 0}
  self.currentState = "init"
  self.numAlerts = 0
  self.alertSoundTimer = self.alertSoundRepDelay

  self.alertSoundObject = {}
  self.alertSoundObject.sound = self.alertSoundName
  self.alertSoundObject.volume = self.alertSoundVolume
end

function StayAlertPaigeObject:StartLogic()
  self:addTask({class = "Task", duration = self.firstAppearanceDelay})
  self:StartAlertCycle()
end

function StayAlertPaigeObject:StartAlertCycle()
  self.clickable = false
  if self.exactAlertsNumber == false
    or (self.numAlerts < self.requiredGoal) then
    self:addTask({class = "CallTask", func = function()
        self:ChooseAlertPosition()
        self:WalkToAlertPosition()
        self:DoAlert()
        self:LeaveScene()
        self:WaitForNextAppearance()
      end
    })
  end
end

function StayAlertPaigeObject:ChooseAlertPosition()
  local randomIndex = 0
  local retries = 100

  if #self.actorAlertPositions > 1 then
    self.previousAlertPosition = self.currentAlertPosition
    repeat
      randomIndex = math.Random(#self.actorAlertPositions)
      self.currentAlertPosition = self.actorAlertPositions[randomIndex]
      retries = retries - 1
    until self.currentAlertPosition ~= self.previousAlertPosition
      or retries == 0
  else
    if #self.actorAlertPositions == 1 then
      self.currentAlertPosition = self.actorAlertPositions[1]
    end
  end
end

function StayAlertPaigeObject:WalkToAlertPosition()
  self.currentState = "walkingToAlert"
  self:addTask({class = "WalkToTask",
                target = self.currentAlertPosition,
                playDefaultOnFinish = true
              })
end

function StayAlertPaigeObject:DoAlert()
  self:addTask({class = "CallTask", func = function()
      self.currentState = "doingAlert"
      self.clickable = true
      if (self.showExclamation) then
        self:createOverhead("exclamation:gamescene")
      end
    end
  })

  self.baseAnimation = self.actorAlertAnimation
  self:addTask({class = "PlayAnimationTask",
                animation = self.actorAlertAnimation,
                cycles = self.actorAlertAnimationCycles,
                frameTime = self.actorAlertAnimationFrameTime,
                wait = true,
              })
  self:addTask({class = "Task", duration = self.alertDuration})
end

function StayAlertPaigeObject:LeaveScene()
  self:addTask({class = "CallTask", func = function()
      self.currentState = "leavingScene"
      self.clickable = false
      self.numAlerts = self.numAlerts + 1
      if (self.showExclamation) then
        self:removeOverhead(true)
      end

      if level.hasShiftFinished and event.numSteps < self.requiredGoal then
        self:eventLost()
      end
    end
  })
  self:addTask({class = "WalkToTask",
                target = self.startLocation,
                playDefaultOnFinish = true,
              })
end

function StayAlertPaigeObject:WaitForNextAppearance()
    if (not self.isEventCompleted) then
        self:addTask({class = "CallTask",
            func = function()
                self.currentState = "waitingForNextAppearance"
            end
        })
        self:addTask({class = "Task", duration = self.appearanceDelay})
        self:addTask({class = "CallTask",
            func = function()
                self:StartAlertCycle()
            end
        })
    else
        taskSystem:cancelAllTasksFor(self)
    end
end

function StayAlertPaigeObject:onClick()
  if self.currentState == "doingAlert" then
    self.workPos = {x = self.x + self.localWorkPosition.x,
      y = self.y + self.localWorkPosition.y}
    self.workDirection = self.localWorkDirection
    StayAlertPaigeObject.super.onClick(self)
    taskSystem:cancelAllTasksFor(self)
    if (self.showExclamation) then
        self:removeOverhead(true)
    end
    self:WaitForHeroToCome()
  end
end

function StayAlertPaigeObject:WaitForHeroToCome()
  self.currentState = "waitingHero"
  self.baseAnimation = self.actorOnClickAnimation
  self:addTask({class = "PlayAnimationTask",
                animation = self.actorOnClickAnimation,
                cycles = self.actorOnClickAnimationCycles,
                frameTime = self.actorOnClickAnimationFrameTime,
                wait = true,
              })
end

function StayAlertPaigeObject:PlaySuccessAnimation()
    self.currentState = "playingSuccess"

    -- Play the animation of the hero
    self.previousHeroAnimation = hero.baseAnimation
    hero.baseAnimation = self.successAnimationResource
    hero:addTaskAtFront({
        class = "PlayAnimationTask",
        animation = self.successAnimationResource,
        cycles = self.successAnimationCycles,
        frameTime = self.successAnimationFrameTime,
        playbackType = "Ping pong",
        wait = true,
        duration = self.successDuration
    })
end

--------------------------------------------------------------------------------
-- StayAlertEvent - PHASE 1                                                   --
--------------------------------------------------------------------------------

StayAlertEvent = deriveLuaClass("StayAlertEvent", "Event")

function StayAlertEvent:init(desc)
  StayAlertEvent.superInit(self, desc)
  self:onInitialized()

  self.eventStarted = false
end

function StayAlertEvent:onInitialized()
  StayAlertEvent.super.onInitialized(self)

  self:SetupDefaultEventObject()
  self:SetupEvent()
end

function StayAlertEvent:setupChallenge()
  if self.eventObject then
    level.challengeManager:setupEventChallenge(
      math.max(1, self.eventObject.requiredGoal), true)
  end
end

function StayAlertEvent:onTick(time)
  if self.eventDone == true then return end

  if self.eventObject then
    if level.isPlaying == true and self.eventStarted == false then
      self.eventObject:StartLogic()
      self.eventStarted = true
    end

    if self.eventObject.currentState == "playingSuccess" then
        if (self.eventObject.visible) then
            -- Hide the event object and remove the highlight temporary
            self.eventObject.visible = false
            self.eventObject.doHighlight = false
        end
    elseif self.eventObject.visible == false then
      -- Reset the hero animation and make the event object visible again
      hero.baseAnimation = self.eventObject.previousHeroAnimation
      self.eventObject.visible = true
      event:StepDone()
      self.eventObject.doHighlight = true
      if self.eventObject.enableHeroLeavePosition then
        hero:addTask({class = "WalkToTask",
                      target = self.eventObject.heroLeavePosition,
                      playDefaultOnFinish = true,
                    })
      end
    end
  end
end

function StayAlertEvent:SetupDefaultEventObject()
  local eventObjectNameUnified = self.configObjectName or self.eventObjectName
  if eventObjectNameUnified then
    self.eventObject = level:getObject(eventObjectNameUnified)
  end
  GH.assert(self.eventObject,
    "ERROR! Cannot link with the StayAlertEvent object in the scene! " ..
    "The StayAlertEvent won't work properly.")
end

function StayAlertEvent:SetupEvent()
  self.numSteps = 0
end

function StayAlertEvent:StepDone()
  self.numSteps = self.numSteps + 1
  self:scoreItemDone({})

  if self.eventObject.stepScore > 0 then
    level:createFloater({
      id = "eventDone",
      textId = "EVENT_SCORE_ONE_DONE",
  		x = self.eventObject.x, y = self.eventObject.y,
  		bonusScore = self.eventObject.stepScore,
  		bonus = {
  			textId = "EVENT_SCORE_ONE_DONE_FMT",
  		},
    })
  end

  if level.onEventStepDone then
    level:onEventStepDone(self, {})
  end
  self:onProgress()
  SpriteExt.playSample(nil, "event_item_complete")

  if self.numSteps == self.eventObject.requiredGoal then
    self:eventWon()
    self.eventDone = true

    self.eventObject.isEventCompleted = true
  end
end

function StayAlertEvent:getProgress()
  if self.numSteps < self.eventObject.requiredGoal then
    return self.numSteps / self.eventObject.requiredGoal
  end
  return 1
end

function StayAlertEvent:eventWon()
  -- TODO: Generic form for calling this!
  self.scoreAllDone = self.eventObject.totalScore
  self.scoreOneDone = self.eventObject.stepScore

  StayAlertEvent.super.eventWon(self)
end
