dofile("script/events/Event.lua")

-- ************************************ Table Order Event ************************************ --
TableOrderEvent = deriveLuaClass("TableOrderEvent", "Event")

function TableOrderEvent:init(desc)
	TableOrderEvent.superInit(self, desc)

	-- Configuration variables
	self.scorePerDeliveredProduct = desc.scorePerDeliveredProduct or 0
	self.scorePerDelivery = desc.scorePerDelivery or 0
	self.randomizeCustomerOrder = desc.randomizeCustomerOrder or false
	self.doOrdersForEmptyCustomersFirst = desc.doOrdersForEmptyCustomersFirst or false
	self.timeBeforeFirstOrder = desc.timeBeforeFirstOrder or 0
	self.minimumTimeBetweenOrders = desc.minimumTimeBetweenOrders or 0
	self.maximumTimeBetweenOrders = desc.maximumTimeBetweenOrders or 0
	self.onlyOneOrderAtATime = self.onlyOneOrderAtATime or false
	self.numProductsPerOrder = desc.numProductsPerOrder or 1
	self.numOrdersPerCustomer = desc.numOrdersPerCustomer or 1
	self.balloonSlotCount = desc.balloonSlotCount or 4
	self.horizontalBalloonSlotCount = desc.horizontalBalloonSlotCount or 2
	self.maxNumOrdersActive = desc.maxNumOrdersActive or 100

	--self.productPool = {{"#cheese",1}, "#sausage"},

	-- Variables for items
	--{
	--	obj = "table1_1",
	--	numOrders = 1,
	-- numProductsPerOrder = 1,
	--	balloonSlotCount = 4,
	--	horizontalBalloonSlotCount = 2,
	-- productPool = {{"#cheese",1}, "#sausage"},
	--},

	-- Status variables
	self.timeBeforeFirstOrder = self.timeBeforeFirstOrder or math.Random(self.minimumTimeBetweenOrders, self.maximumTimeBetweenOrders)
	self.timeUntilNextOrder = self.timeBeforeFirstOrder
	self.nextOrderAllowed = true

	self.minimumTimeBetweenOrders = desc.minimumTimeBetweenOrders or 0
	self.maximumTimeBetweenOrders = desc.maximumTimeBetweenOrders or self.minimumTimeBetweenOrders
end

local function TableOrder_onWorkStart(self, task)
	event:hideHintArrow()
	if (not event.started or event.completed) then -- or self.item.order.currentStep.numProductsLeft <= 0) then
		return false
	end

	local duration = self:getWorkDurationWith(task.actor, "work")
	task.duration = duration
	if (duration > 0) then
		if (self.workDirection) then
			task.actor.direction = self.workDirection
		end
		task.actor:playAnimation(self.workAnimation or "work", self.workCycles or -1, self.workFrameTime or 150, self.workWhenFinished or "default")
	end
end

local function TableOrder_onWorkFinish(obj, task)
	GH.assert(obj.item.order)

	-- Check if all the products for this object are delivered
	local item = obj.item
	local order = item.order
	local numProductsInOrderBefore = order.currentStep.numProductsLeft
	if (numProductsInOrderBefore > 0) then
		order:deliver(item, level:getTray(task.actor))
	end
	local numProductsLeftInOrder = order.currentStep.numProductsLeft

	item.obj:updateOrderOverhead()

	if (numProductsLeftInOrder == 0) then
		if (event.onlyOneOrderAtATime) then
			event.nextOrderAllowed = true
		end
	end

	task.actor:playDefaultAnimation()

	local numDeliveredProducts = numProductsInOrderBefore - numProductsLeftInOrder
	local scoreForThisDelivery = event.scorePerDelivery + event.scorePerDeliveredProduct*numDeliveredProducts
	if (scoreForThisDelivery > 0) then
		local pos = obj.floaterPosition
		level:createFloater({
			id = "eventDone",
			textId = "EVENT_SCORE_ONE_DONE",
			x = pos.x, y = pos.y,
			bonusScore = scoreForThisDelivery,
			bonus = {
				textId = "EVENT_SCORE_ONE_DONE_FMT",
			},
		})
		if (level.BM_eventScore) then level:BM_eventScore(scoreForThisDelivery) end
	end

	if (numDeliveredProducts > 0 or numProductsLeftInOrder == 0) then
		if (item.event_onDeliveredProducts) then
			item:event_onDeliveredProducts(numDeliveredProducts)
		end
		if (item.associatedObjects) then
			for _,v in ipairs(item.associatedObjects) do
				if (v.event_onDeliveredProducts) then
					v:event_onDeliveredProducts(item, numDeliveredProducts)
				end
			end
		end
		if (event.onDeliveredProducts) then
			event:onDeliveredProducts(item, numDeliveredProducts)
		end
	end
end

function TableOrderEvent:onItemDone(item)
	if (item.event_onAllDelivered) then
		item:event_onAllDelivered()
	end
	if (item.associatedObjects) then
		for _,v in ipairs(item.associatedObjects) do
			if (v.event_onAllDelivered) then
				v:event_onAllDelivered(item)
			end
		end
	end
	if (self.event_onAllDelivered) then
		self:event_onAllDelivered(item)
	end
	return TableOrderEvent.super.onItemDone(self, item)
end

function TableOrderEvent:onInitialized()
	TableOrderEvent.super.onInitialized(self)

	for _,item in ipairs(self.items) do
		self:preserveItem(item)
		if (isTable(item.obj)) then
			item.obj.onWorkStart = TableOrder_onWorkStart
			item.obj.onWorkUpdate = nil
			item.obj.onWorkFinish = TableOrder_onWorkFinish

			item.numProductsPerOrder = item.numProductsPerOrder or self.numProductsPerOrder
			item.numOrdersToDo = item.numOrders or self.numOrdersPerCustomer
			item.maxSlotCount = item.balloonSlotCount or self.balloonSlotCount
			item.maxHorizontalSlotCount = item.horizontalBalloonSlotCount or self.horizontalBalloonSlotCount
			item.productPool = item.productPool or self.productPool
			item.obj.orderMaxSlotCount = item.maxSlotCount
			item.obj.orderMaxHorizontalSlotCount = item.maxHorizontalSlotCount
			item.obj.numSteps = item.numOrdersToDo * self.numProductsPerOrder
			item.order = item.obj:createOrder()
		end
	end
end

function TableOrderEvent:getNumStepsTodo()
	return TableOrderEvent.super.getNumStepsTodo(self)
end

function TableOrderEvent:showItem(item)
	TableOrderEvent.super.showItem(self, item)
end

function TableOrderEvent:onTick(time)
	if (self.completed or not self.started or self.won --[[or level.elapsedTimeFraction >= 1]]) then
		-- Now this event doesn't finish when restaurant closes. It can order products until level really finishes (instead of restaurant closes)
		return
	end

	-- Countdown for next event action
	if (self.nextOrderAllowed) then
		self.timeUntilNextOrder = self.timeUntilNextOrder - time
		if (self.timeUntilNextOrder <= 0) then
			self.timeUntilNextOrder = math.Random(self.minimumTimeBetweenOrders, self.maximumTimeBetweenOrders)
			if (self.onlyOneOrderAtATime) then
				self.nextOrderAllowed = false
			end

			local allCustomerChoices = {}
			local preferredCustomerChoices = {}
			for _,v in ipairs(self.items) do
				if (not v.done and v.order and v.numOrdersToDo > 0) then
					table.push_back(allCustomerChoices, v)
					if (self.doOrdersForEmptyCustomersFirst and v.order.currentStep.numProductsLeft == 0) then
						table.push_back(preferredCustomerChoices, v)
					end
				end
			end

			if (not table.empty(preferredCustomerChoices)) then
				allCustomerChoices = preferredCustomerChoices
			end

			if (#allCustomerChoices > 0) then
				if (self.randomizeCustomerOrder) then
					table.randomize(allCustomerChoices)
				end

				local item = allCustomerChoices[1]
				GH.assert(item)

				-- Determine set of valid products
				local validProductPool = {}
				for _,v in ipairs(item.productPool) do
					local product = isTable(v) and v[1] or v
					if (table.contains(level.possibleProducts, product)) then
						table.insert(validProductPool, v)
					end
				end
				GH.assert(#validProductPool > 0, "No valid products to choose from found!")

				GH.assert(item.order)
				local currentStep = item.order.currentStep
				local remainingProducts = currentStep.productNames
				local productCount = {}
				for _,v in ipairs(remainingProducts) do
					productCount[v] = (productCount[v] or 0) + 1
				end
				if (self.maxNumOrdersActive > #remainingProducts) then
					for i=1,self.numProductsPerOrder do
						table.randomize(validProductPool)
						local added = false
						for _,v in ipairs(validProductPool) do
							local product = v
							local maxCount = -1
							if (isTable(v)) then
								product = v[1]
								maxCount = v[2] or maxCount
							end
							local currentProductCount = productCount[product] or 0
							if (product and maxCount < 0 or currentProductCount < maxCount) then
								productCount[product] = currentProductCount + 1
								currentStep:addProduct(product)
								added = true
								break
							end
						end
						GH.assert(added, "Could not get a valid product from: "..serialize("validProductPool", validProductPool).." with: "..serialize("productCount", productCount))
					end
					item.numOrdersToDo = item.numOrdersToDo - 1
					item.obj:updateOrderOverhead()

					if (item.event_onProductsAddedToOrder) then
						item:event_onProductsAddedToOrder()
					end
					if (item.associatedObjects) then
						for _,v in ipairs(item.associatedObjects) do
							if (v.event_onProductsAddedToOrder) then
								v:event_onProductsAddedToOrder(item)
							end
						end
					end
					if (self.onProductsAddedToOrder) then
						self:onProductsAddedToOrder(item)
					end
				end
			end
		end
	end
end

--function TableOrderEvent:getProgress()
--	return
--end

function TableOrderEvent:removeDecals(item)
	TableOrderEvent.super.removeDecals(self, item)
	if (item and item.obj and item.order) then
		item.obj:removeOverhead(true)
		item.order = nil
	end
end

function TableOrderEvent:onDeliver(order, matched, item)
	if (item.obj and not self.completed) then
		if (#matched > 0 or order.currentStep.numProductsLeft == 0) then
			self:onProductsDelivered(order, matched, item)

			for i=1, #matched do
				self:stepDone(item)
			end

			for _,v in ipairs(self.items) do
				if (v.done or self.completed or self.won) then
					self:removeDecals(v)
				end
			end

			self:onProgress()
		end
	end
end

function TableOrderEvent:onProductsDelivered(order, matched, item)
	local consume = level.productSettings[matched[1]]
	item.consumeAnimation = (consume and consume.consumeAnimation) or "sit_eat"
end
