dofile("script/events/Event.lua")
TellStoryEvent = deriveLuaClass("TellStoryEvent", "Event")

function TellStoryEvent:init(desc)
	TellStoryEvent.superInit(self, desc)
	self.targetGoal = desc.targetGoal or 4
	self.currentGoal = 0
	self.eventDone = false
	self.restTime = desc.restTime or 5000
	self.eventState = "Resting"
	self.eventTimer = desc.restTime or 5000
	self.attentionSound = desc.attentionSound or "balloon_pops_up:sound"
	if (desc.attentionIcon) then
		self.attentionIcons = self.attentionIcons or {}
		if (isTable(desc.attentionIcon)) then
			for _,v in ipairs(desc.attentionIcon) do
				GH.assert(isString(v))
				table.insert(self.attentionIcons, v)
			end
		elseif (isString(desc.attentionIcon)) then
			table.insert(self.attentionIcons, desc.attentionIcon)
		end
	end
	self.targetObject = desc.targetObject
	self.targetCharacter = desc.targetCharacter
end

function TellStoryEvent:setupChallenge()
	level.challengeManager:setupEventChallenge(math.max(1, self.targetGoal), true)
end

function TellStoryEvent:onInitialized()
	TellStoryEvent.super.onInitialized(self)
	local object = nil
	if (isTable(self.targetCharacter)) then
		if (isTable(v)) then
			object = level:createCharacter(v)
		elseif (isString(v)) then
			object = level:resolveActor(v)
		end
	elseif (isString(self.targetCharacter)) then
		object = level:resolveActor(self.targetCharacter)
	elseif (isString(self.targetObject)) then
		object = level:getObject(self.targetObject)
	end
	if (object) then
		self.targetObject = object
	end
	GH.assert(self.targetObject, "Problem fetching target object");
	self.targetObject.onClick = TellStoryEvent.targetCharacter_onClick
	self.targetObject.isEventCharacter = true
	self.targetObject.clickable = false

	-- Add this function for the special DisableBlinkTask
	if (self.targetObject) then
		local tobj = self.targetObject
		self.targetObject.disableBlink = function()
			tobj.blinkLikeProduct = false
		end
	end
end

function TellStoryEvent:onStart()
	TellStoryEvent.super.onStart(self)
end

function TellStoryEvent:onTick(time)
	if (self.eventDone or not self.started or level.elapsedTimeFraction >= 1) then
		return
	end
	if (self.eventState == "Resting") then
		self.eventTimer = self.eventTimer - time
		if (self.eventTimer <= 0 and not self.targetObject.clickable) then
			self.targetObject.clickable = true
			event.targetObject:addTask({class="OverheadTask", image=event.attentionIcon, soundId=event.attentionSound})
		end
	end
end

function TellStoryEvent.targetCharacter_onClick(self)
	self.clickable = false
	self:removeOverhead(true)
	event:itemDone({});
	-- Use self because the function targetCharacter_onClick parameter is the
	-- clickable object.
	self.blinkLikeProduct = true
	event.eventState = "Queued"

	self:getWorkCharacter():addTask({class="CallTask",
		func=function()
			event.eventState = "Telling"
			event:onStartTelling()
		end
	})
end

function TellStoryEvent:onStartTelling()
	-- if (self.targetObject) then
	-- 	self.targetObject.blinkLikeProduct = false
	-- end
	local animName = "eventThanks"..self.currentGoal
	if (isTable(taskSystem:getAnimationDesc(animName))) then
		--event.targetObject:playDefaultAnimation()
		local anim = taskSystem:createAnimation(animName, self.sequenceThanksParams)
		anim:atEnd({class="CallTask", func=function() event:onTellingDone() end })
	else
		event:onTellingDone()
	end
end

function TellStoryEvent:onTellingDone()
	self.eventState = "Resting"
	self.eventTimer = self.restTime
end

function TellStoryEvent:itemDone(item)
	self.currentGoal = self.currentGoal + 1
	self:scoreItemDone()

	if (level.onEventStepDone) then
		level:onEventStepDone(self, item)
	end
	self:onProgress()
	SpriteExt.playSample(nil, "event_item_complete")
	if (self.currentGoal == self.targetGoal) then
		self:eventWon()
		self.eventDone = true
	end
end

function TellStoryEvent:getProgress()
	if (self.currentGoal < self.targetGoal) then
		return self.currentGoal / self.targetGoal
	end
	return 1
end

function TellStoryEvent:getFloaterPosition(obj)
	local pos = self.targetObject.floaterPosition
	return {x = pos.x - (self.targetObject.width * 0.5), y = pos.y}
end
