AdjustDrip = deriveLuaClass("AdjustDrip", "Sprite")

AdjustDrip.inputBlockingTutorialStates = {4}
AdjustDrip.dripDurationChange = 20

AdjustDrip.variations = {
	{
		chance = 50,
		dripStart = {min=600, max=700},
		dripTarget = {min=800, max=900},
		allowedDripDurationDeviation = 50,
		hearts = {
			9500,--7500,
			5500,--5000,
			3500,--2500,
			2500,--7500,
		},
	},
	{
		tutorial = true,
		chance = 50,
		dripStart = {min=800, max=900},
		dripTarget = {min=600, max=700},
		allowedDripDurationDeviation = 50,
		hearts = {
			9500,--7500,
			5500,--5000,
			3500,--2500,
			2500,--7500,
		},
	},
--[[
	{
		chance = 50,
		dripStart = {min=800, max=900},
		dripTarget = {min=600, max=700},
		allowedDripDurationDeviation = 20,
		hearts = {
			7000,
			5000,
			3000,
		},
	},
	{
		chance = 50,
		dripStart = {min=600, max=700},
		dripTarget = {min=800, max=900},
		allowedDripDurationDeviation = 20,
		hearts = {
			7000,
			5000,
			3000,
		},
	},
	{
		chance = 50,
		dripStart = {min=800, max=900},
		dripTarget = {min=600, max=700},
		allowedDripDurationDeviation = 10,
		hearts = {
			7500,
			5500,
			3500,
		},
	},
	{
		chance = 50,
		dripStart = {min=600, max=700},
		dripTarget = {min=800, max=900},
		allowedDripDurationDeviation = 10,
		hearts = {
			7500,
			5500,
			3500,
		},
	},
--]]
}

AdjustDrip.dripLimits = {min=30, max=2500}
AdjustDrip.pxMoveBeforeDragging = 8
AdjustDrip.pxPerTick = 30
AdjustDrip.fallDuration = 400
AdjustDrip.dropStartPos = {x=1094, y=595}
AdjustDrip.dropEndY = 827

function AdjustDrip:onShow()
    self.minigame.back.image = "background:minigames"
    local traits = self.minigame.patient and self.minigame.patient.traits or "normal_female"
	self:addSprite({"drip_pillow:minigames", x=0, y=484})
    self.face = self:addSprite({traits.."_close-up-status:".."minigames", x=428.27, y=1026, scale=2, anchor="bottom"})
    self.emotion = self:addSprite({traits.."_close-up-status:".."minigames", x=428.27, y=1026, scale=2, anchor="bottom"})
	self.bag = self:addSprite({"drip_bag:minigames", x=706.67, y=0})
	self:addSprite({"drip_bag_top:minigames", x=706.67, y=0})
    self.dial = self:addSprite({"drip_dial:minigames", anchor="center", x=1306.13, y=520.53})
    self.arrow = self:addSprite({"drip_arrow:minigames", anchor="center", x=1418.13, y=520})
    self.arrow:animate({wave={y=520, amplitude=30, loops=-1}, duration=1000})
	if (self.minigame.tutorialState == 1) then
		self.arrow.alpha = 0
	end

	self.soundTimer = 0

	self.dripTimer = 0
	self.dripDuration = math.Random(self.dripStart.min, self.dripStart.max)
	self.requiredDripDuration = math.Random(self.dripTarget.min, self.dripTarget.max)

    self.okButton = self:addChild({class="Button", name="set", x=1381.53, y=911.31, setup=setups.stdbtn, textId="BUTTON_OK", anchor="center", mobileFlash=false})
    self.okButton.onClick = function(btn)
        if (self.minigame.tutorialState == 3) then
            hints:removeArrow("AdjustDrip_tutorial3")
            self.minigame:showResidentMessage("AdjustDrip_tutorial4", 1, "ok", {x=1012.45, y=770.7})
            self.minigame.tutorialState = 4
        end

        if (self.correctDrip) then
            self.minigame:scoreEffect(btn, {}, 3, true)
            self.minigame:gameWon()
        else
            self.minigame:gameOver()
        end
        btn.enabled = false
    end

	self:changeDrip(0)
end

function AdjustDrip:onResidentClick(name, texId)
	if (self.minigame.tutorialState == 1) then
		hints:removeArrow("AdjustDrip_tutorial1")
		self.minigame:showResidentMessage("AdjustDrip_tutorial2", 1, "", {x=992.9, y=452.06})
		self.arrow:animate({alpha=1, duration=300})
		self.minigame.tutorialState = 2
	elseif (self.minigame.tutorialState == 4) then
		self.minigame:onTutorialFinished()
	end
end

function AdjustDrip:addSprite(desc)
	desc = isString(desc) and {image=desc} or desc or {}
	desc.class = desc.class or "Sprite"
	desc.image = desc.image or desc[1]
	return self:newChild(desc)
end

function AdjustDrip:onStart()
	if (self.minigame.tutorialState == 1) then
		self.okButton.enabled = false
		self.minigame:showResidentMessage("AdjustDrip_tutorial1", 2, "ok", {x=18.2, y=305.57})
		hints:placeArrow({sprite=self.face, arrowId="AdjustDrip_tutorial1", shadowAlpha=0, offset={y=20}})
	end
end

function AdjustDrip:onTick(time)
	self:tick(time)

	if (self.soundTimer > 0) then
		self.soundTimer = self.soundTimer - time
	end

	self.dripTimer = self.dripTimer + time
	if (self.dripTimer >= self.dripDuration) then
		self.dripTimer = self.dripTimer - self.dripDuration

		local drop = self:newChild({class="Sprite", image="drip_drop:minigames",x=1094, y=595, anchor="bottom"})
		drop:moveInFrontOf(self.bag)
		drop:animate({delay=self.fallDuration/2},{frame=1},{delay=self.fallDuration/2},{frame=2},{delay=80},{frame=3},{delay=80},{frame=4},{delay=80},{frame=5},{delay=80},{remove=true})
		drop:animate({y=self.dropEndY, duration=self.fallDuration, ease="in"})
	end
end

function AdjustDrip:onFinish()
    self.arrow:removeAllModifiers()
    self.arrow.visible = false
end

function AdjustDrip:changeDrip(amount, x, y)
	local prevDrip = self.dripDuration
	self.dripDuration = math.clamp(self.dripDuration + amount, self.dripLimits.min, self.dripLimits.max)

	local wasCorrect = self.correctDrip
	self.correctDrip = false
	local dripError = self.dripDuration - self.requiredDripDuration
	if (dripError > self.allowedDripDurationDeviation) then
        self.emotion.visible = false
        self.arrow.rotation = 180--math.pi
        self.arrow.visible = true
		if (self.minigame.tutorialState == 3) then
			self.okButton.enabled = false
			hints:removeArrow("AdjustDrip_tutorial3")
			self.minigame:showResidentMessage("AdjustDrip_tutorial2", -2, "", {x=992.9, y=452.06})
			self.minigame.tutorialState = 2
		end
	elseif (dripError < -self.allowedDripDurationDeviation) then
        self.emotion.visible = true
		self.emotion.frame = 2
        self.arrow.rotation = 0
        self.arrow.visible = true
		if (self.minigame.tutorialState == 3) then
			self.okButton.enabled = false
			hints:removeArrow("AdjustDrip_tutorial3")
			self.minigame:showResidentMessage("AdjustDrip_tutorial2", -2, "", {x=992.9, y=452.06})
			self.minigame.tutorialState = 2
		end
	else
		self.correctDrip = true
        self.emotion.visible = true
        self.emotion.frame = 1
        self.arrow.visible = false
		if (self.minigame.tutorialState == 2) then
			self.okButton.enabled = true
			self.minigame:showResidentMessage("AdjustDrip_tutorial3", 4, "", {x=1012.45, y=770.7})
			hints:placeArrow({sprite=self.okButton, arrowId="AdjustDrip_tutorial3", offset={y=50}})
			self.minigame.tutorialState = 3
		end
	end

	if (prevDrip ~= self.dripDuration) then
		--if (x and y and not wasCorrect and
		--	(amount < 0) == (self.dripDuration > self.requiredDripDuration))
		--then
		--	self.minigame:scoreEffect(nil, {x,y}, 0.5, true)
		--end

		local frame = self.dial.frame
		if (amount < 0) then
			frame = frame + 1
		else
			frame = frame - 1
		end
		if (self.soundTimer <= 0) then
			SpriteExt.playSample(nil, amount < 0 and "adjusting_wheel_upward:sound" or "adjusting_wheel_downward:sound")
			self.soundTimer = 500
		end
		self.dial.frame = math.mod(frame, self.dial.frameCount)
	end
end

function AdjustDrip:onTouchBegin(x, y)
	if (x >= self.dial.left and x <= self.dial.right and
		y >= self.dial.top and y <= self.dial.bottom)
	then
		self.isDragging = false
		self.startY = y
		self.prevY = y
		self.clickedTop = (y - self.dial.top < self.dial.height/2)
	end
end

function AdjustDrip:onTouchMove(x, y)
	if (not self.isDragging and self.startY) then
		local dy = self.startY - y
		if (math.abs(dy) >= self.pxMoveBeforeDragging) then
			self.isDragging = true
		end
	end

	if (self.isDragging) then
		local dy = self.prevY - y
		while (self.prevY - y > self.pxPerTick) do
			self.prevY = self.prevY - self.pxPerTick
			self:changeDrip(-self.dripDurationChange, x, y)
		end
		while (y - self.prevY > self.pxPerTick) do
			self.prevY = self.prevY + self.pxPerTick
			self:changeDrip(self.dripDurationChange, x, y)
		end
	end
end

function AdjustDrip:onTouchEnd(x, y)
	if (self.startY and not self.isDragging) then
		if (self.clickedTop) then
			self:changeDrip(-self.dripDurationChange, x, y)
		else
			self:changeDrip(self.dripDurationChange, x, y)
		end
	end
	self.isDragging = false
	self.startY = nil
	self.prevY = nil
end
