-- ************************************ AttendIngredient ************************************ '--
AttendIngredient = deriveLuaClass("AttendIngredient", "IngredientButton")

function AttendIngredient:init(desc)
	AttendIngredient.superInit(self, desc)
	self.attendAlertTime = desc.attendAlertTime or 5000
	self.actorPrepareAnimation = desc.actorPrepareAnimation or "work"
	self.actorPrepareCycles = desc.actorPrepareCycles or -1
	self.actorPrepareFrameTime = desc.actorPrepareFrameTime or 100
	self.actorPrepareDirection = desc.actorPrepareDirection or "up"
	self.actorPrepareDuration = desc.actorPrepareDuration or 1000
	self.showInactiveButton = desc.showInactiveButton ~= false -- Default true
	self.hintArrowImportance = desc.hintArrowImportance or 450
	self.hintArrowId = desc.hintArrowId or "AttendIngredient "..self.name
end

function AttendIngredient:onProductsInitialized()
	self.ingredientsToAttend = self.ingredientsToAttend or {}
	for i=1,3 do
		local ing = self["ingredientToAttend"..i]
		if (ing) then
			ing = level:getObject(ing)
			table.push_back(self.ingredientsToAttend, ing)
		end
	end
--print(serialize("self.ingredientsToAttend: ", self.ingredientsToAttend))
	self.state = "available"
end

function AttendIngredient:onTick(time)
	self:tick(time)
--print("AttendIngredient:onTick state: "..tostring(self.state).." "..tostring(self.overhead and "overhead" or "NO overhead").." "..tostring(self.attendAlertOverhead))
	if (not self.attendAlertActive and
		self.state == "available" and
		self.workTaskCount <= 0 and
		not self.isPreparing and
		level and level.isPlaying)
	then
		local timer = self.timer
--print("timer active "..tostring(timer.active).." time left "..tostring(timer.clampedTimeLeft))
		if (timer.active and timer.clampedTimeLeft < self.attendAlertTime) then
--print("OK 3")
			if (self.attendAlertOverhead) then
				self:createOverhead(self.attendAlertOverhead, self.attendAlertSound)
				if (self.attendAlertOverheadFrameTime and self.overhead) then
					self.overhead:animate({frameTime=self.attendAlertOverheadFrameTime})
				end
			end
			if (self.attendAlertAnimation) then
				self:playAnimationWithKeywordOnActor("attendAlert", self)
			end
			self.attendAlertActive = true
		end
	end
end

function AttendIngredient:cancelAttendAppearance()
	self:removeOverhead()
	self.attendAlertActive = false
	if (self.attendAlertLoop) then
		self:stopLoopSample()
	end
end

function AttendIngredient:getIngredientNameForTrayIcon()
	if (self.ingredientsToAttend) then
		return ""
	end
	if (self.state == "inactive" or self.state == "decay") then
		return ""
	end
	return AttendIngredient.super.getIngredientNameForTrayIcon(self)
end

function AttendIngredient:onWorkStart(task)
--print("AttendIngredient:onWorkStart")
--print("self.state: "..tostring(self.state))
--print("self.actorPrepareAnimation: "..tostring(self.actorPrepareAnimation))
--print("self.actorPrepareCycles: "..tostring(self.actorPrepareCycles))
--print("self.actorPrepareFrameTime: "..tostring(self.actorPrepareFrameTime))
--print(serialize("self.ingredientsToAttend", self.ingredientsToAttend))
	local hasExternalIngredientSource = #self.ingredientsToAttend > 0
	local isInactive = self.state == "inactive" or self.state == "decay"
	if (hasExternalIngredientSource or isInactive) then
		self:cancelAttendAppearance()
		self.state = "prepare"
		self.isPreparing = true
		local duration
		if (not isInactive) then
			-- Get the NORMAL prepare time
			duration = self:getWorkDurationWith(task.actor, "prepare")
		end
		self:defaultOnWorkStart(task, "actorPrepare", duration)
--print("AttendIngredient:onWorkStart task.duration: "..tostring(task.duration))
		return true
	end
	--GH.assert(not hasExternalIngredientSource)
	local ret = AttendIngredient.super.onWorkStart(self, task)
	if (ret) then
		self:cancelAttendAppearance()
		self.workingOnMe = true
	end
	return ret
end

function AttendIngredient:onWorkUpdate(task, time)
--print("AttendIngredient:onWorkUpdate")
	return AttendIngredient.super.onWorkUpdate(self, task, time)
end

function AttendIngredient:onWorkFinish(task)
--print("AttendIngredient:onWorkFinish")
	if (self.isPreparing) then
		self.isPreparing = false
		self.state = "available"
		self:defaultOnWorkFinish(task, true)

		hints:successfulSequence(self.hintArrowId)
		return
	end
	self.workingOnMe = false
	self.timer:reset()
	if (#self.ingredientsToAttend > 0) then
		return
	end
	return AttendIngredient.super.onWorkFinish(self, task)
end

function AttendIngredient:onWorkCancel(task)
	self.timer:reset()
	self:cancelAttendAppearance()
end

function AttendIngredient:onStartPreparing()
	AttendIngredient.super.onStartPreparing(self)
--print("AttendIngredient:onStartPreparing")
	self:cancelAttendAppearance()
	for _,v in ipairs(self.ingredientsToAttend) do
		local state = v.state
		if (state ~= "preparing" and state ~= "available") then
			v.state = "prepare"
		end
	end
end

function AttendIngredient:onDecay()
	AttendIngredient.super.onDecay(self)
--print("AttendIngredient:onDecay")
	self:createOverhead(self.attendOverhead, self.attendSound)
	if (self.attendOverheadFrameTime and self.overhead) then
		self.overhead:animate({frameTime=self.attendOverheadFrameTime})
	end
	self.clickable = true
	for _,v in ipairs(self.ingredientsToAttend) do
		v.state = "decay"
		if (self.showInactiveButton) then
			v.onDeactivate = function() v.visible = true end
		end
	end
end

function AttendIngredient:onDeactivate(time)
--print("AttendIngredient:onDeactivate")
	self.visible = true
	self.clickable = true
end

function AttendIngredient:needsHintArrow()
	return self.state == "inactive" and self.workTaskCount == 0
end
