dofile("script/objects/CashRegister.lua")

-- ************************************ Broken Cash Register ************************************ --
BrokenCashRegister = deriveLuaClass("BrokenCashRegister", "CashRegister")

function BrokenCashRegister:init(desc)
	BrokenCashRegister.superInit(self, desc)
	self.isInialized = false
	self.maxTimeToBreak = desc.maxTimeToBreak or 10000
	self.remainingTimeToBreak = desc.remainingTimeToBreak or self.maxTimeToBreak
	self.maxTimeToFix = desc.fixTime or 2500
	self.remainingTimeToFix = self.maxTimeToFix
	self.progressBarLocation = desc.progressBarLocation or {x=0,y=0}
	self.progressBarFloor = desc.progressBarFloor or 1000
	self.workAnimation = desc.workAnimation or "work"
	self.defaultAnimation = self.baseAnimation
	self.animationWhenBroken = desc.brokenAnimation
	self.brokenSound = desc.brokenSound
	self.fixedSound = desc.fixedSound
	self.allowCancel = desc.allowCancel ~= false -- Default true
    self.isBroken = false
    self.isBeingFixed = false
end

function BrokenCashRegister:onLevelInitialized()
	self:createProgressBar()
	BrokenCashRegister.super.onLevelInitialized(self)
	self.isInialized = true
end

-- Progress bar will appear when the cash is broken to show the fix progress
function BrokenCashRegister:createProgressBar()
	self.progressBar = self:addChild(blueprints["decreasingbar_requestAttention"])
	self.progressBar.location = self.progressBarLocation
	self.progressBar:setProgress(1)
	self.progressBar.alpha = 0
end

function BrokenCashRegister:onTick(time)
	self:tick(time)
	if self.isInialized then
		if (not self.isBroken and not self.isBeingFixed) then
			-- Break the register after some time
			self.remainingTimeToBreak = self.remainingTimeToBreak - time

			if self.remainingTimeToBreak <= 0 then
				self:changeFixStatus(false)
			end
		else
			-- Fill the bar according to the fix progress
			self.progressBar:setProgress(1 - (self.remainingTimeToFix / self.maxTimeToFix))
		end
	end
end

function BrokenCashRegister:onClick()
	if not self.isBroken then
		BrokenCashRegister.super.onClick(self)
	else
		self:addWorkTaskFor(self:getWorkCharacter())
	end
end

-- Executes the normal register behaviour OR starts the fix sequence depending on the register status
function BrokenCashRegister:onWorkStart(task)
	if not self.isBroken then
		BrokenCashRegister.super.onWorkStart(self, task)
	else
		self.isBeingFixed = true
		task.actor.direction = self.workDirection
		task.actor:playAnimation(self.workAnimation, -1, self.workAnimationFrameTime)
		task.duration = self.remainingTimeToFix
	end
end

-- onWorkUpdate is neccesary to allow the task to be canceled
function BrokenCashRegister:onWorkUpdate(task, time)
	if self.isBroken then
		self.remainingTimeToFix = self.remainingTimeToFix - time
		if self.remainingTimeToFix <= 0 then
			self:changeFixStatus(true)
			self:onWorkEndRestoreActor(task)
		end

		return self:shouldTaskEnd(task)
	end
end

-- Function called every time the hero finished the work, or the interaction is cancelled
function BrokenCashRegister:onWorkFinish(task)
	if not self.isBroken and not self.isBeingFixed then
		BrokenCashRegister.super.onWorkFinish(self, task)
	else
		self.isBeingFixed = false
	end
end

-- Cancel the repairing when the player queues another task
function BrokenCashRegister:shouldTaskEnd(task)
	local result = (self.remainingTimeToBreak >= self.maxTimeToBreak) or
		(taskSystem:hasTaskFor(task.actor) and allowCancel ~= false)
	return result
end

-- Changes the status of the register, and shows or hides the progress bar depending on its status
function BrokenCashRegister:changeFixStatus(param)
	self.isBroken = not param

	if self.isBroken then
		-- When it's broken: show the progress bar and change the sprite
		self.remainingTimeToFix = self.maxTimeToFix
        self.progressBar:animate({alpha = 1, duration = 600})
        self.progressBar:setProgress(0)

        if self.animationWhenBroken ~= "" then
        	self.baseAnimation = self.brokenAnimation
        end
        
        if (self.brokenSound) then
            self:playSample(self.brokenSound)
        end
	else
		-- When it's fixed: hide the progress bar and change the sprite
		self.remainingTimeToBreak = self.maxTimeToBreak
		self.progressBar:animate({alpha = 0, duration = 600})
		self.baseAnimation = self.defaultAnimation
        
        if (self.fixedSound) then
            self:playSample(self.fixedSound)
        end
	end
end

function BrokenCashRegister:onWorkEndRestoreActor(task)
	task.actor.direction = "down"
	task.actor:playDefaultAnimation()
end
