--' ************************************ Butterflies ************************************ '--
Butterflies = deriveLuaClass("Butterflies", "Object")

function Butterflies:init(desc)
	Butterflies.superInit(self, desc)

	--' Settings for the butterflies
    self.numButterflies = desc.numButterflies or 4
    self.numButterflyTypes = desc.numButterflyTypes or 5
    self.butterflySineWaveFrequencyMin = desc.butterflySineWaveFrequencyMin or 1000
    self.butterflySineWaveFrequencyMax = desc.butterflySineWaveFrequencyMax or 1000
    self.butterflySineWaveAmplitudeMin = desc.butterflySineWaveAmplitudeMin or 0
    self.butterflySineWaveAmplitudeMax = desc.butterflySineWaveAmplitudeMax or 0
	self.butterflySineWaveBaseMinY = desc.butterflySineWaveBaseMinY or 0
	self.butterflySineWaveBaseMaxY = desc.butterflySineWaveBaseMaxY or GH.getUnitHeight()
	self.butterflyVelocityMinX = desc.butterflyVelocityMinX or 50
	self.butterflyVelocityMaxX = desc.butterflyVelocityMaxX or 100
	self.butterflyTimeBeforeReappearMin = desc.butterflyTimeBeforeReappearMin or 0
    self.butterflyTimeBeforeReappearMax = desc.butterflyTimeBeforeReappearMax or 0
    self.secondsToFastForward = desc.secondsToFastForward or 60

	--' Members
	self.butterflies = {}

	--' Validate
    assert(self.butterflySineWaveFrequencyMin <= self.butterflySineWaveFrequencyMax)
    assert(self.butterflySineWaveAmplitudeMin <= self.butterflySineWaveAmplitudeMax)
	assert(self.butterflySineWaveBaseMinY <= self.butterflySineWaveBaseMaxY)
	assert(self.butterflyVelocityMinX <= self.butterflyVelocityMaxX)
	assert(self.butterflyTimeBeforeReappearMin <= self.butterflyTimeBeforeReappearMax)
end

function Butterflies:onPreInitializeLevel()
    while (table.count(self.butterflies) < self.numButterflies) do
        local butterfly = level.gameLayer:addChild({class="SpriteExt", anchor="center", floor=300})
        butterfly:animate({frameTime=100})
        butterfly.updateY = function(self)
            self.y = self.sineWaveBaseY + math.sin(self.sineWaveTime / butterfly.sineWaveFrequency) * self.sineWaveAmplitude
        end
        self:resetButterfly(butterfly)
        table.push_back(self.butterflies, butterfly)
    end

    for i=1,self.secondsToFastForward do
        self:onTick(1000)
    end
end

function Butterflies:onTick(time)
	self:tick(time)

    local dt = time/1000
	for _,butterfly in ipairs(self.butterflies) do
        --' Update y
        butterfly.sineWaveTime = butterfly.sineWaveTime + time
        butterfly:updateY()

        --' Update x
        butterfly.x = butterfly.x + dt * butterfly.velocityX

        local shouldReset
        if (butterfly.goingRight) then
            shouldReset = butterfly.x > GH.getUnitWidth() + butterfly.width
        else
            shouldReset = butterfly.x < -butterfly.width
        end
        if (shouldReset) then
            self:resetButterfly(butterfly)
        end
	end
end

function Butterflies:resetButterfly(butterfly)
	butterfly.image = "butterfly"..tostring( math.Random(self.numButterflyTypes) )..":room1"
	butterfly.sineWaveBaseY = math.Random(self.butterflySineWaveBaseMinY, self.butterflySineWaveBaseMaxY)
    butterfly.sineWaveFrequency = math.Random(self.butterflySineWaveFrequencyMin, self.butterflySineWaveFrequencyMax)
    butterfly.sineWaveAmplitude = math.Random(self.butterflySineWaveAmplitudeMin, self.butterflySineWaveAmplitudeMax)
	butterfly.velocityX = math.Random(self.butterflyVelocityMinX, self.butterflyVelocityMaxX)
	butterfly.timeBeforeReappear = math.Random(self.butterflyTimeBeforeReappearMin, self.butterflyTimeBeforeReappearMax)
    butterfly.goingRight = (math.Random(2) == 1)
	if (butterfly.goingRight) then
		butterfly.x = butterfly.timeBeforeReappear * -butterfly.velocityX/1000
		butterfly.flipHorizontal = false
	else
		butterfly.velocityX = -butterfly.velocityX
		butterfly.x = GH.getUnitWidth() + butterfly.timeBeforeReappear * -butterfly.velocityX/1000
		butterfly.flipHorizontal = true
	end

	butterfly.sineWaveTime = 0

    butterfly.fixedFloor = false --' TURN IT OFF
    butterfly:updateY()
    butterfly.fixedFloor = true --' TURN IT ON
end
