-- ******************************* SpecialStationTable ********************************** --

CarerStation = deriveLuaClass("CarerStation", "SpecialStationTable")

function CarerStation:init(desc)
    CarerStation.superInit(self,desc)

    self.carerCharacter = desc.carerCharacter
    self.calmDuration = desc.calmDuration or 0
    self.calmAnimation = desc.calmAnimation
    self.walkEmotion = desc.walkEmotion or "baby"

    -- Sounds
    self.calmBabySound = desc.calmBabySound

    -- There should be 1 baby for each chair
    self.babyArray = desc.babyArray
    self.babyId1 = desc.babyId1 or ""
    self.babyId2 = desc.babyId2 or ""
    if (self.babyId1 ~= "" and self.babyId2 ~= "") then
      self.babyId = { self.babyId1, self.babyId2 }
    else
      self.babyId = {}
    end
    self.actionPositions = desc.actionPositions or {{x=0, y=0}}
    self.deliverPositions = desc.deliverPositions or {{x=0, y=0}}
end

function CarerStation:onLevelInitialized()
    CarerStation.super.onLevelInitialized(self)
    self.fakeCarer = level:getSpriteExt(self.carerCharacter)

    self.babyObjects = {}
    for i=1,table.getn(self.babyArray) do
        self.babyObjects[i] = level:getSpriteExt(self.babyArray[i])
    end
end

function CarerStation:onTableStateChanged()
    CarerStation.super.onTableStateChanged(self)
end

function CarerStation:makeCarerClickable()
    if (self.group and self.group.customers[1]) then
        self.clickableCustomer = self.group.customers[1]
        self.group.customers[1].clickable = true
        self:addObjectToController(self.group.customers[1])
    end
end

function CarerStation:removeCarerClickable()
    if (self.clickableCustomer) then
        -- THE ORDER IS IMPORTANT! If we do first the clickable = false, it
        -- seems all the elements of the controller are set to unclickable.
        self:removeObjectFromController(self.group.customers[1])
        self.clickableCustomer.clickable = false
    end
end

function CarerStation:doneEating()
    self.eatingState = false
    if (self.group) then
        self.group:handleCustomerPay(100)
        self.group:onDoneEating()
        self.group.state = "WalkToCradle"
    end
end

function CarerStation:onStartEating()
    -- Setting the group duration to 999999 means the customer won't leave the
    -- station because he "finishes" eating
    if (not self.clickableWhileGestureActive) then
        self.oldClickable = self.clickable
        self.clickable = false
    end

    self.group.stateTimer.duration = 999999

    -- Set the mechanic depending on the first product
    self:SetMechanicFromProduct(self.group.order.allDeliveredProductNames[1])
    self:CalculateNumberOfMechanics()
    self:AddBabyIdToCustomerAnimations()

    self.eatingState = true

    self.completionGestures = self.mechanic:calculateCompletionTable({},1)
    self.mechanic:start(self)

    -- Play the consume animation of the gesture.
    if (self.mechanic.consumeAnimation) then
        self:playCustomersAnimation(self.mechanic.consumeAnimation, self.mechanic.consumeAnimationCycles)
    end
end


function CarerStation:stopMechanic()
    if (self.mechanic) then
        self.mechanic:stopAndHide()
    end
end

function CarerStation:AddBabyIdToCustomerAnimations()
    -- First reset the animations. If it's the first time, it will do nothing.
    self:resetCustomerAnimations()

    local finish = false
    local actualMechanic = self.mechanic
    local babyId = ""
    if (self.group) then
        babyId = self.group:getBabyId()
    end
    if (actualMechanic) then
        while (not finish) do
            self:modifyCustomerAnimations(actualMechanic, babyId)
            actualMechanic = actualMechanic.nextMechanicObject;
            if (not actualMechanic) then
                finish = true
            end
        end
    end
end

function CarerStation:modifyCustomerAnimations(mechanic, suffix)
    if (mechanic.customerIdleAnimation) then
        mechanic.origCustomerIdleAnimation = mechanic.customerIdleAnimation
        mechanic.customerIdleAnimation = mechanic.customerIdleAnimation .. suffix
    end
    if (mechanic.customerActiveAnimation) then
        mechanic.origCustomerActiveAnimation = mechanic.customerActiveAnimation
        mechanic.customerActiveAnimation = mechanic.customerActiveAnimation .. suffix
    end
    if (mechanic.customerCompleteAnimation) then
        mechanic.origCustomerCompleteAnimation = mechanic.customerCompleteAnimation
        mechanic.customerCompleteAnimation = mechanic.customerCompleteAnimation .. suffix
    end
    if (mechanic.customerLowMissAnimation) then
        mechanic.origCustomerLowMissAnimation = mechanic.customerLowMissAnimation
        mechanic.customerLowMissAnimation = mechanic.customerLowMissAnimation .. suffix
    end
    if (mechanic.customerHighMissAnimation) then
        mechanic.origCustomerHighMissAnimation = mechanic.customerHighMissAnimation
        mechanic.customerHighMissAnimation = mechanic.customerHighMissAnimation .. suffix
    end
end

function CarerStation:resetCustomerAnimations()
    local finish = false
    local actualMechanic = self.mechanic
    if (actualMechanic) then
        while (not finish) do
            self:resetOneCustomerAnimation(actualMechanic)
            actualMechanic = actualMechanic.nextMechanicObject;
            if (not actualMechanic) then
                finish = true
            end
        end
    end
end

function CarerStation:resetOneCustomerAnimation(mechanic)
    if (mechanic.origCustomerIdleAnimation) then
        mechanic.customerIdleAnimation = mechanic.origCustomerIdleAnimation
    end
    if (mechanic.origCustomerActiveAnimation) then
        mechanic.customerActiveAnimation = mechanic.origCustomerActiveAnimation
    end
    if (mechanic.origCustomerCompleteAnimation) then
        mechanic.customerCompleteAnimation = mechanic.origCustomerCompleteAnimation
    end
    if (mechanic.origCustomerLowMissAnimation) then
        mechanic.customerLowMissAnimation = mechanic.origCustomerLowMissAnimation
    end
    if (mechanic.origCustomerHighMissAnimation) then
        mechanic.customerHighMissAnimation = mechanic.origCustomerHighMissAnimation
    end
end
