dofile("script/objects/StatusObject.lua")

-- ************************************ Cooker ************************************ --
Cooker = deriveLuaClass("Cooker", "StatusObject")

function Cooker:init(desc)
	Cooker.superInit(self, desc)
	self.productAppend = desc.productAppend or "ready"
	self.hashReplace = desc.hashReplace or "-"
	self.doThrowJump = desc.doThrowJump ~= false
	self.stockSourceObjectId = desc.stockSourceObjectId or nil

	self.stockSourceObject = nil
end

function Cooker:onLevelInitialized()
	--[[ PR --]] --pn("["..self.name.."] Init cooker")
	Cooker.super.onLevelInitialized(self)

	self.buttons = {}
	for i=1,20 do
		local button = self["button"..i]
		if (button) then
			--[[ PR --]] --pb("["..self.name.."] Checking for button named '"..button.."'")
			button = level:getSpriteExt(button)
			if (button) then
				--[[ PR --]] --pb("["..self.name.."] Found button named '"..button.name.."'")
				button.state = "inactive"
				table.push_back(self.buttons, button)
				-- Save the original durations just in case we later modify them
				if (button.originalPrepareDuration == nil) then
					button.originalPrepareDuration = button.prepareDuration
				else
					button.prepareDuration = button.originalPrepareDuration
				end
				if (button.originalDecayDelay == nil) then
					button.originalDecayDelay = button.decayDelay
				else
					button.decayDelay = button.originalDecayDelay
				end
			end
		end
	end

	if (isString(self.cookerButton)) then
		--[[ PR --]] --pn("["..self.name.."] Cooker button named '"..self.cookerButton.."'")
		self.cookerButton = level:getSpriteExt(self.cookerButton)
		--[[ PR --]] --pr(self.cookerButton and "exists" or "doesn't exist")
	end

	if (not self.buttonPositions or #self.buttonPositions < #self.buttons) then
		self.buttonPositions = self.buttonPositions or {}
		for _,button in ipairs(self.buttons) do
			table.push_back(self.buttonPositions, {place=button.location, throw=button.center})
		end
	end

	if (isString(self.overlayObject)) then
		--[[ PR --]] --pn("["..self.name.."] Overlay object named '"..self.overlayObject.."'")
		self.overlayObject = level:getSpriteExt(self.overlayObject)
		--[[ PR --]] --pr(self.overlayObject and "exists" or "doesn't exist")
	end

	if (self.stockSourceObjectId ~= nil) then
		self.stockSourceObject = level:getSpriteExt(self.stockSourceObjectId)
	end
end

function Cooker:getAvailableButton()
	--[[ PR --]] --pi("["..self.name.."] Looking for button...")
	for k,button in ipairs(self.buttons) do
		if (button.state == "inactive" and not button.isQueued) then
			--[[ PR --]] --pd("["..self.name.."] Using '"..button.name.."'")
			return k,button
		end
		--[[ PR --]] --pn("["..self.name.."] Not chosen '"..button.name.."': state='"..button.state.."'")
	end
	--[[ PR --]] --pd("["..self.name.."] No button found!")
end

function Cooker:notify_onWorkStart(ing, task)
	local newProductName = self:getNewProductName(ing.ingredientName)
	--[[ PR --]] --pi(" -> cooks "..newProductName)
	local tray = level:getTray(task.actor)
	GH.assert(tray, "Actor '"..tostring(actor).."' does not have a tray!")

	if (not self.usedInGameplay) then
		return false
	end

	if (not self.dummyProducts) then
		if (tray and not tray:makesValidProductOrStart(task.actor, newProductName)) then
			--[[ PR --]] --pd("Not my problem")
			return nil -- Let ing handle it itself
		end

		local k,button = self:getAvailableButton()
		if (not isTable(button)) then
			--[[ PR --]] --pr("Spot available? "..ts(isTable(button)))
			if (level:isValidProductOrStart(ing.ingredientName, false)) then
				if (tray) then
					tray:clearCurrentIngredients(task.actor)
				end
				--[[ PR --]] --pd("New product")
				return nil -- Let ing handle it itself
			end
			--[[ PR --]] --pd()
			return false
		end
	end

	ing:defaultOnWorkStart(task)
	if (tray) then
		tray:startIngredient(task.actor, task.duration, ing.compoundPosition, ing.compoundTipType, true, ing)
	end
	--[[ PR --]] --pd("notify_onWorkStart true")
	return true
end

function Cooker:getNewProductAppend()
	if (self.productAppend and self.productAppend ~= "") then
		return "."..self.productAppend
	end
	return ""
end

function Cooker:getNewProductName(productName)
	return productName..self:getNewProductAppend()
end

function Cooker:handleButtonCommit(idx, button, newProductName, x, y)
	if (button ~= nil) then
		if (self.stockSourceObject ~= nil) then
			button.stock = self.stockSourceObject.stock
			self.stockSourceObject.stock = 0
			self.stockSourceObject.blockWork = true
		end
	end

--print("PR SCREEN POS C: ["..ts(x)..", "..ts(y).."]")
	local success = (button ~= nil)
	if (success) then
		if (not self.dummyProducts) then
			button:setIngredientAndResource(newProductName)
		end

		local oldParent = button.parent
		local btnPos = self.buttonPositions[idx]
		local oldButtonLocation = btnPos.place
		local oldAnchor = button.anchor
		button.isOvenProduct = true
		button:becomeChildOf(level.topLayer)
		button.isQueued = true
		local targetLocation = btnPos.throw
		button.x = x -- NOT TOLOCAL ?!?!?!?!!? [button.location = level:toLocal(x, y)]
		button.y = y -- NOT TOLOCAL ?!?!?!?!!? [button.location = level:toLocal(x, y)]
--print("PR SCREEN POS D: ["..ts(button.x)..", "..ts(button.y).."]")
		button.anchor = "center"
		button:relocateAnchor(oldAnchor)
		local throwDuration = self.throwDuration or globalThrowDuration
		if (throwDuration > 0) then
			-- Temporary set values to give the Throw Copy the right appearance
			--throwDuration = globalThrowDuration
			button.trowing = true
			button.old_visible = button.visible
			button.visible = true
			button.old_baseAnimation = button.baseAnimation
			button.baseAnimation = button.baseAnimation.."_prepare"
			button:throwCopy(targetLocation.x, targetLocation.y, level.topLayer, false, nil, self.throwArriveSound)
			button.visible = button.old_visible
			button.baseAnimation = button.old_baseAnimation
			button:animate({
				{delay=throwDuration},
				{call=function()
					oldParent:addChild(button)
					button.location = oldButtonLocation
					button.state = "prepare"
					button.isQueued = false
					button.trowing = false
				end},
			})

			-- Apply the effect of the product to the prepareDuration and the
			-- decayDelay
			self:applyProductEffect(newProductName, button)
		else
			oldParent:addChild(button)
			button.trowing = false
			button.location = oldButtonLocation
			button.state = "prepare"
			button.isQueued = false
		end

		--if (self.cookerButton and self.cookerButton.cooker_onCommit) then
		--	self.cookerButton:cooker_onCommit()
		--end
	end
	--[[ PR --]] --pd()
	return success
end

function Cooker:notify_onCommit(task, ing, productName, x, y)
	--[[ PR --]] --pi("["..self.name.."] Cooker notify on commit: "..productName)
	local newProductName = self:getNewProductName(productName)

	if (not self.dummyProducts) then
		if (not level:isValidProductOrStart(newProductName, false)) then
			--[[ PR --]] --pd("handle it yourself")
			return nil -- Let ing handle it itself
		end
	end

	local success = false
	if (self.handleAllButtons) then
		for k,button in ipairs(self.buttons) do
			if (button.state == "inactive" and not button.isQueued) then
				--[[ PR --]] --pn("["..self.name.."] Button: "..(button and ("'"..button.name.."'") or "<nil>"))
				success = self:handleButtonCommit(k, button, newProductName, x, y) or success
			end
		end
	else
		local idx, button = self:getAvailableButton()
		--[[ PR --]] --pn("["..self.name.."] Button: "..(button and ("'"..button.name.."'") or "<nil>"))
		success = self:handleButtonCommit(idx, button, newProductName, x, y)
	end

	if (success) then
		local tray = level:getTray(task.actor)
		GH.assert(tray, "Actor '"..tostring(actor).."' does not have a tray!")
		if (tray) then
			local num = tray:getCurrentIngredientCount(task.actor)
			for i = 1,num do
				local ing = tray:getIngredientSource(task.actor, i)
				--[[ PR --]] --pi("["..self.name.."] calling defaultOnCommit for ["..ing.name.."]")
				ing:defaultOnCommit(productName, x, y)
				--[[ PR --]] --pd()
			end
			tray:clearCurrentIngredients(task.actor)
		end
	end

	return success
end

function Cooker:status_onStartPreparing(ing)
	--[[ PR --]] --print("["..self.name.."] Cooker:status_onStartPreparing overlayObject="..ts(self.overlayObject and self.overlayObject.name or nil))
	if (self.overlayObject and self.overlayObject.status_onStartPreparing) then
		self.overlayObject:status_onStartPreparing(ing)
	end

	if (self.cookerButton and self.cookerButton.status_onStartPreparing) then
		self.cookerButton:status_onStartPreparing(ing)
	end

	-- Cooker.super.status_onStartPreparing(self, ing)
end

function Cooker:status_onBecomeAvailable(ing)
	--[[ PR --]] --print("["..self.name.."] Cooker:status_onBecomeAvailable overlayObject="..ts(self.overlayObject and self.overlayObject.name or nil))
	if (self.overlayObject and self.overlayObject.status_onBecomeAvailable) then
		self.overlayObject:status_onBecomeAvailable(ing)
	end

	if (self.cookerButton and self.cookerButton.status_onBecomeAvailable) then
		self.cookerButton:status_onBecomeAvailable(ing)
	end

	-- Cooker.super.status_onBecomeAvailable(self, ing)
end

function Cooker:status_onDecay(ing)
	--[[ PR --]] --print("["..self.name.."] Cooker:status_onDecay overlayObject="..ts(self.overlayObject and self.overlayObject.name or nil))
	if (self.overlayObject and self.overlayObject.status_onDecay) then
		self.overlayObject:status_onDecay(ing)
	end

	if (self.cookerButton and self.cookerButton.status_onDecay) then
		self.cookerButton:status_onDecay(ing)
	end

	-- Cooker.super.status_onDecay(self, ing)
end

function Cooker:status_onDeactivate(ing)
	--[[ PR --]] --print("["..self.name.."] Cooker:status_onDeactivate ")
	if (self.overlayObject and self.overlayObject.status_onDeactivate) then
		--[[ PR --]] --pd("-> overlayObject = " .. ts(self.overlayObject and self.overlayObject.name or nil))
		self.overlayObject:status_onDeactivate(ing)
	end

	if (self.cookerButton and self.cookerButton.status_onDeactivate) then
		self.cookerButton:status_onDeactivate(ing)
	end

	if (self.stockSourceObject ~= nil) then
		self.stockSourceObject.blockWork = false
	end

	-- Cooker.super.status_onDeactivate(self, ing)
end

function Cooker:getObjectInUseChainOf(obj, productName)
--print("Cooker getObjectInUseChainOf ["..obj.name.."] -> ["..self.name.."] '"..productName.."'")
	for _,button in ipairs(self.buttons) do
--print("\t"..button.name)
		if (button == obj) then
--print("\ttraverse for ["..self.name.."]")
			return IngredientButton.getFirstObjectInUseChain_traverse(self, productName)
		end
	end
end

function Cooker:getObjectThatCanMakeIngredient(ingredientName, unfinishedQueuedProducts)
	local tail = self:getNewProductAppend()
	local posA, posB = string.find(ingredientName, tail)
	if (posA and posB == #ingredientName) then
		local base = string.sub(ingredientName, 1, posA-1)
		local prepareButton
		for _,button in ipairs(self.buttons) do
			local state = button.state
			if (state == "prepare" or state == "available") then
				--[[ PR --]] --pr("Cooker:getObjectThatCanMakeIngredient ing: "..ingredientName.." button: "..button.ingredientName.." base: "..base)
				if (button.ingredientName and string.find(button.ingredientName, base) == 1) then
					if (state == "prepare") then
						prepareButton = button
					else
						return button
					end
				end
			end
		end
		if (prepareButton) then
			return prepareButton
		end

		return level.gameLayer:traverse(function(_, node)
			if (node ~= self and node.getNotifyObject and node:getNotifyObject() == self) then
				local obj = node:getObjectThatCanMakeIngredient(base, unfinishedQueuedProducts)
				if (obj) then return obj end
			end
		end)
	end
end

function Cooker:notify_getIngredientNameForHints(ing)
	--[[ PR --]] --pb("["..self.name.."] Cooker Notify get ingredient name for hints (from '"..ing.name.."')")
 --return self:getNewProductName(ing.ingredientName)
	return ing.ingredientName
end

function Cooker:notify_getObjectThatCanMakeIngredient(ing)
	return self:getAvailableButton() ~= nil
end

function Cooker:getProductEffect(productName, productGroup)
	if (productGroups and player) then
		if (productGroups[player.episode]) then
			if (productGroups[player.episode][productGroup]) then
				local allEffects = productGroups[player.episode][productGroup]
				local i = 1
				local j = 1
				local groupNum = -1
				while (allEffects[i] and groupNum == -1) do
					if (groupNum == -1) then
						j = 1
						while (allEffects[i].images[j]) do
							if (allEffects[i].images[j] == "button"..productName) then
								if (allEffects[i].effects) then
									return allEffects[i].effects[j]
								else
									return nil
								end
							end
							j = j+1
						end
					end
					i = i+1
				end
			end
		end
	end
	return nil
end

function Cooker:applyProductEffect(productName, button)
	local effect = self:getProductEffect(productName, button.productGroup)
	if (effect) then
		if (effect.prepareDuration) then
			button.prepareDuration = effect.prepareDuration
		end
		if (effect.decayDelay) then
			button.decayDelay = effect.decayDelay
		end
	end
end

-- ************************************ Cooker Button ************************************ --
TapButton = deriveLuaClass("TapButton", "IngredientButton")

function TapButton:init(desc)
	TapButton.superInit(self, desc)

--print("TapButton:init()")
	self.activateOnBecomeAvailable = desc.activateOnBecomeAvailable
end

function TapButton:status_onDeactivate()
--print("TapButton:status_onDeactivate()")
	self.visible = self.activateOnBecomeAvailable
	self.clickable = self.activateOnBecomeAvailable
end

function TapButton:canBeUsedInChain()
--print("===============")
--print("state "..self.state)
--print("clickable "..tostring(self.clickable))
--print("checkStock "..tostring(self:checkStock()))
	return
		self.clickable and
		(self.state == "inactive" or self.state == "available") and
		self:checkStock()
end
