dofile("script/objects/MultiTargetButton.lua")

-- ************************************ Fishing ************************************ --
Fishing = deriveLuaClass("Fishing", "MultiTargetButton")

function Fishing:init(desc)
	Fishing.superInit(self, desc)

	-- Configuration
	self.productList = {}
	self.ingredientNameForTrayIcon = "#fishing"

	-- System
	self.lastFrameDuration = 700
end

function Fishing:randomizeProducts()
	self.productList = table.copy(self.originalProductChoices)
	table.randomize(self.productList)
end

function Fishing:getProductFromList()
	if (#self.productList <= 0) then
		self:randomizeProducts()
	end
	return table.pop_back(self.productList)
end

function Fishing:onLevelInitialized()
	Fishing.super.onLevelInitialized(self)

	if (level:getObject("restock-b")) then
		self.originalProductChoices = {"#restock-a", "#restock-a", "#restock-b", "#restock-b"}
	else
		self.originalProductChoices = {"#restock-a", "#restock-a"}
	end
	self:randomizeProducts()
end

function Fishing:onTick(time)
	-- If we do not add this function, the object doesn't highlight
	self:tick(time)

	if self.originalProductChoices ~= nil then
		if (self.notifyObjectId ~= nil) then
			-- if a notifyObject is set, check if it's stock is still 0
			-- if is greater than 0 find a product with 0 stock
			local object = level:getObject(self.notifyObjectId)
			if (object.stock > 0) then
				self.notifyObjectId = self:findAProductWithOStock()
			end
		else
			self.notifyObjectId = self:findAProductWithOStock()
		end
	end
end

function Fishing:findAProductWithOStock()
	for _,v in ipairs(self.originalProductChoices) do
		v = string.sub(v,-9)
		local object2 = level:getObject(v)
		if (object2 ~= nil)then
			if (object2.stock == 0 and self.notifyObjectId ~= v) then
				return v
			end
		end
	end
	return nil
end

function Fishing:listPossibleIngredients()
	return self.originalProductChoices
end

function Fishing:getIngredientNameForTrayIcon()
	return self.ingredientNameForTrayIcon
end

function Fishing:onWorkStart(task)
	if (self:isStockFull()) then
		self:playSample("no_stock:sound")
		self:doWrongAnimation(task)
		--print("Full stock")
		return false
	end

	self:removeAllModifiers()

	self.chosen = self:getProductFromList()

	self.ingredientName = self.chosen
	local ret = Fishing.super.onWorkStart(self, task)
	if (ret) then
		self.visible = false
		local actor = task.actor
		local d = task.duration
		actor:setAppearance("fish", "fish")
		actor:setAppearance("fishing", self.chosen)
		actor:animate({delay=d},{call=function()
			self.ret = Fishing.super.onWorkFinish(self, task)
			--actor:playAnimation("fish_end", 1, 700, "default")
		end})
		task.duration = d + self.lastFrameDuration
	else
		self.ingredientName = "#restock-a"
	end
	return ret
end

function Fishing:onWorkFinish(task)
	self.visible = true
	self.ingredientName = "#restock-a"
	local actor = task.actor
	actor:removeAppearance("fish")
	actor:removeAppearance("fishing")
	actor:playDefaultAnimation()

	-- This was added to solve the bug where after using the fishing rod once
	-- when it's upgraded, it's sprite changes to the non upgraded version
	if (string.contains(self.animationFull, "_upgrade")) then
		self.baseAnimation = self.animationFull
		self:playAnimation(self.animationFull, -1)
	end
	return self.ret
end

function Fishing:isStockFull()
	-- Checks if all the restock objects have full stock
	local fullStock = true
	for i=1,4 do
	local src = self["notifySource"..i]
		if (src) then
			local obj = self.notifyObjects[src]
			if (obj and obj[1]) then
				if (obj[1].stock < obj[1].maxStock) then
					fullStock = false
				end
			end
		end
	end
	return fullStock
end
