-- ************************************ FixableWithGesturesObject ************************************ --
FixableWithGesturesObject = deriveLuaClass("FixableWithGesturesObject", "EventObject")

function FixableWithGesturesObject:init(desc)
	FixableWithGesturesObject.superInit(self, desc)
	-- Configuration variables
	self.mechanicName = desc.mechanicName
	self.onFixedCallback = nil
	GH.assert(self.mechanicName, "FixableWithGesturesObject must have at least one mechanic name")
	self.objectToSwapName = desc.objectToSwapName
	self.hideWhenFixed = desc.hideWhenFixed
	self.delayBetweenGestures = desc.delayBetweenGestures or 10000
	self.visible = false
	self.eventLost = false
end

-- Get the first mechanic and disable the clicks on the object
function FixableWithGesturesObject:onLevelInitialized(desc)
	FixableWithGesturesObject.super.onLevelInitialized(self)
	self.mechanic = level:getObject(self.mechanicName)
	self.clickable = false
	self.doHighlight = false
end

function FixableWithGesturesObject:activateGestureWithDelay(delayBeforeGesture)
	self:animate({delay=delayBeforeGesture}, {call=function()
		if (not level.hasShiftFinished) then
			-- Do not activate the gesture if the shift has finished
			self:activateGesture()
		end
	end})
end

-- Set the onGestureCompleted callback and activate the gesture
function FixableWithGesturesObject:activateGesture()
	if not eventLost then
		local selfObject = self
		self.mechanic.onCompleteCallback = function()
			selfObject:onGestureCompleted()
		end

		self.mechanic:reset()
		self.mechanic:showGraphic(true)
	end
end

-- Callback triggered when a gesture is completed, it will activate the next one
function FixableWithGesturesObject:onGestureCompleted()
	event:AddStep()

	if (self.mechanic.nextMechanic) then
		self.mechanic = level:getObject(self.mechanic.nextMechanic)
		self:activateGestureWithDelay(self.delayBetweenGestures)
	else
		self:objectFixed()
	end
end

-- When the object is fixed, it will notify the FixWithGesturesEvent and hide/show the corresponding objects
function FixableWithGesturesObject:objectFixed()
	if(self.onFixedCallback) then
		self:onFixedCallback()
	end

	if (self.objectToSwapName and self.objectToSwapName ~= "") then
		level:getObject(self.objectToSwapName).visible = true
		self.visible = false
	else
		self.visible = not self.hideWhenFixed
	end
end


function FixableWithGesturesObject:OnEventLost()
	self.eventLost = true

	if self.mechanic then
		self.mechanic:stopAndHide()
	end
end
