-- *************************** Hammock Station ************************* --

Hammock = deriveLuaClass("Hammock", "Table")

function Hammock:init(desc)
    Hammock.superInit(self,desc)

    self.hammockProduct = desc.hammockProduct or "product_hammock"

    self.minDuration = desc.minDuration or 6000
    self.maxDuration = desc.maxDuration or 10000

    self.layPosition = desc.layPosition
    self.normalLayAnimation = desc.normalLayAnimation
    self.normalLayFrameTime = desc.normalLayFrameTime or 150
    self.lowLayAnimation = desc.lowLayAnimation
    self.lowLayFrameTime = desc.lowLayFrameTime or 150
    self.fallAnimation = desc.fallAnimation
    self.fallFrameTime = desc.fallFrameTime or 150
    self.fallDuration = desc.fallDuration or 1500

    self.waitDecrease = desc.waitDecrease or 2000
    self.progressBarLocation = desc.progressBarLocation
    self.decreaseSpeed = desc.decreaseSpeed or 1
    self.hideProgressBarIfFull = desc.hideProgressBarIfFull ~= false

    self.loopFullSound = desc.loopFullSound
    self.loopHalfSound = desc.loopHalfSound
    self.fallSound = desc.fallSound

    self.playShakeEffect = desc.playShakeEffect or false
    self.shakeDelay = desc.shakeDelay or 1000
    self.shakeDuration = desc.shakeDuration or 250
    self.shakeX = desc.shakeX or 1
    self.shakeY = desc.shakeY or 1

    self.sandSprite = desc.sandSprite

    self.leftRopeSprite = desc.leftRopeSprite
    self.rightRopeSprite = desc.rightRopeSprite

    -- We set the workDuration to 0 so the hero doesn't do the work animation
    -- when she delivers a product
    self.originalWorkDuration = self.workDuration
    self.workDuration = 0

    self.barValue = 1

    self.stopDecrease = false

    self.STATE_EMPTY = 0
    self.STATE_CUSTOMER_NORMAL = 1
    self.STATE_CUSTOMER_LOW = 2
    self.STATE_CUSTOMER_ANGRY = 3
    self.animationState = self.STATE_EMPTY

    self.totalTime = 0

    self.playedShake = false

    self:createProgressBar()
    -- Set the onClick function of the back part of the progress bar
    local hammock = self
    self.progressBar.back.onClick = function()
        hammock:onClick()
    end

    self.showingBar = false
end

function Hammock:onLevelInitialized()
    Hammock.super.onLevelInitialized(self)
    self.hammockButton = level:getObject(self.hammockProduct)
    self.visible = false
    self.hammockButton.visible = true

    if (self.sandSprite) then
        self.sandAnimation = level:getSpriteExt(self.sandSprite)
    end

    if (self.leftRopeSprite) then
        self.leftSprite = level:getSpriteExt(self.leftRopeSprite)
    end
    if (self.rightRopeSprite) then
        self.rightSprite = level:getSpriteExt(self.rightRopeSprite)
    end

    self:hideRopeButtons();
end

function Hammock:onStartEating()
    self.totalDuration = math.Random(self.minDuration, self.maxDuration)
    self.group.stateTimer.duration = 999999

    self:setCharacterPositionToLay()
    self:restartProgressBar()
    self:playNormalLayAnimation()
    self.totalTime = 0
    self.workDuration = self.originalWorkDuration
    self.stopDecrease = false
    self.visible = false
    self.hammockButton.visible = false
    self:showRopeButtons()
    self.barValue = 1
    self.animationState = self.STATE_CUSTOMER_NORMAL
    if (self.loopFullSound) then
        self:loopSample(self.loopFullSound)
    end
end

function Hammock:setCharacterPositionToLay()
    local s = self:getChair(1)
    local customers = self.group.customers
    for _,c in ipairs(customers) do
        c.x = self.x - s.x + self.layPosition.x
        c.y = self.y - s.y + self.layPosition.y
    end
end

function Hammock:onTick(time)
    self:tick(time)
	self.totalTime = self.totalTime + time

    if (self.animationState == self.STATE_CUSTOMER_NORMAL) then
        self.totalDuration = self.totalDuration - time
        self:decreaseBar(time)
        if (self.barValue < 0.5) then
            self.animationState = self.STATE_CUSTOMER_LOW
            self:playLowLayAnimation()
            self:stopLoopSample()
            if (self.loopHalfSound) then
                self:loopSample(self.loopHalfSound)
            end
            if (Hammock.showHint ~= nil) then
                Hammock:showHint({subject=self, id="decreaseHammockHalf"})
            end
        end
    elseif (self.animationState == self.STATE_CUSTOMER_LOW) then
        self.totalDuration = self.totalDuration - time
        self:decreaseBar(time)
        if (self.barValue <= 0) then
            self.animationState = self.STATE_CUSTOMER_ANGRY
            self.totalTime = 0
            self:stopLoopSample()
            if (self.fallSound) then
                self:playSample(self.fallSound)
            end
            self:playFallAnimation()
            self:hideProgressBar()
            self.playedShake = false
        end
    elseif (self.animationState == self.STATE_CUSTOMER_ANGRY) then
        if (self.totalTime >= self.fallDuration) then
            -- IMPORTANT!! First me must change the mood and then the state,
            -- if we don't do it in this order the group is unassigned and
            -- self.group is nil
            if (self.group) then
                self.group.mood = 0
                self.group.state = "Leaving"
            end
            self.animationState = self.STATE_EMPTY
            self.visible = true
            self.frame = self.frameCount -1 -- Put it to the last frame
        elseif (self.totalTime > self.shakeDelay and not self.playedShake) then
            if (self.sandAnimation) then
                self.sandAnimation.visible = true
                self.sandAnimation:playAnimation("oven_steam_animation",1, 200)
                self.sandAnimation.x = self.x
                self.sandAnimation.y = self.y
            end
            self.playedShake = true
            if (self.playShakeEffect) then
                self:addTask({class = "ScreenShakeTask",
                    sprite = level,
                    duration = self.shakeDuration,
                    x = self.shakeX,
                    y = self.shakeY
                })
            end
        end
    end

    if (self:isCustomerLaying()) then
        if (self.totalDuration <= 0) then
            self:doneEating()
            self:hideProgressBar()
            self.animationState = self.STATE_EMPTY
            self.totalTime = 0
        end
    end
end

function Hammock:doneEating()

    self.group:onDoneEating()
    self.visible = true
    self.frame = self.frameCount -1 -- Put it to the last frame (should be 1)
    self:stopLoopSample()
end

function Hammock:decreaseBar(time)
    if (not self.stopDecrease and self.totalTime > self.waitDecrease) then
        if (Hammock.showHint ~= nil) then
            Hammock:showHint({subject=self, id="decreaseHammockFirst"})
        end
        self:showProgressBar()
        self.barValue = self.barValue - time * self.decreaseSpeed / 10000
        if (self.barValue < 0) then
            self.barValue = 0
        end
        self.progressBar:setProgress(self.barValue)
    end
end

function Hammock:playNormalLayAnimation()
    if (self.group) then
        local customers = self.group.customers
        for _,c in ipairs(customers) do
            c:playAnimation(self.normalLayAnimation, -1, self.normalLayFrameTime)
        end
    end
end

function Hammock:playLowLayAnimation()
    if (self.group) then
        local customers = self.group.customers
        for _,c in ipairs(customers) do
            c:playAnimation(self.lowLayAnimation, -1, self.lowLayFrameTime)
        end
    end
end

function Hammock:playFallAnimation()
    if (self.group) then
        local customers = self.group.customers
        for _,c in ipairs(customers) do
            c:playAnimation(self.fallAnimation, 1, self.fallFrameTime)
        end
    end
end

function Hammock:onClick()
    self.super.onClick(self)
    if (self:isCustomerLaying()) then
        self.stopDecrease = true
        self:stopLoopSample()
        if (self.loopFullSound) then
            self:loopSample(self.loopFullSound)
        end
    end
end

function Hammock:onWorkStart(task)
    Hammock.super.onWorkStart(self, task)
end

function Hammock:onWorkUpdate(task)
    -- Added this to solve the bug that the character keeps working if the
    -- customer leaves in the middle of the work.
    if (not self:isCustomerLaying()) then
        Hammock.super.onWorkFinish(self,task)
    end
end

function Hammock:onWorkFinish(task)
    Hammock.super.onWorkFinish(self, task)
    if (self:isCustomerLaying()) then
        self.stopDecrease = false
        self.barValue = 1
        self.progressBar:setProgress(self.barValue)
        self.totalTime = 0
        self:playNormalLayAnimation()
        self.animationState = self.STATE_CUSTOMER_NORMAL
        if (self.hideProgressBarIfFull == true) then
            self:hideProgressBar()
        end
    end

end

function Hammock:onTableStateChanged()
    Hammock.super.onTableStateChanged(self)
    if (self.plateState == "empty") then
        self.hammockButton.visible = true
        self:hideRopeButtons()
        self.visible = false
    elseif (self.plateState == "occupied") then
        if (Hammock.showHint ~= nil) then
            Hammock:showHint({subject=self, id="askForHammock"})
        end
    elseif (self.plateState == "dirty") then
        self.visible = true
        self.frame = self.frameCount -1 -- Put it to the last frame (should be 1)
    end
end


function Hammock:isCustomerLaying()
    return self.animationState == self.STATE_CUSTOMER_NORMAL
        or self.animationState == self.STATE_CUSTOMER_LOW
end

-- #######################
-- Copied from Event class
function Hammock:createProgressBar()
        self.progressBar = level:add(blueprints["progressbar_pinata"])

		self.progressBar.location = self.progressBarLocation or {x=0,y=0}

        self.progressBar.x = self.x + self.progressBar.location.x
        self.progressBar.y = self.y + self.progressBar.location.y

		self.progressBar.floor = 1000--self.worldFloor-self.progressBar.worldY+1
		self.progressBar:setProgress(1)

        self.progressBar.alpha = 0
        self.progressBar.back.clickable = false
end

function Hammock:restartProgressBar()
    if (self.progressBar) then
        self.progressBar:setProgress(1)
        self.actualProgress = 0
    end
end

function Hammock:showProgressBar()
    if (self.progressBar and not self.showingBar) then
        self.showingBar = true
        self.progressBar:animate({alpha = 1, duration = 600})
        self.progressBar.back.clickable = true
    end
end

function Hammock:hideProgressBar()
    if (self.progressBar and self.showingBar) then
        self.showingBar = false
        self.progressBar:animate({alpha = 0, duration = 600})
        self.progressBar.back.clickable = false
    end
end

function Hammock:showRopeButtons()
    if (self.leftSprite) then
        self.leftSprite.visible = true
    end
    if (self.rightSprite) then
        self.rightSprite.visible = true
    end
end

function Hammock:hideRopeButtons()
    if (self.leftSprite) then
        self.leftSprite.visible = false
    end
    if (self.rightSprite) then
        self.rightSprite.visible = false
    end
end
