HideAndSeekObject = deriveLuaClass("HideAndSeekObject", "EventObject")

function HideAndSeekObject:init(desc)
  HideAndSeekObject.superInit(self, desc)
  self.desc = desc
end

function HideAndSeekObject:LoadEditorVariables(desc)
  self:LoadBasicVariables(desc)
  self:LoadAnimationVariables(desc)
end

function HideAndSeekObject:LoadBasicVariables(desc)
  self.clickTimes = {}
  self.sounds = {}

  self.customSection = desc.customSection or ""

  self.clickTimes.min = desc.minClickableTime or 3000
  self.clickTimes.max = desc.maxClickableTime or 6000

  self.doDisappearAfterClick = desc.doDisappearAfterClick ~= false -- Default true
  self.clickableOnce = desc.clickableOnce ~= false -- Default true

  self.sounds.appear = desc.appearSoundName or ""
  self.sounds.hide = desc.hideSoundName or ""
  self.sounds.click = desc.clickSoundName or ""

  self.stepScore = desc.stepScore or 25
end

function HideAndSeekObject:LoadAnimationVariables(desc)
  self.animations = {}

  self.animations.idleAnimation = {}
  self.animations.appearAnimation = {}
  self.animations.hideAnimation = {}
  self.animations.onClickAnimation = {}

  -- Animation names:
  self.animations.idleAnimation.name = desc.idleAnimation or ""
  self.animations.appearAnimation.name = desc.appearAnimation or self.animations.idleAnimation
  self.animations.hideAnimation.name = desc.hideAnimation or self.animations.idleAnimation
  self.animations.onClickAnimation.name = desc.onClickAnimation or self.animations.idleAnimation

  -- Animation start frames:
  self.animations.idleAnimation.startFrame = desc.idleAnimationStartFrame or -1
  self.animations.appearAnimation.startFrame = desc.appearAnimationStartFrame or -1
  self.animations.hideAnimation.startFrame = desc.hideAnimationStartFrame or -1
  self.animations.onClickAnimation.startFrame = desc.onClickAnimationStartFrame or -1

  -- Animation end frames:
  self.animations.idleAnimation.endFrame = desc.idleAnimationEndFrame or -1
  self.animations.appearAnimation.endFrame = desc.appearAnimationEndFrame or -1
  self.animations.hideAnimation.endFrame = desc.hideAnimationEndFrame or -1
  self.animations.onClickAnimation.endFrame = desc.onClickAnimationEndFrame or -1

  -- Animation cycles:
  self.animations.idleAnimation.cycles = desc.idleAnimationCycles or -1
  self.animations.appearAnimation.cycles = desc.appearAnimationCycles or 1
  self.animations.hideAnimation.cycles = desc.hideAnimationCycles or 1
  self.animations.onClickAnimation.cycles = desc.onClickAnimationCycles or 1

  -- Animation playback modes:
  self.animations.idleAnimation.playback = desc.idleAnimationPlayback or "Default"
  self.animations.appearAnimation.playback = desc.appearAnimationPlayback or "Default"
  self.animations.hideAnimation.playback = desc.hideAnimationPlayback or "Default"
  self.animations.onClickAnimation.playback = desc.onClickAnimationPlayback or "Default"

  -- Animation frame times:
  self.animations.idleAnimation.frameTime = desc.idleAnimationFrameTime or 100
  self.animations.appearAnimation.frameTime = desc.appearAnimationFrameTime or 100
  self.animations.hideAnimation.frameTime = desc.hideAnimationFrameTime or 100
  self.animations.onClickAnimation.frameTime = desc.onClickAnimationFrameTime or 100

  -- Animation frozen frames:
  self.animations.idleAnimation.frozenFrames = desc.idleAnimationFrozenFrames or nil
  self.animations.appearAnimation.frozenFrames = desc.appearAnimationFrozenFrames or nil
  self.animations.hideAnimation.frozenFrames = desc.hideAnimationFrozenFrames or nil
  self.animations.onClickAnimation.frozenFrames = desc.onClickAnimationFrozenFrames or nil
end

function HideAndSeekObject:DoErrorControl()
  -- TODO: in phase 2
end

function HideAndSeekObject:SetupObject()
  self:Hide()
  self.shadowResource = ""
  self.showing = false
  self.hasBeenClicked = false
end

function HideAndSeekObject:onLevelInitialized()
  self:LoadEditorVariables(self.desc)
  self:DoErrorControl()
  self:SetupObject()

  if event then
    -- Adds the current hidden object to the challenge event's objects pool:
    event:AddObject(self)
  end
end

function HideAndSeekObject:Show()
  self.defaultShow = true
  self.visible = true
  self.clickable = true
  self.showing = true
end

function HideAndSeekObject:Hide()
  self.defaultShow = false
  self.visible = false
  self.clickable = false
  self.showing = false
end

function HideAndSeekObject:Appear()
  -- Starts the animation chain for this current object, starting by
  -- playing the appear animation and then changing state to Idle:
  self.baseAnimation = self.animations.appearAnimation.name
  self:addTask({class="PlayAnimationTask",
                animation=self.animations.appearAnimation.name,
                startFrame=self.animations.appearAnimation.startFrame,
                endFrame=self.animations.appearAnimation.endFrame,
                cycles=self.animations.appearAnimation.cycles,
                playbackType=self.animations.appearAnimation.playback,
                frameTime=self.animations.appearAnimation.frameTime,
                frozenFrames=self.animations.appearAnimation.frozenFrames,
                wait=true})
  self:addTask({class="CallTask", func=function() self:Idle() end})
  -- Plays the sound when the object appears:
  self:playSample(self.sounds.appear)
end

function HideAndSeekObject:Idle()
  -- Plays the idle animation for a limited time and then changes the state
  -- to Disappear:
  self.baseAnimation = self.animations.idleAnimation.name
  self:addTask({class="PlayAnimationTask",
                animation=self.animations.idleAnimation.name,
                startFrame=self.animations.idleAnimation.startFrame,
                endFrame=self.animations.idleAnimation.endFrame,
                cycles=self.animations.idleAnimation.cycles,
                playbackType=self.animations.idleAnimation.playback,
                frameTime=self.animations.idleAnimation.frameTime,
                frozenFrames=self.animations.idleAnimation.frozenFrames})
  self:addTask({class="Task",
                duration=math.floor(math.Random(self.clickTimes.min,
                                                self.clickTimes.max))})
  self:addTask({class="CallTask", func=function() self:Disappear() end})
end

function HideAndSeekObject:Disappear()
  -- Plays the disappear animation and then hides the object again:
  self.baseAnimation = self.animations.hideAnimation.name
  self:addTask({class="PlayAnimationTask",
                animation=self.animations.hideAnimation.name,
                startFrame=self.animations.hideAnimation.startFrame,
                endFrame=self.animations.hideAnimation.endFrame,
                cycles=self.animations.hideAnimation.cycles,
                playbackType=self.animations.hideAnimation.playback,
                frameTime=self.animations.hideAnimation.frameTime,
                frozenFrames=self.animations.hideAnimation.frozenFrames,
                wait=true})
  self:addTask({class="CallTask", func=function() self:Hide() end})
  -- Plays the sound when the object disappears:
  self:playSample(self.sounds.hide)
end

function HideAndSeekObject:onClick()
  taskSystem:cancelAllTasksFor(self)
  event:AddStep(self) -- Adds one step to the challenge event's goal
  self.clickable = false
  self.hasBeenClicked = true
  self.baseAnimation = self.animations.onClickAnimation.name
  -- Plays the onClick animation and then hides the object if requested:
  self:addTask({class="PlayAnimationTask",
                animation=self.animations.onClickAnimation.name,
                startFrame=self.animations.onClickAnimation.startFrame,
                endFrame=self.animations.onClickAnimation.endFrame,
                cycles=self.animations.onClickAnimation.cycles,
                playbackType=self.animations.onClickAnimation.playback,
                frameTime=self.animations.onClickAnimation.frameTime,
                frozenFrames=self.animations.onClickAnimation.frozenFrames,
                whenFinished="stay",
                wait=true})
  self:addTask({class="CallTask", func=
    function()
      taskSystem:cancelAllTasksFor(self)
      if self.doDisappearAfterClick then
        self:Disappear()
      else
        self:Hide()
      end
    end})
  -- Plays the sound when the object is clicked:
  self:playSample(self.sounds.click)
end
