-- ************************************ Ingredient Button ************************************ --
IngredientButton = getClass("IngredientButton")

IngredientButton.__getters = IngredientButton.__getters or {}

IngredientButton.availableBlink = {startAlpha=1,endAlpha=0,fadeIn=60,fadeOut=60,loops=4}

function IngredientButton:init(desc)
	IngredientButton.superInit(self, desc)

	--[[ PR --]] --pr("["..self.name.."] Init")
	self.statusObjects = self.statusObjects or {}

	self.prepareAnimation = desc.prepareAnimation or "prepare"
	self.prepareVisible = desc.prepareVisible ~= false
	self.prepareFrameTime = desc.prepareFrameTime or -1
	self.prepareDuration = desc.prepareDuration or 0
	self.prepareCycles = desc.prepareCycles or -1
	self.decayAnimation = desc.decayAnimation or "decay"
	self.decayDuration = desc.decayDuration or 1000
	self.decayDelay = desc.decayDelay or -1
	self.decayVisible = desc.decayVisible ~= false
	self.decayFrameTime = desc.decayFrameTime or -1
	self.decayCycles = desc.decayCycles or 1
	self.failChallengeOnDecay = desc.failChallengeOnDecay ~= false
	self.failChallengeOnStockEmpty = desc.failChallengeOnStockEmpty ~= false
	self.inactiveFrameTime = desc.inactiveFrameTime or 100
	self.worth = desc.worth or 1
	self.compoundPosition = desc.compoundPosition or {x=self.x, y=self.y}
	self.blinkWhenAvailable = desc.blinkWhenAvailable ~= false -- Default true
	self.allowRestockOverload = desc.allowRestockOverload ~= false -- Default true
	self.mustBeStartOfCompound = desc.mustBeStartOfCompound or false -- Default FALSE
	self.needsToBeValidProduct = desc.needsToBeValidProduct or false -- Default FALSE
	self.ignoreTray = desc.ignoreTray or false --Default FALSE

	if (not self.ingredientName) then
		self.ingredientName = self:getIngredientNameFromResource()
	end

	self:onDisplayStockChange()
	self:updateAppearance()
end

function IngredientButton:onLevelInitialized()
	IngredientButton.super.onLevelInitialized(self)

	-- Fix a bug with products that did not get the CHOICE EFFECTS...
    self:setupProductChoiceEffects(self)

    if (self.notifyObjectId) then
		--[[ PR --]] --pr("["..self.name.."] Searching notify object... '"..self.notifyObjectId.."'")
		self.notifyObject = level:getSpriteExt(self.notifyObjectId)
		--[[ PR --]] --pr((self.notifyObject and "found.") or "not found!")
		--[[ PR --]] --pn()
	end
	for i=1,10 do
		local so = self["statusObject"..i]
		if (isString(so)) then
			--[[ PR --]] --pr("["..self.name.."] Searching status object... '"..so.."'")
			so = level:getSpriteExt(so)
			--[[ PR --]] --pr((so and "found.") or "not found!")
			if (so) then
				table.push_back(self.statusObjects, so)
			end
		end
	end

	if (self.useWorkChararcterIfUgrade and player:hasUpgrade(self.boughtUpgrade))then
		self.workCharacter = self.workCharacterUpgrade
	end
end

function IngredientButton:onProductsInitialized()
	IngredientButton.super.onProductsInitialized(self)
	if (self.initialState) then
		self:setIngredientState(self.initialState, false)
	end

	if (self.checkNextExtraTasty) then
		self:checkNextExtraTasty()
	end
end

function IngredientButton:getProductCenter()
	return self.center
end

function IngredientButton:setIngredientAndResource(ingredientName)
	--[[ PR --]] --pb("["..self.name.."] Set ingredient name: "..ingredientName)
	self.ingredientName = ingredientName
	self.baseAnimation = "button"..ingredientName
end

function IngredientButton:getIngredientNameFromResource(res)
	res = res or self.baseAnimation
	--[[ PR --]] --pi("["..self.name.."] Get ingredient name from: "..res)
	for _,v in ipairs({"button#", "product#", "icon#"}) do
		if (string.starts_with(res, v)) then
			res = string.sub(res, #v)
		end
	end
	for _,v in ipairs({"_prepare", "_decay"}) do
		if (string.ends_with(res, v)) then
			res = string.sub(res, 1, #res - #v)
		end
	end
	--[[ PR --]] --pd("-> "..res)
	return res
end

function IngredientButton:listPossibleIngredients()
	return {self.ingredientName or ""}
end

function IngredientButton:getNotifyObject(actor)
	return self.notifyObject
end

function IngredientButton:getIngredientNameForLogic()
	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject) then
		if (notifyObject.notify_getIngredientNameForLogic) then
			local r = notifyObject:notify_getIngredientNameForLogic(self)
			if (r) then return r end
		end
	end
	return self.ingredientName or ""
end

function IngredientButton:notify_getIngredientNameForLogic(ing)
	--[[ PR --]] --pb("["..self.name.."] IngredientButton Notify get ingredient name for logic (from '"..ing.name.."') = "..tostring(ing.ingredientName))
	return ing.ingredientName
end

function IngredientButton:getIngredientNameForTrayIcon()
	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject) then
		if (notifyObject.notify_getIngredientNameForTrayIcon) then
			local r = notifyObject:notify_getIngredientNameForTrayIcon(self)
			if (r) then return r end
		end
	end
	return self.ingredientName or ""
end

function IngredientButton:notify_getIngredientNameForTrayIcon(ing)
	--[[ PR --]] --pb("["..self.name.."] IngredientButton Notify get ingredient name for tray icon (from '"..ing.name.."')")
	return ""
end

function IngredientButton:getIngredientNameForHints()
	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject) then
		if (notifyObject.notify_getIngredientNameForHints) then
			local r = notifyObject:notify_getIngredientNameForHints(self)
			if (r) then return r end
		end
	end
	return self.ingredientName or ""
end

function IngredientButton:notify_getIngredientNameForHints(ing)
	--[[ PR --]] --pb("["..self.name.."] IngredientButton Notify get ingredient name for hints (from '"..ing.name.."')")
	return ing.ingredientName
end

function IngredientButton:onStartPreparing()
	--[[ PR --]] --pi("["..self.name.."] Start prepare: "..ts(self.ingredientName))

	local challengeManager = level.challengeManager
	if (challengeManager) then
		challengeManager:onStartPreparing(self.ingredientName)
	end

	for _,v in ipairs(self.statusObjects) do
		--[[ PR --]] --pe("Status object "..tostring(v.name))
		if (v.status_onStartPreparing) then
			--[[ PR --]] --pr("("..v.name..")")
			local handled = v:status_onStartPreparing(self)
			if (handled ~= nil) then
				--[[ PR --]] --pd("Handled")
				return
			end
		end
	end

	if (self.doPrepareAnimation ~= false) then
		self:playAnimation(self.prepareAnimation, self.prepareCycles, self.prepareFrameTime, nil, nil, false)
		if (not self.prepareFrameTime or self.prepareFrameTime < 0) then
			self:matchFrameTimeTo("prepareDuration")
		end
	end
	self.visible = self.prepareVisible
	--[[ PR --]] --pd()
end

function IngredientButton:onBecomeAvailable(fromTimer)
	--[[ PR --]] --pi()
	if (self.name ~= "") then
		--[[ PR --]] --pr("["..self.name.."] Become available: "..ts(self.ingredientName))
	end

	local challengeManager = level.challengeManager
	if (challengeManager) then
		if self.ingredientName then
			challengeManager:onBecomeAvailable(self.ingredientName)
		end
	end

	if (self.statusObjects) then
		for _,v in ipairs(self.statusObjects) do
			if (v.status_onBecomeAvailable) then
				--[[ PR --]] --pr("("..v.name..")")
				local handled = v:status_onBecomeAvailable(self)
				if (handled ~= nil) then
					--[[ PR --]] --pd("Handled")
					return handled
				end
			end
		end
	end
	self.visible = true

	local notifyObject = self:getNotifyObject(nil)
	if (self.autoProcessOnBecomeAvailable) then
		-- TODO: Would prefer to run this through the normal onWorkStart; Unfortunately we have no task here
		--self:onWorkStart()

		--print("onBecomeAvailable")

		-- So: temp HACK!
		if (notifyObject) then
			notifyObject:animate(
				--{call=function()
				--	notifyObject:throwCopy(self.left + self.width/2, self.top + self.height/2, level.topLayer, true)
				--end},
				{delay=self.throwDuration or 0},
				{call=function() notifyObject:incDisplayStock() end}
			)

			notifyObject:incStock()
			self.state = "inactive"
		end
	end

	self:playAnimation(self.defaultAnimation, 0, self.productFrameTime)

	if (self.blinkWhenAvailable and fromTimer) then
		self:doAvailableBlink()
	end

	if (notifyObject and notifyObject.notify_onBecomeAvailable) then
		notifyObject:notify_onBecomeAvailable()
	end
	--[[ PR --]] --pd()
end

function IngredientButton:doAvailableBlink()
	if (self.availableBlink) then
		--print("blinkWhenAvailable: "..serialize("sad", self.availableBlink or "nil"))
		self:animate({blink = self.availableBlink})
	end
end

function IngredientButton:onDecay()
	--[[ PR --]] --pi("["..self.name.."] Decays: "..ts(self.ingredientName))

	local challengeManager = level.challengeManager
	if (challengeManager) then
		challengeManager:onDecay(self.ingredientName)
	end

	for _,v in ipairs(self.statusObjects) do
		if (v.status_onDecay) then
			--[[ PR --]] --pr("("..v.name..")")
			local handled = v:status_onDecay(self)
			if (handled ~= nil) then
				--[[ PR --]] --pd("Handled")
				return handled
			end
		end
	end

	if (self.doDecayAnimation ~= false) then
		self:playAnimation(self.decayAnimation, self.decayCycles, self.decayFrameTime, "stay", nil, false)
		if (not self.decayFrameTime or self.decayFrameTime < 0) then
			self:matchFrameTimeTo("decayDuration")
		end
	end
	self.visible = self.decayVisible

	--if (player.trophyManager and self.isOvenProduct and player.gameStats.incNrProductsBurned) then
	--	player.gameStats:incNrProductsBurned()
	--	player.trophyManager:checkTrophyGoalReached(17)
	--end

	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject and notifyObject.notify_onDecay) then
		notifyObject:notify_onDecay()
	end
	--[[ PR --]] --pd()
end

function IngredientButton:updateStatusObjects(frac)
	for _,v in ipairs(self.statusObjects) do
		if (v.status_onUpdateStatusFraction) then
			v:status_onUpdateStatusFraction(self, frac)
		end
	end
end

function IngredientButton:onDeactivate()
	--[[ PR --]] --pi("["..self.name.."] Becomes deactivated: "..ts(self.ingredientName))

	local challengeManager = level.challengeManager
	if (challengeManager) then
		challengeManager:onDeactivate(self.ingredientName)
	end

	if (self.statusObjects) then
		for _,v in ipairs(self.statusObjects) do
			if (v.status_onDeactivate) then
				--[[ PR --]] --pr("("..v.name..")")
				local handled = v:status_onDeactivate(self)
				if (handled ~= nil) then
					--[[ PR --]] --pd("Handled")
					return handled
				end
			end
		end
	end

	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject) then
		if (notifyObject.notify_onDeactivate) then
			notifyObject:notify_onDeactivate()
		end
	end

	if (self.doInactiveAnimation) then
		self:playAnimation("inactive", 0, self.inactiveFrameTime)
		self.visible = true
	end
	--[[ PR --]] --pd()
end

function IngredientButton:onClick()
	--[[ PR --]] --pi("["..self.name.."] On click")
	if (not level.draggingObject) then
		local notifyObject = self:getNotifyObject(nil)
		if (notifyObject) then
			--[[ PR --]] --pr("-> notifyObject")
			if (notifyObject.notify_onClick) then
				--[[ PR --]] --pr("("..notifyObject.name..")")
				local handled = notifyObject:notify_onClick(self)
				if (handled ~= nil) then
					--[[ PR --]] --pd()
					--self:handleClickEffect(true)
					return true
				end
			end
		end
		--[[ PR --]] --pd()
		local res = IngredientButton.super.onClick(self)
		--self:handleClickEffect(res)
	end
	level.draggingObject = false
	return true
end

function IngredientButton:allowWork(actor)
	--[[ PR --]] --pi("["..self.name.."] Allow work?")
	local notifyObject = self:getNotifyObject(actor)
	if (notifyObject) then
		--[[ PR --]] --pr("-> notifyObject")
		if (notifyObject.notify_allowWork) then
			--[[ PR --]] --pr("("..notifyObject.name..")")
			local allow = notifyObject:notify_allowWork(self, actor)
			if (allow ~= nil) then
				pr("allow = "..tostring(allow))
				--[[ PR --]] --pd()
				return allow
			end
		end
	end

	if (self.deactivateOnWorkFinish and self.workTaskCount > 0) then
		--[[ PR --]] --pd("Cannot click twice on a single-fire button")
		return false
	end

	--[[ PR --]] --pd("-> true")
	return true
end

function IngredientButton:allowIngredientWork(actor)
	--[[ PR --]] --pi("["..self.name.."] IngredientButton allowIngredientWork")
	if (not self.usedInGameplay) then
		-- print ("allowIngredientWork Ingredient(" .. self.name .. ") FALSE -> not used in Gameplay")
		return false
	end

	local tray = level:getTray(actor)
	GH.assert(tray, "Actor '"..tostring(actor).."' does not have a tray!")
	if (tray and self.mustBeStartOfCompound and tray:getCurrentIngredientCount(actor) > 0) then
		--[[ PR --]] --pd("Ingredient must be start of compound. Clearing tray and continue")

		-- Clear the tray and continue
		tray:clearCurrentIngredients(actor)
		-- print ("allowIngredientWork Ingredient(" .. self.name .. ") TRUE -> is start of compound")
		return true
	end

	if (not self.ingredientName) then
		--[[ PR --]] --pd("No ingredient name!")
		-- print ("allowIngredientWork Ingredient(" .. self.name .. ") FALSE -> missing ingredientName")
		return false
	end

	if (self.blockWork == true) then
		-- print ("allowIngredientWork Ingredient(" .. self.name .. ") FALSE -> blocked for work")
		return false
	end

	if (tray and not tray:makesValidProductOrStart(actor, self.ingredientName)) then
		--[[ PR --]]
		--print("Not a valid product or start: "..self.ingredientName)
		-- print ("allowIngredientWork Ingredient(" .. self.name .. ") FALSE -> doesn't make a valid product: " .. self.ingredientName)
		return false
	end

	--[[ PR --]] --pd("-> true")
	return true
end

function IngredientButton:allowIngredientWork_forNotifyObject(actor)
	--[[ PR --]] --pi("["..self.name.."] IngredientButton allowIngredientWork_forNotifyObject")
	if (not self.usedInGameplay) then
		-- print ("allowIngredientWork_forNotifyObject Ingredient(" .. self.name .. ") FALSE -> not used in Gameplay")
		return false
	end

	if (not self.ingredientName) then
		--[[ PR --]] --pd("No ingredient name!")
		-- print ("allowIngredientWork_forNotifyObject Ingredient(" .. self.name .. ") FALSE -> missing ingredientName")
		return false
	end

	if (self.blockWork == true) then
		-- print ("allowIngredientWork_forNotifyObject Ingredient(" .. self.name .. ") FALSE -> blocked for work")
		return false
	end

	local notifyObject = self:getNotifyObject(actor)
	if (notifyObject) then
		--[[ PR --]] --pr("-> notifyObject")
		if (notifyObject.allowIngredientWork_forNotifyObject) then
			--[[ PR --]] --pr("("..notifyObject.name..")")
			-- print ("allowIngredientWork_forNotifyObject Ingredient(" .. self.name .. ") forwarding ..")
			local allow = notifyObject:allowIngredientWork_forNotifyObject(actor)
			-- print ("allowIngredientWork_forNotifyObject Ingredient(" .. self.name .. ") " .. ((function() if allow then return "TRUE" else return "FALSE" end end)()))
			return allow
		end
	end

	--[[ PR --]] --pd("-> true")
	-- print ("allowIngredientWork_forNotifyObject Ingredient(" .. self.name .. ") TRUE")
	return true
end

function IngredientButton:notify_allowWork(ing, actor)
	return true
end

function IngredientButton:onWorkStart_doNotify(task)
	local notifyObject = self:getNotifyObject(task.actor)
	if (notifyObject) then
		--[[ PR --]] --pr("-> notifyObject")
		if (notifyObject.notify_onWorkStart) then
			local success = notifyObject:notify_onWorkStart(self, task)
			if (success ~= nil) then
				return success
			end
		end
		--[[ PR --]] --pr("-> unhandled...")
	end
end

function IngredientButton:onWorkStart(task)
	--[[ PR --]] --pi("["..self.name.."] IngredientButton onWorkStart")
	IngredientButton.super.onWorkStart(self, task)

	self.isDoingWrongAnimation = false
	self:stopLoopSample()

	if (not self:checkStock()) then
		--[[ PR --]] --pd("-> no stock")

		self:playSample("no_stock:sound")
		self:doWrongAnimation(task)
		return false -- Do not start
	end

	if (not self:checkTrayAvailable(task) and not self.ignoreTray) then
		--[[ PR --]] --pd("-> tray full")
		hints:doTrayFullTip()
		self:doWrongAnimation(task, "trayfull")
		local tray = level:getTray(task.actor)
		if (tray) then
			tray:doBlink(true)
		end
		return false -- Do not start
	else
		level.timesClickedWithFullTray = 0
	end

	local allowIngredientWork = self:allowIngredientWork(task.actor)
	if (not self.needsToBeValidProduct or allowIngredientWork) then
		local success = self:onWorkStart_doNotify(task)
		if (success ~= nil) then
			--[[ PR --]] --pd("self:onWorkStart_doNotify success = "..ts(success))
			return success
		end
	end

	if (self.state ~= "available") then
		--[[ PR --]] --pd("-> not available yet")
		--print("-> not available yet")
		return false -- Do not start
	end

	if (not allowIngredientWork) then
		self:doWrongAnimation(task)
		--[[ PR --]] --pd("-> !allow ingredient work")
		return false -- Do not start
	end

	self:defaultOnWorkStart(task)
	self:startIngredient(task)

	return true
end

function IngredientButton:startIngredient(task)
	local tray = level:getTray(task.actor)
	GH.assert(tray, "Actor '"..tostring(task.actor).."' does not have a tray!")
	if (tray) then
		tray:startIngredient(task.actor, task.duration, self.compoundPosition, self.compoundTipType, false, self)
	end
	--[[ PR --]] --pd("started ingredient '"..ts(self.ingredientName).."'")
end

function IngredientButton:checkTrayAvailable(task)

	local notifyObject = self:getNotifyObject(task.actor)
	if (notifyObject) then
		if (notifyObject.notify_checkTrayAvailable) then
			local available = notifyObject:notify_checkTrayAvailable(self, task)
			if (available ~= nil) then
				return available
			end
		end
	end

	local tray = level:getTray(task.actor)
	GH.assert(tray, "Actor '"..tostring(task.actor).."' does not have a tray!")
	return not tray or not tray.full or not tray:makesValidProduct(task.actor, self.ingredientName)
end

function IngredientButton:notify_onWorkStart(ing, task)
	if (ing.state ~= "available") then
		--[[ PR --]] --pd("-> not available yet")
		return false -- Do not start
	end

	if (not ing:allowIngredientWork_forNotifyObject(task.actor)) then
		ing:doWrongAnimation(task)
		return false -- Do not start
	end

	--[[ PR --]] --pi("["..self.name.."] IngredientButton notify_allowWork (from '"..ing.name.."')"..
	--[[ PR --]] --	" allowRestockOverload: "..ts(self.allowRestockOverload).." usesStock: "..ts(self.usesStock)..
	--[[ PR --]] --	" stock: "..ts(self.stock).." maxStock: "..ts(self.maxStock))
	if (self.usesStock and not self.allowRestockOverload and self.stock >= self.maxStock) then
		--[[ PR --]] --pd("-> No filling beyond the maximum stock")
		ing:doWrongAnimation(task)
		return false
	end
	--[[ PR --]] --pd()

	--[[ PR --]] --pb("["..self.name.."] IngredientButton notify_onWorkStart (from '"..ing.name.."')")
	ing:defaultOnWorkStart(task)
	return true
end

function IngredientButton:onWorkUpdate(task, time)
	local notifyObject = self:getNotifyObject(task.actor)
	if (notifyObject) then
		if (notifyObject.notify_onWorkUpdate) then
			return notifyObject:notify_onWorkUpdate(self, task, time)
		end
	end
end

function IngredientButton:onWorkFinish_doNotify(task)
	local notifyObject = self:getNotifyObject(task.actor)
	if (notifyObject) then
		--[[ PR --]] --pr("-> notifyObject")
		if (notifyObject.notify_onWorkFinish) then
			local success = notifyObject:notify_onWorkFinish(self, task)
			if (success ~= nil) then
				return success
			end
		end
		--[[ PR --]] --pr("-> unhandled...")
	end
end

function IngredientButton:onWorkFinish(task)
	--[[ PR --]] --pi("["..self.name.."] Work finish")
	if (self.isDoingWrongAnimation) then
		--[[ PR --]] --pd()
		return
	end

	IngredientButton.super.onWorkFinish(self, task)
	self:stopLoopSample()

	local handled = self:onWorkFinish_doNotify(task)
	if (handled ~= nil) then
		--[[ PR --]] --pd("handled = "..ts(handled))
		return handled
	end

	--[[ PR --]] --pi("COMMITTING PRODUCT: "..tostring(self))
	local tray = level:getTray((task and task.actor) or hero)
	GH.assert(isTable(tray), "Actor '"..tostring((actor and task.actor) or hero).."' does not have a tray!")
	local success = false
	if (tray) then
		success = tray:commitIngredient(task.actor, self)
		--[[ PR --]] --pi("Commit ingredient success: "..ts(success))
		if (success) then
			local productName = self.ingredientName
			local curIngs = {self}
			if (not self.dummyProducts) then
				--[[ PR --]] --pi("getting actual current ingredients")
				productName = tray:getCurrentIngredients(task.actor)
				curIngs = tray:getCurrentIngredientList(task.actor)
				for _,v in ipairs(curIngs) do
					--[[ PR --]] --pb("ing: "..v.name)
					if (v.getProductToCommit) then
						local tmpProductName = v:getProductToCommit(productName)
						if (tmpProductName) then
							--[[ PR --]] --pb("product name: "..tmpProductName)
							productName = tmpProductName
						end
					end
				end
				--[[ PR --]] --pd()
			end
			local productScreenPos = tray:getProductScreenPos(task.actor, self)
			for k,v in ipairs(curIngs) do
				local isLastItem = (k == #curIngs)
				--[[ PR --]] --pi("onCommit "..ts(k).." last? "..ts(isLastItem))
				if (v.onCommit) then
					local isLastItem = (k == #curIngs)
					--[[ PR --]] --pi("=> onCommit")
					success = v:onCommit(task, productName, productScreenPos.x, productScreenPos.y, isLastItem)
					--[[ PR --]] --pd()
				end
				--[[ PR --]] --pd()
			end
			if (success == nil) then
				--[[ PR --]] --pb("onCommit succeeded: "..productName)
				if (level:isValidProduct(productName, true)) then
					tray:commitProduct(task.actor, productName, productScreenPos, self);
					success = true
				elseif (level:isValidProduct(productName, false)) then
					success = true
				end
			end
			if (success) then
				--[[ PR --]] --pb("Commit succeeded: "..productName)
				tray:clearCurrentIngredients(task.actor)
			end
		end
		--[[ PR --]] --pd()
	end
	--[[ PR --]] --pd()

	self:defaultOnWorkFinish(task, success)
	--[[ PR --]] --pd("IngredientButton:onWorkFinish success = "..ts(success))
end

function IngredientButton:defaultOnWorkFinish(task, success)
	for _,v in ipairs(self.statusObjects) do
		if (v.status_onWorkFinish) then
			v:status_onWorkFinish(self, success)
		end
	end

	--[[ PR --]] --pb("["..self.name.."] Default work finish: "..ts(self.ingredientName).." success = "..tostring(success))
	self:stopLoopSample()
	self:characterOnWorkFinish(task, success)
end

function IngredientButton:notify_onWorkFinish(ing, task)
	--[[ PR --]] --pb("["..self.name.."] Notify work finish: (from '"..ing.name.."')")
end

function IngredientButton:isValidProductForCommit(task, productName)
	--[[ PR --]] --pi("["..self.name.."] Is Valid Product For Commit: "..ts(productName))
	local notifyObject = self:getNotifyObject(task.actor)
	if (notifyObject and notifyObject.notify_isValidProductForCommit) then
		--[[ PR --]] --pr("-> notifyObject")
		local success = notifyObject:notify_isValidProductForCommit(task, self, productName)
		if (success ~= nil) then
			--[[ PR --]] --pd("success = "..ts(success))
			return success
		end
	end
	--[[ PR --]] --pd("... no")
	return false
end

function IngredientButton:notify_isValidProductForCommit(task, ing, productName)
	--[[ PR --]] --pb("["..self.name.."] Is Valid Product For Commit: "..productName.." (from '"..ing.name.."')")
	return true
end

function IngredientButton:onCommit(task, productName, x, y, lastIngredient)
	--[[ PR --]] --pi("["..self.name.."] On commit: "..productName)
	local notifyObject = self:getNotifyObject(task.actor)
	if (notifyObject) then
		if (not self.needsToBeValidProduct or level:isValidProduct(productName, true)) then
			--[[ PR --]] --pr("-> notifyObject")
			--[[ PR --]] --pb("["..self.name.."] Is " .. ((lastIngredient and "") or "not ") .. "last element")
			if (lastIngredient and notifyObject.notify_onCommit) then
				local success = notifyObject:notify_onCommit(task, self, productName, x, y)
				if (success ~= nil) then
					--[[ PR --]] --pd("notifyObject:notify_onCommit success = "..ts(success))
					return success
				end
			end
			--[[ PR --]] --pr("-> unhandled...")
		end
	end

	if (level:isValidProduct(productName, true)) then
		--[[ PR --]] --pr("-> valid product!")
		self:defaultOnCommit(productName, x, y)
	end
	--[[ PR --]] --pd()
end

function IngredientButton:defaultOnCommit(productName, x, y)

	--[[ PR --]] --pi("["..self.name.."] Default commit: "..productName)
	if (self.deactivateOnWorkFinish) then
		--[[ PR --]] --pr("-> deactivating!")
		self.state = "inactive"
	end
	if (self.usesStock) then
		--print("3. IngredientButton:defaultOnCommit: "..productName.." : "..self.name)
		if (self.stock > 0) then
			--[[ PR --]] --pr("-> decreased stock")
			self:decStock(true)
			if (self.upgradeGroup and player.decUpgradeConsumableCount) then
				player:decUpgradeConsumableCount(self.upgradeGroup)
			end
		--else
		--	print("["..self.name.."] Error! Decreasing stock <= 0")
		end
	end
	--[[ PR --]] --pd()
end

function IngredientButton:notify_onCommit(task, ing, productName, x, y)
	--[[ PR --]] --pi("["..self.name.."] Notify on commit: "..productName.." (from '"..ing.name.."')")
	local originalBaseAnimation
	if (self.throwProductCopy) then
		originalBaseAnimation = ing.baseAnimation
		ing.baseAnimation = "product"..ing.ingredientName
	end
	local c = self.center
	local t = ing:throwCopy(c.x, c.y, level.topLayer)
	if (originalBaseAnimation) then
		ing.baseAnimation = originalBaseAnimation
	end
	self:incStock(ing.worth)

	local challengeManager = level.challengeManager
	if (challengeManager) then
		challengeManager:onRestock(ing.name, ing.worth)
	end

	self:animateIncDisplayStock(ing.worth, t.mod)
	ing:defaultOnCommit(productName, x, y)

	return t.mod
end

function IngredientButton:onRemoveProductFromTray(tray, product)
	--[[ PR --]] --pi("["..self.name.."] IngredientButton: onRemoveProductFromTray: "..ts(tray and tray.name).." prod: "..ts(product.source.name)) -- Should be same
	local handler = self.removeTrayHandler
	if (handler == "self") then
		handler = self
	elseif (isString(handler)) then
		handler = level:resolveSprite(handler)
	end
	local handled = false
	if (handler and handler.handleRemoveProductFromTray) then
		handled = handler:handleRemoveProductFromTray(tray, product)
	end
	--[[ PR --]] --pd((handled and "-> handled") or "-> NOT handled")
	return handled, false
end

function IngredientButton:handleRemoveProductFromTray(tray, product)
	--[[ PR --]] --pi("["..self.name.."] handleRemoveProductFromTray: "..ts(tray and tray.name).." ")
	Tray.removeProductsToTarget(tray, {product}, self)
	if (self.usesStock) then
		self:incStock(product.source.worth, true)
		if (self.upgradeGroup and player.incUpgradeConsumableCount) then
			for i=1, product.source.worth do
				player:incUpgradeConsumableCount(self.upgradeGroup)
			end
		end

		--[[ PR --]] --pr("-> increased stock by "..product.source.worth)
	end
	if (self.state == "inactive") then
		self.state = "prepare"
	end
	--[[ PR --]] --pd()
	return true
end

function IngredientButton:onDisplayStockChange()
	if (level.onDisplayStockChange) then
		level:onDisplayStockChange(self.name, self.stock)
	end
	if (self.statusObjects) then
		for _,v in ipairs(self.statusObjects) do
			if (v.status_onDisplayStockChange) then
				local handled = v:status_onDisplayStockChange(self)
				if (handled) then return end
			end
		end
	end
	IngredientButton.super.onDisplayStockChange(self)
end

function IngredientButton:onStockChange()
	if (self.deactivateOnWorkFinish) then
		if (self.stock > 0) then
			self.state = "available"
		else
			self.state = "inactive"
		end
	end
end

function IngredientButton:canBeUsedInChain()
	return self.clickable and self.state == "available" and self:checkStock()
end

function IngredientButton:blocksUseChain()
	return not self.clickable and (self.state == "prepare" or self.state == "decay" or self.state == "inactive") and self:checkStock()
end

function IngredientButton:getFirstObjectInUseChain(productName)
--print("IngredientButton getFirstObjectInUseChain ["..self.name.."] '"..productName.."'")
	if (self:canBeUsedInChain()) then
--print("\tcanBeUsed")
		return self
	end
	if (self:blocksUseChain()) then
--print("\tblocksUseChain")
		return nil
	end
--print("\ttraverse for ["..self.name.."]")
	return self:getFirstObjectInUseChain_traverse(productName)
end

function IngredientButton:getFirstObjectInUseChain_traverse(productName)
--print("IngredientButton getFirstObjectInUseChain_traverse ["..self.name.."] '"..productName.."'")
	return level.gameLayer:traverse(function(_, node)
		if (node ~= self and node.getObjectInUseChainOf) then
			local obj = node:getObjectInUseChainOf(self, productName)
			if (obj) then return obj end
		end
	end)
end

function IngredientButton:getObjectInUseChainOf(obj, productName)
--print("IngredientButton getObjectInUseChainOf ["..obj.name.."] -> ["..self.name.."] '"..productName.."'")
	if (self:getNotifyObject(nil) == obj) then
		return self:getFirstObjectInUseChain(productName)
	end
end

function IngredientButton:getObjectThatCanMakeIngredient(ingredientName, unfinishedQueuedProducts)
--print("IngredientButton getObjectThatCanMakeIngredient ["..self.name.."] '"..tostring(ingredientName).."' == '"..self.ingredientName.."'? "..ts(ingredientName == self.ingredientName))
	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject) then
--print("NOTIFY")
		if (notifyObject.notify_getObjectThatCanMakeIngredient) then
			local r = notifyObject:notify_getObjectThatCanMakeIngredient(self, ingredientName)
			if (r == false) then return nil end
		end
	end
	if (self.ingredientName == ingredientName) then
--print("IngredientButton getObjectThatCanMakeIngredient ["..self.name.."] '"..tostring(ingredientName).."' FOUND")
--print("SELF: "..ts(self))
		return self
	end
--print("NIL")
end
