-- ************************ IngredientButtonWithTool *********************** --

IngredientButtonWithTool = deriveLuaClass("IngredientButtonWithTool", "IngredientButton")

function IngredientButtonWithTool:init(desc)
    IngredientButtonWithTool.superInit(self, desc)

    self.tool = desc.tool
    self.isFinalProduct = desc.isFinalProduct or false
    self.burnAnimation = desc.burnAnimation
    self.burnFrameTime = desc.burnFrameTime or 150
    self.burnDuration = desc.burnDuration or 2000
    self.burnSound = desc.burnSound

    self.orig_workDuration = self.workDuration
    self.orig_workAnimation = self.workAnimation
    self.orig_workFrameTime = self.workFrameTime
    self.orig_workDirection = self.workDirection


    self.totalTime = 0
    self.animationState = 0
    self.burnedHands = false
end

function IngredientButtonWithTool:onLevelInitialized()
    IngredientButtonWithTool.super.onLevelInitialized(self)

    -- We get the tool ingredientName for future comparations
    self.toolButton = level:getObject(self.tool)
    self.toolProduct = self.toolButton.ingredientName

    self.burnedHands = false
end

function IngredientButtonWithTool:onTick(time)
    self:tick(time)
    self.totalTime = self.totalTime + time

    if (self.animationState == 1) then
        if (self.totalTime >= self.orig_workDuration) then
            self.animationState = 0
            self.burnedHands = true
            if (self.actor) then
                self.actor:playAnimation(self.burnAnimation, -1, self.burnFrameTime)
                if (self.burnSound) then
                    self:loopSample(self.burnSound)
                end
                if (IngredientButtonWithTool.showHint ~= nil) then
                    IngredientButtonWithTool:showHint({subject=self.toolButton, id="burnAnimation"})
                end
            end
        end
    end
end

function IngredientButtonWithTool:onWorkStart(task)
    local actor = task.actor
    local result = true
    local tray = level:getTray(actor)
    local prodName = tray:getCurrentIngredients(actor)

    self.workDuration = self.orig_workDuration
    self.workAnimation = self.orig_workAnimation
    self.workFrameTime = self.orig_workFrameTime
    self.workDirection = self.orig_workDirection

    if (prodName == self.toolProduct) then
        -- The actual product is the tool
        if (self.isFinalProduct) then
            result = true
        else
            -- This is used to break the compound if we click on the wrong
            -- element of the compound
            tray:clearCurrentIngredients(actor)
        end
    else
        -- The actual product is not the tool
        if (self.isFinalProduct) then
            -- Hero burns his/her hands
            self.workDuration = self.orig_workDuration + self.burnDuration
            self.totalTime = 0
            self.animationState = 1
            self.actor = actor
        else
            result = true -- "Normal case"
        end
    end

    if (result) then
        -- If the result was true, then call the default onWorkStart function
        result = IngredientButtonWithTool.super.onWorkStart(self, task)
    end
    return result
end

-- Copy&Paste from Ingredient Button code, added the line productName =
-- productName:gsub(self.toolProduct, "")
function IngredientButtonWithTool:onWorkFinish(task)
	if (self.isDoingWrongAnimation) then
		return
	end

	IngredientButton.super.onWorkFinish(self, task)
	self:stopLoopSample()

    -- Used for the challenge do not burn your hands.
    if (self.burnedHands) then
        if (level and level.challengeManager) then
            level.challengeManager:onHandsBurned()
        end
    end
    self.burnedHands = false

	local handled = self:onWorkFinish_doNotify(task)
	if (handled ~= nil) then
		return handled
	end

	local tray = level:getTray((task and task.actor) or hero)
	GH.assert(isTable(tray), "Actor '"..tostring((actor and task.actor) or hero).."' does not have a tray!")
	local success = false
	if (tray) then
		success = tray:commitIngredient(task.actor, self)
		if (success) then
			local productName = self.ingredientName
			local curIngs = {self}
			if (not self.dummyProducts) then
				productName = tray:getCurrentIngredients(task.actor)
				curIngs = tray:getCurrentIngredientList(task.actor)
				for _,v in ipairs(curIngs) do
					if (v.getProductToCommit) then
						local tmpProductName = v:getProductToCommit(productName)
						if (tmpProductName) then
							productName = tmpProductName
						end
					end
				end
			end
			local productScreenPos = tray:getProductScreenPos(task.actor, self)
			for k,v in ipairs(curIngs) do
				local isLastItem = (k == #curIngs)
				if (v.onCommit) then
					local isLastItem = (k == #curIngs)
                    success = v:onCommit(task, productName, productScreenPos.x, productScreenPos.y, isLastItem)
				end
			end
			if (success == nil) then
				if (level:isValidProduct(productName, true)) then
                    -- Remove the peel ingredient if there is one
                    productName = productName:gsub(self.toolProduct, "")
					tray:commitProduct(task.actor, productName, productScreenPos, self);
					success = true
				elseif (level:isValidProduct(productName, false)) then
					success = true
				end
			end
			if (success) then
				tray:clearCurrentIngredients(task.actor)
			end
		end
	end
	self:defaultOnWorkFinish(task, success)
end
