-- ************************************ InstantIngredientButton ************************************ --
InstantIngredientButton = deriveLuaClass("InstantIngredientButton", "IngredientButton")

function InstantIngredientButton:init(desc)
	InstantIngredientButton.superInit(self, desc)
	self.trigger = desc.trigger
	self.triggerObject = desc.triggerObject
	self.fireToLevel = desc.fireToLevel or false -- Default FALSE
	self.particleOffset = desc.particleOffset or {x=0, y=0}
end

function InstantIngredientButton:onClick()
	if (not self:checkStock()) then
		return true
	end

	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject) then
--		local baseAnim = self.baseAnimation
--		self.baseAnimation = self.throwBaseAnimation or self.baseAnimation or ""

		local c = notifyObject.center
		local t = self:throwCopy(c.x, c.y, level.topLayer)
--		self.baseAnimation = baseAnim
		notifyObject:incStock(self.worth)
		notifyObject:animateIncDisplayStock(self.worth, t.mod)
		self.state = "inactive"

		if (self.floaterId) then
			local pos = self.floaterPosition
			level:createGenericFloater(self.floaterId, pos.x, pos.y)
		end
	end

	-- "effects:particles"
	-- "water_splash"
	if (self.particleEffect and self.emitterName) then
		level:addChild({class="ParticleEffect", x=self.x+self.particleOffset.x, y=self.y+self.particleOffset.y, library=self.particleEffect, emitter=self.emitterName, removeWhenFinished=true, start=true})
	end

	if (self.workSoundId) then
		self:playSample(self.workSoundId)
	end

	if (self.upgradeGroup and player.decUpgradeConsumableCount) then
		player:decUpgradeConsumableCount(self.upgradeGroup)
	end

	if (self.usesStock) then
		self:decStock(1, true)
	end

	if (self.trigger) then
		local triggerTable = {triggerObject=self, trigger=self.trigger}
		if (isString(self.triggerParams)) then
			local r = tryParseAsTable(self.triggerParams)
			if (isTable(r)) then
				table.copy_into(triggerTable, r)
			end
		end

		if (self.fireToLevel) then
			level:fireTrigger(self.trigger, triggerTable)
		end

		for i=1,10 do
			local objName = self["objectToTrigger"..i]
			if (objName) then
				triggerTable.name = objName
				level:fireTrigger("object", triggerTable)
			end
		end
	end

	return true
end
