-- ************************* Instrument Station **************************** --

InstrumentStation = deriveLuaClass("InstrumentStation", "Table")

function InstrumentStation:init(desc)
    InstrumentStation.superInit(self, desc)

    self.increaseHearts = desc.increaseHearts or 1

    self.playDuration = desc.playDuration or 10000
    self.playInstrumentSound = desc.playInstrumentSound
    self.waitDuration = desc.waitduration or 1000
    self.waitAnimation = desc.waitAnimation
    self.totalTime = 0

    self.STATE_NOTHING = 0
    self.STATE_PLAY = 1
    self.STATE_WAIT = 2
    self.animationState = self.STATE_NOTHING
end

function InstrumentStation:onLevelInitialized()
	InstrumentStation.super.onLevelInitialized(self)
end

function InstrumentStation:onTick(time)
    self:tick(time)
    self.totalTime = self.totalTime + time
    if (self.animationState == self.STATE_PLAY) then
        if (self.totalTime >= self.playDuration) then
            self.animationState = self.STATE_WAIT
            self.totalTime = 0
            self:donePlaying()
        end
    elseif (self.animationState == self.STATE_WAIT) then
        if (self.totalTime >= self.waitDuration) then
            self.animationState = self.STATE_NOTHING
            self.totalTime = 0
            self:doneEating()
        end
    end
end

function InstrumentStation:donePlaying()
    -- Increase the mood of all the customers that are on the counter
    -- for _,v in ipairs(level.customerGroups) do
    --     self:increaseMoodOfCustomer(v)
    -- end
    -- Increase the mood of all the table customers but myself
    for _,v in ipairs(level.tables) do
        if (v.group ~= nil and v ~= self) then
            if (v.plateState ~= "empty") then
                self:increaseMoodOfCustomer(v.group)
                v.group.alreadyInstrument = false
            end
        end
    end
    -- Play the wait animation of the customer
    for _,c in ipairs(self.group.customers) do
        c:playAnimation(self.waitAnimation,1)
    end
    -- Show the tip
    if (InstrumentStation.showHint ~= nil) then
        InstrumentStation:showHint({subject=self, id="increaseMood"})
    end

end

function InstrumentStation:increaseMoodOfCustomer(customer)
    if (customer.alreadyInstrument ~= true) then
        customer.mood = customer.mood + self.increaseHearts
        customer.alreadyInstrument = true
        customer:resetMoodDropTimer()
        customer:createFloatingHeart()
    end
end

function InstrumentStation:onStartEating()
    self.group.stateTimer.duration = 999999
    self.totalTime = 0
    self.animationState = self.STATE_PLAY
    self:playSample(self.playInstrumentSound)
end

function InstrumentStation:doneEating()
    if (self.group) then
        self.group:onDoneEating()
        self.group.state = "Leaving"
    end
end
