-- ************************************ Mouse ************************************ --
Mouse = deriveLuaClass("Mouse", "Object")

function Mouse:init(desc)
	desc.defaultSection = desc.defaultSection or "gamescene"

	--desc.hitMaskThreshold = -1
	Mouse.superInit(self, desc)
	--self.hitMaskThreshold = -1

	self.showTimes = 0
	self.showing = false
	self.visible = false
	self.clickable = true
	self.baseAnimation = desc.baseAnimation or "mouse"
	self.moveType = desc.moveType or "pop_up"
	self.hitType = desc.hitType or "hit"
	self.mouseShowTime = desc.mouseShowTime or 5000
	self.minShowDelay = desc.minShowDelay or 25000
	self.maxShowDelay = desc.maxShowDelay or 60000
	self.delayMultiplier = desc.delayMultiplier or 1.0
	self.floaterPosition = desc.floaterPosition or {x=0,y=-30}
	self.scoreWhenFound = desc.scoreWhenFound or globalMouseScore
	self.isInsidePriority = 99999
	self.useLargeClickArea = true

	self.showMouseSound = desc.showMouseSound or "mouse_squeak"
	self.findMouseSound = desc.findMouseSound or "mouse"

	self:pickNewShowDelay()
	self:hideMouse()

	if (self.targetPos) then
		self.location = self.targetPos
	end

	self.shadowResource = ""

	self.onTick = DelLevel.mouseTick

if(level:isMetadifficultyActivated())then
	self.mouseShowTime = self.mouseShowTime * level:calcValueWithMetadifficultyLevel("metadifficultyMouseShowingTime")
end
end

function Mouse:onClick()
	if (self.visible and self.clickable) then
		self.clickable = false
		hints:successfulSequence("mouse")

		taskSystem:cancelAllTasksFor(self)
		self:addTask({class="PlayAnimationTask", animation=self.moveType.."_"..self.hitType, cycles=1, frameTime=200, whenFinished="stay", wait=true})
		self:addTask({class="PlayAnimationTask", animation=self.moveType, cycles=1, frameTime=100, whenFinished="stay", wait=true, reversed=true})
		self:addTask({class="CallTask", func=function() self:hideMouse() end})

		self:playSample(self.findMouseSound..":sound")

		local pos = {x=self.left + self.width/2, y=self.top}
		local floaterDesc = {
			id = "mouse",
			textId = "MOUSE_FOUND",
			x = pos.x, y = pos.y,
		}
		if (self.scoreWhenFound > 0) then
			floaterDesc.bonusScore = self.scoreWhenFound
			floaterDesc.bonus = {textId = "MOUSE_FOUND_FMT"}
		end
		level:createFloater(floaterDesc)

		if (level.challengeManager) then
			if (not level.challengeManager:onMouseFound(self)) then
				player:foundMouse()
			end
		else
			player:foundMouse()
		end

		level.foundMouse = true
		local guiMouse = level:getChild("gui_mouse")
		if (guiMouse) then
            local effectPos = {x=guiMouse.x-(guiMouse.width/2), y=guiMouse.y}
            local effect = guiMouse:addChild({class="ParticleEffect", x=effectPos.x, y=effectPos.y, anchor="center", library="effects:particles", emitter="flashing", play=true, scale=2})
            effect:sendToBack()
            guiMouse.scale = 0
            guiMouse:animate({visible=true, scale=1, style="over", duration=200})
		end
		if (level.BM_mouseScore) then level:BM_mouseScore(self.scoreWhenFound) end
	end
	return true
end

function Mouse:doShow()
	GH.assert(self.visible == false and self.showing == false, "The mouse was already showing, the show delay is lower than the hide delay.")

	self.showing = true
	self.clickable = true
	self:pickNewShowDelay()
	self:addTask({class="SetTask", visible=true, showing=true})
	self:addTask({class="PlaySoundTask", sound=self.showMouseSound..":sound"})
	self:addTask({class="PlayAnimationTask", animation=self.moveType, cycles=1, frameTime=100, whenFinished="stay", wait=true})
	self:addTask({class="Task", duration=self:calcValue("mouseShowTime")})
	self:addTask({class="PlayAnimationTask", animation=self.moveType, cycles=1, frameTime=100, whenFinished="stay", wait=true, reversed=true})
	self:addTask({class="CallTask", func=function() self:hideMouse() end})

	self.showTimes = self.showTimes + 1
end

function Mouse:hideMouse()
	self.showing = false
	self.visible = false
end

function Mouse:pickNewShowDelay()
	if(level:isMetadifficultyActivated())then
		self.showDelay = math.Random(self.minShowDelay, self.maxShowDelay)*level:calcValueWithMetadifficultyLevel("metadifficultyMouseShowingRateTime")
	else
	self.showDelay = math.Random(self.minShowDelay, self.maxShowDelay)*self:calcValue("delayMultiplier")
	end
end
