-- ************************************ Multi Target Button ************************************ --
MultiTargetButton = deriveLuaClass("MultiTargetButton", "IngredientButton")

function MultiTargetButton:init(desc)
	MultiTargetButton.superInit(self, desc)
    self.dummyImageHolder = level:newChild({class="Sprite", image=self.image})
    self.dummyImageHolder.visible = false
end

function MultiTargetButton:onLevelInitialized()
	MultiTargetButton.super.onLevelInitialized(self)
	self.notifyObjects = self.notifyObjects or {}
	for i=1,4 do
	local src = self["notifySource"..i]
		if (src) then
			for j=1,3 do
				local obj = self["notifyObject"..i.."_"..j]
				if (obj) then
					if (isString(obj)) then
						obj = level:getObject(obj)
					end
					if (obj) then
						self.notifyObjects[src] = self.notifyObjects[src] or {}
						table.push_back(self.notifyObjects[src], obj)
					end
				end
			end
		end
	end
end

function MultiTargetButton:getNotifyObjects(actor)
	if (not self.notifyObjects) then
		return nil
	end

	local objects = self.notifyObjects[self.ingredientName]
	if (objects) then
		return objects
	end

	actor = actor or hero
	if (actor) then
		local tray = level:getTray(actor)
		if (tray) then
			local curIngs = tray:getCurrentIngredientList(actor)
			for _,v in ipairs(curIngs) do
				objects = self.notifyObjects[v.name]
				if (objects) then
					return objects
				end
			end
		end
	end

	--' If nothing found, return that
	return nil
end

function MultiTargetButton:getNotifyObject(actor)
	local objects = self:getNotifyObjects(actor)
	if (objects) then return objects[1] end
	return nil
end

function MultiTargetButton:getIngredientNameForLogic()
	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject) then
		if (notifyObject.notify_getIngredientNameForLogic) then
			local r = notifyObject:notify_getIngredientNameForLogic(self)
			if (r) then return r end
		end
	end
	return MultiTargetButton.super.getIngredientNameForLogic(self)
end

function MultiTargetButton:getIngredientNameForTrayIcon()
	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject) then
		if (notifyObject.notify_getIngredientNameForTrayIcon) then
			local r = notifyObject:notify_getIngredientNameForTrayIcon(self)
			if (r) then return r end
		end
	end
	return MultiTargetButton.super.getIngredientNameForTrayIcon(self)
end

function MultiTargetButton:getIngredientNameForHints()
	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject) then
		if (notifyObject.notify_getIngredientNameForHints) then
			local r = notifyObject:notify_getIngredientNameForHints(self)
			if (r) then return r end
		end
	end
	return MultiTargetButton.super.getIngredientNameForHints(self)
end

function MultiTargetButton:onBecomeAvailable()
	local handled = MultiTargetButton.super.onBecomeAvailable(self)
	if (handled ~= nil) then return handled end

	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject) then
		if (notifyObject.notify_onBecomeAvailable) then
			notifyObject:notify_onBecomeAvailable()
		end
	end
end

function MultiTargetButton:onDecay()
	local handled = MultiTargetButton.super.onDecay(self)
	if (handled ~= nil) then return handled end

	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject) then
		if (notifyObject.notify_onDecay) then
			notifyObject:notify_onDecay()
		end
	end
end

function MultiTargetButton:onDeactivate()
	local handled = MultiTargetButton.super.onDeactivate(self)
	if (handled ~= nil) then return handled end

	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject) then
		if (notifyObject.notify_onDeactivate) then
			notifyObject:notify_onDeactivate()
		end
	end
end

function MultiTargetButton:onClick()
	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject) then
		if (notifyObject.notify_onClick) then
			local handled = notifyObject:notify_onClick(self)
			if (handled ~= nil) then
				return true
			end
		end
	end
	return MultiTargetButton.super.onClick(self)
end

function MultiTargetButton:allowWork(actor)
	local notifyObject = self:getNotifyObject(actor)
	if (notifyObject) then
		if (notifyObject.notify_allowWork) then
			local allow = notifyObject:notify_allowWork(self, actor)
			if (allow ~= nil) then
				return allow
			end
		end
	end
	return MultiTargetButton.super.allowWork(self, actor)
end

function MultiTargetButton:onWorkStart_doNotify(task)
	local notifyObject = self:getNotifyObject(task.actor)
	if (notifyObject) then
		if (notifyObject.notify_onWorkStart) then
			local success = notifyObject:notify_onWorkStart(self, task)
			if (success ~= nil) then
				return success
			end
		end
	end
	return MultiTargetButton.super.onWorkStart_doNotify(self, task)
end

function MultiTargetButton:checkTrayAvailable(task)
	return true
--	local notifyObject = self:getNotifyObject(task.actor)
--	if (notifyObject) then
--		if (notifyObject.notify_checkTrayAvailable) then
--			local available = notifyObject:notify_checkTrayAvailable(self, task)
--			if (available ~= nil) then return available end
--		end
--	end
--	return MultiTargetButton.super.checkTrayAvailable(self, task)
end
 
function MultiTargetButton:onWorkUpdate(task, time)
	local notifyObject = self:getNotifyObject(task.actor)
	if (notifyObject) then
		if (notifyObject.notify_onWorkUpdate) then
			local handled = notifyObject:notify_onWorkUpdate(self, task, time)
			if (handled ~= nil) then return handled end
		end
	end
	return MultiTargetButton.super.onWorkUpdate(self, task, time)
end

function MultiTargetButton:onWorkFinish_doNotify(task)
	local notifyObject = self:getNotifyObject(task.actor)
	if (notifyObject) then
		if (notifyObject.notify_onWorkFinish) then
			local success = notifyObject:notify_onWorkFinish(self, task)
			if (success ~= nil) then
				return success
			end
		end
	end
	return MultiTargetButton.super.onWorkFinish_doNotify(self, task)
end

function MultiTargetButton:isValidProductForCommit(task, productName)
	local notifyObject = self:getNotifyObject(task.actor)
	if (notifyObject) then
		if (notifyObject.notify_isValidProductForCommit) then
			local success = notifyObject:notify_isValidProductForCommit(task, self, productName)
			if (success ~= nil) then
				return success
			end
		end
	end
	return MultiTargetButton.super.isValidProductForCommit(self, task, productName)
end

function MultiTargetButton:onCommit(task, productName, x, y, lastIngredient)
	local notifyObjects = self:getNotifyObjects(task.actor)
	if (notifyObjects) then
		self.originalThrowDuration = self.originalThrowDuration or self.throwDuration or 300
		local throwDuration = self.originalThrowDuration
		if (lastIngredient) then
			for _,v in ipairs(notifyObjects) do
				if (v.notify_onCommit) then
					self.throwDuration = throwDuration
					self.image = v.image
					local c = v.center
					local t = self:throwCopy(c.x, c.y, level.topLayer)
					v:incStock(self.worth)
					v:animateIncDisplayStock(self.worth, t.mod)
					throwDuration = throwDuration + 200
				end
			end
		end

		self:defaultOnCommit(productName, x, y)
		self.image = self.dummyImageHolder.image
		return true
	end

	if (level:isValidProduct(productName, true)) then
		self:defaultOnCommit(productName, x, y)
	end
end

function MultiTargetButton:getObjectInUseChainOf(obj, productName)
	for _,v in pairs(self.notifyObjects) do
		if (v == obj[1]) then
			return self:getFirstObjectInUseChain(productName)
		end
	end
end

function MultiTargetButton:getObjectThatCanMakeIngredient(ingredientName, unfinishedQueuedProducts)
	local notifyObject = self:getNotifyObject(nil)
	if (notifyObject) then
		if (notifyObject.notify_getObjectThatCanMakeIngredient) then
			local r = notifyObject:notify_getObjectThatCanMakeIngredient(self, ingredientName)
			if (r == false) then return nil end
		end
	end
	return MultiTargetButton.super.getObjectThatCanMakeIngredient(self, ingredientName)
end
