-- ************************************ SpriteExt ************************************ --
SpriteExt = getClass("SpriteExt")

function SpriteExt:onLevelInitialized()
end

function SpriteExt:onProductsInitialized()
end

-- ************************************ Object ************************************ --
Object = getClass("Object")

function Object:init(desc)
	Object.superInit(self, desc)

	--self.workDuration = desc.workDuration or 400 -- TODO: check with Roel implications on Upgrades and Choices?
	self.workAnimation = desc.workAnimation or "work"
	self.workDirection = desc.workDirection or "up"
	self.workCycles = desc.workCycles or -1
	self.workWhenFinished = desc.workWhenFinished or "default"
	self.workFrameTime = desc.workFrameTime or 100

	self.workSoundId = desc.workSoundId or ""
	self.workSoundLooping = desc.workSoundLooping or false
	self.wrongDuration = desc.wrongDuration or 800
	self.wrongAnimation = desc.wrongAnimation or "shakeno"
	self.wrongCycles = desc.wrongCycles or 2
	self.wrongWhenFinished = desc.wrongWhenFinished or "default"
	self.wrongFrameTime = desc.wrongFrameTime or -1
	self.wrongIsBlocking = desc.wrongIsBlocking or false -- Default FALSE

	self.additionalHighlightObject = desc.additionalHighlightObject or ""
	self.overwriteHighlightOnGameplay = false

	self.specialShadowSuffix = desc.specialShadowSuffix or ""
	if (self.specialShadowSuffix and self.specialShadowSuffix ~= "") then
		self.shadowResource = self.baseAnimation .. "_" .. self.specialShadowSuffix .. ":" .. self.defaultSection
	end

	self.idleAnimations = desc.idleAnimations or {
		{a="idle1", f=150, c=1},
		{a="idle2", f=150, c=1},
		{a="idle3", f=150, c=1},
	}

	if (self.initialClickable ~= nil) then
		self.clickable = desc.initialClickable
	end

	--if (self.upgradeGroup and self.usesStock and player.getUpgradeConsumableCount) then
	--	local consumableCount = player:getUpgradeConsumableCount(self.upgradeGroup)
	--	if (consumableCount >= 0) then
	--		self.initialMaxStock = 9
	--		if (consumableCount > 0) then
	--			self.stockAndDisplay = consumableCount
	--		end
	--	end
	--end
end

function Object:onLevelInitialized()
	Object.super.onLevelInitialized(self)

	for i=1,5 do
		local aco = self["additionalClick"..i]
		if (isString(aco)) then
			aco = level:getObject(aco)
			if (aco) then
				self["additionalClick"..i] = aco
				aco.clickable = self.clickable and self.visible and self.alpha > 0
				self:addObjectToController(aco)
			end
		end
	end

	-- During gameplay, this object will get the same highlight as the self object
 	if (self.additionalHighlightObject) then
		self.additionalHighlight = level:getObject(self.additionalHighlightObject)
		if (self.additionalHighlight) then
			self.additionalHighlight.overwriteHighlightOnGameplay = true
		end
	end
end

function Object:defaultOnWorkStart(task, keyword, duration)
	keyword = keyword or "work"
	--[[ PR --]] --pr("["..self.name.."] Default work start: keyword: "..tostring(keyword))
	task.duration = duration or self:getWorkDurationWith(task.actor, keyword)
	--[[ PR --]]

	if (self.statusObjects) then
		for _,v in ipairs(self.statusObjects) do
			if (v.status_onWorkStart) then
				v:status_onWorkStart(self, task.duration)
			end
		end
	end

	if (task.duration > 0) then
		GH.assert(task.actor)
		--[[ PR --]] --pr("-> playing work anim '"..ts(self[keyword.."Animation"]).."'")
		self:playAnimationWithKeywordOnActor(keyword, task.actor, task.duration)
		if (self.workOnAnimation) then
			self.workOnWhenFinished = self.workOnWhenFinished or "default"
			self:playAnimationWithKeywordOnActor("workOn", self, task.duration)
		end
	end
	--[[ PR --]] --pd()
end

function Object:getWorkDurationWith(actor, keyword)
	local actorWorkDurationMultiplier = 1
	if (actor and actor.calcValue) then
		actorWorkDurationMultiplier = actor:calcValue("actorWorkDurationMultiplier", 1)
	end
	GH.assert(isString(keyword))
	return self:calcValue(keyword.."Duration") * actorWorkDurationMultiplier
end

function Object:playAnimationWithKeywordOnActor(keyword, actor, duration)
	assert(actor)
	local direction = self[keyword.."Direction"]
	if (direction) then
		actor.direction = direction
	end
--print("["..self.name.."] PAWKON keyword: "..ts(keyword).." name: "..ts(actor.name).. " duration: "..ts(duration))
	local anim = self[keyword.."Animation"]
	if (anim) then
		local frameTime = self[keyword.."FrameTime"]
		actor:playAnimation(anim, self[keyword.."Cycles"], frameTime, self[keyword.."WhenFinished"])
		duration = duration or self[keyword.."Duration"]
		if (duration and (not frameTime or frameTime < 0)) then
			actor:matchFrameTimeTo(duration)
		end
	end
	local sound = self[keyword.."Sound"]
	if (sound and sound ~= "") then
		self:playSample(sound)
	else
		local loop = self[keyword.."Loop"]
		if (loop and loop ~= "") then
			self:loopSample(loop)
		end
	end
end

function Object:allowWork(actor)
	return true
end

function Object:getWorkCharacter()
	if (self.workCharacter) then
		if (isString(self.workCharacter)) then
			self.workCharacter = level:resolveSprite(self.workCharacter)
		end
		if (isTable(self.workCharacter)) then
			if (self.workCharacter.blockWork == true) then
				return nil
			else
				local workActive = self.workCharacter.isWorkActive
				if (isCallable(workActive)) then workActive = workActive(self.workCharacter) end
				if (workActive ~= false) then -- Default true
					return self.workCharacter
				end
			end
		end
	end
	return self:cpp_getWorkCharacter()
end

function Object:onClick()
	-- Added this to avoid the bug of clicking an object when releasing
	-- the touch of a drag gesture.
	if (not level.draggingObject) then
		local res = Object.cpp_onClick(self)
		if (not res) then
			self:onClickAddWorkTask();
		end
		return res
	end
	-- If an object calls this function, it means we have released the drag.
	-- reset it just in case.
	level.draggingObject = false
end

function Object:onClickAddWorkTask()
	local workCharacter = self:getWorkCharacter()
	if (workCharacter and self:allowWork(workCharacter)) then
		if (workCharacter == level.entertainer) then
			workCharacter:addTask({class="EntertainTask", target=self})
			hints:successfulSequence("entertainer")
			hints:successfulSequence("entertainer"..level.entertainer.name)
		else
			self:addWorkTaskFor(workCharacter)
		end
		self:playSample("product_clicked:sound")
		res = true
	--elseif (workCharacter) then
	--	-- EMPAT
	--	if (empat and empat.selected) then
	--		empat:shakeNo()
	--	end
	end
end

function Object:addWorkTaskFor(workCharacter)
	if (level.beforeAddWorkTaskOnClick) then
		local handled = level:beforeAddWorkTaskOnClick(self, workCharacter)
		if (handled) then return handled end
	end
	workCharacter:addTask({class="WorkTask", target=self})
end

function Object:onActorWorkStart(task, success)
	if (success) then
		local workObject = task.workObject
		local workSoundId = workObject and workObject.workSoundId
		if (workSoundId and workSoundId ~= "") then
			if (workObject.workSoundLooping) then
				self:loopSample(workSoundId)
			else
				self:playSample(workSoundId)
			end
		end
	end
end

function Object:onActorWorkFinish(task)
	self:stopLoopSample()
end

function Object:onWorkStart(task)
	if (task) then
		task.actor:playDefaultAnimation()
	end
	return self:cpp_onWorkStart(task)
end

function Object:doWrongAnimation(task, wrongAnimationName)
	if (self.wrongDuration > 0) then
		task.duration = self.wrongDuration
		task.actor.direction = self.wrongDirection
		wrongAnimationName = wrongAnimationName or self.wrongAnimation or "shakeno"
		task.actor:playAnimation(wrongAnimationName, self.wrongCycles, self.wrongFrameTime, self.wrongWhenFinished)
		if (self.wrongFrameTime and self.wrongFrameTime < 0) then
			task.actor:matchFrameTimeTo(task.duration)
		end

		if (self.wrongSoundId and self.wrongSoundId ~= "") then
			self:playSample(self.wrongSoundId)
		end

		if (self.wrongIsBlocking) then
			self.isDoingWrongAnimation = true
			return true
		end
	end
end

function Object:doWrongAnimationOnCharacter(actor,wrongAnimationName)
	self:doWrongAnimation({actor=actor,},wrongAnimationName)
end

function Object:characterOnWorkFinish(task, success)
--print("YEEEEEEEEEEEEEEEEEEEEEEEYYYYYYYYYY")
	if (task.actor.pendingWorkTaskWithCount == 0) then
	--print("YEEEEEEEEEEEEEEEEEEEEEEEYYYYYYYYYY")
		task.actor:playDefaultAnimation()
		task.actor.direction = "down"
	end
end

function Object:addTasks(t)
	if (not taskSystem) then return {} end
	t = taskSystem:wrapSingleTable(t)
	t = table.copy(t, 1)
	for k,v in pairs(t) do
		v.actor = v.actor or self
	end
	return taskSystem:addTasks(t)
end

Object.addTask = function(self, t)
	return Object.addTasks(self, t)[1]
end

function Object:addTaskAtFront(t)
	--t = table.copy(t, 1)
	t.actor = t.actor or self
	taskSystem:addTaskAtFront(t)
end

function Object:onDisplayStockChange()
	self:updateAppearance()
end

function Object:showInputMessage(desc)
	GH.assert(false, "Text typing in dialog bubbles has not been implemented yet!")
	return self:showMessageDialog(desc)
end

--function Object:showOptionListMessage(desc)
--	desc = table.copy(desc)
--	desc.content = desc.content or { children = {y}, }
--	desc.spacing = desc.spacing or 40
--	desc.width = desc.width or 40
--	local num = table.count(desc.options)
--	local totalWidth = num*desc.width + (num - 1)*desc.spacing
--	local curX = -(totalWidth-desc.width)/2
--	local aOption
--	local scale = ((isPhone and 1.15) or 1)
--	for _,v in pairs(desc.options) do
--		aOption = v.id
--		table.insert(desc.content.children, {
--			class="Button",
--			name=v.id,
--			image=v.img,
--			x=curX, y=0,
--			anchor="top",
--			setup=setups.silent_btn,
--			scale=scale
--		})
--		curX = curX + desc.width + desc.spacing
--	end
--	desc.onTaskSkip = function(self)
--		self:getChild(aOption):onClick()
--	end
--	return self:showMessageDialog(desc)
--end

--function Object:showOptionMessage(desc)
--	desc.optionId = tonumber(desc.optionId)
--	desc.groupId = "room"..player.episodeId.."_option"..desc.optionId
--	local description = optionGroups[desc.optionId]
--	GH.assert(description, "Could not find option description for option '"..tostring(desc.optionId).."'")
--	desc.dialogId = desc.dialogId or desc.textId or math.Random(999999)
--	desc.options = description
--	desc.onClick = function(sender)
--		level:setUpgradeSelection(desc.optionId, sender.name)
--		if (self.hideMessageDialog) then
--			self:hideMessageDialog(desc.dialogId, false)
--		end
--	end
--	return self:showOptionListMessage(desc);
--end

function Object:showYesNoMessage(desc)
	local scale = ((isPhone and 1.2) or 1)
	desc = table.copy(desc)
	desc.yesImage = desc.yesImage or "balloon_yes:gamescene"
	desc.noImage = desc.noImage or "balloon_no:gamescene"
	desc.spacing = desc.spacing or 30
	desc.contentOffsetY = desc.contentOffsetY or 0
	desc.content = desc.content or {
		y=desc.contentOffsetY, anchor="bottom",
		children = {
			{ class="Button", name="yes", x=-desc.spacing/2, y=32, anchor="topRight", setup=setups.silent_btn, scale=scale, matchSizeToLabel=true,
					children={{class="Label", name="balloon_yes", anchor="topRight", textSize=66, setup=setups.outline, color1=0x0097b7, color0=0xa5f8fd, textId="BUTTON_YES", align="right"}}
			},
			{ class="Button", name="no", x=desc.spacing/2, y=32, anchor="topLeft", setup=setups.silent_btn, scale=scale, matchSizeToLabel=true,
					children={{class="Label", name="balloon_no", anchor="topLeft", textSize=66, setup=setups.outline, color1=0x0097b7, color0=0xa5f8fd, textId="BUTTON_NO", align="right"}}
			},
		},
	}
	desc.onTaskSkip = function(self)
		self:getChild("no"):onClick()
	end
	return self:showMessageDialog(desc)
end

function Object:showOkMessage(desc)
	local scale = ((isPhone and 1.2) or 1)
	desc = table.copy(desc)
	desc.image = desc.image or "balloon_ok:gamescene"
	desc.contentOffsetY = desc.contentOffsetY or 0
	desc.content = desc.content or {
		y=desc.contentOffsetY,
		children = {
			{ class="Button", name="ok", anchor="top", y=32, x=0, setup=setups.btn, scale=scale, matchSizeToLabel=true,
				children={{class="Label", name="balloon_ok", anchor="top", textSize=66, setup=setups.outline, color1=0x0097b7, color0=0xa5f8fd, textId="BUTTON_OK", align="center"}}
			},
		},
	}
	desc.onTaskSkip = function(self)
		self:getChild("ok"):onClick()
	end
	return self:showMessageDialog(desc)
end

function toPoint(v, w)
	if (isTable(v)) then
		return {x = v.x or v[1] or 0, y = v.y or v[2] or 0}
	end
	return {x = v or 0, y = w or 0}
end

function Object:showMessageDialog(desc)
--	local dialogDesc = createMessageDialogDescription(desc)

	desc = table.copy(desc)
	if (not desc.text) then
		desc.textId = desc.textId or ""
	end
	desc.dialogId = desc.dialogId or desc.textId or math.Random(999999)
	desc.tipType = desc.tipType or "point"
	desc.tipSide = desc.tipSide or "center"
	desc.bubbleLayer = desc.bubbleLayer or ""
	desc.maxLineCount = desc.maxLineCount or 4
	desc.labelWidth = (desc.labelWidth or defaultMessageBalloonWidth) * ((isPhone and 1.15) or 1.0)

	if (desc.content) then
		desc.content.y = desc.content.y or 10
		desc.content.class = desc.content.class or "Sprite"
	end

	local textSize = desc.textSize or 40
	textSize = textSize * ((isPhone and 1.15) or 1)

	local textColor = desc.textColor or 0x000000

	local balloonChildren = {}
	balloonChildren[1] = {
		class="Sprite",
		name="tip",
	}
	balloonChildren[2] = {
		class="TypeLabel",
		name="message",
		text=desc.text,
		maxWidth=desc.labelWidth,
		wrapStyle="word",
		font=desc.font or "main_font:fonts",
		color0=0, alpha1=0, alpha2=0, alpha3=0,
		anchor=desc.anchor or "top",
		x=desc.labelX,
		textSize=textSize,
		color0 = textColor,
		lineHeight=textSize,
		y=-15
	}
	-- If there is portrait
	if (desc.portraitSection and desc.portraitSection ~= ""
		and desc.portraitAnimation and desc.portraitAnimation ~= "") then
		balloonChildren[3] = {
			x=0, y=0,
			class="SpriteExt",
			name="portrait",
			defaultSection=desc.portraitSection,
			baseAnimation=desc.portraitAnimation,
			anchor="right",
		}
	end

	local dialogDef = {
		class="BalloonDialog",
		owner = self,
		dialogId = desc.dialogId,
		anchor = "top",
		topPadding = desc.topPadding or 11,
		sidePadding = desc.sidePadding or 20,
		bottomPadding = desc.bottomPadding or 24,
		contentOffsetY = desc.contentOffsetY or 0,
		textId = desc.textId,
		choiceId = desc.choiceId,
		tipType = desc.tipType,
		customTip = desc.customTip,
		tipSide = desc.tipSide,
		maxLineCount = desc.maxLineCount or 4,
		restrictScreenPosition = desc.restrict,
		doPopIn = desc.doPopIn,
		doPopOut = desc.doPopOut,
		children = balloonChildren,
		content = desc.content,
		customImage = desc.customImage,
		offset = toPoint(desc.offset),
		onTaskSkip = desc.onTaskSkip,
		bubbleLayer = desc.bubbleLayer,
	}

	local dlg = self:newChild(dialogDef)
	local setupBalloonDialog = self.setupBalloonDialog or Object.setupBalloonDialog
	return setupBalloonDialog(self, dlg, desc)
end

function Object:setupBalloonDialog(dlg, desc)
	desc = desc or {}
	desc.onClick = desc.onClick or function(sender)
		local handled = false
		if (desc.task and desc.task.onMessageClick) then
			handled = handled or desc.task:onMessageClick(sender.name, desc.dialogId, self)
		end
		handled = handled or (self and self:onMessageClick(sender.name, desc.dialogId))
		handled = handled or (level and level.onMessageClick and level:onMessageClick(sender.name, desc.dialogId, self))
		handled = handled or (event and event.onMessageClick and event:onMessageClick(sender.name, desc.dialogId, self))
		if (not handled and scene and scene.cpp_handleCommand) then
			scene:cpp_handleCommand(sender.name, self)
		end
		if (not handled and self and self.hideMessageDialog) then
			self:hideMessageDialog(dlg, false, desc.delayBeforeHide)
		end
		return true
	end

	if (self) then
		self.messageDialogs = self.messageDialogs or {}
		local oldDlg = self.messageDialogs[desc.dialogId]
		if (isTable(oldDlg)) then
			--print("Warning: Removing message with dup ID: "..tostring(oldDlg.dialogId))
			if (self.removeMessageDialog) then
				self:removeMessageDialog(oldDlg)
			end
		end
		self.messageDialogs[desc.dialogId] = dlg
	end

	dlg:traverse(function(_, node) node.onClick = desc.onClick end)
	dlg:onShow()
	if (dlg.name == "") then
		dlg.name = desc.dialogId or "Unnamed Message Dialog"
	end
	if (self and self.calcMessagePosition) then
		self:calcMessagePosition()
	end
	return dlg
end

function Object:getMessageDialogAndId(dlg)
	local theDialog, theDialogId

	if (isTable(dlg)) then
		theDialog = dlg
		if (self.messageDialogs) then
			for k,v in pairs(self.messageDialogs) do
				if (v == theDialog) then
					theDialogId = k
					break
				end
			end
		end
		if (theDialogId == nil) then
			theDialogId = theDialog.dialogId
		end

	elseif (isString(dlg)) then
		theDialogId = dlg
		if (self.messageDialogs) then
			theDialog = self.messageDialogs[dlg]
			if (theDialog == nil) then
				for k,v in pairs(self.messageDialogs) do
					if (v.dialogId == theDialogId) then
						theDialog = v
						break
					end
				end
			end
		end
	end

	return theDialog, theDialogId
end

function Object:findMessageDialog(dlg)
	if (isTable(dlg)) then
		dlg = dlg.dialogId or dlg.textId
	end
	return (self.messageDialogs and dlg and self.messageDialogs[dlg])
end

function Object:removeMessageDialog(dlg, all)
	all = all ~= false
	local theDialog, theDialogId = self:getMessageDialogAndId(dlg)

	if (theDialog and theDialog.removeFromParent) then
		theDialog:removeFromParent()
	end
	if (theDialogId and self.messageDialogs) then
		self.messageDialogs[theDialogId] = nil
		if (all) then
			for k,v in pairs(self.messageDialogs) do
				if (v.dialogId == theDialogId) then
					self.messageDialogs[k] = nil
					if (v.removeFromParent) then
						v:removeFromParent()
					end
				end
			end
		end
	end
end

function Object:hideMessageDialog(dlg, all, duration)
	all = all ~= false
	duration = duration or 0
	local theDialog, theDialogId = self:getMessageDialogAndId(dlg)

	if (theDialog and theDialog.onHide and not theDialog.__deleted) then
		theDialog:onHide(duration)
	end
	if (theDialogId and all and self.messageDialogs) then
		for k,v in pairs(self.messageDialogs) do
			if (v ~= theDialog and v.dialogId == theDialogId and v.onHide and not v.__deleted) then
				v:onHide(duration)
			end
		end
	end
end
