-- ************************** Old Well Station **************************** --
OldWell = deriveLuaClass("OldWell", "Table")

function OldWell:init(desc)
    OldWell.superInit(self, desc)

    self.clicksNumber = desc.clicksNumber or 5
    self.downDuration = desc.downDuration or 2000
    self.pullAnimation = desc.pullAnimation
    self.pullFrameTime = desc.pullFrameTime or 150
    self.fullBucketObject = desc.fullBucketObject or "full_bucket"
    self.showFullBucketDuration = self.showFullBucketDuration or 1000
    self.showFullBucketCustomerAnimation = desc.showFullBucketCustomerAnimation or "sit"
    self.allowDecrease = desc.allowDecrease or false
    self.decreaseTime = desc.decreaseTime or 1000
    self.yOverheadAdjustment = desc.yOverheadAdjustment or -74 -- This value is an adjustment for the overhead position

    self.pullSound = desc.pullSound or nil
    self.delayBetweenSound = desc.delayBetweenSound or 0

    self.splashSound = desc.splashSound or nil

    -- Progress bar parameters
    self.progressBarLocation = desc.progressBarLocation

    -- Set the frame to the last one, where the rope is on top
    self.frame = self.frameCount - 1

    self.STATE_DEFAULT = 1
    self.STATE_GOING_DOWN = 2
    self.STATE_PULLING = 3
    self.STATE_SHOW_FULL_BUCKET = 4
    self.animationState = self.STATE_DEFAULT

    self.totalTime = 0
    self.unclickedTime = 0
    self.actualClicks = 0

    self.overwriteCustomerHighlight = false
    self.oldGroup = nil
    self.playedSoundTime = 0

    self.showingArrowHint = false
    self.showingArrowTime = 0
end

function OldWell:onLevelInitialized()
	OldWell.super.onLevelInitialized(self)
    self.fullBucket = level:getSpriteExt(self.fullBucketObject)
    self.fullBucket.visible = false

    self:createProgressBar()
    self:hideProgressBar(false)
end

function OldWell:onClick()
    if (self.animationState == self.STATE_PULLING) then
        self.unclickedTime = 0
        self.showingArrowTime = 0
        self.actualClicks = self.actualClicks + 1
        self.progressBar:setProgress(self.actualClicks / self.clicksNumber)
        if (self.actualClicks < self.clicksNumber) then
            -- This is used to calculate the frame number depending on the
            -- number of clicks needed to take the bucket
            local frameNum = (self.frameCount-1) * self.actualClicks/self.clicksNumber
            self.frame = math.floor(frameNum)
            --self:playPullAnimation();
            if (self.group and self.pullAnimation) then
                for _,c in ipairs(self.group.customers) do
                    c.frame = (self.actualClicks % c.frameCount)
                end
            end
        else
            self.frame = self.frameCount - 1
            self.animationState = self.STATE_SHOW_FULL_BUCKET
            self.fullBucket.visible = true
            self.totalTime = 0
            self.actualClicks = 0
            for _,c in ipairs(self.group.customers) do
                c:playAnimation(self.showFullBucketCustomerAnimation,1)
            end
        end
        if (self.playedSoundTime <= 0 and self.pullSound) then
            self.playedSoundTime = self.delayBetweenSound
            self:playSample(self.pullSound)
        end
    elseif (self.animationState == self.STATE_SHOW_FULL_BUCKET) then
        -- Do nothing, to prevent the hero moving to the OldWell if the player
        -- clicks too quick on it when pulling the rope.
    else
        self.super.onClick(self)
    end

    if (self.showingArrowHint) then
        level:removeHintArrow(self)
        self.showingArrowHint = false
    end
end

function OldWell:onTick(time)
    self:tick(time)

    if (self.playedSoundTime > 0) then
        self.playedSoundTime = self.playedSoundTime - time
    end
	self.totalTime = self.totalTime + time
    self.unclickedTime = self.unclickedTime + time

    if (self.animationState == self.STATE_DEFAULT) then
        if (self.group and not self.playedAngry) then
            if (self.group.mood <= 1) then
                self.playedAngry = true
                for _,c in ipairs(self.group.customers) do
                    c:playAnimation("angry")
                end
            end
        end
    elseif (self.animationState == self.STATE_GOING_DOWN) then
        if (self.totalTime < self.downDuration) then
            -- Used to calculate which frame of the rope we should show
            -- depending on the remaining duration of the down animation
            local frameNum = (self.frameCount - 1) * (1 - self.totalTime / self.downDuration)
            self.frame = math.floor(frameNum)
        elseif (self.totalTime >= self.downDuration) then
            self.unclickedTime = 0
            self.animationState = self.STATE_PULLING
            self.frame = 0
            self.totalTime = 0
            self.group.state = "PullLever"

            if (self.splashSound) then
                self:playSample(self.splashSound)
            end

            if (OldWell.showHint ~= nil) then
                OldWell:showHint({subject=self, id="clickLever"})
            end
            -- Set the customer animation to frame 0
            -- if (self.group and self.pullAnimation) then
            --     for _,c in ipairs(self.group.customers) do
            --         c.frame = 0
            --     end
            -- end
        end
    elseif (self.animationState == self.STATE_PULLING and self.allowDecrease) then
        if (self.unclickedTime >= self.decreaseTime) then
            self.unclickedTime = 0
            if (self.actualClicks > 0) then
                if (OldWell.showHint ~= nil) then
                    OldWell:showHint({subject=self, id="ropeDown"})
                end
                self.actualClicks = self.actualClicks - 1

                -- Play a sound when the bucket falls to the bottom
                if (self.actualClicks == 0) then
                    if (self.splashSound) then
                        self:playSample(self.splashSound)
                    end
                end

                self.progressBar:setProgress(self.actualClicks / self.clicksNumber)
                local frameNum = (self.frameCount-1) * self.actualClicks/self.clicksNumber
                self.frame = math.floor(frameNum)
                --self.frame = self.actualClicks
                if (self.group and self.pullAnimation) then
                    for _,c in ipairs(self.group.customers) do
                        c.frame = (self.actualClicks % c.frameCount)
                    end
                end
            end
        end
    elseif (self.animationState == self.STATE_SHOW_FULL_BUCKET) then
        if (self.totalTime >= self.showFullBucketDuration) then
            self.animationState = self.STATE_DEFAULT
            self.fullBucket.visible = false
            -- If we have completed the station mechanic, clean it
            -- automatically
            self.autoClean = true
            self.group:onDoneEating()
            self:hideProgressBar(true)
        end
    end
    -- Make the highlight of the customer the same that the station has,
    -- only when the overwriteHighlightOnGameplay is true
    if (self.group and self.overwriteCustomerHighlight) then
        local customers = self.group.customers
        for _,c in ipairs(customers) do
            c.overwriteHighlightOnGameplay = true
            c.highlightCurrentAlpha = self.highlightCurrentAlpha
        end
    else
        self.overwriteCustomerHighlight = false
        if (self.oldGroup) then
            -- Do this to remove this special highlight mechanic when the
            -- customer has left the station (e.g. when going to pay).
            local customers = self.oldGroup.customers
            for _,c in ipairs(customers) do
                c.overwriteHighlightOnGameplay = false
            end
            self.oldGroup = nil
        end
    end

    -- Hint arrow control
    self:HintArrowControl(time)
end

function OldWell:onStartEating()
    --self.group.state = "PullLever"
    self.group.stateTimer.duration = 999999

    self:playPullAnimation();

    self.totalTime = 0
    self.animationState = self.STATE_GOING_DOWN

    -- Set the autoClean to false so if the customer leaves angry, the station
    -- is set to dirty
    self.autoClean = false

    self:restartProgressBar()
    self:showProgressBar(true)
end

function OldWell:playPullAnimation()
    -- Play the pull animation for all the customers (it will only be 1)
    if (self.group and self.pullAnimation) then
        for _,c in ipairs(self.group.customers) do
            c:playAnimation(self.pullAnimation,1,self.pullFrameTime)
        end
    end
end

function OldWell:onTableStateChanged()
    self.super.onTableStateChanged(self)
    if (self.plateState == "empty") then
        self.animationState = self.STATE_DEFAULT
        self.fullBucket.visible = false
        self.frame = self.frameCount - 1
        self.totalTime = 0
        self.unclickedTime = 0
        self.actualClicks = 0
        -- Set the autoClean to false so if the customer leaves angry, the station
        -- is set to dirty
        self.autoClean = false
        self.playedAngry = false
    elseif (self.plateState == "occupied") then
        -- Adjust the y position of the overhead to the height of the customer
        self.playedAngry = false
        self.overwriteCustomerHighlight = true
        self.oldGroup = self.group
        self.group:modifyOverheadPosition({
            y = self.yOverheadAdjustment + self:getChair(1).y
                - self.group.customers[1].height - self.group.overheadObject.height
        })
    elseif (self.plateState == "dirty") then
        -- The table will be dirty only if a customer leaves it angry
        self:hideProgressBar(true)
        if (OldWell.showHint ~= nil) then
            OldWell:showHint({subject=self, id="wellDirty"})
        end
        self.animationState = self.STATE_DEFAULT
        self.fullBucket.visible = false
        self.frame = 0
        self.totalTime = 0
        self.unclickedTime = 0
        self.actualClicks = 0
    end
end


function OldWell:HintArrowControl(time)
    if (self.group) then
        if (self.animationState == self.STATE_PULLING) then
            self.showingArrowTime = self.showingArrowTime + time
            if (not self.showingArrowHint and self.showingArrowTime > 10000) then
                self.showingArrowHint = true
                level:placeHintArrow(self)
            end
        end
    else
        if (self.showingArrowHint) then
            self.showingArrowHint = false
            level:removeHintArrow(self)
        end
    end
end
-- #######################
-- Copied from Event class
function OldWell:createProgressBar()
        self.progressBar = level:add(blueprints["progressbar_pinata"])

		self.progressBar.location = self.progressBarLocation or {x=0,y=0}

        self.progressBar.x = self.x + self.progressBar.location.x
        self.progressBar.y = self.y + self.progressBar.location.y

		self.progressBar.floor = 1000
		self.progressBar:setProgress(0)

        self.progressBar.alpha = 0
        self.progressBar.back.clickable = false

        local oldWellSelf = self
        self.progressBar.back.onClick = function()
            oldWellSelf:onClick()
        end
end

function OldWell:restartProgressBar()
    if (self.progressBar) then
        self.progressBar:setProgress(0)
        self.actualProgress = 0
    end
end

function OldWell:showProgressBar(animate)
    if (self.progressBar) then
        if (animate == true and self.progressBar.alpha < 1) then
            self.progressBar:animate({alpha=1, duration=600})
        elseif (self.progressBar.alpha < 1) then
            self.progressBar.alpha = 1
        end
        self.progressBar.back.clickable = true
    end
end

function OldWell:hideProgressBar(animate)
    if (self.progressBar) then
        if (animate == true and self.progressBar.alpha > 0) then
            self.progressBar:animate({alpha=0, duration=600})
        elseif (self.progressBar.alpha > 0) then
            self.progressBar.alpha = 0
        end
        self.progressBar.back.clickable = false
    end
end
