-- ************************ Painting Paige Station ************************* --

PaintingStation = deriveLuaClass("PaintingStation", "Table")

function PaintingStation:init(desc)
    PaintingStation.superInit(self, desc)

    self.painter = desc.painter
    self.idleAnimation = desc.idleAnimation or "idle"

    self.thinkAnimation = desc.thinkAnimation or "think"
    self.thinkFrameTime = desc.thinkFrameTime or 150

    self.orderPainterAnimation = desc.orderPainterAnimation or "order"
    self.orderPainterFrameTime = desc.orderPainterFrameTime or 150

    self.paintAnimation = desc.paintAnimation or "paint"
    self.paintFrameTime = desc.paintFrameTime or 150
    self.paintDuration = desc.paintDuration or 8000

    self.showAnimation = desc.showAnimation or "show"
    self.showTime = desc.showTime or 1000
    self.showPosition = desc.showPosition

    self.paintingSound = desc.paintingSound

    self.STATE_NOTHING = 0
    self.STATE_ORDERING = 1
    self.STATE_PAINTING = 2
    self.STATE_SHOW = 3

    self.animationState = self.STATE_NOTHING
    self.totalTime = 0
    self.originalDuration = 0
    self.productName = nil
    self.drawSprite = nil
end

function PaintingStation:onLevelInitialized()
    PaintingStation.super.onLevelInitialized(self)
    self.painter = level:getSpriteExt(self.painter)
    if (self.drawSprite == nil) then
        local xPos = self.x + self.showPosition.x
        local yPos = self.y + self.showPosition.y
        self.drawSprite = level:add({
            class = "SpriteExt",
            x = xPos, y = yPos,
            anchor = "center",
            baseAnimation = self.baseAnimation,
            defaultSection = levelDesc.paintingImages.section,
            doHighlight = false,
            floor = 0
        })
        self.drawSprite.visible = false
    end
end

function PaintingStation:onTick(time)
    self:tick(time)
    self.totalTime = self.totalTime + time

    if (self.group) then
       if (self.group.state == "Ordering" and self.animationState == self.STATE_NOTHING) then
            self.animationState = self.STATE_ORDERING
            self.painter:playAnimation(self.orderPainterAnimation, -1, self.orderPainterFrameTime)
            if (PaintingStation.showHint ~= nil) then
                PaintingStation:showHint({subject=self, id="orderProduct"})
            end
       end
    end

    if (self.animationState == self.STATE_PAINTING) then
        if (self.totalTime >= self.originalDuration) then
            self.animationState = self.STATE_SHOW
            self.painter:playAnimation(self.showAnimation,1)
            -- Change the draw picture depending on the given product
            self.drawSprite.baseAnimation = levelDesc.paintingImages[self.productName]
            -- We calculate this floor so the draw will be always over the painter
            local calculatedFloor = self.painter.y - (self.y + self.showPosition.y) + 1
            self.drawSprite.floor = calculatedFloor
            self.drawSprite.visible = true
            -- Stop the painting sound when the artist shows the painting
            self:stopLoopSample()
        end
    elseif (self.animationState == self.STATE_SHOW) then
        if (self.group) then
            if (self.totalTime >= self.group.stateTimer.duration) then
                self.animationState = self.STATE_NOTHING
                self.drawSprite.visible = false
                self.painter:playAnimation(self.idleAnimation,-1)
            end
        else
            self.animationState = self.STATE_NOTHING
            self.drawSprite.visible = false
            self.painter:playAnimation(self.idleAnimation,-1)
        end
    end
end

function PaintingStation:onStartEating()
    self.originalDuration = self.paintDuration
    self.group.stateTimer.duration = self.paintDuration + self.showTime
    self.animationState = self.STATE_PAINTING
    self.totalTime = 0
    self.painter:playAnimation(self.paintAnimation,-1, self.paintFrameTime)

    local splittedProductName = string.split(self.group.order.allDeliveredProductNames[1], "#")
    self.productName = splittedProductName[1]
    -- Play the painting sound
    self:loopSample(self.paintingSound)
end

function PaintingStation:onTableStateChanged()
    self.super.onTableStateChanged(self)
    if (self.plateState == "occupied" and self.group) then
        -- Since the orderAnimation will be the customer seated, we play this
        -- animation also when the customer is on state thinking.
        if (PaintingStation.showHint ~= nil) then
            PaintingStation:showHint({subject=self, id="customerArrived"})
        end
        local customers = self.group.customers
        for _,c in ipairs(customers) do
            c:playAnimation(self.orderAnimation,1)
        end
        self.painter:playAnimation(self.thinkAnimation, -1, self.thinkFrameTime)
    elseif (self.plateState == "empty") then
        self.painter:playAnimation(self.idleAnimation,-1)
        self.drawSprite.visible = false
    end
end
