-- ***************************** Pinata Station **************************** --

PinataStation = deriveLuaClass("PinataStation", "Table")

function PinataStation:init(desc)
    PinataStation.superInit(self,desc)
    self.pinataObject = desc.pinataObject or pinata
    self.pinataCandy = desc.pinataCandy or "pinata_candy"
end

function PinataStation:onLevelInitialized()
    PinataStation.super.onLevelInitialized(self)
    self.pinata = level:getObject(self.pinataObject)
    self.pinata.candy = level:getSpriteExt(self.pinata.pinataCandy)
    self.pinata:hidePinata()
    self.pinata.station = self

    self.eatingState = false
    self.clickOverwrite = false
end

function PinataStation:onStartEating()
    -- We create this new state called PinataBreaking. This way the TableQueue
    -- decreases the mood because it's not one of the states inside the
    -- isMoodDropping function
    self.group.state = "PinataBreaking"
    -- Setting the group duration to 999999 means the customer won't leave the
    -- station because he "finishes" eating
    self.group.stateTimer.duration = 999999

    -- Get the first product which always will be one of the pinata products
    -- and set the frame
    local pinataName = string.split(self.group.order.allDeliveredProductNames[1], "_")
    if (table.getn(pinataName) >= 2) then
        self.pinata:showPinata(self, pinataName[2])
    else
        --print("ERROR: the product delivered does not have a correct pinata name: "..self.group.order.allDeliveredProductNames[1])
    end
    self.eatingState = true
    self.clickOverwrite = true
end

function PinataStation:onTableStateChanged()
    self.super.onTableStateChanged(self)
    -- If state is empty it's probably because the customer has left so hide
    -- the pinata
    if (self.plateState == "empty") then
        self.eatingState = false
        self.clickOverwrite = false
        if (self.pinata) then
            self.pinata:hidePinata()
        end
    elseif (self.plateState == "occupied") then
        self.originalOverheadY = self.group.overheadObject.y
        self.group.overheadObject.y = self.group.customers[1].y + self:getChair(1).y
            - self.group.customers[1].height - self.group.overheadObject.height
    end
end

function PinataStation:doneEating()
    self.group:onDoneEating()
end

function PinataStation:stopClickOverwrite()
    self.eatingState = false
end

function PinataStation:onClick()
    if (self.eatingState == true) then
        self.pinata:onClick()
    elseif (self.clickOverwrite == true) then
        -- Do nothing
    else
        self.super.onClick(self)
    end
end
