-- ************************************ Progress Bar ************************************ --
ProgressBar = deriveLuaClass("ProgressBar", "SpriteExt")

function ProgressBar:init(desc)
	self.progress = 0
	self.maxPercentage = 99

	ProgressBar.superInit(self, desc)

--	self.horizontal = true
	self.horizontal = desc.horizontal ~= false

	if (desc.back) then
		self.back = self:newChild(desc.back)
	end
	if (desc.front) then
		self.front = self:newChild(desc.front)
	end
	if (desc.label) then
		self.label = self:newChild(desc.label)
	end

	self.startClip = self.startClip or 0
	self.endClip = self.endClip or 0

	self:update()
end

function ProgressBar:setProgress(progress)
	self.progress = ((progress <= 0) and 0) or progress
	if (self.label) then
		if (progress >= 1) then
			self.label.text = "100%"
		elseif (progress <= 0) then
			self.label.text = "0%"
		elseif (self.maxPercentage) then
			self.label.text = string.format("%.0f", math.min(self.maxPercentage, progress*100)).."%"
		end
	end
	self:update()
end

function ProgressBar:update()
	local s = self.front
	if (s) then
		if (self.progress >= 1) then
			s:removeClipShape()
		else
			local anchor = s.anchor
			if (self.horizontal) then
				local w = s.width - self.startClip - self.endClip
				s.clipRect = { x=s.x - anchor.x, y=s.y - anchor.y, width=self.startClip + w * self.progress, height=s.height }
			else
				local hh = s.height
				local h = (hh - self.startClip - self.endClip)
				local ph = h * self.progress + self.endClip
				s.clipRect = { x=s.x - anchor.x, y=s.y - anchor.y + hh - ph, width=s.width, height=ph }
			end
		end
	end
end

function ProgressBar:finish()
	self.progress = 1
	if (self.label) then
		self.label.text = "100%"
	end
	self:update()
end