-- ************************************ Reaction Bar ************************************ --
ReactionBar = deriveLuaClass("ReactionBar", "Object")

function ReactionBar:init(desc)
    if (desc.verticalOrientation == nil) then
        self.verticalOrientation = true
    else
        self.verticalOrientation = desc.verticalOrientation
    end
    self.arrowSpeed = desc.arrowSpeed or 5

    -- The divisions are ordered from top to bottom/left to right depending
    -- on the orientation
    self.firstOffset = desc.firstOffset or 0
    self.firstDivision = desc.firstDivision
    self.secondDivision = desc.secondDivision
    self.secondOffset = desc.secondOffset or 0

    self.STATE_STOP = 0
    self.STATE_MOVE_1 = 1
    self.STATE_MOVE_2 = 2
    self.state = self.STATE_STOP
    self.visible = false
    self.clickable = false

    -- Use the width/2 and height/2 because the anchor is center
    self.leftX = self.x - self.width / 2 + self.firstOffset
    self.rightX = self.x + self.width / 2 - self.secondOffset
    self.topY = self.y - self.height / 2 + self.firstOffset
    self.bottomY = self.y + self.height / 2 - self.secondOffset
end

function ReactionBar:setArrow(arrow)
    self.arrow = arrow
    self.arrow.visible = false
end

function ReactionBar:show()
    self.visible = true
    self.arrow.visible = true
end

function ReactionBar:hide()
    self.visible = false
    self.arrow.visible = false
    self.clickable = false
end

function ReactionBar:start(randomPosition)
    -- Select between the state 1 and 2
    self.state = 1
    self:show()
    self.clickable = true
    -- Set a random initial position for the arrow
    if (randomPosition == true) then
        if (self.verticalOrientation) then
            self.arrow.y = math.Random(self.y - self.height / 2, self.y + self.height / 2)
        else
            self.arrow.x = math.Random(self.x - self.width / 2, self.x + self.width / 2)
        end
    else
        if (self.verticalOrientation) then
            self.arrow.y = self.y
        else
            self.arrow.x = self.x
        end
    end
end

function ReactionBar:stop()
    self.state = self.STATE_STOP
    self.clickable = false
end

function ReactionBar:isInsideFirstArea()
    if (self.verticalOrientation) then
        -- VERTICAL: see if the arrow is between the top part of the bar and the
        -- first division.
        return self.arrow.y <= self.y + self.firstDivision - self.height / 2
    else
        -- HORIZONTAL: see if the arrow is between the left part of the bar and
        -- the first division.
        return self.arrow.x <= self.x + self.firstDivision - self.width / 2
    end
end

function ReactionBar:isInsideSecondArea()
    -- BOTH CASES: see if the arrow is between the first and second divisions.
    if (self.verticalOrientation) then
        return (self.arrow.y > self.y + self.firstDivision - self.height / 2)
            and (self.arrow.y <= self.y + self.secondDivision - self.height / 2)
    else
        return (self.arrow.x > self.x + self.firstDivision - self.width / 2)
            and (self.arrow.x <= self.x + self.secondDivision - self.width / 2)
    end
end

function ReactionBar:isInsideThirdArea()
    if (self.verticalOrientation) then
        -- VERTICAL: see if the arrow is between the second division and the
        -- bottom part of the bar.
        return self.arrow.y > self.y + self.firstDivision - self.height / 2
    else
        -- HORIZONTAL: see if the arrow is between the second division and the
        -- rightmost part of the bar.
        return self.arrow.x > self.x + self.firstDivision - self.width / 2
    end
end

function ReactionBar:onMouseDown()
    self.reactionObject:onClick()
end

function ReactionBar:onClick()
    --self.reactionObject:onClick()
end

function ReactionBar:onTick(time)
    self:tick(time)
    -- Make the arrow move up/down or left/right
    if (self.verticalOrientation) then
        self:verticalMovement()
    else
        self:horizontalMovement()
    end
end

function ReactionBar:horizontalMovement()
    -- Arrow horizontal movement
    local finalPosition = 0
    if (self.state == STATE_MOVE_1 or self.state == STATE_MOVE_2) then
        finalPosition = self.arrow.x + self.arrowSpeed
        if (finalPosition > self.rightX) then
            self.state = self.STATE_MOVE_2
            self.arrowSpeed = - self.arrowSpeed
            finalPosition = self.rightX
        elseif (finalPosition < self.leftX) then
            self.state = self.STATE_MOVE_1
            self.arrowSpeed = - self.arrowSpeed
            finalPosition = self.leftX
        end
        self.arrow.x = finalPosition
    end
end

function ReactionBar:verticalMovement()
    -- Arrow vertical movement
    local finalPosition = 0
    if (self.state == self.STATE_MOVE_1 or self.state == self.STATE_MOVE_2) then
        finalPosition = self.arrow.y + self.arrowSpeed
        if (finalPosition > self.bottomY) then
            self.state = self.STATE_MOVE_2
            self.arrowSpeed = - self.arrowSpeed
            finalPosition = self.bottomY
        elseif (finalPosition < self.topY) then
            self.state = self.STATE_MOVE_1
            self.arrowSpeed = - self.arrowSpeed
            finalPosition = self.topY
        end
        self.arrow.y = finalPosition
    end
end
