dofile("script/events/RequestAttentionEvent.lua")

RequestAttentionObjectR3 = deriveLuaClass("RequestAttentionObjectR3", "RequestAttentionObject")

function RequestAttentionObjectR3:init(desc)
  RequestAttentionObjectR3.superInit(self, desc)

  self.leavingPercentage = desc.leavingPercentage or 15
  self.leavePosition = desc.leavePosition or {x = 0, y = 0}
  self.leaveAlertSound = desc.leaveAlertSound or "balloon_pops_up"
  self.leaveAlertSoundVolume = desc.leaveAlertSoundVolume or 1.0

  self.leaveWalkAnim = desc.leaveWalkAnim or "walk"
  self.leaveWalkSpeed = desc.leaveWalkSpeed or 100
  self.animationWhenLeft = desc.animationWhenLeft or "idle"

  self.returnWalkAnim = desc.returnWalkAnim or "walk"
  self.returnWalkSpeed = desc.returnWalkSpeed or 250
  self.stopDecreasingWhenReturning = desc.stopDecreasingWhenReturning ~= false

  self.ignoreBlocking = desc.ignoreBlocking or false

  self.idleAnimation = desc.idleAnimation or "vittorio_disguise_hat_off_confused"
  self.idleCycles = desc.idleCycles or -1
  self.idleFrameTime = desc.idleFrameTime or 150

  self.defaultAnimation = "vittorio_disguise_hat_off_confused"

  self.x = self.x + self.anchor.x
  self.y = self.y + self.anchor.y

  self.leaveSoundObject = {}
  self.leaveSoundObject["sound"] = self.leaveAlertSound
  self.leaveSoundObject["volume"] = self.leaveAlertSoundVolume

  self.startLocation = self.location
  self.failEvent = false
  self.stopped = false
end

function RequestAttentionObjectR3:onLevelInitialized()
  RequestAttentionObjectR3.super.onLevelInitialized(self)
end

function RequestAttentionObjectR3:onTick(time)
  RequestAttentionObjectR3.super.onTick(self, time)

  if (self.currentBarLevel <= (self.leavingPercentage * 0.01) and not self.stopped) then
    if not self.leavingTask then
      self:LeavePosition()
    end
  end
end

function RequestAttentionObjectR3:LeavePosition()
    if (not self.leavingTask) then
        -- If the event has finished, all the other kids leave and so
        -- we don't want to play this sound.
        if (not self.eventFinished) then
            self:playSample(self.leaveSoundObject)
        end

        taskSystem:cancelAllTasksFor(self)
        self.baseAnimation = self.defaultSection
        self.walkSpeed = self.leaveWalkSpeed
        self.leavingTask = self:addTask({
            class = "WalkToTask",
            target = self.leavePosition,
            walkAnimation = self.leaveWalkAnim,
            playDefaultOnFinish = false,
            ignoreBlocking = self.ignoreBlocking,
        })
        self.failTask = self:addTask({
            class = "CallTask",
            func = function()
                self.failEvent = true
                self.clickable = false
                self:playAnimation(self.animationWhenLeft, self.idleCycles, self.idleFrameTime)
            end
        })
    end
end

function RequestAttentionObjectR3:onClick()
    RequestAttentionObjectR3.super.onClick(self)
    if (self.currentBarLevel >= (self.leavingPercentage * 0.01)) then
        if (self.leavingTask) then
            taskSystem:cancelAllTasksFor(self)
            self.leavingTask = nil
            self.enableDecreasing = false
            self:HideBar(true)
            self.clickable = false
            self.walkSpeed = self.returnWalkSpeed
            self.returningTask = self:addTask({
                class = "WalkToTask",
                target = self.startLocation,
                walkAnimation = self.returnWalkAnim,
                playDefaultOnFinish = false,
                ignoreBlocking = self.ignoreBlocking,
            })
            self:addTask({
                class = "CallTask",
                func = function()
                    self.baseAnimation = self.idleAnimation
                    self:playAnimation(self.idleAnimation, self.idleCycles, self.idleFrameTime)

                end
            })
            -- Wait the delay and then activate another random character
            self:addTask({class = "Task", duration = self.barClickDrainDelay})
            self:addTask({class = "CallTask", func =
              function()
                self.parentEventObject.eventObjects[math.Random(1, table.getn(self.parentEventObject.eventObjects))]:ActivateAttention()
              end})
        end
    end
end

function RequestAttentionObjectR3:StopObject(easingEnabled)
    taskSystem:cancelAllTasksFor(self)
    self.clickable = false
    if (self.leavingTask) then
        -- If the character was leaving when we call this function, make it
        -- return to teh initial position, because this function is only called
        -- when the event is won.
        self.walkSpeed = self.returnWalkSpeed
        self.returningTask = self:addTask({
            class = "WalkToTask",
            target = self.startLocation,
            walkAnimation = self.returnWalkAnim,
            playDefaultOnFinish = false,
            ignoreBlocking = self.ignoreBlocking,
        })
        self:addTask({
            class = "CallTask",
            func = function()
                self.baseAnimation = self.idleAnimation
                self:playAnimation(self.idleAnimation, self.idleCycles, self.idleFrameTime)
            end
        })
    elseif (self.returningTask) then
        -- If the character was returning, add again the returning task.
        self.walkSpeed = self.returnWalkSpeed
        self.returningTask = self:addTask({
            class = "WalkToTask",
            target = self.startLocation,
            walkAnimation = self.returnWalkAnim,
            playDefaultOnFinish = false,
            ignoreBlocking = self.ignoreBlocking,
        })
        self:addTask({
            class = "CallTask",
            func = function()
                self.baseAnimation = self.idleAnimation
                self:playAnimation(self.idleAnimation, self.idleCycles, self.idleFrameTime)
            end
        })
    end

    self.leavingTask = nil
    self.stopped = true
    if (self.decreasingBar) then
        self.decreasingBar:cancelAnimations()
        if easingEnabled == true then
            self.decreasingBar:animate({alpha = 0, duration = self.barEasingDuration})
        else
            self.decreasingBar.alpha = 0
        end
    end
end
