dofile("script/events/RequestAttentionEvent.lua")

RequestAttentionObjectR5 = deriveLuaClass("RequestAttentionObjectR5", "RequestAttentionObject")

function RequestAttentionObjectR5:init(desc)
  RequestAttentionObjectR5.superInit(self, desc)

  self.onClickAnimation = {}
  self.onClickAnimation.enabled = desc.onClickAnimationEnabled ~= false
  self.onClickAnimation.name = desc.onClickAnimationName or self.defaultAnimation
  self.onClickAnimation.range = desc.onClickAnimationRange or {x = 1, y = -1}
  self.onClickAnimation.cycles = desc.onClickAnimationCycles or 1
  self.onClickAnimation.frameTime = desc.onClickAnimationFrameTime or -1
  self.onClickAnimation.playback = desc.onClickAnimationPlayback or "Default"
  self.onClickAnimation.frozenFrames = desc.onClickAnimationFrozenFrames or nil

  self.eventAnimations = {}

  local numEditorEventAnimations = 3 -- Add more if needed!
  local indexCounter = 1
  for i = 1, numEditorEventAnimations do
    local eventAnimEnabled = desc["eventAnimation" .. i .. "Enabled"] or false
    if (eventAnimEnabled == true) then
      self.eventAnimations[indexCounter] = {}

      -- Read the event animation parameters from the editor:
      self.eventAnimations[indexCounter].percentage =
        desc["eventAnimation" .. i .. "Percent"] or (100 - (25 * indexCounter))
      self.eventAnimations[indexCounter].name =
        desc["eventAnimation" .. i .. "Name"] or self.defaultAnimation
      self.eventAnimations[indexCounter].range =
        desc["eventAnimation" .. i .. "Range"] or {x = 1, y = -1}
      self.eventAnimations[indexCounter].cycles =
        desc["eventAnimation" .. i .. "Cycles"] or -1
      self.eventAnimations[indexCounter].frameTime =
        desc["eventAnimation" .. i .. "FrameTime"] or -1
      self.eventAnimations[indexCounter].playback =
        desc["eventAnimation" .. i .. "Playback"] or "Default"
      self.eventAnimations[indexCounter].frozenFrames =
        desc["eventAnimation" .. i .. "FrozenFrames"] or nil

      self.eventAnimations[indexCounter].canBePlayed = true
      indexCounter = indexCounter + 1
    end
  end

  self.defaultAnimation = self.baseAnimation

  self.x = self.x + self.anchor.x
  self.y = self.y + self.anchor.y

  self.failEvent = false
  self.stopped = false
end

function RequestAttentionObjectR5:onLevelInitialized()
  RequestAttentionObjectR5.super.onLevelInitialized(self)
end

function RequestAttentionObjectR5:onTick(time)
    RequestAttentionObjectR5.super.onTick(self, time)

    if (not self.stopped) then
        if (self.currentBarLevel <= 0 ) then
            self.failEvent = true
            self.stopped = true
        elseif (self.playIfNotGameplay == true or
            (self.playIfNotGameplay == false and level.isPlaying)) then
            for _, v in ipairs(self.eventAnimations) do
                if (self.currentBarLevel <= v.percentage * 0.01) then
                    if v.canBePlayed == true then
                        if self.currentEventAnim then taskSystem:cancelAllTasksFor(self) end
                        self.baseAnimation = v.name
                        self.currentEventAnim = self:addTask({class = "PlayAnimationTask",
                            animation = v.name,
                            cycles = v.cycles,
                            frameTime = v.frameTime,
                            playbackType = v.playback,
                            startFrame = v.range.x,
                        endFrame = v.range.y,
                        frozenFrames = v.frozenFrames,
                        onFinishedCallback = function()
                            -- Play back the basic idle animation after the event:
                                local defaultAnim = {}
                                defaultAnim.animation = self.basicAnimName
                                defaultAnim.cycles = self.basicAnimCycles
                                defaultAnim.frameTime = self.basicAnimFrameTime
                                self.baseAnimation = self.basicAnimName
                                self:playAnimation(defaultAnim)
                            end,
                        })
                        -- Mark the current event animation as already played:
                        v.canBePlayed = false
                    end
                    break
                end
            end
        end
    end
end

function RequestAttentionObjectR5:onClick()
  RequestAttentionObjectR5.super.onClick(self)

  -- Play the onClick animation if enabled:
  if (self.currentBarLevel >= 0) then
      if (self.onClickAnimation.enabled == true) then
          taskSystem:cancelAllTasksFor(self)
          self:HideBar(true)
          self.baseAnimation = self.onClickAnimation.name
          self:addTask({class = "PlayAnimationTask",
                        animation = self.onClickAnimation.name,
                        cycles = self.onClickAnimation.cycles,
                        frameTime = self.onClickAnimation.frameTime,
                        playbackType = self.onClickAnimation.playback,
                        startFrame = self.onClickAnimation.range.x,
                        endFrame = self.onClickAnimation.range.y,
                        frozenFrames = self.onClickAnimation.frozenFrames,
                        onFinishedCallback = function()
                        -- Play back the basic idle animation after the event:
                            local defaultAnim = {}
                            defaultAnim.animation = self.basicAnimName
                            defaultAnim.cycles = self.basicAnimCycles
                            defaultAnim.frameTime = self.basicAnimFrameTime
                            self.baseAnimation = self.basicAnimName
                            self:playAnimation(defaultAnim)
                            self:addTask({class = "Task", duration = self.barClickDrainDelay})
                            self:addTask({class = "CallTask", func =
                                function()
                                    self.parentEventObject.eventObjects[math.Random(1, table.getn(self.parentEventObject.eventObjects))]:ActivateAttention()
                                end})
                        end,
                        })
        end
  end

  -- Reset already played event animations for the bar to reach them again:
  for _, v in ipairs(self.eventAnimations) do
    if ((v.canBePlayed == false) and
      ((v.percentage * 0.01) <= self.currentBarLevel)) then
        v.canBePlayed = true
    end
  end
end

function RequestAttentionObjectR5:LeavePosition()
    -- Empty but necessary function to avoid a crash from RequestAttentionObject
end
