dofile("script/objects/Object.lua")

-- ************************************ Shelf ************************************ --
Shelf = deriveLuaClass("Shelf", "IngredientButton")

function Shelf:init(desc)
	Shelf.superInit(self, desc)

	self.allowBreak = desc.allowBreak ~= false -- Default true
	self.onlyWhenEmpty = desc.onlyWhenEmpty or false -- Default FALSE
	self.restockAllSimultaneously = desc.restockAllSimultaneously or false -- Defauult FALSE
	self.alwaysCompleteOne = desc.alwaysCompleteOne ~= false -- Default true
	self.onlyRestockOnce = desc.onlyRestockOnce or false -- Default FALSE
	self.clickable = true
end

function Shelf:onLevelInitialized()
	Shelf.super.onLevelInitialized(self)

	self.items = {}
	for i=1,20 do
		local item = self["item"..i]
		if (item) then
			item = level:resolveSprite(item)
			if (item) then
				GH.assert(isNumber(item.stock))
				GH.assert(isNumber(item.maxStock))
				GH.assert(item.usesStock)
				table.push_back(self.items, item)
			end
		end
	end
end

function Shelf:getIngredientNameForLogic()
	return ""
end

function Shelf:hasWorkTodo()
	return self:getLevelsToRestock() > 0
end

function Shelf:allowIngredientWork(actor)
	return true
end

function Shelf:isStockEmpty()
	for _,v in ipairs(self.items) do
		if (v.usesStock and v.stock == 0) then
			return true
		end
	end
	return false
end

function Shelf:getLevelsToRestock()
	local levels = 0
	for _,v in ipairs(self.items) do
		if (v.usesStock and (v.stock == 0 or not self.onlyWhenEmpty)) then
			levels = levels + v.maxStock - v.stock
		end
	end
	return levels
end

function Shelf:getWorkDurationWith(actor, keyword)
	self.levelDuration = Shelf.super.getWorkDurationWith(self, actor, keyword)
	return self.levelDuration * self:getLevelsToRestock()
end

function Shelf:onWorkStart(task)
	--[[ PR --]] --print("Trying to start stocking")
	if (not self:checkStock()) then
		--[[ PR --]] --print("Cannot start: not enough stock MYSELF")
		return false
	end
	self.levelsToRestock = self:getLevelsToRestock()
	if (self.levelsToRestock <= 0) then
		--[[ PR --]] --print("Cannot start: nothing to restock")
		self:doWrongAnimation(task)
		return false
	end
	if (self.workFrameTime == nil or self.workFrameTime < 0) then
		self.workCycles = self.levelsToRestock
	end
	self:defaultOnWorkStart(task)
	self.currentLevel = 1
	--[[ PR --]] --print("Started stocking")
	return true
end

function Shelf:getSmallestStock()
	local smalleststock
	for k,v in ipairs(self.items) do
		if (v.usesStock and
			(v.stock < v.maxStock  or v.allowRestockOverload) and
			(not smalleststock or v.stock < smalleststock.stock))
		then
			smalleststock = v
			self.currentResourceIndex = k
		end
	end
	return smalleststock
end

function Shelf:incRestockLevel(workDuration, currentTime)
	--[[ PR --]] --pr("workDuration: "..ts(workDuration).." currentTime: "..ts(currentTime))
	if (not self:checkStock()) then
		--[[ PR --]] --pr("Stopping stocking: not enough stock MYSELF")
		return false
	end
	while (workDuration * self.currentLevel / self.levelsToRestock <= currentTime) do
		local doneSomething = false
		if (self.restockAllSimultaneously) then
			for k,v in ipairs(self.items) do
				if (v.usesStock and v.stock < v.maxStock) then
					self.currentLevel = self.currentLevel + 1
					if (self:doIncStockOf(v)) then
						doneSomething = true
					end
				end
			end
		else
			local v = self:getSmallestStock()
			if (v) then
				self.currentLevel = self.currentLevel + 1
				if (self:doIncStockOf(v)) then
					doneSomething = true
				end
			else
				-- Added this to make the restocker with worth work properly
				return false
			end
		end
		if (doneSomething) then
			if (self.usesStock) then
				self:decStock(1, true)
				if (not self:checkStock()) then
					--[[ PR --]] --pr("Stopping stocking: not enough stock MYSELF")
					return false
				end
			end
		end
	end
	return true
end

function Shelf:doIncStockOf(v)
	local throwPos = self.throwPos or {x=self.width/2, y=self.height/2}
	local t = v:throwCopy(self.left + throwPos.x, self.top + throwPos.y, level.topLayer, true)
	-- Added the v.worth for the increases so the shelf works as expected.
	t.mod:add({class="CallTask", func=function() v:incDisplayStock(v.worth) end})
	v:incStock(v.worth)
	local challengeManager = level.challengeManager
	if (challengeManager) then
		challengeManager:onRestock(self.name, 1)
	end
	return true
end

function Shelf:onWorkUpdate(task, time)
	if (not self:incRestockLevel(task.timer.duration, task.timer.time)) then
		--[[ PR --]] --pr("Stopping stocking: done")
		return true
	end
	if (not self:checkStock()) then
		--[[ PR --]] --pr("Stopping stocking: not enough stock MYSELF")
		return true
	end
	if (self.allowBreak) then
		if ((taskSystem:hasTaskFor(task.actor) or self.onlyRestockOnce) and (not self.alwaysCompleteOne or self.currentLevel > 1)) then
			--[[ PR --]] --pr("Stopping stocking: other tasks to be done")
			return true
		end
	end
end

function Shelf:onWorkFinish(task)
	--[[ PR --]] --pr("Stopped stocking.")
	self:defaultOnWorkFinish(task, true)
	self:incRestockLevel(task.timer.duration, task.timer.time + 1)
end
