-- Mirar que no s'hagi de fer subclase de gesture per evitar problemes
-- d'ordre d'execució.

SpecialStationDragMobile = deriveLuaClass("SpecialStationDragMobile", "SpecialStationDrag")


function SpecialStationDragMobile:init(desc)
    SpecialStationDragMobile.superInit(self, desc)

    -- Editor parameters
    self.nightBackground = desc.nightBackground or "ss_blue_night"
    self.nightStars = desc.nightStars or "ss_blue_stars"
    self.mobileGlow = desc.mobileGlow or "ss_blue_mobile1_glow"

    self.glowAlpha = desc.glowAlpha or 1

    self.appearDuration = desc.appearDuration or 500
    self.hideDuration = desc.hideDuration or 1000
    self.mobileYDisplacement = desc.mobileYDisplacement or 500

    self.maxAlpha = desc.maxAlpha or 1
	self.minAlpha = desc.minAlpha or 0.3
    self.fadingSpeed = desc.fadingSpeed or 0.002

    self.fadeAmplitude = self.maxAlpha - self.minAlpha
    self.fadeTime = 0
end

function SpecialStationDragMobile:onLevelInitialized()
    SpecialStationDragMobile.super.onLevelInitialized(self)
    local suffix = "_regular"
    if (player:hasUpgrade("shop_room2_moodcribs")) then
        suffix = "_upgrade"
    end
    if (self.nightBackground) then
        self.nightBackgroundObject = level:getSpriteExt(self.nightBackground .. suffix)
    end
    if (self.nightStars) then
        self.nightStarsObject = level:getSpriteExt(self.nightStars)
    end
    if (self.mobileGlow) then
        self.mobileGlowObject = level:getSpriteExt(self.mobileGlow)
        self.originalGlowY = self.mobileGlowObject.y
    end
    if (self.targetObject) then
        self.originalTargetY = self.targetObject.y
    end
end

function SpecialStationDragMobile:start(station)
    SpecialStationDragMobile.super.start(self,station)
    if (self.targetObject) then
        self.targetObject.y = self.originalTargetY
    end
    if (self.mobileGlowObject) then
        self.mobileGlowObject.alpha = 0
        self.mobileGlowObject.y = self.originalGlowY
    end
    self.fadeTime = 0
end

function SpecialStationDragMobile:completeGraphicAnimation()
    SpecialStationDragMobile.super.completeGraphicAnimation(self)

    if (self.nightBackgroundObject) then
        self.nightBackgroundObject.visible = true
        self.nightBackgroundObject.alpha = 0
        self.nightBackgroundObject:animate({alpha=1, duration=self.hideDuration})
    end
    if (self.nightStarsObject) then
        self.nightStarsObject.visible = true
        self.nightStarsObject.alpha = 0
        self.nightStarsObject:animate({alpha=1, duration=self.completeDuration})
    end
    if (self.mobileGlowObject) then
        self.mobileGlowObject.visible = true
        self.mobileGlowObject.alpha = 0
        self.mobileGlowObject:animate({alpha=self.glowAlpha, duration=self.hideDuration})
    end
end

function SpecialStationDragMobile:lastComplete()
    SpecialStationDragMobile.super.lastComplete(self)
    if (self.nightBackgroundObject) then
        self.nightBackgroundObject:animate({alpha=0, duration=self.hideDuration})
    end
    if (self.nightStarsObject) then
        self.nightStarsObject:animate({alpha=0, duration=self.hideDuration})
    end
    if (self.targetObject) then
        -- Set the target object visibility to true and move it upwards
        self.targetObject.visible = true
        self.targetObject:animate({y=self.targetObject.y - self.mobileYDisplacement,
            duration=self.hideDuration})
    end
    if (self.mobileGlowObject) then
        self.mobileGlowObject:animate({y=self.targetObject.y - self.mobileYDisplacement,
            duration=self.hideDuration})
    end
end

function SpecialStationDragMobile:completeAnimationState(time)
    SpecialStationDragMobile.super.completeAnimationState(self, time)
    if (self.nightStarsObject) then
        --self.actualTime
        -- self.nightStarsObject.alpha = self.nightStarsObject.alpha
        --     + math.sin(self.actualTime) * 0.1
    end
end

function SpecialStationDragMobile:lastAnimationState(time)
    SpecialStationDragMobile.super.lastAnimationState(self, time)
    if (self.nightStarsObject) then
        self.fadeTime = self.fadeTime + time
        self.nightStarsObject.alpha = self.maxAlpha
            - math.abs(math.sin(self.fadingSpeed * self.fadeTime))
            * self.fadeAmplitude
    end
end
