-- *************************** SpecialGesture ****************************** --

SpecialStationGesture = deriveLuaClass("SpecialStationGesture", "Object")

function SpecialStationGesture:init(desc)
    SpecialStationGesture.superInit(self,desc)

    -- Editor parameters
    -- Additional sprites
    self.gestureArea = desc.gestureArea or nil
    self.highlightButton = desc.highlightButton or nil

    -- If both strings are empty, this tries to get the object named "", which
    -- can introduce a bug on the Drag gesture, since the arrow of the drag
    -- have no name (so it's "")
    if (self.gestureArea == "") then
        self.gestureArea = nil
    end
    if (self.highlightButton == "") then
        self.highlightButton = nil
    end
    -- Appearing effects
    self.appearAlphaDuration = desc.appearAlphaDuration or 500
    self.appearScale = desc.appearScale or 2

    -- Action label
    self.actionLabelText = desc.actionLabelText
    self.actionLabelPosition = desc.actionLabelPosition
    self.rotationSpeed = 0.05--desc.rotationSpeed or 0.05

    -- Customer animations
    self.customerCompleteAnimation = desc.customerCompleteAnimation
    self.customerCompleteLoop = desc.customerCompleteLoop or 1
    self.customerCompleteFrameTime = desc.customerCompleteFrameTime or 150
    self.customerCompleteMode = desc.customerCompleteMode or "Default"
    self.customerIdleAnimation = desc.customerIdleAnimation
    self.customerIdleLoop = desc.customerIdleLoop or -1
    self.customerIdleFrameTime = desc.customerIdleFrameTime or 150
    self.customerIdleMode = desc.customerIdleMode or "Default"
    self.customerActiveAnimation = desc.customerActiveAnimation
    self.customerActiveLoop = desc.customerActiveLoop or -1
    self.customerActiveFrameTime = desc.customerActiveFrameTime or 150
    self.customerActiveMode = desc.customerActiveMode or "Default"

    -- Worker animations
    self.workerCompleteAnimation = desc.workerCompleteAnimation
    self.workerCompleteLoop = desc.workerCompleteLoop or 1
    self.workerCompleteFrameTime = desc.workerCompleteFrameTime or 150
    self.workerCompleteMode = desc.workerCompleteMode or "Default"
    self.workerIdleAnimation = desc.workerIdleAnimation
    self.workerIdleLoop = desc.workerIdleLoop or 1
    self.workerIdleFrameTime = desc.workerIdleFrameTime or 150
    self.workerIdleMode = desc.workerIdleMode or "Default"
    self.workerActiveAnimation = desc.workerActiveAnimation
    self.workerActiveLoop = desc.workerActiveLoop or 1
    self.workerActiveFrameTime = desc.workerActiveFrameTime or 150
    self.workerActiveMode = desc.workerActiveMode or "Default"

    -- Target object
    self.targetObjectName = desc.targetObjectName
    self.targetDisabledAnimation = desc.targetDisabledAnimation
    self.targetDisabledLoop = desc.targetDisabledLoop or 1
    self.targetDisabledFrameTime = desc.targetDisabledFrameTime or 150
    self.targetDisabledMode = desc.targetDisabledMode or "Default"
    self.targetIdleAnimation = desc.targetIdleAnimation
    self.targetIdleLoop = desc.targetIdleLoop or 1
    self.targetIdleFrameTime = desc.targetIdleFrameTime or 150
    self.targetIdleMode = desc.targetIdleMode or "Default"
    self.targetActiveAnimation = desc.targetActiveAnimation
    self.targetActiveLoop = desc.targetActiveLoop or 1
    self.targetActiveFrameTime = desc.targetActiveFrameTime or 150
    self.targetActiveMode = desc.targetActiveMode or "Default"
    self.targetVisibleOnStop = desc.targetVisibleOnStop ~= false
    self.targetVisibleOnAppearing = desc.targetVisibleOnAppearing ~= false
    self.targetVisibleOnComplete = desc.targetVisibleOnComplete ~= false

    -- Sounds
    self.gestureEnableSound = desc.gestureEnableSound
    self.gestureIdleSound = desc.gestureIdleSound
    self.gestureActiveSound = desc.gestureActiveSound
    self.gestureUnlockedSound = desc.gestureUnlockedSound
    self.completeSound = desc.completeSound

    -- Special effects
    self.gestureIdleParticle = desc.gestureIdleParticle
    self.hintAnimation = desc.hintAnimation
    self.hintAppearanceTime = desc.hintAppearanceTime or 10000
    self.hintAnimationPosition = desc.hintAnimationPosition
    self.moodHeartsGesturePosition = desc.moodHeartsGesturePosition or nil

    self.completeDuration = desc.completeDuration or 1500

    -- Final effects
    self.particleName = desc.particleName
    self.floaterName = desc.floaterName
    self.completeShowTime = desc.completeShowTime or 500
    self.particleDelay = desc.particleDelay or 0

    self.nextMechanic = desc.nextMechanic or nil
    self.nextMechanicMode = desc.nextMechanicMode or "sequential"

    self.interactiveObjectWhenCompleteName = desc.interactiveObjectWhenCompleteName or nil
    self.interactiveObjectWhenComplete = nil

    -- Last animation state parameters
    self.lastDuration = desc.lastDuration or 0
    self.customerLastAnimation = desc.customerLastAnimation
    self.customerLastLoop = desc.customerLastLoop or 1
    self.customerLastFrameTime = desc.customerLastFrameTime or 150
    self.customerLastMode = desc.customerLastMode or "Default"
    self.targetLastAnimation = desc.targetLastAnimation
    self.targetLastLoop = desc.targetLastLoop or 1
    self.targetLastFrameTime = desc.targetLastFrameTime or 150
    self.targetLastMode = desc.targetLastMode or "Default"
    self.workerLastAnimation = desc.workerLastAnimation
    self.workerLastLoop = desc.workerLastLoop or 1
    self.workerLastFrameTime = desc.workerLastFrameTime or 150
    self.workerLastMode = desc.workerLastMode or "Default"
    self.gestureLastSound = desc.gestureLastSound

    -- States
    self.STATE_STOP = 0
    self.STATE_APPEARING = 1
    self.STATE_COMPLETE_ANIMATION = 2
    self.STATE_LAST_ANIMATION = 3

    self.actualState = self.STATE_STOP

    self.nextMechanicObject = nil
    self.completeTime = 0
    self.showingComplete = false
    self.isInStory = false
    self.showHintTime = 0
    self.shouldRemoveWaitHeroTask = false

    -- Set the anchor to center so it's not necessary to do it in the editor.
    self.anchor = "center"

    -- Function to be called when the gesture is completed
    self.onCompleteCallback = nil
end

function SpecialStationGesture:createActionLabel()
    -- Create the action label in a separate function because we want to
    -- add it as a child after the other graphics.
    if (self.actionLabelText) then
        if (not self.actionLabelPosition) then
            self.actionLabelPosition = {x=0, y=0}
        end
        local suffix = ""
        if (isWindows or isWin32 or isOSX) then
            suffix = "_DESKTOP"
        end
        self.actionLabel = self:newChild({
            class="Label", textSize=50, setup=setups.stdbtn_txt,
            textId=self.actionLabelText .. suffix, anchor="center",
            x=self.actionLabelPosition.x, y=self.actionLabelPosition.y
        })
    end
end

function SpecialStationGesture:onLevelInitialized()
    SpecialStationGesture.super.onLevelInitialized(self)
    if (self.gestureArea) then
        self.gestureAreaObject = level:getSpriteExt(self.gestureArea)
    end
    if (self.highlightButton) then
        self.highlightButtonObject = level:getObject(self.highlightButton)
    end
    if (self.targetObjectName) then
        self.targetObject = level:getObject(self.targetObjectName)
    end
    if (self.hintAnimation) then
        self.hintAnimationObject = level:getSpriteExt(self.hintAnimation)
        if (self.hintAnimationObject) then
            self.hintAnimationObject.alpha = 0
        end
    end
    if (self.interactiveObjectWhenCompleteName) then
        self.interactiveObjectWhenComplete = level:getObject(self.interactiveObjectWhenCompleteName)
    end

    -- Added this just in case, to make sure the draggingObject is false
    -- when the level is started.
    level.draggingObject = false
end

function SpecialStationGesture:onClick()
    -- This onClick function is necessary to overwrite the default behaviour of
    -- onClick
end

-- We set the 3 parameters of playAnimation to nil because we don't need their value
function SpecialStationGesture:playIdleAnimations()
    if(self.isInStory == true and self.storyActor and self.storyIdleAnimation) then
      self.storyActor:playAnimation(self.storyIdleAnimation, self.storyIdleLoop,
          self.storyIdleFrameTime, nil, nil, nil, self.storyIdleMode)
    end
    if (self.workerIdleAnimation and self.station) then
        local char = self.station:getWorkCharacter()
        if (char) then
            char:playAnimation(self.workerIdleAnimation, self.workerIdleLoop,
                self.workerIdleFrameTime, nil, nil, nil, self.workerIdleMode)
        end
    end
    if (self.customerIdleAnimation and self.station) then
        self.station:playCustomersAnimation(self.customerIdleAnimation, self.customerIdleLoop,
                                            self.customerIdleFrameTime, self.customerIdleMode)
    end
    if (self.targetObject and self.targetIdleAnimation) then
        self.targetObject.baseAnimation = self.targetIdleAnimation
        self.targetObject:playAnimation(self.targetIdleAnimation, self.targetIdleLoop,
            self.targetIdleFrameTime, nil, nil, nil, self.targetIdleMode)
    end
    if (self.gestureIdleSound) then
        self:loopSample(self.gestureIdleSound)
    end
end

function SpecialStationGesture:playActiveAnimations()
    if(self.isInStory == true and self.storyActor and self.storyActiveAnimation) then
      self.storyActor:playAnimation(self.storyActiveAnimation, self.storyActiveLoop,
          self.storyActiveFrameTime, nil, nil, nil, self.storyActiveMode)
    end
    if (self.workerActiveAnimation and self.station) then
        local char = self.station:getWorkCharacter()
        if (char) then
            char:playAnimation(self.workerActiveAnimation, self.workerActiveLoop,
                self.workerActiveFrameTime, nil, nil, nil, self.workerActiveMode)
        end
    end
    if (self.customerActiveAnimation and self.station) then
        self.station:playCustomersAnimation(self.customerActiveAnimation, self.customerActiveLoop,
                                            self.customerActiveFrameTime, self.customerActiveMode)
    end
    if (self.targetObject and self.targetActiveAnimation) then
        self.targetObject.baseAnimation = self.targetActiveAnimation
        self.targetObject:playAnimation(self.targetActiveAnimation, self.targetActiveLoop,
            self.targetActiveFrameTime, nil, nil, nil, self.targetActiveMode)
    end
    self:stopLoopSample();
    if (self.gestureActiveSound) then
        self:playSample(self.gestureActiveSound)
    end
end

function SpecialStationGesture:playCompleteAnimations()
    -- Play the hero, customer and target animations when the gesture is finished.
    if(self.isInStory == true and self.storyActor and self.storyCompleteAnimation) then
      self.storyActor:playAnimation(self.storyCompleteAnimation, self.storyCompleteLoop,
          self.storyCompleteFrameTime, nil, nil, nil, self.storyCompleteMode)
    end
    if (self.workerCompleteAnimation and self.station) then
        local char = self.station:getWorkCharacter()
        if (char) then
            char:playAnimation(self.workerCompleteAnimation, self.workerCompleteLoop,
                self.workerCompleteFrameTime, nil, nil, nil, self.workerCompleteMode)
        end
    end
    if (self.customerCompleteAnimation and self.station) then
        self.station:playCustomersAnimation(self.customerCompleteAnimation, self.customerCompleteLoop,
                                            self.customerCompleteFrameTime, self.customerCompleteMode)
    end
    if (self.targetObject) then
        self.targetObject.visible = self.targetVisibleOnComplete
        if (self.targetDisabledAnimation) then
            self.targetObject.baseAnimation = self.targetDisabledAnimation
            self.targetObject:playAnimation(self.targetDisabledAnimation, self.targetDisabledLoop,
                self.targetDisabledFrameTime, nil, nil, nil, self.targetDisabledMode)
        end
    end
    self:stopLoopSample();
    if (self.completeSound) then
        self:addTask({class = "Task", duration = 1000})
        self:addTask({class = "CallTask", func = function() self:playSample(self.completeSound) end})
    end

    if(self.interactiveObjectWhenComplete) then
        self.interactiveObjectWhenComplete:gestureComplete()
    end
end

function SpecialStationGesture:start(station)
    self.station = station
    if (self.station.group) then
        -- Set the gesture
        self.station.group.tipTarget = self.actionLabel
    end
    self:reset()
    self:showGraphic(true)
    if (self.station.group) then
        if (self.moodHeartsGesturePosition) then
            self.station.group.moodHeartsGesturePosition = self.moodHeartsGesturePosition
        end
    end
    self.station.pendingMechanics = self.station.pendingMechanics + 1
    if (self.nextMechanicObject and self.nextMechanicMode == "simultaneous") then
        self.nextMechanicObject:start(self.station)
    elseif (self.nextMechanicObject and self.nextMechanicMode == "one_active") then
        self.nextMechanicObject:showInactive(station)
    end

    if (self.gestureEnableSound) then
        self:playSample(self.gestureEnableSound)
    end

    self:playIdleAnimations()
end

function SpecialStationGesture:showInactive(station)
    self.station = station
    self:reset()
    self:showGraphic(true)
    self:setInactiveGraphics()
    self:setState(self.STATE_STOP)
    self.actualTime = 0
    if (self.targetObject) then
        self.targetObject.visible = self.targetVisibleOnStop
    end
    if (self.nextMechanicObject) then
        self.nextMechanicObject:showInactive(self.station)
    end
    if (self.gestureEnableSound) then
        self:playSample(self.gestureEnableSound)
    end

end

function SpecialStationGesture:stopAndHide()
    if (self.actualState ~= self.STATE_STOP
        and self.actualState ~= self.STATE_COMPLETE_ANIMATION) then
        self:hideHintAnimation()
        self:setState(self.STATE_STOP)
        if (self.targetObject) then
            self.targetObject.visible = self.targetVisibleOnStop
        end
        self.actualTime = 0
        self:hideGraphic(true)
        -- Stop the loop sample just in case we were playing something in loop
        self:stopLoopSample()
    end
    if (self.nextMechanicObject) then
        self.nextMechanicObject:stopAndHide()
    end
end

function SpecialStationGesture:goalAchieved()
    -- All taps have been done.
    self:setState(self.STATE_COMPLETE_ANIMATION)
    self.actualTime = 0
    self:completeGraphicAnimation()
    self:playCompleteAnimations()
    self:hideGestureArea(true);
    -- Start or activate the next mechanic
    if(self.isInStory) then
        --self:restoreAllObjects()
        return
    elseif (self.station and self:gesturesNotCompleted() <= 1) then
        -- If there's only 1 mechanic pending, this means it's the last one.
        if (self.station and self.station.group) then
            -- Use the state HidingGesture so the customer does not lose patience.
            self.station.group.state = "HidingGesture"
            -- We should remove the wait hero task only when the complete
            -- function is called. This way the worker also plays the complete
            -- animations.
            self.shouldRemoveWaitHeroTask = true
        end
        self:completionEffects()
    elseif (self.station and self:gesturesNotCompleted() > 1) then
        -- There are still mechanics missing.
        if (self.station.group and self.station.group.resetMoodStep) then
            self.station.group:resetMoodStep()
        end
    end
    -- Call the group completeStep to update the graphical part of the order.
    if (self.station and self.station.group and self.station.group.completeStep) then
        self.station.group:completeStep()
    end
end

function SpecialStationGesture:complete()
    self:setState(self.STATE_STOP)
    if (self.station and self.shouldRemoveWaitHeroTask) then
        self.shouldRemoveWaitHeroTask = false
        self.station:removeWaitHeroTask()
    end
    if (self.targetObject) then
        self.targetObject.visible = self.targetVisibleOnStop
    end
    if(self.isInStory == true) then
      taskSystem:cancelAllTasksFor(self)
    end
    if(self.station) then
        self.station.pendingMechanics = self.station.pendingMechanics - 1
        if (not self.nextMechanicObject or self.nextMechanicMode == "simultaneous") then
            if (self.station.pendingMechanics <= 0) then
                if (self.station) then
                    self.station:doneEating()
                end
            end
        end
    end

    if(self.onCompleteCallback) then
        self:onCompleteCallback()
    end
end

function SpecialStationGesture:completionEffects()
    -- Play the particles and floater when the gesture is finished.
    if (self.floaterName) then
        local floater = floaters[self.floaterName]
        if (floater) then
            floater.x = self.floaterPosition.x
            floater.y = self.floaterPosition.y
            level:createFloater(floater)
        end
    end

    if (self.particleName and level.topLayer and level.topLayer.animate) then
        level.topLayer:animate({delay=self.particleDelay},{
            particle="effects:particles",
            emitter=self.particleName,
            ex=self.floaterPosition.x,
            ey=self.floaterPosition.y
        })
    end
end

function SpecialStationGesture:showGraphic(animate)
    if (animate) then
        if (self.highlightButtonObject) then
            self.highlightButtonObject.alpha = 0
            self.highlightButtonObject:animate({alpha=1, duration=self.appearAlphaDuration})
        end
        if (self.gestureAreaObject) then
            self.gestureAreaObject.alpha = 0
            self.gestureAreaObject:animate({alpha=1, duration=self.appearAlphaDuration})
        end
    else
        if (self.highlightButtonObject) then
            self.highlightButtonObject.alpha = 1
        end
        if (self.gestureAreaObject) then
            self.gestureAreaObject.alpha = 1
        end
    end
    if (self.targetObject) then
        self.targetObject.visible = self.targetVisibleOnAppearing
    end
    if(self.isInStory) then
        if(self.storyActorName)then
          self.storyActor = level:getObject(self.storyActorName)
        end
      self:playIdleAnimations()
    end
end

function SpecialStationGesture:hideGraphic(animate)
    if (animate) then
        self.alpha = 1
        self:animate({alpha=0, duration=250})
        if (self.gestureAreaObject) then
            self.gestureAreaObject:animate({alpha=0, duration=self.appearAlphaDuration})
        end
    else
        self.alpha = 0
        if (self.gestureAreaObject) then
            self.gestureAreaObject.alpha = 0
        end
    end
    self:hideHighlight(animate)
end


function SpecialStationGesture:showHighlight(animate)
    if (animate) then
        if (self.highlightButtonObject) then
            self.highlightButtonObject:animate({alpha=1, duration=self.appearAlphaDuration})
        end
    else
        if (self.highlightButtonObject) then
            self.highlightButtonObject.alpha = 1
        end
    end
end

function SpecialStationGesture:hideHighlight(animate)
    if (animate) then
        if (self.highlightButtonObject) then
            self.highlightButtonObject:animate({alpha=0, duration=self.appearAlphaDuration})
        end
    else
        if (self.highlightButtonObject) then
            self.highlightButtonObject.alpha = 0
        end
    end
end

function SpecialStationGesture:hideGestureArea(animate)
    if (animate) then
        if (self.gestureAreaObject) then
            self.gestureAreaObject:animate({alpha=0, duration=self.appearAlphaDuration})
        end
    else
        if (self.gestureAreaObject) then
            self.gestureAreaObject.alpha = 0
        end
    end

end

function SpecialStationGesture:playShowGraphicAnimation()
    -- Animate the graphics with an alpha and a scale down.
    self.alpha = 0
    self.scaleX = self.appearScale
    self.scaleY = self.appearScale
    self:animate({alpha=1, duration=self.appearAlphaDuration})
    self:animate({scaleX=1, duration=self.appearAlphaDuration})
    self:animate({scaleY=1, duration=self.appearAlphaDuration})
    self:setState(self.STATE_APPEARING)
end

function SpecialStationGesture:controlCompleteGraphicAnimation(time)
    if (self.showingComplete) then
        self.completeTime = self.completeTime + time
        if (self.completeTime >= self.completeShowTime) then
            self:hideGraphic(true)
            self.showingComplete = false
            -- Start the next gesture only after we have finished the completeanimations
            if (self.station) then
              self.station.completionGestures[self.completionIndex] = self.station.completionGestures[self.completionIndex] -1
              if (self.station.completionGestures[self.completionIndex] <= 0) then
                  self:startNextSequential()
              end
            end
        end
    end
end

function SpecialStationGesture:startNextSequential()
    local foundSequential = false
    local actual = self
    while (not foundSequential and actual) do
        if (actual.nextMechanicObject and actual.nextMechanicMode == "sequential") then
            foundSequential = true
            actual.nextMechanicObject:start(self.station)
        elseif (actual.nextMechanicObject and actual.nextMechanicMode == "one_active") then
            foundSequential = true
            actual.nextMechanicObject:startFromInactive(self.station)
        end
        actual = actual.nextMechanicObject
    end
end

function SpecialStationGesture:restoreAllObjects()
	local objs = level:getObjects()
	for _,v in ipairs(objs) do
    if (v ~= nil) then
			   v.clickable = v.old_clickable
    end
	end

	local tray = level:getTray(hero)
	if (tray) then
		tray.clickable = tray.old_clickable
	end
end

function SpecialStationGesture:disableAllObjects()
	local objs = level:getObjects()
	for _,v in ipairs(objs) do
			if (v.old_clickable == nil) then
				v.old_clickable = v.clickable == true or v.additionalClick1 ~= nil
                 or v.onlyClickable == true
			end
			if(v.name ~= self.name) then
				v.clickable = false
			end
	end
  local tray = level:getTray(hero)
	if (tray) then
		if (tray.old_clickable == nil) then
			tray.old_clickable = tray.old_clickable or (tray.clickable == true)
		end
		tray.clickable = false
	end
end

function SpecialStationGesture:completeGraphicAnimation()
    self.showingComplete  = true
    self.completeTime = 0
    -- Implement the graphical part on children gestures.
end

function SpecialStationGesture:modifyPropertiesfromProduct(properties)
    -- Implement on children gestures.
end

function SpecialStationGesture:resetProperties()
    -- Implement on children gestures.
end

function SpecialStationGesture:showHintAnimation()
    if (self.hintAnimationObject) then
        if (not self.hintAnimationPosition) then
            self.hintAnimationPosition = {x=self.x, y=self.y}
        end
        self.hintAnimationObject.x = self.hintAnimationPosition.x
        self.hintAnimationObject.y = self.hintAnimationPosition.y
        self.hintAnimationObject:playAnimation(self.hintAnimationObject.baseAnimation, -1)
        self.hintAnimationObject:animate({alpha=1, duration=self.appearAlphaDuration})
    end
end

function SpecialStationGesture:hideHintAnimation()
    self.showHintTime = 0
    if (self.hintAnimationObject) then
        self.hintAnimationObject:animate({alpha=0, duration=self.appearAlphaDuration})
    end
end

function SpecialStationGesture:hintControl(time)
    if (self.showHintTime ~= -1) then
        self.showHintTime = self.showHintTime + time
        if (self.showHintTime >= self.hintAppearanceTime) then
            self:showHintAnimation()
            self:showHighlight(true);
            -- We set the showHintTime to -1 so it does not show again the hint
            -- Until we manually set the self.showHintTime to 0
            self.showHintTime = -1
        end
    end
end

-- This function calculates the number simultaneous groups needed to be
-- completed. For example, if we have 2 simultaneous drags, 1 sequential range
-- and 2 simultaneous drags, this will return {2,1,2}.
function SpecialStationGesture:calculateCompletionTable(completionTable, index)
    completionTable[index] = 1
    local foundSequential = false
    local actualMechanic = self
    while not foundSequential and actualMechanic do
        actualMechanic.completionIndex = index
        if (actualMechanic.nextMechanicMode == "sequential") then
            foundSequential = true
            if (actualMechanic.nextMechanicObject) then
                completionTable = actualMechanic.nextMechanicObject:calculateCompletionTable(completionTable, index+1)
            end
        elseif (actualMechanic.nextMechanicMode == "simultaneous") then
            completionTable[index] = completionTable[index] + 1
        end
        actualMechanic = actualMechanic.nextMechanicObject
    end
    return completionTable
end

function SpecialStationGesture:gesturesNotCompleted()
    local result = 0
    if (self.station) then
        for i=1,table.getn(self.station.completionGestures) do
            result = result+self.station.completionGestures[i]
        end
    end
    return result
end

function SpecialStationGesture:onMouseDown()
    if (not self.isInStory) then
        level.draggingObject = true
    end
    if (self.actionLabel) then
        self.actionLabel.scaleX = 0.8
        self.actionLabel.scaleY = 0.8
    end
end

function SpecialStationGesture:onMouseUp()
    level.draggingObject = false
    if (self.actionLabel) then
        self.actionLabel.scaleX = 1
        self.actionLabel.scaleY = 1
    end
end

function SpecialStationGesture:onMouseOut()
    if (self.actionLabel) then
        self.actionLabel.scaleX = 1
        self.actionLabel.scaleY = 1
    end
end

function SpecialStationGesture:completeAnimationState(time)
    self.actualTime = self.actualTime + time
    if (self.actualTime >= self.completeDuration) then
        if (self.lastDuration <= 0) then
            self:complete()
        else
            self:setState(self.STATE_LAST_ANIMATION)
            self.actualTime = 0
            self:playLastAnimations()
        end
    end
end

function SpecialStationGesture:lastAnimationState(time)
    self.actualTime = self.actualTime + time
    if (self.actualTime >= self.lastDuration) then
        self:lastComplete()
    end
end

function SpecialStationGesture:lastComplete()
    self:setState(self.STATE_STOP)
    if (self.station and self.shouldRemoveWaitHeroTask) then
        self.shouldRemoveWaitHeroTask = false
        self.station:removeWaitHeroTask()
    end
    if (self.targetObject) then
        self.targetObject.visible = self.targetVisibleOnStop
    end
    if(self.isInStory == true) then
      taskSystem:cancelAllTasksFor(self)
    end
    if(self.station) then
        self.station.pendingMechanics = self.station.pendingMechanics - 1
        if (not self.nextMechanicObject or self.nextMechanicMode == "simultaneous") then
            if (self.station.pendingMechanics <= 0) then
                if (self.station) then
                    self.station:doneEating()
                end
                --self:completionEffects()
            end
        end
    end
end

function SpecialStationGesture:playLastAnimations()
    if (self.workerLastAnimation and self.station) then
        local char = self.station:getWorkCharacter()
        if (char) then
            char:playAnimation(self.workerLastAnimation, self.workerLastLoop,
                self.workerLastFrameTime, nil, nil, nil, self.workerLastMode)
        end
    end
    if (self.customerLastAnimation and self.station) then
        self.station:playCustomersAnimation(self.customerLastAnimation, self.customerLastLoop,
                                            self.customerLastFrameTime, self.customerLastMode)
    end
    if (self.targetObject and self.targetLastAnimation) then
        self.targetObject.baseAnimation = self.targetLastAnimation
        self.targetObject:playAnimation(self.targetLastAnimation, self.targetLastLoop,
            self.targetLastFrameTime, nil, nil, nil, self.targetLastMode)
    end
    if (self.gestureLastSound) then
        self:playSample(self.gestureLastSound)
    end
end

function SpecialStationGesture:rotateSprite(sprite, time)
    if (sprite) then
        sprite.rotation = sprite.rotation + time * self.rotationSpeed
        if (sprite.rotation > 360) then
            sprite.rotation = sprite.rotation - 360
        end
    end
end

function SpecialStationGesture:setState(state)
    local previousState = self.actualState
    self.actualState = state
    if (self.station and previousState == self.STATE_APPEARING) then
        -- Check that the customer has this function just in case it's
        -- not a GestureCustomerGroup
        if (self.station.group.doGestureTip) then
            self.station.group:doGestureTip()
        else
            print("### ERROR!! A customer that goes to the special station doesn't have the doGestureTip function. ###")
        end
    end
end
