-- ******************************* SpecialStationTable ********************************** --

--SpecialStationTable = deriveLuaClass("SpecialStationTable", "Table")
SpecialStationTable = getClass("SpecialStationTable")

function SpecialStationTable:init(desc)
    SpecialStationTable.superInit(self,desc)

    -- Editor parameters
    self.mechanicName = desc.mechanicName
    self.adaptHeartsPosition = desc.adaptHeartsPosition or false
    self.waitAtStation = desc.waitAtStation or false
    self.waitAnimation = desc.waitAnimation or ""
    self.customAnimation = ""
    self.clickableWhileGestureActive = desc.clickableWhileGestureActive or false

    self.eatingState = false
    self.pendingMechanics = 0

    self.oldClickable = self.clickable
end

function SpecialStationTable:onLevelInitialized()
    SpecialStationTable.super.onLevelInitialized(self)
	if(self.mechanicName) then
		self.mechanic = level:getObject(self.name .. "-" .. self.mechanicName)
	end
end

function SpecialStationTable:onStartEating()
    -- Setting the group duration to 999999 means the customer won't leave the
    -- station because he "finishes" eating
	
    if (not self.clickableWhileGestureActive) then
        self.oldClickable = self.clickable
        if (not levelDesc.clickTableToPay) then
          self.clickable = false
        end
    end

    if(self.customAnimation ~= "") then
        for _,c in ipairs(self.group.customers) do
            c:playAnimation(self.customAnimation,-1,self.eatFrameTime)
        end
end

	local count = 0
  for _ in pairs(self.group.order.allDeliveredProductNames) do count = count + 1 end

		-- Set the mechanic depending on the first product
	self:SetMechanicFromProduct(self.group.order.allDeliveredProductNames[count])
	
	if(self.mechanic) then
		self.group.stateTimer.duration = 999999
		self:CalculateNumberOfMechanics()

		self.eatingState = true

		self.completionGestures = self.mechanic:calculateCompletionTable({},1)
		self.mechanic:start(self)

		-- Play the consume animation of the gesture.
		if (self.mechanic.consumeAnimation) then
			self:playCustomersAnimation(self.mechanic.consumeAnimation, self.mechanic.consumeAnimationCycles)
		end
		
	else
	
		self.eatingState = true
	end

end

function SpecialStationTable:onTableStateChanged()
    SpecialStationTable.super.onTableStateChanged(self)
    if (self.plateState == "empty" or self.plateState == "dirty") then
        -- If state is empty it's because the customer has left
        self.clickable = self.oldClickable
        self.eatingState = false
        if (self.mechanic) then
            self.mechanic:stopAndHide()
        end
        self.pendingMechanics = 0
    elseif (self.plateState == "occupied") then
        local numMechanics = self:CalculateNumberOfMechanics()
        if (self.group and self.group.setNumberOfSteps) then
            self.group:setNumberOfSteps(numMechanics)
        end
    end
end

function SpecialStationTable:doneEating()
    self.eatingState = false
    if (self.group) then
        self.group:onDoneEating()
    end
end

function SpecialStationTable:onClick()
    -- If we are eating, overwrite the normal onClick so the hero does not
    -- go there when clicking on the customer/station
    if (levelDesc.clickTableToPay) then
      SpecialStationTable.super.onClick(self)
    else
    --  if (self.eatingState == false) then
          SpecialStationTable.super.onClick(self)
     -- end
    end
end

function SpecialStationTable:playCustomersAnimation(animationName, animationCycles, frameTime, playback)
    if (not frameTime) then
        playback = 150
    end
    if (not playback) then
        playback = "Default"
    end
    if (self.group and self.group.customers) then
        for _,c in ipairs(self.group.customers) do
            c:playAnimation(animationName,animationCycles, frameTime,
                nil, nil, nil, playback)
        end
    end
end

function SpecialStationTable:SetMechanicFromProduct(productName)
local gesture;
--print(levelDesc.productSettings[productName]);
if(levelDesc.productSettings[productName]) then
    gesture = levelDesc.productSettings[productName].gesture
	end
	self.mechanic = nill
    if (gesture) then
        self.mechanic = level:getObject(self.name .. "-" .. gesture)
        -- Reset the properties before modifying them
		
        if (self.mechanic) then
		--GH.assert(dddd,self.name .. "-" .. gesture)
            self.mechanic:resetProperties()
            local properties = levelDesc.productSettings[productName].gestureProps
            if (properties) then
                self.mechanic:modifyPropertiesfromProduct(properties)
            end
        end
    end
  --  if (not gesture or not self.mechanic) then
        -- If there is no gesture or couldn't find the object,
        -- use the default mechanic
--        self.mechanic = level:getObject(self.name .. "-" .. self.mechanicName)
--    end
end

function SpecialStationTable:CalculateNumberOfMechanics()
    if (self.mechanic) then
        return self:CalculateNumberOfMechanics_REC(self.mechanic)
    end
    return 0
end

function SpecialStationTable:CalculateNumberOfMechanics_REC(element)
    if (element.nextMechanic) then
        local nextMec = level:getObject(element.nextMechanic)
        if (nextMec) then
            return 1 + self:CalculateNumberOfMechanics_REC(nextMec)
        end
    end
    return 1
end
