-- ******************************* SpecialStationTapTox ********************************** --

SpecialStationTapTox = deriveLuaClass("SpecialStationTapTox", "SpecialStationGesture")

function SpecialStationTapTox:init(desc)
    SpecialStationTapTox.superInit(self, desc)

    -- Editor parameters
    self.totalTapSteps = desc.totalTapSteps or 1

    -- Delay between the customer and target tap animations
    self.tapDuration = desc.tapDuration or 500

    -- Original data for resetting
    self.originalTotalTapSteps = self.totalTapSteps

    -- States (states 1 to 10 reserved for the generic gesture).
    self.STATE_CLICKABLE = 11
    self.actualState = self.STATE_STOP

    -- Setting for playing idle after tap animation
    self.playIdleAfter = desc.playIdleAfter or 0
    self.playedTap = true

    -- Internal parameters
    self.actualTime = 0
    self.actualSteps = 0
    self.graphicsCreated = false
    self.lastTapAnimation = self.tapDuration
    if (self.nextMechanic == "") then
        self.nextMechanic = nil
    end

    -- Set it's alpha to 0 so it's hidden during the menu chooser screen
    self.alpha = 0

    self:createGestureGraphic(self.totalTapSteps)
    self:createActionLabel()
end

function SpecialStationTapTox:onLevelInitialized()
    SpecialStationTapTox.super.onLevelInitialized(self)
    if (self.animationTarget) then
        self.animationTargetObject =  level:getObject(self.animationTarget)
    end
    self:hideGraphic(false)

    if (self.nextMechanic) then
        self.nextMechanicObject = level:getObject(self.nextMechanic)
    end

    if (self.highlightButtonObject) then
        self.highlightButtonObject.clickable = true
        local selfObject = self
        self.highlightButtonObject.onClick = function()
            selfObject:onClick()
        end
    end
end

function SpecialStationTapTox:onTick(time)
    self:tick(time)
    self.lastTapAnimation = self.lastTapAnimation + time

    if (self.actualState == self.STATE_APPEARING) then
        self.actualTime = self.actualTime + time
        if (self.actualTime >= self.appearAlphaDuration) then
            self:setState(self.STATE_CLICKABLE)
            self.actualTime = 0
            self.showHintTime = 0
        end
    elseif (self.actualState == self.STATE_CLICKABLE) then
        if (self.playIdleAfter > 0 and not self.playedTap) then
            if (self.lastTapAnimation > self.playIdleAfter) then
                self.playedTap = true
                self:playIdleAnimations()
            end
        end
    elseif (self.actualState == self.STATE_COMPLETE_ANIMATION) then
        self:completeAnimationState(time)
    elseif (self.actualState == self.STATE_LAST_ANIMATION) then
        self:lastAnimationState(time)
    end

    -- Hint animation show
    if (self.actualState == self.STATE_CLICKABLE) then
        self:hintControl(time)
    end

    self:controlCompleteGraphicAnimation(time)
  --  self:rotateSprite(self.borderSprite, time)
end

function SpecialStationTapTox:createGestureGraphic(totalTapSteps)
    self.clickable = true
    self.fillSprite = self:newChild({
        class="Sprite", image="tap_tox_fill:special_stations",
        anchor="center", scaleX=0, scaleY=0,
    })
    self.borderSprite = self:newChild({
        class="Sprite", image="tap_tox_border:special_stations", anchor="center"
    })
end

function SpecialStationTapTox:onClick()
    if (self.actualState == self.STATE_CLICKABLE) then
        self:hideHintAnimation()
        self:hideHighlight(false)
        self.actualSteps = self.actualSteps + 1
        if (self.actualSteps < self.totalTapSteps) then
            -- Scale the fill graphic
            self.fillSprite.scaleX = self.actualSteps / self.totalTapSteps
            self.fillSprite.scaleY = self.actualSteps / self.totalTapSteps
            self:playTapAnimations()
            self.playedTap = false
        else
            self.fillSprite.scaleX = 1
            self.fillSprite.scaleY = 1
            self:goalAchieved()
        end
    end
end

function SpecialStationTapTox:showGraphic(animate)
    SpecialStationTapTox.super.showGraphic(self,animate)
    if (animate) then
        self:playShowGraphicAnimation()
    else
        self.scaleX = 1
        self.scaleY = 1
        self.alpha = 1
        self:setState(self.STATE_CLICKABLE)
    end
    self.actualTime = 0
end

function SpecialStationTapTox:setActiveGraphics()
    -- Sets the graphics to active. It's used when for the one_active mode of
    -- chaining gestures.
    -- self.tapLabel:animate({alpha=1, duration=self.appearAlphaDuration})
end

function SpecialStationTapTox:setInactiveGraphics()
    -- Sets the graphics to inactive. It's used when for the one_active mode of
    -- chaining gestures.
    -- self.tapLabel.alpha = 0
end

function SpecialStationTapTox:playTapAnimations()
    -- Play the customer and target animations when a tap is done.
    if (self.lastTapAnimation >= self.tapDuration) then
        self.lastTapAnimation = 0
        self:playActiveAnimations()
    end
end

function SpecialStationTapTox:startFromInactive(station)
    self:setActiveGraphics()
    self:setState(self.STATE_CLICKABLE)
end

function SpecialStationTapTox:reset()
    self.actualSteps = 0
    self.scaleX = 1
    self.scaleY = 1
    self.fillSprite.scaleX = 0
    self.fillSprite.scaleY = 0
    self.doHighlight = true
    self.clickable = true
end

function SpecialStationTapTox:modifyPropertiesfromProduct(properties)
    if (properties.totalTapSteps) then
        self.totalTapSteps = properties.totalTapSteps
    end
end

function SpecialStationTapTox:resetProperties()
    self.totalTapSteps = self.originalTotalTapSteps
end

function SpecialStationTapTox:completeGraphicAnimation()
    SpecialStationTapTox.super.completeGraphicAnimation(self)
    self.doHighlight = false
    self.clickable = false
end
