-- ******************************* SpecialStationWait ********************************** --

SpecialStationWait = deriveLuaClass("SpecialStationWait", "SpecialStationGesture")

function SpecialStationWait:init(desc)
    SpecialStationWait.superInit(self,desc)

    -- Editor parameters
    self.waitDuration = desc.waitDuration or 4000
    self.enableMoodDrop = desc.enableMoodDrop or false

    -- States (states 1 to 10 reserved for the generic gesture).
    self.STATE_ACTIVE = 11
    self.actualState = self.STATE_STOP

    -- Internal parameters
    self.actualTime = 0
    self.actualSteps = 0

    -- Set it's alpha to 0 so it's hidden during the menu chooser screen
    self.alpha = 0

    self.previousGroupState = ""
end

function SpecialStationWait:onLevelInitialized()
    SpecialStationWait.super.onLevelInitialized(self)
    if (self.animationTarget) then
        self.animationTargetObject =  level:getObject(self.animationTarget)
    end

    if (self.nextMechanic) then
        self.nextMechanicObject = level:getObject(self.nextMechanic)
    end
end

function SpecialStationWait:onClick()
    -- This onClick function is necessary to overwrite the default behaviour of
    -- onClick
end

function SpecialStationWait:onTick(time)
    self:tick(time)

    if (self.actualState == self.STATE_APPEARING) then
        self:setState(self.STATE_ACTIVE)
    elseif (self.actualState == self.STATE_ACTIVE) then
        if (self.station and self.station.group and not self.enableMoodDrop) then
            -- Disable the mood drop if we don't want the customer to lose
            -- mood while the gesture is active.
            self.station.group.moodDropEnabled = self.enableMoodDrop
        end
        self.actualTime = self.actualTime + time
        if (self.actualTime >= self.waitDuration) then
            self.station.group.moodDropEnabled = true
            self:goalAchieved()
        end
    elseif (self.actualState == self.STATE_COMPLETE_ANIMATION) then
        self:completeAnimationState(time)
    elseif (self.actualState == self.STATE_LAST_ANIMATION) then
        self:lastAnimationState(time)
    end

    self:controlCompleteGraphicAnimation(time)
end

function SpecialStationWait:showGraphic(animate)
    SpecialStationWait.super.showGraphic(self,animate)
    self:setState(self.STATE_ACTIVE)
    self.actualTime = 0
end

function SpecialStationWait:reset()

end
