-- ************************************ Status Object ************************************ --
StatusObject = deriveLuaClass("StatusObject", "Object")

StatusObject.statusTables = {
	reactToProductState = { "prepare", "ready", "decay", "inactive", },
	reactToStockState = { "stockEmpty", "stocked", "maxStock" },
	reactToWorkState = { "workstart", "workfinish" },
	reactToTableState = { "tableEmpty", "tableOccupied", "tableBusy", "tableDirty" },
}

function StatusObject:init(desc)
	StatusObject.superInit(self, desc)

	self.useBaseFromIngredient = desc.useBaseFromIngredient
	self.startState = desc.startState or "inactive"
	self.curStatus = desc.startState

--	print("self.baseAnimation of \""..self.name.."\": "..self.baseAnimation.." ("..desc.baseAnimation..")")
	self.start_baseAnimation = desc.baseAnimation
	self.statusTable = {}
	for k,t in pairs(self.statusTables) do
		local hasStartState = false
		for _,v in ipairs(t) do
			self:statusInit(v)
			if (v == self.startState) then
				hasStartState = true
			end
		end
		if (hasStartState or self[k]) then
			for _,v in ipairs(t) do
				self.statusTable[v] = true
			end
		end
	end

	self:statusUpdate(self.startState, true, nil)
end

function StatusObject:shouldShowState(state)
	return self.statusTable[state]
end

function StatusObject:statusInit(status)
	self[status.."Visible"] = self[status.."Visible"] ~= false
	self[status.."Alpha"] = self[status.."Alpha"] or 1
	self[status.."FadeTime"] = self[status.."FadeTime"] or 120
	self[status.."Animation"] = self[status.."Animation"] or ""
	self[status.."BaseFromIngredient"] = self[status.."BaseFromIngredient"] or false
end

function StatusObject:statusUpdate(status, init, ing, duration)
--if (not init) then
--print(self.name)
--print(status)
--print(serialize("self.statusTable", self.statusTable))
--end

	if (init or self:shouldShowState(status)) then
		self.curStatus = status
		self:cancelAnimations()
		self.visible = self[status.."Visible"]
		if (init) then
			self.alpha = self[status.."Alpha"]
		else
		--print("{alpha="..tostring(self[status.."Alpha"])..", duration="..tostring(self[status.."FadeTime"]).."}")
			self:animate({alpha=self[status.."Alpha"], duration=self[status.."FadeTime"]})
		end
		if (self[status.."BaseFromIngredient"]) then
			GH.assert(self.baseAnimationPrefix, "no baseAnimation Prefix set")
			self.baseAnimation = self.baseAnimationPrefix..ing.ingredientName
	--	else
		--	self.baseAnimation = self.start_baseAnimation
		end
		if (self[status.."Frame"] and self[status.."Frame"] >= 0) then
			--print(status.."Frame: "..tostring(self[status.."Frame"]))
			self.frame = self[status.."Frame"]
		end
		if (self[status.."DoAnimation"]) then
-- print(status.."DoAnimation: "..tostring(self[status.."DoAnimation"]))
-- print(status.."Animation: "..tostring(self[status.."Animation"]))
-- print(status.."Cycles: "..tostring(self[status.."Cycles"]))
-- print(status.."FrameTime: "..tostring(self[status.."FrameTime"]))
			self:playAnimation(self[status.."Animation"], self[status.."Cycles"], self[status.."FrameTime"], self[status.."WhenFinished"])
			if (duration and duration > 0) then
				self:matchFrameTimeTo(duration)
-- print("duration: "..duration)
			end
		end
		if (not init) then
			local shouldStopLoop = true
			local snd = self[status.."Sound"]
			local obj = ing or self
			if (snd) then
				if (self[status.."LoopSound"]) then
					shouldStopLoop = false
					obj:loopSample(snd)
				else
					obj:playSample(snd)
				end
			end
			if (shouldStopLoop) then
				obj:stopLoopSample()
			end
		end

		if (ing) then
			for i=1,5 do
				if (ing["additionalClick"..i] == self) then
					self.clickable = ing and ing.clickable and ing.visible and ing.alpha > 0
					break
				end
			end
		end
	end
end

function StatusObject:status_onStartPreparing(ing)
-- print("StatusObject:status_onStartPreparing "..self.name.." (from "..ing.name..")")
	self:statusUpdate("prepare", nil, ing)
end

function StatusObject:status_onBecomeAvailable(ing)
-- print("StatusObject:status_onBecomeAvailable "..self.name.." (from "..ing.name..")")
	self:statusUpdate("ready", nil, ing)
end

function StatusObject:status_onDecay(ing)
-- print("StatusObject:status_onDecay "..self.name.." (from "..ing.name..")")
	self:statusUpdate("decay", nil, ing)
end

function StatusObject:status_onDeactivate(ing)
-- print("StatusObject:status_onDeactivate "..self.name.." (from "..ing.name..")")
	self:statusUpdate("inactive", nil, ing)
end

function StatusObject:status_onDisplayStockChange(obj, ing)
	if (obj.usesStock) then
		if (not obj:checkStock()) then
			self:statusUpdate("stockEmpty", nil, ing)
		elseif (obj.stock == obj.maxStock) then
			self:statusUpdate("maxStock", nil, ing)
		else
			self:statusUpdate("stocked", nil, ing)
		end
	end
end

function StatusObject:status_onWorkStart(ing, duration)
-- print("StatusObject:status_onWorkStart "..self.name.." (from "..ing.name..")")
	self:statusUpdate("workstart", nil, ing, duration)
end

function StatusObject:status_onWorkFinish(ing, success)
-- print("StatusObject:status_onWorkFinish "..self.name.." (from "..ing.name..")")
	self:statusUpdate("workfinish", nil, ing)
end

--function StatusObject:status_onUpdateStatusFraction(ing, frac)
--end

function StatusObject:status_onTableEmpty(table)
-- print("StatusObject:status_onTableEmpty "..self.name.." (from "..table.name..")")
	self:statusUpdate("tableEmpty", nil, table)
end

function StatusObject:status_onTableOccupied(table)
-- print("StatusObject:status_onTableOccupied "..self.name.." (from "..table.name..")")
	self:statusUpdate("tableOccupied", nil, table)
end

function StatusObject:status_onTableBusy(table, duration)
-- print("StatusObject:status_onTableBusy "..self.name.." (from "..table.name..")")
	self:statusUpdate("tableBusy", nil, table, duration)
end

function StatusObject:status_onTableDirty(table)
-- print("StatusObject:status_onTableDirty "..self.name.." (from "..table.name..")")
	self:statusUpdate("tableDirty", nil, table)
end

function StatusObject:onWinFinishDay()
	self:statusUpdate("inactive", nil, nil)
end
