StepOver = deriveLuaClass("StepOver", "Table")

function StepOver:init(desc)
    StepOver.superInit(self, desc)

    self.animationSprite = desc.animationSprite
    self.productName1 = desc.productName1
    self.productAnimation1 = desc.productAnimation1
    self.productName2 = desc.productName2
    self.productAnimation2 = desc.productAnimation2

    self.overwriteCustomerHighlight = false
    self.oldGroup = nil
end

function StepOver:onLevelInitialized()
	StepOver.super.onLevelInitialized(self)
    self.animationSprite = level:getSpriteExt(self.animationSprite)
    if (self.animationSprite) then
        self.animationSprite.visible = false
        defaultSection = levelDesc.stepOverAnimations.section
    end
end

function StepOver:onStartEating()
    self.overwriteCustomerHighlight = false
    -- Play the animation of the object depending on the delivered product
    if (self.animationSprite) then
        self.animationSprite.visible = true
        local frameTime = self.group.stateTimer.duration
        if (self.eatTimeOverride) then
            frameTime = self.eatTimeOverride
        end
        local productName = self.group.order.allDeliveredProductNames[1]
        productName = string.sub(productName,2)
        local animationName = levelDesc.stepOverAnimations[productName]
        if (animationName) then
            self.animationSprite.baseAnimation = animationName
            frameTime = frameTime / self.animationSprite.frameCount
            self.animationSprite:playAnimation(animationName, 1, frameTime)
        end
    end
end

function StepOver:onTableStateChanged()
    self.super.onTableStateChanged(self)
    if (self.plateState == "empty" or self.plateState == "dirty") then
        if (self.animationSprite) then
            self.animationSprite.visible = false
        end
    elseif (self.plateState == "occupied") then
        self.overwriteCustomerHighlight = true
        self.oldGroup = self.group
    end
end

function StepOver:onTick(time)
    self:tick(time)

    -- Make the highlight of the customer the same that the station has,
    -- only when the overwriteHighlightOnGameplay is true
    if (self.group and self.overwriteCustomerHighlight) then
        local customers = self.group.customers
        for _,c in ipairs(customers) do
            c.overwriteHighlightOnGameplay = true
            c.highlightCurrentAlpha = self.highlightCurrentAlpha
        end
    else
        self.overwriteCustomerHighlight = false
        if (self.oldGroup) then
            -- Do this to remove this special highlight mechanic when the
            -- customer has left the station (e.g. when going to pay).
            local customers = self.oldGroup.customers
            for _,c in ipairs(customers) do
                c.overwriteHighlightOnGameplay = false
            end
            self.oldGroup = nil
        end
    end
end
