dofile("script/objects/StatusObject.lua")

-- ************************************ Stocker ************************************ --
Stocker = deriveLuaClass("Stocker", "StatusObject")

function Stocker:init(desc)
	Stocker.superInit(self, desc)
	self.doThrowJump = desc.doThrowJump ~= false

	self.stockIngredient1 = desc.stockIngredient1 or ""
	self.stockIngredient2 = desc.stockIngredient2 or ""
	self.stockIngredient3 = desc.stockIngredient3 or ""
	self.stockIngredient4 = desc.stockIngredient4 or ""

	self.sourceProductContains1 = desc.sourceProductContains1 or ""
	self.sourceProductContains2 = desc.sourceProductContains2 or ""
	self.sourceProductContains3 = desc.sourceProductContains3 or ""
	self.sourceProductContains4 = desc.sourceProductContains4 or ""

end

function Stocker:onLevelInitialized()
	--[[ PR --]] --pn("["..self.name.."] Init Stocker")

	Stocker.super.onLevelInitialized(self)

	self.stockIngredientButtons = {}
	for i=1,20 do
		local buttonName = self["stockIngredient"..i]
		local sourceProductContains = self["sourceProductContains"..i]
		if (buttonName and buttonName ~= "" and sourceProductContains and sourceProductContains ~= "") then
			local button = level:getSpriteExt(buttonName)
			if (button) then
				local item = {ing=sourceProductContains, btn=button}
				item.ings = {}
				while (true) do
					local p1 = sourceProductContains:find("#", 2)
					if (not p1) then break end
					table.push_back(item.ings, sourceProductContains:sub(1, p1-1))
					sourceProductContains = sourceProductContains:sub(p1)
				end
				table.push_back(item.ings, sourceProductContains)
				table.push_back(self.stockIngredientButtons, item)
			end
		end
	end

end

function Stocker:notify_onWorkStart(ing, task)
	local newProductName = self:getNewProductName(ing.ingredientName)

	local tray = level:getTray(task.actor)
	GH.assert(tray, "Actor '"..tostring(actor).."' does not have a tray!")

	if (not self.usedInGameplay) then
		return false
	end

	if (not self.dummyProducts) then
		if (tray and not tray:makesValidProductOrStart(task.actor, newProductName)) then
			return nil -- Let ing handle it itself
		end

	end

	ing:defaultOnWorkStart(task)
	if (tray) then
		tray:startIngredient(task.actor, task.duration, ing.compoundPosition, ing.compoundTipType, true, ing)
	end

	return true
end

function Stocker:notify_onCommit(task, ing, productName, x, y)
	--[[ PR --]] --pi("["..self.name.."] Cooker notify on commit: "..productName)
	local newProductName = self:getNewProductName(productName)
	local success = true
	for _,v in ipairs(self.stockIngredientButtons) do
		local toStockObject = v.btn
		local productNameNoSlash = string.gsub(productName, "-", "")
		local ingNameNoSlash = string.gsub(v.ing, "-", "")

		if (string.contains(productNameNoSlash, ingNameNoSlash)) then
			toStockObject:animate(
				{call=function()
					local throwPos = {x=ing.left + ing.width/2, y=ing.top + ing.height/2}
					toStockObject:throwCopy(throwPos.x, throwPos.y, level.topLayer, true)
				end},
				{delay=self.throwDuration or 0},
				{call=function() toStockObject:incDisplayStock() end}
			)

			toStockObject:incStock()
			break
		end
	end

	if (success) then
		local tray = level:getTray(task.actor)
		GH.assert(tray, "Actor '"..tostring(actor).."' does not have a tray!")
		if (tray) then
			local num = tray:getCurrentIngredientCount(task.actor)
			for i = 1,num do
				local ing = tray:getIngredientSource(task.actor, i)
				ing:defaultOnCommit(productName, x, y)
			end
			tray:clearCurrentIngredients(task.actor)
		end
	end

	return success
end

function Stocker:getNewProductName(productName)
	return productName
end

function Stocker:handleButtonCommit(idx, button, newProductName, x, y)
--print("handleButtonCommit: newProductName: "..newProductName)
	local success = (button ~= nil)
	return success
end

function Stocker:status_onStartPreparing(ing)
	--[[ PR --]] --pb("["..self.name.."] Stocker:status_onStartPreparing overlayObject="..ts(self.overlayObject and self.overlayObject.name or nil))
	if (self.overlayObject and self.overlayObject.status_onStartPreparing) then
		self.overlayObject:status_onStartPreparing(ing)
	end

	if (self.StockerButton and self.StockerButton.status_onStartPreparing) then
		self.StockerButton:status_onStartPreparing(ing)
	end

	Stocker.super.status_onStartPreparing(self, ing)
end

function Stocker:status_onBecomeAvailable(ing)
	--[[ PR --]] --pb("["..self.name.."] Stocker:status_onBecomeAvailable overlayObject="..ts(self.overlayObject and self.overlayObject.name or nil))
	if (self.overlayObject and self.overlayObject.status_onBecomeAvailable) then
		self.overlayObject:status_onBecomeAvailable(ing)
	end

	if (self.StockerButton and self.StockerButton.status_onBecomeAvailable) then
		self.StockerButton:status_onBecomeAvailable(ing)
	end

	Stocker.super.status_onBecomeAvailable(self, ing)
end

function Stocker:status_onDecay(ing)
	--[[ PR --]] --pb("["..self.name.."] Stocker:status_onDecay overlayObject="..ts(self.overlayObject and self.overlayObject.name or nil))
	if (self.overlayObject and self.overlayObject.status_onDecay) then
		self.overlayObject:status_onDecay(ing)
	end

	if (self.StockerButton and self.StockerButton.status_onDecay) then
		self.StockerButton:status_onDecay(ing)
	end

	Stocker.super.status_onDecay(self, ing)
end

function Stocker:status_onDeactivate(ing)
	--[[ PR --]] --pb("["..self.name.."] Stocker:status_onDeactivate ")
	if (self.overlayObject and self.overlayObject.status_onDeactivate) then
		--[[ PR --]] --pd("-> overlayObject = " .. ts(self.overlayObject and self.overlayObject.name or nil))
		self.overlayObject:status_onDeactivate(ing)
	end

	if (self.StockerButton and self.StockerButton.status_onDeactivate) then
		self.StockerButton:status_onDeactivate(ing)
	end

	Stocker.super.status_onDeactivate(self, ing)
end

function Stocker:notify_getIngredientNameForHints(ing)
	--[[ PR --]] --pb("["..self.name.."] Stocker Notify get ingredient name for hints (from '"..ing.name.."')")
 --return self:getNewProductName(ing.ingredientName)
	return ing.ingredientName
end

function Stocker:getObjectThatCanMakeIngredient(ingredientName, unfinishedQueuedProducts)
	for _,v in ipairs(self.stockIngredientButtons) do
		if (string.contains(ingredientName, v.btn.ingredientName)) then
			for _,ing in ipairs(v.ings) do
				local obj = level:getObjectInLevelThatCanMakeIngredient(ing, v.ing, unfinishedQueuedProducts)
--print("Stocker:getObjectThatCanMakeIngredient: "..ing.." prod: "..v.ing.." name: "..self.name.." obj: "..ts(obj))
				if (obj and (obj:blocksUseChain() or not unfinishedQueuedProducts or not table.contains(unfinishedQueuedProducts, obj.ingredientName))) then
--print("FOUND: "..ts(obj.name))
					return obj
				end
			end
		end
	end
end

function Stocker:notify_getObjectThatCanMakeIngredient(ing)
	return true
end
