-- ************************************ Table ************************************ --
Table = getClass("Table")

function Table:init(desc)
	Table.superInit(self, desc)
	self.statusObjects = desc.statusObjects or {}
end

function Table:onLevelInitialized()
	self:cpp_onLevelInitialized()
	Table.super.onLevelInitialized(self)

	for i=1,10 do
		local so = self["statusObject"..i]
		if (isString(so)) then
			so = level:getSpriteExt(so)
			if (so) then
				table.push_back(self.statusObjects, so)
			end
		end
	end
end

function Table:updateStatusObjects(frac)
	for _,v in ipairs(self.statusObjects) do
		if (v.status_onUpdateStatusFraction) then
			v:status_onUpdateStatusFraction(self, frac)
		end
	end
end

function Table:onActivate()
	--[[ PR --]] --pi("["..self.name.."] Becomes activated: "..ts(self.ingredientName))
	if (self.statusObjects) then
		for _,v in ipairs(self.statusObjects) do
			if (v.status_onActivate) then
				--[[ PR --]] --pr("("..v.name..")")
				v:status_onActivate(self)
			end
		end
	end
	--[[ PR --]] --pd()
end

function Table:onDeactivate()
	--[[ PR --]] --pi("["..self.name.."] Becomes deactivated: "..ts(self.ingredientName))
	if (self.statusObjects) then
		for _,v in ipairs(self.statusObjects) do
			if (v.status_onDeactivate) then
				--[[ PR --]] --pr("("..v.name..")")
				v:status_onDeactivate(self)
			end
		end
	end
	--[[ PR --]] --pd()
end

function Table:onTableStateChanged()
	local plateState = self.plateState
	--[[ PR --]] --pi("["..self.name.."] onTableStateChanged: "..ts(plateState))
	if (self.statusObjects) then
		for _,v in ipairs(self.statusObjects) do
			if (plateState == "empty" and v.status_onTableEmpty) then
				v:status_onTableEmpty(self)
			elseif (plateState == "occupied" and v.status_onTableOccupied) then
				v:status_onTableOccupied(self)
			elseif (plateState == "busy" and v.status_onTableBusy) then
				v:status_onTableBusy(self, self.group.stateTimer.duration)
			elseif (plateState == "dirty" and v.status_onTableDirty) then
				v:status_onTableDirty(self)
			end
		end
	end
	--[[ PR --]] --pd()
end

function Table:getWorkCharacter()
	if (level.entertainer and level.entertainer.selected) then
		return level.entertainer
	--elseif (empat and empat.selected) then
	--		return empat.main
	end

	local workCharacter = Table.super.getWorkCharacter(self)
	if (self.plateState == "dirty") then
		if (workCharacter and workCharacter.helperType == "cleaner") then
			return workCharacter
		end

		return level.cleaner or hero
	end
	return workCharacter
end

--function Table:addWorkTaskFor(workCharacter)
--	if (empat and workCharacter == empat.main) then
--		if (self.group) then
--			self.group:cancelWalkToTable()
--		end
--		empat.table = self
--		empat.state = "WalkingToTable"
--	else
--		Table.super.addWorkTaskFor(self, workCharacter)
--	end
--end

function Table:makeReservationWith(tray, product)
	self.hasReservation = true

	local reservationImage = self.reservationImage
	if (not reservationImage) then
		reservationImage = "button#reservation"
	end
	self.reservationSign = self.reservationSign or self:newChild({
		class="IngredientButton",
		baseAnimation=reservationImage,
		defaultSection=level.iconSection,
		location=self.reservationPos,
		source=product.source,
		removeTrayHandler=product.source.removeTrayHandler,
		deactivateOnWorkFinish=true,
		workPos=self.workPos,
		defaultOnWorkFinish = function(me, task, success)
			IngredientButton.defaultOnWorkFinish(me, task, success)
			if (success) then
				self.hasReservation = false
			end
		end,
		returnToStock = function(me)
			if (me.remove) then
				level:onRemoveProductFromTray(nil, me)
			end
		end,
	})
	self.reservationSign.state = "available"
	--self.reservationSign.clickable = false

	Tray.removeProductsToTarget(tray, {product}, self.reservationSign)

	local pos = self.floaterPosition
	level:createGenericFloater("RESERVATION_MADE", pos.x, pos.y)
end

function Table:notify_onCommit(task, ing, productName, x, y)
	local group = self.group
	if (ing and group) then
		if (group:deliverProductsFromTray(task, productName)) then
			ing:defaultOnCommit(productName, x, y)
		end
	end
	return true
end
