-- ************************************ Tree ************************************ --
Tree = deriveLuaClass("Tree", "IngredientButton")

function Tree:init(desc)
	Tree.superInit(self, desc)
	self.regrowDelayMin = desc.regrowDelayMin or 0
	self.regrowDelayMax = desc.regrowDelayMax or 0
	self.regrowDelay = desc.regrowDelay or 0
	self.rotationMin = desc.rotationMin or 0
	self.rotationMax = desc.rotationMax or 0
	self.startFruitAvailable = desc.startFruitAvailable ~= false -- Default true
	self.pickIndividual = desc.pickIndividual or false -- Default FALSE
	self.doRepositionFruits = desc.doRepositionFruits ~= false -- Default true
	self.relativeToTree = desc.relativeToTree ~= false -- Default true

	self.useLargeClickArea = false
	self.clickable = true
end

function Tree:onLevelInitialized()
	--[[ PR --]] --pi("Tree level initialized")
	Tree.super.onLevelInitialized(self)

	self.fruits = {}
	for i=1,20 do
		local fruit = self["fruit"..i]
		if (fruit) then
			fruit = level:getSpriteExt(fruit)
			--[[ PR --]] --pb("fruit"..ts(i)..": "..self["fruit"..i].." = "..ts(fruit)..",")
			if (fruit) then
				fruit:setIngredientState("inactive", false)
				table.push_back(self.fruits, fruit)
			end
		end
	end

	if (not self.growPositions or #self.growPositions < #self.fruits) then
		self.growPositions = self.growPositions or {}
		for _,v in ipairs(self.fruits) do
			table.push_back(self.growPositions, v.location)
		end
	end

	if (self.relativeToTree) then
		for _,v in ipairs(self.fruits) do
			v:becomeChildOf(self)
		end
	end

	if (table.empty(self.fruits)) then
		--[[ PR --]] --pr("Empty tree.")
		self.clickable = false
	end

	if (self.startFruitAvailable) then
		for _,v in ipairs(self.fruits) do
			self:showFruit(v)
			v:setIngredientState("available", false)
		end
	end
	self:resetRegrowDelay()
	--[[ PR --]] --pd()
end

function Tree:resetRegrowDelay()
	self.regrowDelay = math.Random(self.regrowDelayMin, self.regrowDelayMax)
end

function Tree:hasActiveFruit()
	for _,v in ipairs(self.fruits) do
		if (v.state == "available" or v.state == "prepare") then
			return true
		end
	end
	return false
end

--[[
function Tree:getNewFruitPos()
	local selfPos = {x=0, y=0}
	if (self.relativeToTree) then
		selfPos = self.screenLocation
	end

	local pos = {x=0, y=0}
	local maxDist = 0
	if (self.growPositions and #self.growPositions > 0) then
		local num = #self.growPositions
		local idxOffset = math.Random(num)
		for i=1,num do
			local loc = self.growPositions[((i + idxOffset) % num) + 1]
			local minDist
			for _,v in ipairs(self.fruits) do
				if (v.state ~= "inactive") then
					local dist = {x=v.x - (loc.x-selfPos.x), y=v.y - (loc.y-selfPos.y)}
					dist = dist.x*dist.x + dist.y*dist.y
					if (not minDist or dist < minDist) then
						minDist = dist
					end
				end
			end

			if (not minDist) then
				return loc
			end
			if (minDist > maxDist) then
				maxDist = minDist
				pos = loc
			end
			i = i + 1
		end
	end
	return pos
end
]]

function Tree:getNewFruitPosIdx()
	if (self.growPositions and #self.growPositions > 0) then
		local num = #self.growPositions
		local idxOffset = math.Random(num)
		for i=1,num do
			local idx = ((i + idxOffset) % num) + 1
			local found = true
			for _,v in ipairs(self.fruits) do
				if (v.state ~= "inactive" and v.treeIdx == idx) then
					found = false
					break
				end
			end

			if (found) then
				return idx
			end
		end
	end
	return -1
end

function Tree:showFruit(fruit)
	if (self.doRepositionFruits) then
		local newPosIdx = self:getNewFruitPosIdx()
		if (newPosIdx == -1) then
			return
		end
		fruit.treeIdx = newPosIdx
		local newPos = self.growPositions[newPosIdx]
		local wp = fruit.workPos
		local loc = fruit.location
		local workOffset = {x=wp.x - loc.x, y=wp.y - loc.y}
		if (self.relativeToTree) then
			loc = {x=newPos.x - self.worldX, y=newPos.y - self.worldY}
		else
			loc = newPos
		end
		fruit.location = loc
		fruit.workPos = {x=loc.x + workOffset.x, y=loc.y + workOffset.y}
	end
	fruit.rotation = math.Random(self.rotationMin, self.rotationMax)
	fruit.flipGrowth = self.allowFlipFruit and math.Random(1000) < 500
	fruit.scaleX = self.allowFlipFruit and ((fruit.flipGrowth and -1) or 1) or fruit.scaleX
end

function Tree:growFruit(fruit)
	self:showFruit(fruit)
	fruit.state = "prepare"
	self:resetRegrowDelay()
	if (not self.allowSimultaneous) then
		self.regrowDelay = self.regrowDelay + fruit.prepareDuration
	end
end

function Tree:onTick(time)
	self:tick(time)
	if (level and level.needsToAddLevelTime and self.fruits) then
		local offset = math.Random(#self.fruits)
		local count = 0;
		for i = 1,#self.fruits do
			local v = self.fruits[((i + offset) % #self.fruits) + 1]
			if (v.state == "available") then
				count = count+1
				if (count == #self.fruits) then
					self:resetRegrowDelay()
				end
			end
			if (v.state == "inactive") then
				self.regrowDelay = self.regrowDelay - time
				if (self.regrowDelay <= 0) then
					self:growFruit(v)
				end
				break
			end
		end
	end
end

function Tree:notify_onClick(ing)
	if self.fruits then
		for _,v in ipairs(self.fruits) do
			v.clickable = false
		end
	end
	if (not self.pickIndividual) then
		-- Catch the click and handle it yourself
		self:onClick()
		return true
	end
end

function Tree:allowWork(actor)
	local notifyObject = self:getNotifyObject(actor)
	--[[ PR --]] --pb("Tree allow work? "..tostring(isTable(notifyObject)))
	return isTable(notifyObject)
end

function Tree:allowIngredientWork(actor)
	return true
end

function Tree:onWorkStart(task)
	self.todoFruits = {}

	local totalFruits = 0
	local availableFruits = 0

	for _,v in ipairs(self.fruits) do
		totalFruits = totalFruits + 1
		if (v.state == "available") then
			table.push_back(self.todoFruits, v)
			availableFruits = availableFruits + 1
		end
	end

	-- Check fruit challenge fail
	local challengeManager = level.challengeManager
	if (challengeManager) then
		challengeManager:onGrowProductsPicked(availableFruits, totalFruits)
	end

	if (table.empty(self.todoFruits)) then
		self:defaultOnWorkFinish(task, false)
		if self.fruits then
			for _,v in ipairs(self.fruits) do
				v.clickable = true
			end
		end
		return false
	end

	return Tree.super.onWorkStart(self, task)
end

function Tree:defaultOnWorkStart(task)
	Tree.super.defaultOnWorkStart(self, task)
	task.duration = task.duration * #self.todoFruits
end

function Tree:onWorkFinish(task,success)
	self:defaultOnWorkFinish(task, success)
end

function Tree:onWorkCancel(task)
	if self.fruits then
		for _,v in ipairs(self.fruits) do
			v.clickable = true
		end
	end
end

function Tree:commitFruits(task, num)
	--[[ PR --]] --pi("["..self.name.."] Commit fruits:")
	local notifyObject = self:getNotifyObject(task.actor)
	if (notifyObject) then
		--[[ PR --]] --pr("-> notifyObject")
		for i=1,num do
			local v = self.todoFruits[i]
			if (v and v.state == "available") then
				--[[ PR --]] --pr("#"..ts(i))
				--print ("v.workSoundId: "..ts(v.workSoundId))
				if (v.workSoundId) then
					v:playSample(v.workSoundId)
				end
				local pos = v.screenCenter
				v.flipGrowth = false
				v.scaleX = math.abs(v.scaleX)
				notifyObject:notify_onCommit(task, v, v.ingredientName, pos.x, pos.y)
			end
		end
	end
	--[[ PR --]] --pd()
end

function Tree:onWorkUpdate(task, time)
	--[[ PR --]] --pb("Tree work update")
	self:commitFruits(task, #self.todoFruits * task.timer.fraction)
	return Tree.super.onWorkUpdate(self, task, time)
end

function Tree:onCommit(task, productName, x, y)
	--[[ PR --]] --pi("Tree on commit")
	self:commitFruits(task, #self.todoFruits)
	--[[ PR --]] --pd()
end

function Tree:getObjectThatCanMakeIngredient(ingredientName, unfinishedQueuedProducts)
	if (self.pickIndividual) then
		local prepareButton
		for _,fruit in ipairs(self.fruits) do
			local state = fruit.state
			if (state == "available") then
				if (fruit.ingredientName == ingredientName) then
					return fruit
				end
			elseif (state == "prepare") then
				if (fruit.ingredientName == ingredientName) then
					prepareButton = fruit
				end
			end
		end
		if (prepareButton) then
			return prepareButton
		end
	end
end
