-- ************************************ TriggerObject ************************************ --
TriggerObject = deriveLuaClass("TriggerObject", "SpriteExt")

function TriggerObject:init(desc)
	TriggerObject.superInit(self, desc)

	self.triggerPrefix = desc.triggerPrefix or "trigger"
	self.active = desc.active ~= false
	self.charactersInside = {}
end

function TriggerObject:onLevelInitialized()
	TriggerObject.super.onLevelInitialized(self)
end

function TriggerObject:onTick(time)
	self:tick(time)

	if (self.active ~= false and level) then
		local previousCharactersInside = self.charactersInside
		self.charactersInside = {}

		local characters = level:getCharacters()
		local left = self.left
		local top = self.top
		local width = self.width
		local height = self.height

		for _,character in ipairs(characters) do
			local cx, cy = character.x - left, character.y - top
			if (cx >= 0 and cy >= 0 and cx < width and cy < height) then
				self.charactersInside[character.name] = character
			end
		end

		--' Check for new characters
		for name,character in pairs(self.charactersInside) do
			if (not previousCharactersInside[enterCharacterName]) then
				self:fireTrigger("onEnter", name, character)
			end
		end

		--' Check for characters gone
		for name,character in pairs(previousCharactersInside) do
			if (not self.charactersInside[leaveCharacterName]) then
				self:fireTrigger("onLeave", name, character)
			end
		end

		if (table.empty(previousCharactersInside) and not table.empty(self.charactersInside)) then
			self:fireTrigger("onFirstEnter")
		end

		if (not table.empty(previousCharactersInside) and table.empty(self.charactersInside)) then
			self:fireTrigger("onLastLeave")
		end
	end
end

function TriggerObject:fireTrigger(postfix, characterName, character)

	for i=1,20 do
		local objName = self["objectToTrigger"..i]
		if (objName) then
			level:fireTrigger("object", {type=self.triggerPrefix.."_"..postfix, name=objName, triggerObject=self, characterName=characterName, character=character})
		end
	end

	if (self.fireToLevel) then
--print("fireTrigger: "..self.triggerPrefix.."_"..postfix)
		level:fireTrigger(self.triggerPrefix.."_"..postfix, {triggerObject=self, characterName=characterName, character=character})
	end
end

function TriggerObject:getNumCharactersInside()
	return table.count(self.charactersInside)
end