-- ***************************** WishFountain ****************************** --

-- This object can be the same as the graphical part only if there is no
-- graphical update in the wish fountain. If not, it should be an empty
-- object

WishFountain = deriveLuaClass("WishFountain", "Object")

-- WishFountainStates
-- 0 -> Nothing
-- 1 -> Waiting for coin animation

function WishFountain:init(desc)
    WishFountain.superInit(self, desc)
    self.reactionBar = nil
    self.totalTime = 0

    self.STATE_NOTHING = 0
    self.STATE_THROW_WAIT = 3
    self.STATE_COIN_FLY = 1
    self.STATE_COIN_LAND = 2

    self.animationState = self.STATE_NOTHING

    self.finalCoinTime = 0
    self.callIncreaseMood = false
    self.clickState = 0
    self.visible = false
end

function WishFountain:onTick(time)
    self:tick(time)
    self.totalTime = self.totalTime + time

    if (self.animationState == self.STATE_THROW_WAIT) then
        if (self.totalTime > self.station.coinWaitTime) then
            self:throwCoin()
        end
    elseif (self.animationState == self.STATE_COIN_FLY) then
        if (self.totalTime > self.finalCoinTime) then
            local animName
            if (self.clickState == 1) then
                animName = self.station.customerHighAnimation
            elseif (self.clickState == 2) then
                self:increaseMood()
                self.visible = true
                self:playAnimation(self.baseAnimation, self.defaultCycles)
                animName = self.station.customerMediumAnimation
            elseif (self.clickState == 3) then
                animName = self.station.customerLowAnimation
            end
            self.animationState = self.STATE_COIN_LAND
            self.totalTime = 0
            self.finalCoinTime = self.station.coinLandTime
            if (animName and self.station.group) then
                self.station.group:playAnimation(animName, 1)
            end
            -- Do the splash animationState
            self:PlaySplashAnimation()
        end
    elseif (self.animationState == self.STATE_COIN_LAND) then
        if (self.totalTime > self.finalCoinTime) then
            self:doneEating()
            self.visible = false
            self.animationState = self.STATE_NOTHING
            self.reactionBar:hide()
        end
    end

    -- Make the highlight of the customer the same that the station has
    if (self.station) then
        if (self.station.group) then
            local customers = self.station.group.customers
            for _,c in ipairs(customers) do
                c.highlightCurrentAlpha = self.station.highlightCurrentAlpha
            end
        end
    end
end

function WishFountain:onClick(desc)
    -- Check the value of the ReactionBar
    if (self.reactionBar) then
        self.callIncreaseMood = false
        if (self.station.group) then
            -- Stop the eating sound, because the arrow has stopped moving
            self.station.group:stopEatingSound()
            -- Using the state None the customer does not lose patience.
            self.station.group.state = "None"
        end
        self.station:stopClickOverwrite()

        -- Make the characters play the throw animation
        local customers = self.station.group.customers
        for _,c in ipairs(customers) do
            c:playAnimation(self.station.customerThrowAnimation,1)
        end

        self.animationState = self.STATE_THROW_WAIT
        self.totalTime = 0
        self.reactionBar:stop()
    end
end

function WishFountain:throwCoin()
    -- Check the value of the ReactionBar
    if (self.reactionBar) then
        self.callIncreaseMood = false
        self.useTail = false
        if (self.reactionBar:isInsideFirstArea()) then
            -- Too much
            self.finalCoinTime = self.station.coinHighTime
            self.x = self.station.coinHighTarget.x
            self.y = self.station.coinHighTarget.y
            self.coinSound = 1
            self.clickState = 1
        elseif (self.reactionBar:isInsideSecondArea()) then
            -- Perfect
            self.finalCoinTime = self.station.coinMediumTime
            self.callIncreaseMood = true

            self.x = self.station.coinMediumTarget.x
            self.y = self.station.coinMediumTarget.y
            self.useTail = true
            self.coinSound = 2
            self.clickState = 2

            -- Create the +X floater for the WishFountain
            level:createFloater({
                id = "eventDone",
                x = self.station.coinSpawn.x - 30, -- Added this displacement to adjust it better
                y = self.station.coinSpawn.y + 70, -- Same as above
                bonusScore = self.station.mediumScoreBonus,
                bonus = {
                    textId = "EVENT_SCORE_ONE_DONE_FMT",
                },
            })
            if (level.BM_eventScore) then level:BM_eventScore(score) end
        else --if (self.reactionBar:isInsideThirdArea()) then
            -- Too low
            self.finalCoinTime = self.station.coinLowTime
            self.x = self.station.coinLowTarget.x
            self.y = self.station.coinLowTarget.y
            self.coinSound = 3
            self.clickState = 3
        end
        -- Move the splash effect to its final position
        if (self.splashEffect) then
            self.splashEffect.x = self.x
            self.splashEffect.y = self.y
        end
        -- Call the function that makes the coin appear
        local stringParams = {
            origX=self.station.coinSpawn.x,
            origY=self.station.coinSpawn.y,
            duration=self.finalCoinTime,
            height=self.station.coinHeightFactor,
            useTail=self.useTail,
            coinSound=self.coinSound
        }
        self.station.group:createCoinWithTrail(self, stringParams)
        self.station:stopClickOverwrite()

        self.animationState = self.STATE_COIN_FLY
        self.totalTime = 0
    end
end

function WishFountain:increaseMood()
    -- This function sets the mood to the maximum and shows the floating
    -- heart. It is used only when the power of the coin is MEDIUM.
    if (self.station.group) then
        self.station.group.mood = 4 -- the max is 4, not 5!
        self.station.group:resetMoodDropTimer()
        self.station.group:createFloatingHeart()
    end
end

function WishFountain:doneEating()
    self.station:doneEating()
end

function WishFountain:PlaySplashAnimation()
    if ((self.coinSound == 1 and self.station.coinSplashLow == true)
        or (self.coinSound == 2 and self.station.coinSplashMedium == true)
        or (self.coinSound == 3 and self.station.coinSplashHigh == true)) then
        self.splashEffect.visible = true
        self.splashEffect:playAnimation(self.splashEffect.baseAnimation, 1)
    end
end

function WishFountain:doCoinAnimation(duration)

    self.finalCoinTime = self.station.coinMediumTime
    self.callIncreaseMood = true
    self.useTail = true
    self.coinSound = 2
    self.clickState = 2

    -- Mov self to the target of the coin
    self.x = self.station.coinMediumTarget.x
    self.y = self.station.coinMediumTarget.y
    -- Move the splash effect to its final position
    if (self.splashEffect) then
        self.splashEffect.x = self.x
        self.splashEffect.y = self.y
    end
    -- Call the function that makes the coin appear
    local stringParams = {
        origX=self.station.coinSpawn.x,
        origY=self.station.coinSpawn.y,
        duration=self.finalCoinTime,
        height=self.station.coinHeightFactor,
        useTail=true,
        coinSound=2
    }
    self.station:doCoinAnimation(self, stringParams)

    self.animationState = self.STATE_COIN_FLY
    self.totalTime = 0
end
