-- ************************* Wish Fountain Station ************************* --

WishFountainStation = deriveLuaClass("WishFountainStation", "Table")

function WishFountainStation:init(desc)
    WishFountainStation.superInit(self,desc)

    self.wishFountainObject = desc.wishFountainObject
    self.customerThrowAnimation = desc.customerThrowAnimation or "throw_coin"
    self.customerLowAnimation = desc.customerLowAnimation
    self.customerMediumAnimation = desc.customerMediumAnimation
    self.customerHighAnimation = desc.customerHighAnimation

    self.coinSpawn = desc.coinSpawn
    self.coinLowTarget = desc.coinLowTarget
    self.coinLowTime = desc.coinLowTime or 1500
    self.coinMediumTarget = desc.coinMediumTarget
    self.coinMediumTime = desc.coinMediumTime or 1500
    self.coinHighTarget = desc.coinHighTarget
    self.coinHighTime = desc.coinHighTime or 1500
    self.coinHeightFactor = desc.coinHeightFactor or 50
    self.coinWaitTime = desc.coinWaitTime or 500
    self.coinLandTime = desc.coinLandTime or 1000

    self.coinSplashLow = desc.coinSplashLow or false
    self.coinSplashMedium = desc.coinSplashMedium ~= false
    self.coinSplashHigh = desc.coinSplashHigh or false

    self.mediumScoreBonus = desc.mediumScoreBonus or 5

    --Correct the positions using self position
    self.coinSpawn.x = self.coinSpawn.x + self.x
    self.coinSpawn.y = self.coinSpawn.y + self.y
    self.coinLowTarget.x = self.coinLowTarget.x + self.x
    self.coinLowTarget.y = self.coinLowTarget.y + self.y
    self.coinMediumTarget.x = self.coinMediumTarget.x + self.x
    self.coinMediumTarget.y = self.coinMediumTarget.y + self.y
    self.coinHighTarget.x = self.coinHighTarget.x + self.x
    self.coinHighTarget.y = self.coinHighTarget.y + self.y

    self.eatingState = false
    self.randomPos = false
end

function WishFountainStation:onLevelInitialized()
    WishFountainStation.super.onLevelInitialized(self)

    -- Add the wish fountain element
    self.wishFountain = level:getObject(self.wishFountainObject)

    self.wishFountain.station = self
    self.wishFountain.reactionBar = level:getSpriteExt("reaction_bar")
    self.wishFountain.reactionBar.reactionObject = self.wishFountain
    self.wishFountain.reactionBar:setArrow(level:getSpriteExt("reaction_bar_arrow"))

    self.wishFountain.splashEffect = level:getSpriteExt("coin_splash")
end

function WishFountainStation:onStartEating()
    -- We create this new state called CoinAiming. This way the TableQueue
    -- decreases the mood because it's not one of the states inside the
    -- isMoodDropping function
    self.group.state = "CoinAiming"
    -- Setting the group duration to 999999 means the customer won't leave the
    -- station because he "finishes" eating
    self.group.stateTimer.duration = 999999
    self.wishFountain.reactionBar:start(self.randomPos)
    self.randomPos = true
    -- Show the tip the first time the WishFountain is shown
    if (WishFountain.showHint ~= nil) then
        WishFountain:showHint({subject=self.wishFountain.reactionBar, id="showBar"})
    end
    self.clickOverwrite = true
    self.eatingState = true
end

function WishFountainStation:onTableStateChanged()
    self.super.onTableStateChanged(self)

    if (self.plateState == "occupied" and self.group) then
        -- If the state is occupied, set the MoodEverDrop to true, so the mood
        -- of the customer does not increase when we deliver him/her an order.
        self.group:setMoodEverDropped(true)
        --self.additionalHighlightObject = self.group.customers[1]
        --self.group.customers[1].doHighlight = true
    elseif (self.plateState == "empty") then
        -- If state is empty it's because the customer has left so hide the
        -- reaction bar
        self.clickOverwrite = false
        self.eatingState = false
        self.wishFountain.reactionBar:stop()
        self.wishFountain.reactionBar:hide()
    end
end

function WishFountainStation:doneEating()
    self.eatingState = false
    if (self.group) then
        self.group:onDoneEating()
    end
end

function WishFountainStation:stopClickOverwrite()
    self.clickOverwrite = false
end

function WishFountainStation:onMouseDown()
    if (self.clickOverwrite == true) then
        self.wishFountain:onClick();
    end
end

function WishFountainStation:onClick()
    -- If we are eating, overwrite the normal onClick so the hero does not
    -- go there when clicking on the customer/station
    if (self.eatingState == false) then
        self.super.onClick(self)
    end
end
